-------------------------------------------------------------------------------------------
-- Copyright © 2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    10:14:57 03/22/2017 
-- Design Name: 
-- Module Name:    ads7947core - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ads7947core is
	generic
	(
		C_CLK_FREQ_HZ  : integer := 100_000_000;
		C_ACQTIME_NS   : integer := 3200
	);
	port 
	( 
		clock  : in  std_logic;
		reset  : in  std_logic;
		sdo    : in  std_logic_vector(3 downto 0);
		sclk   : out std_logic;
		cs     : out std_logic;
		ch_sel : out std_logic;
		pdo00  : out std_logic_vector(11 downto 0);
		pdo01  : out std_logic_vector(11 downto 0);
		pdo10  : out std_logic_vector(11 downto 0);
		pdo11  : out std_logic_vector(11 downto 0);
		pdo20  : out std_logic_vector(11 downto 0);
		pdo21  : out std_logic_vector(11 downto 0);
		pdo30  : out std_logic_vector(11 downto 0);
		pdo31  : out std_logic_vector(11 downto 0);
		finish : out std_logic
	);
end ads7947core;

architecture Behavioral of ads7947core is

    function lcpceil(v : real) return integer is
        variable t : integer;
    begin
        t := integer(v);
        if (real(t) < v) then
            t := t + 1;
        end if;
        return t;
    end function;
    
    function int2bits(v : integer) return integer is
        variable b : integer := 1;
        variable n : integer := 2;
    begin
        while(v >= n) loop
            b := b + 1;
            n := n * 2;
        end loop;
        return b;
    end function;

    -- frequence range is 0.4 to 40 Mhz
    constant C_SCLK_FREQ_HZ  : integer := 12_500_000;
    
    constant C_SCLKCNT_MAX   : integer := integer(real(C_CLK_FREQ_HZ) / real(C_SCLK_FREQ_HZ) / 2.0 - 1.0);
    constant C_SCLKCNT_WIDTH : integer := int2bits(C_SCLKCNT_MAX);
    
    constant K_SCLKCNT_MAX   : std_logic_vector(C_SCLKCNT_WIDTH - 1 downto 0) := std_logic_vector(to_unsigned(C_SCLKCNT_MAX,C_SCLKCNT_WIDTH));
    constant K_SCLKCNT_0     : std_logic_vector(C_SCLKCNT_WIDTH - 1 downto 0) := std_logic_vector(to_unsigned(0,C_SCLKCNT_WIDTH));
    signal   sclkcnt         : std_logic_vector(C_SCLKCNT_WIDTH - 1 downto 0) := K_SCLKCNT_0;
    signal   sclk_ev         : std_logic;
    
    -- clocks per acquistion, 16 active + 1 idle
    constant C_BITSACQ       : integer := 17; -- tconv+tacq sclk equivalent (16+1)
    constant C_BITCNT_MAX    : integer := C_BITSACQ - 1;
    signal   bitcnt          : natural range 0 to C_BITCNT_MAX := 0;
    
    -- acqisition cycle
    constant C_SYNCNT_MAX    : integer := lcpceil(real(C_ACQTIME_NS) * 1.0e-9 * real(C_CLK_FREQ_HZ)) - 1;
    signal   syncnt          : natural range 0 to C_SYNCNT_MAX := 0;
    signal   syn_ev          : std_logic;

	type     stm_t is        (sW4SYN,sSHIFT,sTAIL0,sTAIL1);
	signal   stm             : stm_t := sW4SYN;


	-- parked data
	signal tempval_0 : std_logic_vector(11 downto 0) := (others => '0');
	signal tempval_1 : std_logic_vector(11 downto 0) := (others => '0');
	signal tempval_2 : std_logic_vector(11 downto 0) := (others => '0');
	signal tempval_3 : std_logic_vector(11 downto 0) := (others => '0');

	-- data-in shift registers
	signal shiftin_0 : std_logic_vector(11 downto 0) := (others => '0');
	signal shiftin_1 : std_logic_vector(11 downto 0) := (others => '0');
	signal shiftin_2 : std_logic_vector(11 downto 0) := (others => '0');
	signal shiftin_3 : std_logic_vector(11 downto 0) := (others => '0');

	-- output values

	signal sclk_i    : std_logic := '0';
	signal cs_i      : std_logic := '1';
	signal ch_sel_i  : std_logic := '0';
	signal pdo00_i   : std_logic_vector(11 downto 0) := (others => '0');
	signal pdo01_i   : std_logic_vector(11 downto 0) := (others => '0');
	signal pdo10_i   : std_logic_vector(11 downto 0) := (others => '0');
	signal pdo11_i   : std_logic_vector(11 downto 0) := (others => '0');
	signal pdo20_i   : std_logic_vector(11 downto 0) := (others => '0');
	signal pdo21_i   : std_logic_vector(11 downto 0) := (others => '0');
	signal pdo30_i   : std_logic_vector(11 downto 0) := (others => '0');
	signal pdo31_i   : std_logic_vector(11 downto 0) := (others => '0');
	signal finish_i  : std_logic := '0';


begin

	-- fwd output

	sclk   <= sclk_i;
	cs     <= cs_i;
	ch_sel <= ch_sel_i;
	pdo00  <= pdo00_i;
	pdo01  <= pdo01_i;
	pdo10  <= pdo10_i;
	pdo11  <= pdo11_i;
	pdo20  <= pdo20_i;
	pdo21  <= pdo21_i;
	pdo30  <= pdo30_i;
	pdo31  <= pdo31_i;
	finish <= finish_i;
	
	-- change channel mux
	
	-- CH SEL after first SCLK rising edge and before the second SCLK rising edge
	-- the middle position is the first SCLK falling edge
	chan_proc : process(clock)
    begin
        if rising_edge(clock) then
            if sclk_ev = '1' and sclk_i = '1' and bitcnt = 1 then
                ch_sel_i <= not ch_sel_i;
            end if;
        end if;
    end process;
    
    -- end of acq, transfer
    
	load_proc : process(clock)
	   variable ev : std_logic;
    begin
        if rising_edge(clock) then
            ev := '0';
            if sclk_ev = '1' and sclk_i = '0' and bitcnt = 12 then
                if ch_sel_i = '1' then
                    pdo00_i   <= tempval_0;
                    pdo10_i   <= tempval_1;
                    pdo20_i   <= tempval_2;
                    pdo30_i   <= tempval_3;
                    pdo01_i   <= shiftin_0;
                    pdo11_i   <= shiftin_1;
                    pdo21_i   <= shiftin_2;
                    pdo31_i   <= shiftin_3;
                    ev        := '1';
                else
                    tempval_0 <= shiftin_0;
                    tempval_1 <= shiftin_1;
                    tempval_2 <= shiftin_2;
                    tempval_3 <= shiftin_3;
                end if;
            end if;
            finish_i <= ev;
        end if;
    end process;
    
    -- shift input
    
	shin_proc : process(clock)
    begin
        if rising_edge(clock) then
            if sclk_ev = '1' and sclk_i = '1' then
                shiftin_0 <= shiftin_0(10 downto 0) & sdo(0);
                shiftin_1 <= shiftin_1(10 downto 0) & sdo(1);
                shiftin_2 <= shiftin_2(10 downto 0) & sdo(2);
                shiftin_3 <= shiftin_3(10 downto 0) & sdo(3);
            end if;
        end if;
    end process;
    
    
	-- spi clock
    
    sclk_ev <= '1' when sclkcnt = K_SCLKCNT_MAX else '0';
    
    sclk_proc : process(clock)
    begin
        if rising_edge(clock) then
            if reset = '1' or sclk_ev = '1' or stm = sW4SYN then
                sclkcnt <= K_SCLKCNT_0;
            else
                sclkcnt <= std_logic_vector(unsigned(sclkcnt) + 1);
            end if;
        end if;
    end process;
    
    
	-- acq clock
    
    syn_ev <= '1' when syncnt = C_SYNCNT_MAX else '0';
    
    cycle_proc : process(clock)
    begin
        if rising_edge(clock) then
            if reset = '1' or syn_ev = '1' then
                syncnt <= 0;
            else
                syncnt <= syncnt + 1;
            end if;
        end if;
    end process;
	
	-- main process
	
	syn_ev <= '1' when syncnt = C_SYNCNT_MAX else '0';
	
    time_proc : process(clock)
    begin
        if rising_edge(clock) then
            if reset = '1' then
                sclk_i <= '0';
                cs_i   <= '1';
                bitcnt <= 0;
                stm    <= sW4SYN;
            else
                case stm is
                    when sW4SYN =>
						bitcnt <= 0;
                        if syn_ev = '1' or ch_sel_i = '1' then
							cs_i   <= '0';
							stm    <= sSHIFT;
                        end if;
                    when sSHIFT =>
                            if sclk_ev = '1' then
                                if sclk_i = '0' then
                                    if bitcnt = C_BITCNT_MAX then
                                        cs_i <= '1';
                                        stm  <= sTAIL0;
                                    else
                                        sclk_i <= '1';
                                        bitcnt <= bitcnt + 1;
                                    end if;
                                else
                                    sclk_i <= '0';
                                end if;
                            end if;
                    when sTAIL0 =>
                        if sclk_ev = '1' then
                            stm  <= sTAIL1;
                        end if;
                    when others =>
                        if sclk_ev = '1' then
                            stm  <= sW4SYN;
                        end if;
                end case;
            end if;
        end if;
    end process;
    
end Behavioral;
