/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2014, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ds1820
/// \file ds1820.c
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2014

#include "xparameters.h"
#include "xil_assert.h"

#include "ds1820.h"

/************************** Function Definitions ***************************/

ds1820_Config *ds1820_LookupConfig(u16 DeviceId)
{
	ds1820_Config *CfgPtr = NULL;
	u32 Index;

	for (Index = 0; Index < XPAR_DS1820_NUM_INSTANCES; Index++) {
		if (ds1820_ConfigTable[Index].DeviceId == DeviceId) {
			CfgPtr = &ds1820_ConfigTable[Index];
			break;
		}
	}

	return CfgPtr;
}

int ds1820_Initialize(ds1820 *InstancePtr, u16 DeviceId)
{
	ds1820_Config *ds1820_ConfigPtr;/* Pointer to Configuration data. */

	/*
	 * Verify that each of the inputs are valid.
	 */
	Xil_AssertNonvoid(InstancePtr != NULL);

	/*
	 * Lookup the device configuration in the configuration table. Use this
	 * configuration info down below when initializing this driver.
	 */
	ds1820_ConfigPtr = ds1820_LookupConfig(DeviceId);
	if (ds1820_ConfigPtr == NULL) {
		return XST_DEVICE_NOT_FOUND;
	}
	
	// save param
	InstancePtr->Config = ds1820_ConfigPtr;
	
    // driver base address
    u32 BaseAddress = ds1820_ConfigPtr->BaseAddress;
    
    // initial value
    DS1820_mWrite_control_reg(BaseAddress,0);
	
	// Process state machine
	InstancePtr->state = DS1820_STATE_INIT;

    // driver Ready
	InstancePtr->IsReady = XIL_COMPONENT_IS_READY;

	return XST_SUCCESS;
}

int ds1820_Down(u16 DeviceId)
{
	ds1820_Config *ds1820_ConfigPtr;
    
    // get configuration address
	ds1820_ConfigPtr = ds1820_LookupConfig(DeviceId);
	if (ds1820_ConfigPtr == NULL) {
		return XST_DEVICE_NOT_FOUND;
	}
        
    // driver base address
    u32 BaseAddress = ds1820_ConfigPtr->BaseAddress;
    
    // initial value
    DS1820_mWrite_control_reg(BaseAddress,0);
    
	return XST_SUCCESS;
}

/***************************************************************************/

// ===========================================================================

#define DS1820_LC_BYTE_INDEX(bix)   ((bix) >> 3)
#define DS1820_LC_BIT_MASK(bix)     (1 << ((bix) & 7))


#define DS1820_LC_SET_BIT(au8,bix)   { au8[DS1820_LC_BYTE_INDEX(bix)] |=   DS1820_LC_BIT_MASK(bix) ; }
#define DS1820_LC_CLR_BIT(au8,bix)   { au8[DS1820_LC_BYTE_INDEX(bix)] &= ~ DS1820_LC_BIT_MASK(bix) ; }

#define DS1820_LC_GET_BIT(au8,bix)   ((au8[DS1820_LC_BYTE_INDEX(bix)] & DS1820_LC_BIT_MASK(bix)) != 0)

// ===========================================================================

#define DS1820_DEFLADDR                u32 ds1820_laddr = InstancePtr->Config->BaseAddress;
#define DS1820_REFLADDR                ds1820_laddr

// ===========================================================================

#define DS1820_SET_STATE(s) { InstancePtr->state = (s); }
#define DS1820_STATE        ( InstancePtr->state )

#define DS1820_SET_COMMAND(command,argument) { InstancePtr->cmd = (command); InstancePtr->arg = (argument); }

// ===========================================================================

// CRC = X^8 + X^5 + X^4 + 1
static int ds1820_crc8_1wire_bit(const u8 crc, const u8 value)
{
	u8 tcrc = crc;
	u8 mul;
	if (tcrc & (1 << (8-1)))
		mul = value == 0;
	else
		mul = value != 0;
	tcrc <<= 1;
	if (mul)
		tcrc ^= (1 << 5) | (1 << 4) | (1 << 0);
	return tcrc;
}

// ===========================================================================

static void ds1820_stack_init(ds1820 *InstancePtr)
{
	int i;
	for(i = 0;i < DS1020_STACK_SIZE;i++)
		InstancePtr->stack[i] = DS1820_STATE_INIT;
	InstancePtr->stkpnt = 0;
}

static void ds1820_stack_push(ds1820 *InstancePtr, u8 st)
{
	InstancePtr->stack[InstancePtr->stkpnt] = st;
	InstancePtr->stkpnt = (InstancePtr->stkpnt + DS1020_STACK_SIZE + 1) % DS1020_STACK_SIZE;
}

static u8 ds1820_stack_pop(ds1820 *InstancePtr)
{
	InstancePtr->stkpnt = (InstancePtr->stkpnt + DS1020_STACK_SIZE - 1) % DS1020_STACK_SIZE;
	return InstancePtr->stack[InstancePtr->stkpnt];
}

#define ds1820_rom_rewind(InstancePtr) { InstancePtr->lastk = 0; InstancePtr->lastd = 0; }


int ds1820_Process_Run(ds1820 *InstancePtr, u8 cmd)
{
	DS1820_DEFLADDR;
	int rc = XST_SUCCESS;
	switch(cmd)
	{
		default:
		case DS1820_CMD_INIT:
			{
				DS1820_SET_STATE(DS1820_STATE_INIT);
			}
			break;
		case DS1820_CMD_RUN:
			{
				switch(DS1820_STATE)
				{
					default:
					case DS1820_STATE_INIT:
						{
							ds1820_stack_init(InstancePtr);
							DS1820_SET_STATE(DS1820_STATE_TCONV);
						}
						break;

					// Temperature Conversion
					case DS1820_STATE_TCONV:
						{
							DS1820_SET_COMMAND(DS1820_CONVERT_T,DS1820_SKIP_ROM);
							ds1820_stack_push(InstancePtr,DS1820_STATE_TCONV_STARTED);
							DS1820_SET_STATE(DS1820_STATE_COMMAND);
						}
						break;
					case DS1820_STATE_TCONV_STARTED:
						{
							DS1820_SET_STATE(DS1820_STATE_TCONV_W4END);
						}
						break;

					case DS1820_STATE_TCONV_W4END:
						{
							// w4 application
						}
						break;

					case DS1820_STATE_TCONV_DONE:
						{
							DS1820_SET_STATE(DS1820_STATE_SEARCH);
						}
						break;

					case DS1820_STATE_SEARCH:
						{
							ds1820_rom_rewind(InstancePtr);
							DS1820_SET_STATE(DS1820_STATE_SEARCH_INIT);
						}
						break;

					case DS1820_STATE_SEARCH_INIT:
						{
							ds1820_stack_push(InstancePtr,DS1820_STATE_SEARCH_ITIALIZED);
							DS1820_SET_STATE(DS1820_STATE_XINIT);
						}
						break;

					case DS1820_STATE_SEARCH_ITIALIZED:
						{
							// data=1 is "no move devices"
							if (InstancePtr->indata)
							{
								DS1820_SET_STATE(DS1820_STATE_INIT);
								rc = XST_FAILURE;
								break;
							}

							ds1820_stack_push(InstancePtr,DS1820_STATE_SEARCH_LOOP);
							InstancePtr->wribyte = DS1820_SEARCH_ROM;
							DS1820_SET_STATE(DS1820_STATE_WRITE_BYTE);

							InstancePtr->bangpos = 0xFF;
							InstancePtr->crc     = 0;
							InstancePtr->idx     = 0;
						}
						break;

					case DS1820_STATE_SEARCH_LOOP:
						{
							// reach bit counts ?
							if (InstancePtr->idx == (DS1820_IDENT_SIZE * 8))
							{
								// save conflict position
								InstancePtr->lastk = InstancePtr->bangpos;

								// device is last found
								InstancePtr->lastd = InstancePtr->bangpos == 0xFF;

							    // CRC : 0=OK, else error
							    if (InstancePtr->crc)
							    {
									DS1820_SET_STATE(DS1820_STATE_INIT);
									rc = XST_FAILURE;
									break;
							    }
						    	DS1820_SET_STATE(DS1820_STATE_SEARCH_DONE_FOUND);
							}
							else
							{
								ds1820_stack_push(InstancePtr,DS1820_STATE_SEARCH_LOOP_B0);
								DS1820_SET_STATE(DS1820_STATE_READ_BIT);
							}
						}
						break;

					case DS1820_STATE_SEARCH_LOOP_B0:
						{
							InstancePtr->b0 = InstancePtr->indata;
							ds1820_stack_push(InstancePtr,DS1820_STATE_SEARCH_LOOP_B1);
							DS1820_SET_STATE(DS1820_STATE_READ_BIT);
						}
						break;

					case DS1820_STATE_SEARCH_LOOP_B1:
						{
							// both 1 is no device
							if (InstancePtr->b0 && InstancePtr->indata)
							{
								DS1820_SET_STATE(DS1820_STATE_INIT);
								rc = XST_FAILURE;
								break;
							}

							// both 0 is conflict
							if (!InstancePtr->b0 && !InstancePtr->indata)
							{
					    		// distinguish 1st , others loops
					    		// on first loop the zeros' wins!

								// all before conflict confirm values.....
								if (InstancePtr->idx < InstancePtr->lastk)
								{
									// already processed bits will be confirmed !
									InstancePtr->b0 = DS1820_LC_GET_BIT(InstancePtr->ident,InstancePtr->idx);
								}
								else
								{
									InstancePtr->b0 = InstancePtr->idx == InstancePtr->lastk;
								}

								if (!InstancePtr->b0)
								{
									InstancePtr->bangpos = InstancePtr->idx;
								}
							}

							InstancePtr->outdata = InstancePtr->b0;
							ds1820_stack_push(InstancePtr,DS1820_STATE_SEARCH_LOOP_SAVEBIT);
							DS1820_SET_STATE(DS1820_STATE_WRITE_BIT);
						}
						break;

					case DS1820_STATE_SEARCH_LOOP_SAVEBIT:
						{
					    	if (InstancePtr->b0)
					    	{
					    		DS1820_LC_SET_BIT(InstancePtr->ident,InstancePtr->idx);
					    	}
					    	else
					    	{
					    		DS1820_LC_CLR_BIT(InstancePtr->ident,InstancePtr->idx);
					    	}

					    	InstancePtr->crc = ds1820_crc8_1wire_bit(InstancePtr->crc,InstancePtr->b0);

					    	InstancePtr->idx++;

							DS1820_SET_STATE(DS1820_STATE_SEARCH_LOOP);
						}
						break;

					case DS1820_STATE_SEARCH_DONE_FOUND:
						{
							DS1820_SET_STATE(DS1820_STATE_READ_TEMP);
						}
						break;

					case DS1820_STATE_READ_TEMP:
						{
							DS1820_SET_COMMAND(DS1820_READ,DS1820_MATCH_ROM);
							ds1820_stack_push(InstancePtr,DS1820_STATE_READ_TEMP_COMD);
							DS1820_SET_STATE(DS1820_STATE_COMMAND);
						}
						break;

					case DS1820_STATE_READ_TEMP_COMD:
						{
							InstancePtr->idx = 0;
							InstancePtr->crc = 0;
							DS1820_SET_STATE(DS1820_STATE_READ_TEMP_LOOP);
						}
						break;

					case DS1820_STATE_READ_TEMP_LOOP:
						{
							// reach bit count?
							if (InstancePtr->idx == (DS1820_SCRPD_SIZE * 8))
							{
							    // CRC : 0=OK, else error
							    if (InstancePtr->crc)
							    {
									DS1820_SET_STATE(DS1820_STATE_INIT);
									rc = XST_FAILURE;
									break;
							    }
						    	DS1820_SET_STATE(DS1820_STATE_READ_TEMP_DONE);
							}
							else
							{
								ds1820_stack_push(InstancePtr,DS1820_STATE_READ_TEMP_LOOP_BIT);
								DS1820_SET_STATE(DS1820_STATE_READ_BIT);
							}
						}
						break;

					case DS1820_STATE_READ_TEMP_LOOP_BIT:
						{
					    	if (InstancePtr->indata)
					    	{
					    		DS1820_LC_SET_BIT(InstancePtr->scrpd,InstancePtr->idx);
					    	}
					    	else
					    	{
					    		DS1820_LC_CLR_BIT(InstancePtr->scrpd,InstancePtr->idx);
					    	}

					    	InstancePtr->crc = ds1820_crc8_1wire_bit(InstancePtr->crc,InstancePtr->indata);

					    	InstancePtr->idx++;

							DS1820_SET_STATE(DS1820_STATE_READ_TEMP_LOOP);
						}
						break;

					case DS1820_STATE_READ_TEMP_DONE:
						{
							if (InstancePtr->lastd)
							{
								DS1820_SET_STATE(DS1820_STATE_LAST_DEVICE);
							}
							else
							{
								DS1820_SET_STATE(DS1820_STATE_SEARCH_INIT);
							}
						}
						break;

					case DS1820_STATE_LAST_DEVICE:
						{
							DS1820_SET_STATE(DS1820_STATE_INIT);
						}
						break;

					// subroutine command
					case DS1820_STATE_COMMAND:
						{
							ds1820_stack_push(InstancePtr,DS1820_STATE_COMMAND_INITALIZED);
							DS1820_SET_STATE(DS1820_STATE_XINIT);
						}
						break;

					case DS1820_STATE_COMMAND_INITALIZED:
						{
							// data=1 is "no device"
							if (InstancePtr->indata)
							{
								DS1820_SET_STATE(DS1820_STATE_INIT);
								rc = XST_FAILURE;
								break;
							}
							InstancePtr->wribyte = InstancePtr->arg;
							DS1820_SET_STATE(DS1820_STATE_WRITE_BYTE);
							ds1820_stack_push(InstancePtr,DS1820_STATE_COMMAND_DOCMD);
							if (InstancePtr->arg == DS1820_MATCH_ROM)
							{
								InstancePtr->idx = 0;
								ds1820_stack_push(InstancePtr,DS1820_STATE_COMMAND_ADRMATCH);
							}
						}
						break;

					case DS1820_STATE_COMMAND_ADRMATCH:
						{
							InstancePtr->wribyte = InstancePtr->ident[InstancePtr->idx];
							InstancePtr->idx ++;
							if (InstancePtr->idx <= DS1820_IDENT_SIZE)
							{
								ds1820_stack_push(InstancePtr,DS1820_STATE_COMMAND_ADRMATCH);
								DS1820_SET_STATE(DS1820_STATE_WRITE_BYTE);
							}
							else
							{
								DS1820_SET_STATE(ds1820_stack_pop(InstancePtr));
							}
						}
						break;

					case DS1820_STATE_COMMAND_DOCMD:
						{
							InstancePtr->wribyte = InstancePtr->cmd;
							DS1820_SET_STATE(DS1820_STATE_WRITE_BYTE);
						}
						break;

					// subroutine initialize
					case DS1820_STATE_XINIT:
						{
							ds1820_stack_push(InstancePtr,DS1820_STATE_XINIT_W4READY);
							DS1820_SET_STATE(DS1820_STATE_W4READY);
						}
						break;

					case DS1820_STATE_XINIT_W4READY:
						{
							DS1820_mWrite_control_reg(DS1820_REFLADDR, (1 << DS1820_DEF_CONTROL_REG_XINIT));
							ds1820_stack_push(InstancePtr,DS1820_STATE_XINIT_W4DONE);
							DS1820_SET_STATE(DS1820_STATE_W4DONE);
						}
						break;

					case DS1820_STATE_XINIT_W4DONE:
						{
							DS1820_SET_STATE(DS1820_STATE_W4DONE);
						}
						break;

					// Write byte
					case DS1820_STATE_WRITE_BYTE:
						{
							InstancePtr->maskbit = 0x01;
							DS1820_SET_STATE(DS1820_STATE_WRITE_BYTE_LOOP);
						}
						break;

					case DS1820_STATE_WRITE_BYTE_LOOP:
						{
							InstancePtr->outdata = InstancePtr->wribyte & InstancePtr->maskbit;
							InstancePtr->maskbit <<= 1;
							InstancePtr->maskbit &= 0xFF;
							if (InstancePtr->maskbit)
							{
								ds1820_stack_push(InstancePtr,DS1820_STATE_WRITE_BYTE_LOOP);
							}
							DS1820_SET_STATE(DS1820_STATE_WRITE_BIT);
						}
						break;

					// Write bit
					case DS1820_STATE_WRITE_BIT:
						{
							DS1820_mWrite_control_reg(DS1820_REFLADDR,
								(1 << DS1820_DEF_CONTROL_REG_XIOBIT) |
								((InstancePtr->outdata) ? (1 << DS1820_DEF_CONTROL_REG_WDATA) : 0));
							DS1820_SET_STATE(DS1820_STATE_W4DONE);
						}
						break;

					// Read Bit
					case DS1820_STATE_READ_BIT:
						{
							DS1820_mWrite_control_reg(DS1820_REFLADDR,
								(1 << DS1820_DEF_CONTROL_REG_XIOBIT) |
								(1 << DS1820_DEF_CONTROL_REG_WDATA));
							DS1820_SET_STATE(DS1820_STATE_W4DONE);
						}
						break;

					// Wait for Done
					case DS1820_STATE_W4DONE:
						{
							ds1820_stack_push(InstancePtr,DS1820_STATE_W4DONE_EPLG);
							DS1820_SET_STATE(DS1820_STATE_W4READY);
						}
						break;

					case DS1820_STATE_W4DONE_EPLG:
						{
							u32 sr = DS1820_mRead_status_reg(DS1820_REFLADDR);
							if (0 == (sr & (1 << DS1820_DEF_STATUS_REG_DONE)))
							{
								DS1820_SET_STATE(DS1820_STATE_INIT);
								rc = XST_FAILURE;
								break;
							}
							InstancePtr->indata = 0 != (sr & (1 << DS1820_DEF_STATUS_REG_RDATA));
							DS1820_SET_STATE(ds1820_stack_pop(InstancePtr));
							break;
						}
						break;

					// Wait for Ready
					case DS1820_STATE_W4READY:
						{
							u32 sr = DS1820_mRead_status_reg(DS1820_REFLADDR);
							if (0 != (sr & (1 << DS1820_DEF_STATUS_REG_BUSY)))
							{
								rc = XST_DEVICE_BUSY;
								break;
							}
							DS1820_SET_STATE(ds1820_stack_pop(InstancePtr));
							break;
						}
						break;

				}
			}
			break;
		case DS1820_CMD_KICK:
			{
				switch(DS1820_STATE)
				{
					default:
						{
						}
						break;
					case DS1820_STATE_TCONV_W4END:
						{
							DS1820_SET_STATE(DS1820_STATE_TCONV_DONE);
						}
						break;
				}
			}
			break;
	}
	return rc;
}

int ds1820_Process_Event(ds1820 *InstancePtr)
{
	if (NULL == InstancePtr) {
		return DS1820_EVENT_ERROR;
	}
	switch(InstancePtr->state)
	{
		case DS1820_STATE_TCONV_STARTED      : return DS1820_EVENT_TCONV_STARTED;
		case DS1820_STATE_TCONV_W4END        : return DS1820_EVENT_TCONV_RUNNING;
		case DS1820_STATE_SEARCH_DONE_FOUND  : return DS1820_EVENT_DEVICE_FOUND;
		case DS1820_STATE_READ_TEMP_DONE     : return DS1820_EVENT_TEMP_READED;
		case DS1820_STATE_LAST_DEVICE        : return DS1820_EVENT_END_OF_SCAN;
		default: break;
	}
	return DS1820_EVENT_NONE;
}

