/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2018, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief motorfoc ap lib
/// \file motorfoc_apc.c
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2018

#include "xparameters.h"
#include "xil_types.h"
#include "xstatus.h"

#include "motorfoc_utl.h"
#include "motorfoc.h"
#include "motorfoc_apc.h"

#include <math.h>


// ===========================================================================

#define LM_TWOPI  (M_PI * 2.0)
#define LM_SQRT3  (1.7320508075688772935274463415059)
#define LM_2X32   (4294967296.0)
#define LM_2X16   (65536.0)

// ===========================================================================

#define K_RPM_TO_HZ    (1.0f / 60.0f * (float)prm->mot.paipol)
#define K_HZ_TO_RPM    (1.0f * 60.0f / (float)prm->mot.paipol)

#define K_SPD32_TO_HZ  (1.0f / (float)LM_2X32 / prm->sys.ts)
#define K_HZ_TO_SPD32  (1.0f * (float)LM_2X32 * prm->sys.ts)

#define K_RAD_TO_ANG32 (1.0f / (float)LM_TWOPI * (float)LM_2X32)
#define K_ANG32_TO_RAD (1.0f * (float)LM_TWOPI / (float)LM_2X32)

#define K_RAD_TO_ANG16 (1.0f / (float)LM_TWOPI * (float)LM_2X16)

#define K_POS_TO_ROUND (1.0f / (float)(1 << MOTORFOC_POSITION_FRAQ_WIDTH) / (float)prm->mot.paipol)

// ===========================================================================

#define K_SPD32_TO_RPM (K_SPD32_TO_HZ * K_HZ_TO_RPM)
#define K_RPM_TO_SPD32 (K_RPM_TO_HZ * K_HZ_TO_SPD32)

// ===========================================================================

#define K_RAD_TO_DEG   (180.0f / (float)M_PI)
#define K_ANG16_TO_DEG (360.0f / (float)LM_2X16)
#define K_ANG32_TO_DEG (360.0f / (float)LM_2X32)

// ===========================================================================


#define QD_MOTORFOC_DEFLADDR                u32 qd_motorfoc_laddr = InstancePtr->Config->BaseAddress;
#define QD_MOTORFOC_REFLADDR                qd_motorfoc_laddr


//-

int motorfoc_ap_init(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// cold init
	prm->sys.ts         = 0.0f;

	return XST_SUCCESS;
}

//-

int motorfoc_ap_sts_read(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
    // driver base address
	QD_MOTORFOC_DEFLADDR;
	
	// read status registers
	
	// main
	sts->glb.ip_ident  = MOTORFOC_mRead_ip_ident(QD_MOTORFOC_REFLADDR);
	sts->glb.tsclocks  = MOTORFOC_mRead_tsclocks(QD_MOTORFOC_REFLADDR);
	sts->glb.motstatus = MOTORFOC_mRead_motstatus(QD_MOTORFOC_REFLADDR);

	// phy
	sts->phy.val_iphs_a = motorfoc_adjval_s18(MOTORFOC_mRead_val_iphs_a(QD_MOTORFOC_REFLADDR));
	sts->phy.val_iphs_b = motorfoc_adjval_s18(MOTORFOC_mRead_val_iphs_b(QD_MOTORFOC_REFLADDR));
	sts->phy.val_iphs_c = motorfoc_adjval_s18(MOTORFOC_mRead_val_iphs_c(QD_MOTORFOC_REFLADDR));
	sts->phy.val_ibus_x = motorfoc_adjval_s18(MOTORFOC_mRead_val_ibus_x(QD_MOTORFOC_REFLADDR));
	sts->phy.val_vphs_a = motorfoc_adjval_s18(MOTORFOC_mRead_val_vphs_a(QD_MOTORFOC_REFLADDR));
	sts->phy.val_vphs_b = motorfoc_adjval_s18(MOTORFOC_mRead_val_vphs_b(QD_MOTORFOC_REFLADDR));
	sts->phy.val_vphs_c = motorfoc_adjval_s18(MOTORFOC_mRead_val_vphs_c(QD_MOTORFOC_REFLADDR));
	sts->phy.val_vbus_x = motorfoc_adjval_s18(MOTORFOC_mRead_val_vbus_x(QD_MOTORFOC_REFLADDR));
	sts->phy.val_vphs_n = motorfoc_adjval_s18(MOTORFOC_mRead_val_vphs_n(QD_MOTORFOC_REFLADDR));
	
	sts->phy.guswcnt = MOTORFOC_mRead_guswcnt(QD_MOTORFOC_REFLADDR);

	// speed eval
	sts->spd.spd_speed  = MOTORFOC_mRead_spd_speed(QD_MOTORFOC_REFLADDR);

	// rotor position high resolution 0:32
	sts->rot.rot_angle  = MOTORFOC_mRead_rot_angle(QD_MOTORFOC_REFLADDR);

	// sliding mode observer
	sts->smo.smo_es2_kflt = MOTORFOC_mRead_smo_es2_kflt(QD_MOTORFOC_REFLADDR);
	sts->smo.smo_bemf_p   = MOTORFOC_mRead_smo_bemf_p(QD_MOTORFOC_REFLADDR);
	sts->smo.smo_bemf_m   = MOTORFOC_mRead_smo_bemf_m(QD_MOTORFOC_REFLADDR);
	sts->smo.smo_angle    = MOTORFOC_mRead_smo_angle(QD_MOTORFOC_REFLADDR);
	
	// position low resolution 16:16
	sts->pos.pos_position  = MOTORFOC_mRead_pos_position(QD_MOTORFOC_REFLADDR);

	// external sensor
	sts->ext.ext_angle = MOTORFOC_mRead_ext_angle(QD_MOTORFOC_REFLADDR);
	
	// resolver sensor
	sts->rsv.rsv_angle = MOTORFOC_mRead_rsv_angle(QD_MOTORFOC_REFLADDR);
	
	// hall sensor
	sts->hls.hls_angle = MOTORFOC_mRead_hls_angle(QD_MOTORFOC_REFLADDR);

	// encoder
	sts->enc.enc_index = MOTORFOC_mRead_enc_index(QD_MOTORFOC_REFLADDR);
	sts->enc.enc_phase = MOTORFOC_mRead_enc_phase(QD_MOTORFOC_REFLADDR);
	sts->enc.enc_phcpt = MOTORFOC_mRead_enc_phcpt(QD_MOTORFOC_REFLADDR);
	sts->enc.enc_angle = MOTORFOC_mRead_enc_angle(QD_MOTORFOC_REFLADDR);

	// scalar mode
	sts->mrt.mrt_spdout = MOTORFOC_mRead_mrt_spdout(QD_MOTORFOC_REFLADDR);
	sts->mrt.mrt_angle = MOTORFOC_mRead_mrt_angle(QD_MOTORFOC_REFLADDR);

	// Is incoming
	sts->iis.ismodval = MOTORFOC_mRead_ismodval(QD_MOTORFOC_REFLADDR);
	sts->iis.isangval = MOTORFOC_mRead_isangval(QD_MOTORFOC_REFLADDR);

	// Vs outgoing
	sts->ovs.vsmodval = MOTORFOC_mRead_vsmodval(QD_MOTORFOC_REFLADDR);
	sts->ovs.vsangval = MOTORFOC_mRead_vsangval(QD_MOTORFOC_REFLADDR);
	
	// pwm
	sts->pwm.pwm_mdval = MOTORFOC_mRead_pwm_mdval(QD_MOTORFOC_REFLADDR);
	
	// eoj

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_read(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
    // driver base address
	QD_MOTORFOC_DEFLADDR;
	
	// read control registers
	
	int i;
	
	// global
	ctl->glb.motcontrol = MOTORFOC_mRead_motcontrol(QD_MOTORFOC_REFLADDR);
	ctl->glb.paipol     = MOTORFOC_mRead_paipol(QD_MOTORFOC_REFLADDR);

	// phy
	ctl->phy.xbus_fk1   = MOTORFOC_mRead_xbus_fk1(QD_MOTORFOC_REFLADDR);
	ctl->phy.xbus_fk2   = MOTORFOC_mRead_xbus_fk2(QD_MOTORFOC_REFLADDR);

	ctl->phy.xphs_fk1   = MOTORFOC_mRead_xphs_fk1(QD_MOTORFOC_REFLADDR);
	ctl->phy.xphs_fk2   = MOTORFOC_mRead_xphs_fk2(QD_MOTORFOC_REFLADDR);

	ctl->phy.ibus_limit = MOTORFOC_mRead_ibus_limit(QD_MOTORFOC_REFLADDR);
	ctl->phy.iphs_limit = MOTORFOC_mRead_iphs_limit(QD_MOTORFOC_REFLADDR);
	
	ctl->phy.deadtval   = MOTORFOC_mRead_deadtval(QD_MOTORFOC_REFLADDR);

	// speed eval
	ctl->spd.spd_fktau1 = MOTORFOC_mRead_spd_fktau1(QD_MOTORFOC_REFLADDR);
	ctl->spd.spd_fktau2 = MOTORFOC_mRead_spd_fktau2(QD_MOTORFOC_REFLADDR);
	
	// rotor position
	ctl->rot.rot_angdef = MOTORFOC_mRead_rot_angdef(QD_MOTORFOC_REFLADDR);
	
	// smo
	ctl->smo.smo_vs_mult  = MOTORFOC_mRead_smo_vs_mult(QD_MOTORFOC_REFLADDR);
	ctl->smo.smo_is_mult  = MOTORFOC_mRead_smo_is_mult(QD_MOTORFOC_REFLADDR);
	ctl->smo.smo_zs_max   = MOTORFOC_mRead_smo_zs_max(QD_MOTORFOC_REFLADDR);
	ctl->smo.smo_es1_kflt = MOTORFOC_mRead_smo_es1_kflt(QD_MOTORFOC_REFLADDR);
	ctl->smo.smo_es2_kflo = MOTORFOC_mRead_smo_es2_kflo(QD_MOTORFOC_REFLADDR);
	ctl->smo.smo_es2_kfmx = MOTORFOC_mRead_smo_es2_kfmx(QD_MOTORFOC_REFLADDR);
	ctl->smo.smo_angofs   = MOTORFOC_mRead_smo_angofs(QD_MOTORFOC_REFLADDR);
		
	// speed loop
	ctl->slp.slp_spdset  = MOTORFOC_mRead_slp_spdset(QD_MOTORFOC_REFLADDR);
	ctl->slp.slp_kmpro.u = MOTORFOC_mRead_slp_kmpro(QD_MOTORFOC_REFLADDR);
	ctl->slp.slp_kmint.u = MOTORFOC_mRead_slp_kmint(QD_MOTORFOC_REFLADDR);
	ctl->slp.slp_outlim  = MOTORFOC_mRead_slp_outlim(QD_MOTORFOC_REFLADDR);
	ctl->slp.slp_kmultx  = motorfoc_adjval_s18(MOTORFOC_mRead_slp_kmultx(QD_MOTORFOC_REFLADDR));
	ctl->slp.slp_kmulty  = motorfoc_adjval_s18(MOTORFOC_mRead_slp_kmulty(QD_MOTORFOC_REFLADDR));
	
	// resolver
	ctl->rsv.rsv_m2rppk = MOTORFOC_mRead_rsv_m2rppk(QD_MOTORFOC_REFLADDR);
	ctl->rsv.rsv_angofs = MOTORFOC_mRead_rsv_angofs(QD_MOTORFOC_REFLADDR);
	
	// hall sensor
	ctl->hls.hls_ctolim = MOTORFOC_mRead_hls_ctolim(QD_MOTORFOC_REFLADDR);
	for(i=0;i<MOTORFOC_HLS_DPR_ANGLE_ELEM;i++)
		ctl->hls.hls_dprangle[i] = MOTORFOC_mRead_hall_sensor_angle_lut(QD_MOTORFOC_REFLADDR,(i << 2));
	
	// encoder
	ctl->enc.enc_cyprnd = MOTORFOC_mRead_enc_cyprnd(QD_MOTORFOC_REFLADDR);
	ctl->enc.enc_angphs = MOTORFOC_mRead_enc_angphs(QD_MOTORFOC_REFLADDR);

	// scalar control
	ctl->mrt.mrt_speed  = MOTORFOC_mRead_mrt_speed(QD_MOTORFOC_REFLADDR);
	ctl->mrt.mrt_accmax = MOTORFOC_mRead_mrt_accmax(QD_MOTORFOC_REFLADDR);
	ctl->mrt.mrt_fktau1 = MOTORFOC_mRead_mrt_fktau1(QD_MOTORFOC_REFLADDR);
	ctl->mrt.mrt_fktau2 = MOTORFOC_mRead_mrt_fktau2(QD_MOTORFOC_REFLADDR);
	
	// incoming is current
	ctl->iis.ismodmax  = MOTORFOC_mRead_ismodmax(QD_MOTORFOC_REFLADDR);

	// current loop
	ctl->pi.pi_setvalx = motorfoc_adjval_s18(MOTORFOC_mRead_pi_setvalx(QD_MOTORFOC_REFLADDR));
	ctl->pi.pi_setvaly = motorfoc_adjval_s18(MOTORFOC_mRead_pi_setvaly(QD_MOTORFOC_REFLADDR));
	ctl->pi.pi_kmpro   = MOTORFOC_mRead_pi_kmpro(QD_MOTORFOC_REFLADDR);
	ctl->pi.pi_kmint   = MOTORFOC_mRead_pi_kmint(QD_MOTORFOC_REFLADDR);
	
	// modulator
	ctl->mod.mod2angskw = MOTORFOC_mRead_mod2angskw(QD_MOTORFOC_REFLADDR);
	
	// pwm
	ctl->pwm.pwm_presc = MOTORFOC_mRead_pwm_presc(QD_MOTORFOC_REFLADDR);
	ctl->pwm.pwm_kmod  = MOTORFOC_mRead_pwm_kmod(QD_MOTORFOC_REFLADDR);
	ctl->pwm.pwm_mdmax = MOTORFOC_mRead_pwm_mdmax(QD_MOTORFOC_REFLADDR);
	ctl->pwm.pwm_ctrl = MOTORFOC_mRead_pwm_ctrl(QD_MOTORFOC_REFLADDR);

	// rpfm
	ctl->rpfm.rpfm3p_ctrl = MOTORFOC_mRead_rpfm3p_ctrl(QD_MOTORFOC_REFLADDR);
	
	//-

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_write(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
    // driver base address
	QD_MOTORFOC_DEFLADDR;
	
	// write control registers
	
	int i;
	
	// global
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol);
	MOTORFOC_mWrite_paipol(QD_MOTORFOC_REFLADDR,ctl->glb.paipol);

	// physical
	MOTORFOC_mWrite_xbus_fk1(QD_MOTORFOC_REFLADDR,ctl->phy.xbus_fk1);
	MOTORFOC_mWrite_xbus_fk2(QD_MOTORFOC_REFLADDR,ctl->phy.xbus_fk2);

	MOTORFOC_mWrite_xphs_fk1(QD_MOTORFOC_REFLADDR,ctl->phy.xphs_fk1);
	MOTORFOC_mWrite_xphs_fk2(QD_MOTORFOC_REFLADDR,ctl->phy.xphs_fk2);

	MOTORFOC_mWrite_ibus_limit(QD_MOTORFOC_REFLADDR,ctl->phy.ibus_limit);
	MOTORFOC_mWrite_iphs_limit(QD_MOTORFOC_REFLADDR,ctl->phy.iphs_limit);

	MOTORFOC_mWrite_deadtval(QD_MOTORFOC_REFLADDR,ctl->phy.deadtval);


	// speed eval
	MOTORFOC_mWrite_spd_fktau1(QD_MOTORFOC_REFLADDR,ctl->spd.spd_fktau1);
	MOTORFOC_mWrite_spd_fktau2(QD_MOTORFOC_REFLADDR,ctl->spd.spd_fktau2);
	
	// rotor position
	MOTORFOC_mWrite_rot_angdef(QD_MOTORFOC_REFLADDR,ctl->rot.rot_angdef);
	
	// smo
	MOTORFOC_mWrite_smo_vs_mult(QD_MOTORFOC_REFLADDR,ctl->smo.smo_vs_mult );
	MOTORFOC_mWrite_smo_is_mult(QD_MOTORFOC_REFLADDR,ctl->smo.smo_is_mult );
	MOTORFOC_mWrite_smo_zs_max(QD_MOTORFOC_REFLADDR,ctl->smo.smo_zs_max  );
	MOTORFOC_mWrite_smo_es1_kflt(QD_MOTORFOC_REFLADDR,ctl->smo.smo_es1_kflt);
	MOTORFOC_mWrite_smo_es2_kflo(QD_MOTORFOC_REFLADDR,ctl->smo.smo_es2_kflo);
	MOTORFOC_mWrite_smo_es2_kfmx(QD_MOTORFOC_REFLADDR,ctl->smo.smo_es2_kfmx);
	MOTORFOC_mWrite_smo_angofs(QD_MOTORFOC_REFLADDR,ctl->smo.smo_angofs);
		
	// speed loop
	MOTORFOC_mWrite_slp_spdset(QD_MOTORFOC_REFLADDR,ctl->slp.slp_spdset);
	MOTORFOC_mWrite_slp_kmpro(QD_MOTORFOC_REFLADDR,ctl->slp.slp_kmpro.u);
	MOTORFOC_mWrite_slp_kmint(QD_MOTORFOC_REFLADDR,ctl->slp.slp_kmint.u);
	MOTORFOC_mWrite_slp_outlim(QD_MOTORFOC_REFLADDR,ctl->slp.slp_outlim);
	MOTORFOC_mWrite_slp_kmultx(QD_MOTORFOC_REFLADDR,ctl->slp.slp_kmultx);
	MOTORFOC_mWrite_slp_kmulty(QD_MOTORFOC_REFLADDR,ctl->slp.slp_kmulty);
	
	// resolver
	MOTORFOC_mWrite_rsv_m2rppk(QD_MOTORFOC_REFLADDR,ctl->rsv.rsv_m2rppk);
	MOTORFOC_mWrite_rsv_angofs(QD_MOTORFOC_REFLADDR,ctl->rsv.rsv_angofs);
	
	// hall sensor
	MOTORFOC_mWrite_hls_ctolim(QD_MOTORFOC_REFLADDR,ctl->hls.hls_ctolim);
	for(i=0;i<MOTORFOC_HLS_DPR_ANGLE_ELEM;i++)
		MOTORFOC_mWrite_hall_sensor_angle_lut(QD_MOTORFOC_REFLADDR, (i << 2),ctl->hls.hls_dprangle[i]);
	
	// encoder
	MOTORFOC_mWrite_enc_cyprnd(QD_MOTORFOC_REFLADDR,ctl->enc.enc_cyprnd);
	MOTORFOC_mWrite_enc_angphs(QD_MOTORFOC_REFLADDR,ctl->enc.enc_angphs);

	// scalar control
	MOTORFOC_mWrite_mrt_speed(QD_MOTORFOC_REFLADDR,ctl->mrt.mrt_speed );
	MOTORFOC_mWrite_mrt_accmax(QD_MOTORFOC_REFLADDR,ctl->mrt.mrt_accmax);
	MOTORFOC_mWrite_mrt_fktau1(QD_MOTORFOC_REFLADDR,ctl->mrt.mrt_fktau1);
	MOTORFOC_mWrite_mrt_fktau2(QD_MOTORFOC_REFLADDR,ctl->mrt.mrt_fktau2);
	
	// incoming is current
	MOTORFOC_mWrite_ismodmax(QD_MOTORFOC_REFLADDR,ctl->iis.ismodmax);

	// current loop
	MOTORFOC_mWrite_pi_setvalx(QD_MOTORFOC_REFLADDR,ctl->pi.pi_setvalx);
	MOTORFOC_mWrite_pi_setvaly(QD_MOTORFOC_REFLADDR,ctl->pi.pi_setvaly);
	MOTORFOC_mWrite_pi_kmpro(QD_MOTORFOC_REFLADDR,ctl->pi.pi_kmpro);
	MOTORFOC_mWrite_pi_kmint(QD_MOTORFOC_REFLADDR,ctl->pi.pi_kmint);
	
	// modulator
	MOTORFOC_mWrite_mod2angskw(QD_MOTORFOC_REFLADDR,ctl->mod.mod2angskw);
	
	// pwm
	MOTORFOC_mWrite_pwm_presc(QD_MOTORFOC_REFLADDR,ctl->pwm.pwm_presc);
	MOTORFOC_mWrite_pwm_kmod(QD_MOTORFOC_REFLADDR,ctl->pwm.pwm_kmod );
	MOTORFOC_mWrite_pwm_mdmax(QD_MOTORFOC_REFLADDR,ctl->pwm.pwm_mdmax);
	MOTORFOC_mWrite_pwm_ctrl(QD_MOTORFOC_REFLADDR,ctl->pwm.pwm_ctrl);

	// rpfm
	MOTORFOC_mWrite_rpfm3p_ctrl(QD_MOTORFOC_REFLADDR,ctl->rpfm.rpfm3p_ctrl);
	
	//-

	return XST_SUCCESS;
}

int motorfoc_ap_compile(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;
	
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

    // not configured
	if (prm->sys.ts == 0.0f) return XST_FAILURE;

	// elaboration
	
	// Global
	{
		// control mode
		{
			u32 val_on = 0;
			switch(prm->mot.controlmode)
			{
				default:
				case MOTORFOC_AP_CTRLMODE_TORQUE:
					{
					}
					break;
				case MOTORFOC_AP_CTRLMODE_SPEED:
					{
						val_on |= MOTORFOC_BC_slp_spd2trq;
					}
					break;
			}
			ctl->glb.motcontrol &= ~(MOTORFOC_BC_slp_spd2trq);
			ctl->glb.motcontrol |= val_on;
		}
        
        // MRT control
        {
			u32 val_on = 0;
			if (prm->mrt.ctrls & MOTORFOC_AP_MRT_CTRLS_TOSPEED) val_on |= MOTORFOC_BC_slp_mrt2spd;

			ctl->glb.motcontrol &= ~(MOTORFOC_BC_slp_mrt2spd);
			ctl->glb.motcontrol |= val_on;
        }

		// incremental encoder control
		{
			u32 val_on = 0;
			if (prm->enc.ctrls & MOTORFOC_AP_ENCODER_CTRLS_INTEN) val_on |= MOTORFOC_BC_enc_inten;
			if (prm->enc.ctrls & MOTORFOC_AP_ENCODER_CTRLS_XMCHA) val_on |= MOTORFOC_BC_enc_xmcha;
			if (prm->enc.ctrls & MOTORFOC_AP_ENCODER_CTRLS_XMCHB) val_on |= MOTORFOC_BC_enc_xmchb;
			if (prm->enc.ctrls & MOTORFOC_AP_ENCODER_CTRLS_XMCHI) val_on |= MOTORFOC_BC_enc_xmchi;

			ctl->glb.motcontrol &= ~(MOTORFOC_BC_enc_inten|MOTORFOC_BC_enc_xmcha|MOTORFOC_BC_enc_xmchb|MOTORFOC_BC_enc_xmchi);
			ctl->glb.motcontrol |= val_on;
		}
		
		// SMO control
		{
			u32 val_on = 0;
			if (prm->smo.ctrls & MOTORFOC_AP_SMO_CTRLS_USEVSACT) val_on |= MOTORFOC_BC_smo_usevsact;

			ctl->glb.motcontrol &= ~(MOTORFOC_BC_smo_usevsact);
			ctl->glb.motcontrol |= val_on;
		}

		// windings
		{
			u32 val_on = 0;
			switch(prm->mot.windings)
			{
                default:
				case MOTORFOC_AP_WINDINGS_PMSM   :
				case MOTORFOC_AP_WINDINGS_BLDC   :
                    {
                        val_on = MOTORFOC_BC_is3phases;
                    }
                    break;
				case MOTORFOC_AP_WINDINGS_BIPOLAR:
                    {
                    }
                    break;
			}
			ctl->glb.motcontrol &= ~(MOTORFOC_BC_is3phases);
			ctl->glb.motcontrol |= val_on;
		}

		// modulator type & levels
		{
			u32 val_on = 0;
			if (prm->mod.modulator == MOTORFOC_AP_MODULATOR_RPFM && InstancePtr->Config->rpfm_modulator > 0)
			{
				val_on |= MOTORFOC_BC_modtype0;

				if (prm->mod.modlevels == 3 && InstancePtr->Config->rpfm_3_level > 0)
				{
					val_on |= MOTORFOC_BC_modlevels0;
				}
			}

			ctl->glb.motcontrol &= ~(MOTORFOC_BC_modtype0 | MOTORFOC_BC_modlevels0);
			ctl->glb.motcontrol |= val_on;
		}

		// rotor position mode
		{
			u32 val_on = 0;
			switch(prm->mot.rotposmode)
			{
				case MOTORFOC_AP_ROTPOSMODE_MANUAL   : val_on = MOTORFOC_BC_ROT_POS_MODE_MANUAL   ; break;
				case MOTORFOC_AP_ROTPOSMODE_EMULATOR : val_on = MOTORFOC_BC_ROT_POS_MODE_EMULATOR ; break;
				case MOTORFOC_AP_ROTPOSMODE_ENCODER  : val_on = MOTORFOC_BC_ROT_POS_MODE_ENCODER  ; break;
				case MOTORFOC_AP_ROTPOSMODE_EVALUATOR: val_on = MOTORFOC_BC_ROT_POS_MODE_EVALUATOR; break;
				case MOTORFOC_AP_ROTPOSMODE_HALLSNS  : val_on = MOTORFOC_BC_ROT_POS_MODE_HALLSNS  ; break;
				case MOTORFOC_AP_ROTPOSMODE_RESOLVER : val_on = MOTORFOC_BC_ROT_POS_MODE_RESOLVER ; break;
				case MOTORFOC_AP_ROTPOSMODE_EXTERNAL : val_on = MOTORFOC_BC_ROT_POS_MODE_EXTERNAL ; break;
			}
			ctl->glb.motcontrol &= ~MOTORFOC_BC_ROT_POS_MODE_MASK;
			ctl->glb.motcontrol |= val_on;
		}


		// # of pair poles
		ctl->glb.paipol = prm->mot.paipol;
	}
    
	// Physical
	{
		// Bus filter
		ctl->phy.xbus_fk1 = motorfoc_lpf1_Fcut2IK(prm->sys.ts,prm->sys.xbus_fcut1);
		ctl->phy.xbus_fk2 = motorfoc_lpf1_Fcut2IK(prm->sys.ts,prm->sys.xbus_fcut2);
		
		// Motor Phase filters
		ctl->phy.xphs_fk1 = motorfoc_lpf1_Fcut2IK(prm->sys.ts,prm->sys.xphs_fcut1);
		ctl->phy.xphs_fk2 = motorfoc_lpf1_Fcut2IK(prm->sys.ts,prm->sys.xphs_fcut2);
		
		// I-bus Limit
		ctl->phy.ibus_limit = motorfoc_clip_round_u17(prm->sys.ibus_limit / prm->sys.ibus_gain);
		
		// I-phase_x Limit
		ctl->phy.iphs_limit = motorfoc_clip_round_u17(prm->sys.iphs_limit / prm->sys.iphs_gain);
		
		// gate unit dead time
		u32 deadtval = (u32)((prm->sys.deadtime * (float)InstancePtr->Config->systemclock) + 0.5f) - 1;
		if (deadtval > 255)
			deadtval = 255;
		ctl->phy.deadtval = deadtval;
	}
	
	// current limit
	{
		// Is
		ctl->iis.ismodmax = motorfoc_clip_round_u17(prm->mot.iscurrmax / prm->sys.iphs_gain);
	}
	
	
	// speed filter
	{
		float fcut1 = 1.0e4f;
		float fcut2 = 1.0e4f;
		
		switch(prm->mot.rotposmode)
		{
			default:
			case MOTORFOC_AP_ROTPOSMODE_MANUAL:
				fcut1 = prm->man.spfcut1;
				fcut2 = prm->man.spfcut2;
				break;
			case MOTORFOC_AP_ROTPOSMODE_EMULATOR:
				fcut1 = prm->mrt.spfcut1;
				fcut2 = prm->mrt.spfcut2;
				break;
			case MOTORFOC_AP_ROTPOSMODE_ENCODER:
				fcut1 = prm->enc.spfcut1;
				fcut2 = prm->enc.spfcut2;
				break;
			case MOTORFOC_AP_ROTPOSMODE_EVALUATOR:
				fcut1 = prm->smo.spfcut1;
				fcut2 = prm->smo.spfcut2;
				break;
			case MOTORFOC_AP_ROTPOSMODE_HALLSNS:
				fcut1 = prm->hls.spfcut1;
				fcut2 = prm->hls.spfcut2;
				break;
			case MOTORFOC_AP_ROTPOSMODE_RESOLVER:
				fcut1 = prm->rsv.spfcut1;
				fcut2 = prm->rsv.spfcut2;
				break;
			case MOTORFOC_AP_ROTPOSMODE_EXTERNAL:
				fcut1 = prm->ext.spfcut1;
				fcut2 = prm->ext.spfcut2;
				break;
		}
		
		ctl->spd.spd_fktau1 = motorfoc_lpf1_Fcut2IK(prm->sys.ts,fcut1);
		ctl->spd.spd_fktau2 = motorfoc_lpf1_Fcut2IK(prm->sys.ts,fcut2);
	}
	
	// SMO
	{
        // es1
        ctl->smo.smo_es1_kflt = motorfoc_lpf1_Fcut2IK(prm->sys.ts,prm->smo.es1_fcut);
        
        // es2 lo
		ctl->smo.smo_es2_kflo = motorfoc_lpf1_Fcut2IK(prm->sys.ts,prm->smo.es2_fclow);
		
        // es2 mult
		{
            float max_hz = prm->mot.maxspeed * K_RPM_TO_HZ;
            float max_fcut = max_hz * prm->smo.es2_fcmul + prm->smo.es2_fclow;
            
            uint32_t max_f_k = motorfoc_lpf1_Fcut2IK(prm->sys.ts,max_fcut);
            
            uint32_t delta_f_k = max_f_k - ctl->smo.smo_es2_kflo;
            float hz_k_mul = (float)delta_f_k / max_hz;
            
            ctl->smo.smo_es2_kfmx = motorfoc_clip_round_u32(hz_k_mul * (float)(1 << InstancePtr->Config->smo_f2_dln2) * K_SPD32_TO_HZ);
		}
		
        // zs max
		{
			uint32_t zmi = motorfoc_clip_round_u17(prm->smo.zs_maxval / prm->sys.vbus_gain);
			if (zmi > MOTORFOC_SMO_EVAL_ZS_MAX_INT_MAX)
				zmi = MOTORFOC_SMO_EVAL_ZS_MAX_INT_MAX;
				
			ctl->smo.smo_zs_max = zmi;
		}
		
        // vs & is
		if (prm->smo.is_maxerr > 0.0f && prm->smo.zs_maxval > 0.0f)
		{
			float dvz = (float)(1 << InstancePtr->Config->smo_zs_dln2);
			
			float v2ig_kv = prm->sys.ts / prm->mot.inductance * prm->sys.vbus_gain;
			float v2ig_ki = prm->sys.ts / prm->mot.inductance * prm->mot.resistance * prm->sys.iphs_gain;
			
			float i2zg = prm->smo.zs_maxval / prm->smo.is_maxerr / prm->sys.vbus_gain;
			
			// k multiplier
			float kv = i2zg * v2ig_kv * dvz;
			float ki = i2zg * v2ig_ki * dvz;
			
			// if one(both) overflow...
			{
				float xx = (kv > ki) ? kv : ki;
				xx /= (float)LM_2X32;
				if (xx > 1.0f)
				{
					// scale both!
					kv /= xx;
					ki /= xx;
				}
			}
			
			ctl->smo.smo_vs_mult = motorfoc_clip_round_u32(kv);
			ctl->smo.smo_is_mult = motorfoc_clip_round_u32(ki);
		}
		
        // angle offset
		{
			ctl->smo.smo_angofs = (int16_t)(remainderf(prm->smo.angofs,(float)LM_TWOPI) * K_RAD_TO_ANG16);
		}
	}
	
	
	// 2nd modulator offset
	{
		ctl->mod.mod2angskw = (int16_t)(remainderf(prm->mod.mod2angskw,(float)LM_TWOPI) * K_RAD_TO_ANG16);
	}

	// PWM setup
	if (prm->pwm.freq > 0.0f)
	{
		// pwm frequency
		float pwm_presc = (float)InstancePtr->Config->systemclock / (float)prm->pwm.freq / 2.0f - 1.0f;
		ctl->pwm.pwm_presc = motorfoc_clip_round_u17(pwm_presc);
		if (ctl->pwm.pwm_presc > MOTORFOC_PWM_COUNTER_MAX)
			ctl->pwm.pwm_presc = MOTORFOC_PWM_COUNTER_MAX;
		
		// pwm gain
		float pwm_kmod = (float)prm->pwm.gain;
		pwm_kmod *= 2.0f;
		pwm_kmod *= (float)ctl->pwm.pwm_presc;
		pwm_kmod *= (float)(1 << MOTORFOC_PWM_KMOD_DLN2);
		ctl->pwm.pwm_kmod = motorfoc_clip_round_u32(pwm_kmod);
		
		// pwm modulator limit
		ctl->pwm.pwm_mdmax = motorfoc_clip_round_u17((float)ctl->pwm.pwm_presc * (float)prm->pwm.mdmax);
		
		// pwm control register
		ctl->pwm.pwm_ctrl = 0
				| ((prm->pwm.ctrls & MOTORFOC_AP_PWM_CTRLS_CMMOFS) ? MOTORFOC_PWM_ctrl_cmmofs : 0)
				| ((prm->pwm.ctrls & MOTORFOC_AP_PWM_CTRLS_TABLE) * MOTORFOC_PWM_ctrl_table)
				;
	}
	
	// RPFM setup
	{
		// control
		ctl->rpfm.rpfm3p_ctrl = 0;
		
		// add vector 7 hold control
		if (prm->rpfm.ctrls & MOTORFOC_AP_RPFM_CTRLS_V7H)
			ctl->rpfm.rpfm3p_ctrl |= MOTORFOC_RPFM3P_ctrl_v7h;
			
		// add vector 0/7 nice control
		if (prm->rpfm.ctrls & MOTORFOC_AP_RPFM_CTRLS_V07N)
			ctl->rpfm.rpfm3p_ctrl |= MOTORFOC_RPFM3P_ctrl_v07n;
		
		// pulse width
		float reqpwd = prm->rpfm.pwdth;
		if (prm->mod.modlevels == 3 && InstancePtr->Config->rpfm_3_level > 0)
		{
			reqpwd /= (float)(3 - 1); // ML - 1
		}

		// requesting clock divisor
		int rqdv = motorfoc_clip_round_u17(reqpwd * (float)InstancePtr->Config->systemclock);
		
		if (rqdv < MOTORFOC_RPFM3P_clocks_width_min)
			rqdv = MOTORFOC_RPFM3P_clocks_width_min;
		
		if (rqdv > MOTORFOC_RPFM3P_clocks_width_max)
			rqdv = MOTORFOC_RPFM3P_clocks_width_max;
			
		// worst/safe case
		int khi = MOTORFOC_RPFM3P_ctrl_mdkd_max;
		int klo = MOTORFOC_RPFM3P_ctrl_mdpr_max;
		int ker = MOTORFOC_RPFM3P_clocks_width_max;

		int slo;
		for(slo = MOTORFOC_RPFM3P_ctrl_mdpr_max; slo >= MOTORFOC_RPFM3P_ctrl_mdpr_min; slo--)
		{
			int jhi = rqdv / slo;
			int shi;
			for(shi = jhi; shi <= (jhi + 1); shi++)
			{
				if (shi >= MOTORFOC_RPFM3P_ctrl_mdkd_min && shi <= MOTORFOC_RPFM3P_ctrl_mdkd_max)
				{
					int ttdv = slo * shi;
					int ser = motorfoc_abs_int(ttdv - rqdv);
					if (ser < ker)
					{
						khi = shi;
						klo = slo;
						ker = ser;
					}
				}
			}
		}
		
		int mdkd = khi - 1;
		int mdpr = klo - 1;
		
		ctl->rpfm.rpfm3p_ctrl |= mdpr * MOTORFOC_RPFM3P_ctrl_mdpr;
		ctl->rpfm.rpfm3p_ctrl |= mdkd * MOTORFOC_RPFM3P_ctrl_mdkd;
	}
	

	// PI
	{
		float k = prm->sys.iphs_gain / prm->sys.vbus_gain;
		
		ctl->pi.pi_kmpro = motorfoc_clip_round_u17(prm->pi.pgain * (float)(1 << InstancePtr->Config->pi_erpro_dln2) * k);
		ctl->pi.pi_kmint = motorfoc_clip_round_u17(prm->pi.igain * (float)(1 << InstancePtr->Config->pi_erint_dln2) * k);
	}
	
	// MRT
	{
		uint32_t accmax_v = 0;
		if (prm->mrt.accmax > 0.0f)
		{
			accmax_v = motorfoc_clip_round_u32(prm->mrt.accmax * K_RPM_TO_SPD32 * prm->sys.ts * (float)(1 << InstancePtr->Config->mrt_accmax_dln2));
			if (accmax_v == 0)
				accmax_v = 1;
		}
		ctl->mrt.mrt_accmax = accmax_v;
		ctl->mrt.mrt_fktau1 = motorfoc_lpf1_Fcut2IK(prm->sys.ts,prm->mrt.sofcut1);
		ctl->mrt.mrt_fktau2 = motorfoc_lpf1_Fcut2IK(prm->sys.ts,prm->mrt.sofcut2);
	}
	
	// SLP
	{
		float k = K_SPD32_TO_RPM / prm->sys.iphs_gain;
		
		ctl->slp.slp_kmpro.f = prm->slp.pgain * (float)(1 << InstancePtr->Config->slp_pro_dln2) * k;
		ctl->slp.slp_kmint.f = prm->slp.igain * (float)(1 << InstancePtr->Config->slp_int_dln2) * k;
	}
	
	// RSV
	{
		// motor to resolver pair poles
		ctl->rsv.rsv_m2rppk = (prm->mot.paipol > prm->rsv.paipol && prm->rsv.paipol > 0) ? (prm->mot.paipol / prm->rsv.paipol) : 1;

		// offset mounting
		ctl->rsv.rsv_angofs = (int16_t)(remainderf(prm->rsv.angofs,(float)LM_TWOPI) * K_RAD_TO_ANG16);
	}
	
	// HLS
	{
		u32 ncys = prm->hls.ctolim / prm->sys.ts;
		if (ncys > MOTORFOC_hls_max_ctolim)
			ncys = MOTORFOC_hls_max_ctolim;
		ctl->hls.hls_ctolim = ncys;
		
		int i;
		for(i=0;i<MOTORFOC_HLS_DPR_ANGLE_ELEM;i++)
		{
			ctl->hls.hls_dprangle[i] = (int16_t)(remainderf(prm->hls.anglelut[i],(float)LM_TWOPI) * K_RAD_TO_ANG16);
		}
		
	}
	
	
	// incremental encoder
	{
		if (prm->enc.cyprnd > 0)
		{
			// rotor angle per phase
			ctl->enc.enc_angphs = motorfoc_clip_round_u32(1.0f
					* (float)LM_2X32
					* (float)prm->mot.paipol
					/ (float)prm->enc.cyprnd
					/ 4.0f);
		}
		
		// pulses per round
		ctl->enc.enc_cyprnd = prm->enc.cyprnd;
        
	}
	
	return XST_SUCCESS;
}

//-

int motorfoc_ap_prm_set_modlevels(motorfoc *InstancePtr, int modlevels)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// modulator levels
	prm->mod.modlevels = modlevels;

	return XST_SUCCESS;
}

int motorfoc_ap_prm_get_modlevels(motorfoc *InstancePtr, int * modlevels)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

    if (modlevels == NULL) {
        return XST_FAILURE;
    }

	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// modulator levels
	* modlevels = prm->mod.modlevels;

	return XST_SUCCESS;
}

//-

int motorfoc_ap_sts_get_iscurr(motorfoc *InstancePtr, float * iscurr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (iscurr == NULL) {
        return XST_FAILURE;
    }
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
		
	// stator current
	* iscurr = (float)sts->iis.ismodval * prm->sys.iphs_gain;
	
	return XST_SUCCESS;
}


int motorfoc_ap_sts_get_vsvolts(motorfoc *InstancePtr, float * vsvolts)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (vsvolts == NULL) {
        return XST_FAILURE;
    }
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	// modulation |Vs|
	* vsvolts = (float)sts->ovs.vsmodval * prm->sys.vbus_gain;
	
	return XST_SUCCESS;
}

int motorfoc_ap_sts_get_vs2israd(motorfoc *InstancePtr, float * vs2israd)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (vs2israd == NULL) {
        return XST_FAILURE;
    }
		
	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	float vs2is_reg = (int)(sts->ovs.vsangval - sts->iis.isangval);
	
	// modulation |Vs|
	* vs2israd = vs2is_reg * K_ANG32_TO_RAD;
	
	return XST_SUCCESS;
}

int motorfoc_ap_sts_get_speedrpm(motorfoc *InstancePtr, float * speedrpm)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (speedrpm == NULL) {
        return XST_FAILURE;
    }
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	// chk prm
	if (prm->sys.ts     <= 0.0f) return XST_FAILURE;
	if (prm->mot.paipol <= 0   ) return XST_FAILURE;
	
	// rotor speed
	* speedrpm = (float)sts->spd.spd_speed * K_SPD32_TO_RPM;
	
	return XST_SUCCESS;
}

int motorfoc_ap_sts_get_mrt_spdout(motorfoc *InstancePtr, float * speed)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (speed == NULL) {
        return XST_FAILURE;
    }
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	// chk prm
	if (prm->sys.ts     <= 0.0f) return XST_FAILURE;
	if (prm->mot.paipol <= 0   ) return XST_FAILURE;
	
	// speed mrt output
	* speed = (float)sts->mrt.mrt_spdout * K_SPD32_TO_RPM;
	
	return XST_SUCCESS;
}

int motorfoc_ap_sts_get_rotpos(motorfoc *InstancePtr, float * rotpos)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (rotpos == NULL) {
        return XST_FAILURE;
    }
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	// chk prm
	if (prm->mot.paipol <= 0) return XST_FAILURE;
	
	// rotor position
	* rotpos = (float)sts->pos.pos_position * K_POS_TO_ROUND;
	
	return XST_SUCCESS;
}


int motorfoc_ap_sts_get_modidx(motorfoc *InstancePtr, float * modidx)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (modidx == NULL) {
        return XST_FAILURE;
    }
		
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	// modulator depending
	int modulator;
	motorfoc_ap_sts_get_modulator(InstancePtr,&modulator);
	
	switch(modulator)
	{
		case MOTORFOC_AP_MODULATOR_RPFM:
			{
				// space vector modulation index
				* modidx = (sts->phy.val_vbus_x > 0) ? ((float)sts->ovs.vsmodval / (float)sts->phy.val_vbus_x * (float)LM_SQRT3) : 0.0f;
			}
			break;
		case MOTORFOC_AP_MODULATOR_PWM:
			{
				// pwm mode: modulation index is table depending
				* modidx = (ctl->pwm.pwm_presc > 0) ? ((float)sts->pwm.pwm_mdval / (float)ctl->pwm.pwm_presc) : 0.0f;
			}
			break;
		default:
			{
				// sinwave modulation index
				* modidx = (sts->phy.val_vbus_x > 0) ? ((float)sts->ovs.vsmodval / (float)sts->phy.val_vbus_x * 2.0f) : 0.0f;
			}
			break;

	}
	
	return XST_SUCCESS;
}

int motorfoc_ap_sts_get_vbus(motorfoc *InstancePtr, float * vbus)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (vbus == NULL) {
        return XST_FAILURE;
    }
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
		
	* vbus = (float)sts->phy.val_vbus_x * prm->sys.vbus_gain;

	return XST_SUCCESS;
}

int motorfoc_ap_sts_get_ivd(motorfoc *InstancePtr, float * ivd)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

    if (ivd == NULL) {
        return XST_FAILURE;
    }

	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;

	ivd[MOTORFOC_IVD_IPHS_A] = (float)sts->phy.val_iphs_a * prm->sys.iphs_gain;
	ivd[MOTORFOC_IVD_IPHS_B] = (float)sts->phy.val_iphs_b * prm->sys.iphs_gain;
	ivd[MOTORFOC_IVD_IPHS_C] = (float)sts->phy.val_iphs_c * prm->sys.iphs_gain;
	ivd[MOTORFOC_IVD_IBUS_X] = (float)sts->phy.val_ibus_x * prm->sys.ibus_gain;
	ivd[MOTORFOC_IVD_VPHS_A] = (float)sts->phy.val_vphs_a * prm->sys.vphs_gain;
	ivd[MOTORFOC_IVD_VPHS_B] = (float)sts->phy.val_vphs_b * prm->sys.vphs_gain;
	ivd[MOTORFOC_IVD_VPHS_C] = (float)sts->phy.val_vphs_c * prm->sys.vphs_gain;
	ivd[MOTORFOC_IVD_VBUS_X] = (float)sts->phy.val_vbus_x * prm->sys.vbus_gain;
	ivd[MOTORFOC_IVD_VPHS_N] = (float)sts->phy.val_vphs_n * prm->sys.vphs_gain;
	
	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_get_smo_lpfsimout(motorfoc *InstancePtr,  float * module, float * angle, float * angofs)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;

	// chk prm
	if (prm->sys.ts     <= 0.0f) return XST_FAILURE;

	// smo lpf emulation
	float spd_hz     = (float)sts->spd.spd_speed * K_SPD32_TO_HZ;
	float smoes1fcut = motorfoc_lpf1_IK2Fcut(prm->sys.ts,ctl->smo.smo_es1_kflt);
	float smoes2fcut = motorfoc_lpf1_IK2Fcut(prm->sys.ts,sts->smo.smo_es2_kflt);

	float rf1 = spd_hz / smoes1fcut;
	float rf2 = spd_hz / smoes2fcut;

	float gain1 = 1.0f / sqrtf(1.0f + (rf1 * rf1));
	float gain2 = 1.0f / sqrtf(1.0f + (rf2 * rf2));

	float ang1 = - atanf(rf1);
	float ang2 = - atanf(rf2);

	float l_module = gain1 * gain2;
	float l_angle  = ang1  + ang2 ;

	float l_angofs = -(float)(M_PI / 2.0) - ((spd_hz >= 0.0f) ? l_angle : (-l_angle));

	* module = l_module;
	* angle  = l_angle;
	* angofs = l_angofs;

	return XST_SUCCESS;
}


//-

int motorfoc_ap_ctl_get_slp_is2pi_rec(motorfoc *InstancePtr, float *x, float *y)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	// chk prm
	if (prm->sys.iphs_gain <= 0.0f) return XST_FAILURE;
	
	float module = (float)ctl->slp.slp_outlim  * prm->sys.iphs_gain;
	
	*x = (float)ctl->slp.slp_kmultx * module / (float)MOTORFOC_SLP_YX_MODULO;
	*y = (float)ctl->slp.slp_kmulty * module / (float)MOTORFOC_SLP_YX_MODULO;

	return XST_SUCCESS;
}

int motorfoc_ap_ctl_set_slp_is2pi_rec(motorfoc *InstancePtr, const float x, const float y, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	// chk prm
	if (prm->sys.iphs_gain <= 0.0f) return XST_FAILURE;
	
	int r_m = 0;
	int r_x = 0;
	int r_y = 0;

	if (x != 0.0f || y != 0.0f)
	{
		// module
		float module = sqrtf(x * x + y * y);

		r_m = motorfoc_clip_round_u17(module / prm->sys.iphs_gain);
		
		float dx = x / module * (float)MOTORFOC_SLP_YX_MODULO;
		float dy = y / module * (float)MOTORFOC_SLP_YX_MODULO;

		dy = fabs(dy);  // want positive Y!

		r_x = motorfoc_clip_round_int(dx, -MOTORFOC_SLP_YX_MODULO, MOTORFOC_SLP_YX_MODULO);
		r_y = motorfoc_clip_round_int(dy, -MOTORFOC_SLP_YX_MODULO, MOTORFOC_SLP_YX_MODULO);
	}

	// m/x/y
	ctl->slp.slp_outlim = r_m;
	ctl->slp.slp_kmultx = r_x;
	ctl->slp.slp_kmulty = r_y;

	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_slp_outlim(QD_MOTORFOC_REFLADDR,ctl->slp.slp_outlim);
		MOTORFOC_mWrite_slp_kmultx(QD_MOTORFOC_REFLADDR,ctl->slp.slp_kmultx);
		MOTORFOC_mWrite_slp_kmulty(QD_MOTORFOC_REFLADDR,ctl->slp.slp_kmulty);
	}

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_get_slp_is2pi_pol(motorfoc *InstancePtr, float *module, float *angle)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	if (module == NULL) {
		return XST_FAILURE;
	}

	if (angle == NULL) {
		return XST_FAILURE;
	}

	float x = 0.0f;
	float y = 0.0f;

	int rc = motorfoc_ap_ctl_get_slp_is2pi_rec(InstancePtr,&x,&y);

	*module = sqrtf(x * x + y * y);
	*angle  = atan2f(y,x);

	return rc;
}

int motorfoc_ap_ctl_set_slp_is2pi_pol(motorfoc *InstancePtr, const float module, const float angle, int wnow)
{
	float x = cosf(angle) * fabs(module);
	float y = sinf(angle) * module;

	return motorfoc_ap_ctl_set_slp_is2pi_rec(InstancePtr,x,y,wnow);
}

//-

int motorfoc_ap_ctl_get_pi_curset_rec(motorfoc *InstancePtr, float *x, float *y)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	// chk prm
	if (prm->sys.iphs_gain <= 0.0f) return XST_FAILURE;

	*x = (float)ctl->pi.pi_setvalx * prm->sys.iphs_gain;
	*y = (float)ctl->pi.pi_setvaly * prm->sys.iphs_gain;

	return XST_SUCCESS;
}

int motorfoc_ap_ctl_set_pi_curset_rec(motorfoc *InstancePtr, const float x, const float y, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	// chk prm
	if (prm->sys.iphs_gain <= 0.0f) return XST_FAILURE;

	// default value is 0,0
	int r_x = 0;
	int r_y = 0;

	// scaling
	{
		r_x = motorfoc_clip_round_s18(x / prm->sys.iphs_gain);
		r_y = motorfoc_clip_round_s18(y / prm->sys.iphs_gain);
	}

	// x/y
	ctl->pi.pi_setvalx = r_x;
	ctl->pi.pi_setvaly = r_y;

	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_pi_setvalx(QD_MOTORFOC_REFLADDR,ctl->pi.pi_setvalx);
		MOTORFOC_mWrite_pi_setvaly(QD_MOTORFOC_REFLADDR,ctl->pi.pi_setvaly);
	}

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_get_pi_curset_pol(motorfoc *InstancePtr, float *module, float *angle)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	if (module == NULL) {
		return XST_FAILURE;
	}

	if (angle == NULL) {
		return XST_FAILURE;
	}

	float x = 0.0f;
	float y = 0.0f;

	int rc = motorfoc_ap_ctl_get_pi_curset_rec(InstancePtr,&x,&y);

	*module = sqrtf(x * x + y * y);
	*angle  = atan2f(y,x);

	return rc;
}

//-

int motorfoc_ap_ctl_set_pi_curset_pol(motorfoc *InstancePtr, const float module, const float angle, int wnow)
{
	float x = cosf(angle) * fabs(module);
	float y = sinf(angle) * module;

	return motorfoc_ap_ctl_set_pi_curset_rec(InstancePtr,x,y,wnow);
}

//-

int motorfoc_ap_ctl_set_mrt_speed(motorfoc *InstancePtr, float speed, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	// chk prm
	if (prm->sys.ts     <= 0.0f) return XST_FAILURE;
	if (prm->mot.paipol <= 0   ) return XST_FAILURE;
	
	// speed setpoint
	ctl->mrt.mrt_speed = motorfoc_clip_round_int(speed * K_RPM_TO_SPD32,-MOTORFOC_mrt_max_spd,MOTORFOC_mrt_max_spd);
	
	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_mrt_speed(QD_MOTORFOC_REFLADDR,ctl->mrt.mrt_speed);
	}
	
	return XST_SUCCESS;
}


int motorfoc_ap_ctl_get_mrt_speed(motorfoc *InstancePtr, float * speed)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (speed == NULL) {
        return XST_FAILURE;
    }
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	// chk prm
	if (prm->sys.ts     <= 0.0f) return XST_FAILURE;
	if (prm->mot.paipol <= 0   ) return XST_FAILURE;
	
	// speed setpoint
	* speed = (float)ctl->mrt.mrt_speed * K_SPD32_TO_RPM;
	
	return XST_SUCCESS;
}


//-

int motorfoc_ap_ctl_set_slp_speed(motorfoc *InstancePtr, float speed, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	// chk prm
	if (prm->sys.ts     <= 0.0f) return XST_FAILURE;
	if (prm->mot.paipol <= 0   ) return XST_FAILURE;
	
	
	// speed setpoint
	float speed_reg = speed * K_RPM_TO_SPD32;
	ctl->slp.slp_spdset = (int32_t)speed_reg;
	
	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_slp_spdset(QD_MOTORFOC_REFLADDR,ctl->slp.slp_spdset);
	}
	
	return XST_SUCCESS;
}


int motorfoc_ap_ctl_get_slp_speed(motorfoc *InstancePtr, float * speed)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (speed == NULL) {
        return XST_FAILURE;
    }
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	// chk prm
	if (prm->sys.ts     <= 0.0f) return XST_FAILURE;
	if (prm->mot.paipol <= 0   ) return XST_FAILURE;
	
	// speed setpoint
	* speed = (float)ctl->slp.slp_spdset * K_SPD32_TO_RPM;
	
	return XST_SUCCESS;
}

//-

int motorfoc_ap_sts_get_failure(motorfoc *InstancePtr, int * failure)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (failure == NULL) {
        return XST_FAILURE;
    }
		
	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	* failure = (sts->glb.motstatus & MOTORFOC_BS_syserr) != 0;
	
	return XST_SUCCESS;
}


int motorfoc_ap_sts_get_modulator(motorfoc *InstancePtr, int * modulator)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (modulator == NULL) {
        return XST_FAILURE;
    }
		
	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	if ((sts->glb.motstatus & MOTORFOC_BS_modtype0) == 0)
		* modulator = MOTORFOC_AP_MODULATOR_PWM;
	else
		* modulator = MOTORFOC_AP_MODULATOR_RPFM;
	
	return XST_SUCCESS;
}


//-

int motorfoc_ap_ctl_set_ctrlmode(motorfoc *InstancePtr, int ctrlmode, int mrtopt, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	// save PRM
	prm->mot.controlmode = ctrlmode;
	prm->mrt.ctrls       = mrtopt;

	u32 val_on = 0;
	
	switch(prm->mot.controlmode)
	{
		default:
		case MOTORFOC_AP_CTRLMODE_TORQUE:
			{
			}
			break;
		case MOTORFOC_AP_CTRLMODE_SPEED:
			{
				val_on |= MOTORFOC_BC_slp_spd2trq;
			}
			break;
	}
	
	if (0 != (prm->mrt.ctrls & MOTORFOC_AP_MRT_CTRLS_TOSPEED))
	{
		val_on |= MOTORFOC_BC_slp_mrt2spd;
	}
	
	ctl->glb.motcontrol &= ~(MOTORFOC_BC_slp_spd2trq | MOTORFOC_BC_slp_mrt2spd);
	ctl->glb.motcontrol |= val_on;
	
	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol);
	}

	
	return XST_SUCCESS;
}

int motorfoc_ap_ctl_get_ctrlmode(motorfoc *InstancePtr, int * ctrlmode, int * mrtopt)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (ctrlmode == NULL) {
        return XST_FAILURE;
    }
		
    if (mrtopt == NULL) {
        return XST_FAILURE;
    }
		
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	
	if (0 != (ctl->glb.motcontrol & MOTORFOC_BC_slp_spd2trq))
	{
		* ctrlmode = MOTORFOC_AP_CTRLMODE_SPEED;
	}
	else
	{
		* ctrlmode = MOTORFOC_AP_CTRLMODE_TORQUE;
	}
	
	* mrtopt = 0;
	
	if (0 != (ctl->glb.motcontrol & MOTORFOC_BC_slp_mrt2spd))
	{
		* mrtopt |= MOTORFOC_AP_MRT_CTRLS_TOSPEED;
	}
	
	return XST_SUCCESS;
}


int motorfoc_ap_ctl_set_power(motorfoc *InstancePtr, int power, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	if (power)
	{
		ctl->glb.motcontrol |= MOTORFOC_BC_coilenab;
	}
	else
	{
		ctl->glb.motcontrol &= ~MOTORFOC_BC_coilenab;
	}
	
	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol);
	}
	
	return XST_SUCCESS;
}

int motorfoc_ap_ctl_get_power(motorfoc *InstancePtr, int * power)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (power == NULL) {
        return XST_FAILURE;
    }
		
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	* power = (ctl->glb.motcontrol & MOTORFOC_BC_coilenab) == MOTORFOC_BC_coilenab;
	
	return XST_SUCCESS;
}


int motorfoc_ap_ctl_set_modulator(motorfoc *InstancePtr, int modulator, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	// save PRM
	prm->mod.modulator = modulator;

	// set control register
	if (prm->mod.modulator == MOTORFOC_AP_MODULATOR_RPFM && InstancePtr->Config->rpfm_modulator > 0)
	{
		ctl->glb.motcontrol |= MOTORFOC_BC_modtype0;
	}
	else
	{
		ctl->glb.motcontrol &= ~MOTORFOC_BC_modtype0;
	}
	
	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol);
	}
	
	return XST_SUCCESS;
}

int motorfoc_ap_ctl_get_modulator(motorfoc *InstancePtr, int * modulator)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (modulator == NULL) {
        return XST_FAILURE;
    }
		
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	if ((ctl->glb.motcontrol & MOTORFOC_BC_modtype0) == 0)
		* modulator = MOTORFOC_AP_MODULATOR_PWM;
	else
		* modulator = MOTORFOC_AP_MODULATOR_RPFM;
	
	return XST_SUCCESS;
}

int motorfoc_ap_ctl_set_rotposmode(motorfoc *InstancePtr, int rotposmode, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	// save PRM
	prm->mot.rotposmode = rotposmode;

	u32 val_on = 0;
	switch(prm->mot.rotposmode)
	{
		case MOTORFOC_AP_ROTPOSMODE_MANUAL   : val_on = MOTORFOC_BC_ROT_POS_MODE_MANUAL   ; break;
		case MOTORFOC_AP_ROTPOSMODE_EMULATOR : val_on = MOTORFOC_BC_ROT_POS_MODE_EMULATOR ; break;
		case MOTORFOC_AP_ROTPOSMODE_ENCODER  : val_on = MOTORFOC_BC_ROT_POS_MODE_ENCODER  ; break;
		case MOTORFOC_AP_ROTPOSMODE_EVALUATOR: val_on = MOTORFOC_BC_ROT_POS_MODE_EVALUATOR; break;
		case MOTORFOC_AP_ROTPOSMODE_HALLSNS  : val_on = MOTORFOC_BC_ROT_POS_MODE_HALLSNS  ; break;
		case MOTORFOC_AP_ROTPOSMODE_RESOLVER : val_on = MOTORFOC_BC_ROT_POS_MODE_RESOLVER ; break;
		case MOTORFOC_AP_ROTPOSMODE_EXTERNAL : val_on = MOTORFOC_BC_ROT_POS_MODE_EXTERNAL ; break;
	}
	ctl->glb.motcontrol &= ~MOTORFOC_BC_ROT_POS_MODE_MASK;
	ctl->glb.motcontrol |= val_on;
	
	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol);
	}
	
	return XST_SUCCESS;
}

int motorfoc_ap_ctl_get_rotposmode(motorfoc *InstancePtr, int * rotposmode)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
    if (rotposmode == NULL) {
        return XST_FAILURE;
    }
		
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	switch(ctl->glb.motcontrol & MOTORFOC_BC_ROT_POS_MODE_MASK)
	{
		default:
		case MOTORFOC_BC_ROT_POS_MODE_MANUAL   : * rotposmode = MOTORFOC_AP_ROTPOSMODE_MANUAL   ; break;
		case MOTORFOC_BC_ROT_POS_MODE_EMULATOR : * rotposmode = MOTORFOC_AP_ROTPOSMODE_EMULATOR ; break;
		case MOTORFOC_BC_ROT_POS_MODE_ENCODER  : * rotposmode = MOTORFOC_AP_ROTPOSMODE_ENCODER  ; break;
		case MOTORFOC_BC_ROT_POS_MODE_EVALUATOR: * rotposmode = MOTORFOC_AP_ROTPOSMODE_EVALUATOR; break;
		case MOTORFOC_BC_ROT_POS_MODE_HALLSNS  : * rotposmode = MOTORFOC_AP_ROTPOSMODE_HALLSNS  ; break;
		case MOTORFOC_BC_ROT_POS_MODE_RESOLVER : * rotposmode = MOTORFOC_AP_ROTPOSMODE_RESOLVER ; break;
		case MOTORFOC_BC_ROT_POS_MODE_EXTERNAL : * rotposmode = MOTORFOC_AP_ROTPOSMODE_EXTERNAL ; break;
	}
	
	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_set_mrt_drive(motorfoc *InstancePtr, int drive, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	if (drive)
	{
		ctl->glb.motcontrol &= ~(MOTORFOC_BC_mrt_rstang);
	}
	else
	{
		ctl->glb.motcontrol |= (MOTORFOC_BC_mrt_rstang);
	}

	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol);
	}

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_set_smo_drive(motorfoc *InstancePtr, int drive, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	if (drive)
	{
		ctl->glb.motcontrol &= ~(MOTORFOC_BC_smo_rstang);
	}
	else
	{
		ctl->glb.motcontrol |= (MOTORFOC_BC_smo_rstang);
	}

	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol);
	}

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_set_enc_drive(motorfoc *InstancePtr, int drive, int wnow)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
    
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
	if (drive)
	{
		ctl->glb.motcontrol &= ~(MOTORFOC_BC_enc_rstang);
	}
	else
	{
		ctl->glb.motcontrol |= (MOTORFOC_BC_enc_rstang);
	}
	
	if (wnow)
	{
		QD_MOTORFOC_DEFLADDR;
		MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol);
	}
	
	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_zerovphsx(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	QD_MOTORFOC_DEFLADDR;
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol | MOTORFOC_BC_zerovphsx);
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol & (~MOTORFOC_BC_zerovphsx));

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_zerovbusx(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	QD_MOTORFOC_DEFLADDR;
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol | MOTORFOC_BC_zerovbusx);
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol & (~MOTORFOC_BC_zerovbusx));

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_reseterr(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	QD_MOTORFOC_DEFLADDR;
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol | MOTORFOC_BC_reseterr);
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol & (~MOTORFOC_BC_reseterr));

	return XST_SUCCESS;
}

//-

int motorfoc_ap_ctl_zerocurr(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

	QD_MOTORFOC_DEFLADDR;
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol | MOTORFOC_BC_zerocurr);
	MOTORFOC_mWrite_motcontrol(QD_MOTORFOC_REFLADDR,ctl->glb.motcontrol & (~MOTORFOC_BC_zerocurr));

	return XST_SUCCESS;
}
