/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2018, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief motorfoc ap lib
/// \file motorfoc_apd.c
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2018

#include <stdio.h>
#include <math.h>

#include "xparameters.h"
#include "xil_types.h"
#include "xstatus.h"

#include "motorfoc_utl.h"
#include "motorfoc.h"
#include "motorfoc_apc.h"
#include "motorfoc_apd.h"

// ===========================================================================

#define LM_TWOPI  (M_PI * 2.0)
#define LM_SQRT3  (1.7320508075688772935274463415059)
#define LM_2X32   (4294967296.0)
#define LM_2X16   (65536.0)

// ===========================================================================

#define K_RPM_TO_HZ    (1.0f / 60.0f * (float)prm->mot.paipol)
#define K_HZ_TO_RPM    (1.0f * 60.0f / (float)prm->mot.paipol)

#define K_SPD32_TO_HZ  (1.0f / (float)LM_2X32 / prm->sys.ts)
#define K_HZ_TO_SPD32  (1.0f * (float)LM_2X32 * prm->sys.ts)

#define K_RAD_TO_ANG32 (1.0f / (float)LM_TWOPI * (float)LM_2X32)
#define K_ANG32_TO_RAD (1.0f * (float)LM_TWOPI / (float)LM_2X32)

#define K_RAD_TO_ANG16 (1.0f / (float)LM_TWOPI * (float)LM_2X16)

#define K_POS_TO_ROUND (1.0f / (float)(1 << MOTORFOC_POSITION_FRAQ_WIDTH) / (float)prm->mot.paipol)

// ===========================================================================

#define K_SPD32_TO_RPM (K_SPD32_TO_HZ * K_HZ_TO_RPM)
#define K_RPM_TO_SPD32 (K_RPM_TO_HZ * K_HZ_TO_SPD32)

// ===========================================================================

#define K_RAD_TO_DEG   (180.0f / (float)M_PI)
#define K_ANG16_TO_DEG (360.0f / (float)LM_2X16)
#define K_ANG32_TO_DEG (360.0f / (float)LM_2X32)

// ===========================================================================

int motorfoc_ap_prm_disp(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;
	
	int i;

	//-
	printf("\n");
	//-
	
	// sys
	printf("System parameters\n");
	printf(" ts          =%e\n",prm->sys.ts   );

	printf(" V-Phs Gain  =%e\n",prm->sys.vphs_gain);
	printf(" V-Bus Gain  =%e\n",prm->sys.vbus_gain);
	printf(" I-Phs Gain  =%e\n",prm->sys.iphs_gain);
	printf(" I-Bus Gain  =%e\n",prm->sys.ibus_gain);

	// Bus filter
	printf("Bus Filter\n");
	printf(" fcut1       =%.1f\n",prm->sys.xbus_fcut1);
	printf(" fcut2       =%.1f\n",prm->sys.xbus_fcut1);

	// Motor Phase filter
	printf("Phases Filter\n");
	printf(" fcut1       =%.1f\n",prm->sys.xphs_fcut1);
	printf(" fcut2       =%.1f\n",prm->sys.xphs_fcut1);

	// motor
	printf("Motor parameters\n");
	printf(" manufacturer=%s\n",(char *)prm->mot.manufacturer);
	printf(" motorcode   =%s\n",(char *)prm->mot.motorcode);
	printf(" windings    =%u\n",prm->mot.windings);
	printf(" paipol      =%u\n",prm->mot.paipol);
	printf(" inductance  =%e\n",prm->mot.inductance);
	printf(" resistance  =%e\n",prm->mot.resistance);
	printf(" bemfke      =%e\n",prm->mot.bemfke);
	printf(" torquemax   =%e\n",prm->mot.torquemax);
	printf(" torquekc    =%e\n",prm->mot.torquekc);
	printf(" maxspeed    =%.1f\n",prm->mot.maxspeed);
	printf(" iscurrmax   =%.3f\n",prm->mot.iscurrmax);
	printf(" iscurrhigh  =%.3f\n",prm->mot.iscurrhigh);

	printf(" controlmode =%u\n",prm->mot.controlmode);
	printf(" rotposmode  =%u\n",prm->mot.rotposmode);

	// speed loop
	printf("Speed Loop\n");
	printf(" pgain       =%e\n",prm->slp.pgain);
	printf(" igain       =%e\n",prm->slp.igain);

	// PI regulator
	printf("Current Loop\n");
	printf(" pgain       =%e\n",prm->pi.pgain );
	printf(" igain       =%e\n",prm->pi.igain );

	// MOD
	printf("MOD\n");
	printf(" mod2angskw  =%e Rad =%.1f Deg\n",
			prm->mod.mod2angskw,
			prm->mod.mod2angskw * K_RAD_TO_DEG);
	
	// PWM
	printf("PWM\n");
	printf(" freq        =%.1f\n",prm->pwm.freq );
	printf(" gain        =%e\n",prm->pwm.gain );
	printf(" mdmax       =%e\n",prm->pwm.mdmax);
	printf(" ctrls       =x%X\n",prm->pwm.ctrls);

	// rpfm
	printf("RPFM\n");
	printf(" pwdth       =%e\n",prm->rpfm.pwdth);
	printf(" ctrls       =x%X\n",prm->rpfm.ctrls);

	// Angle Manual Mode
	printf("Manual Mode\n");
	printf(" s-fcut1     =%.1f\n",prm->man.spfcut1);
	printf(" s-fcut2     =%.1f\n",prm->man.spfcut2);

	// Angle Scalar Mode
	printf("Scalar Mode\n");
	printf(" accmax      =%e\n",prm->mrt.accmax );
	printf(" o-fcut1     =%.1f\n",prm->mrt.sofcut1);
	printf(" o-fcut2     =%.1f\n",prm->mrt.sofcut2);
	printf(" ctrls       =x%x\n",prm->mrt.ctrls);
	printf(" s-fcut1     =%.1f\n",prm->mrt.spfcut1);
	printf(" s-fcut2     =%.1f\n",prm->mrt.spfcut2);

	// Angle Encoder Mode
	printf("Encoder Mode\n");
	printf(" cyprnd      =%u\n",prm->enc.cyprnd);
	printf(" ctrls       =x%X\n",prm->enc.ctrls);
	printf(" s-fcut1     =%.1f\n",prm->enc.spfcut1);
	printf(" s-fcut2     =%.1f\n",prm->enc.spfcut2);
	
	// Angle Sliding Mode Mode Observer
	printf("SMO Mode\n");
	printf(" is_maxerr   =%e\n",prm->smo.is_maxerr);
	printf(" zs_maxval   =%e\n",prm->smo.zs_maxval);
	printf(" es1_fcut    =%.1f\n",prm->smo.es1_fcut );
	printf(" es2_fclow   =%.1f\n",prm->smo.es2_fclow);
	printf(" es2_fcmul   =%.3f\n",prm->smo.es2_fcmul);
	printf(" angofs      =%e Rad =%.1f Deg\n",
			prm->smo.angofs,
			prm->smo.angofs * K_RAD_TO_DEG);
	printf(" ctrls       =x%X\n",prm->smo.ctrls);
	printf("\n");
	printf(" s-fcut1     =%.1f\n",prm->smo.spfcut1);
	printf(" s-fcut2     =%.1f\n",prm->smo.spfcut2);

	// Angle Hall Sensor Mode
	printf("Hall Sensor Mode\n");
	printf(" ctolim      =%e\n",prm->hls.ctolim);
	printf(" anglelut     : ");
	for(i=0;i<MOTORFOC_HLS_DPR_ANGLE_ELEM;i++)
		printf("%.1f ",prm->hls.anglelut[i] * K_RAD_TO_DEG);
	printf(" s-fcut1     =%.1f\n",prm->hls.spfcut1);
	printf(" s-fcut2     =%.1f\n",prm->hls.spfcut2);
	
	// Angle Resolver Mode
	printf("Resolver Sensor Mode\n");
	printf(" paipol      =%u\n",prm->rsv.paipol);
	printf(" angofs      =%e Rad =%.1f Deg\n",
			prm->rsv.angofs,
			prm->rsv.angofs * K_RAD_TO_DEG);
	printf(" s-fcut1     =%.1f\n",prm->rsv.spfcut1);
	printf(" s-fcut2     =%.1f\n",prm->rsv.spfcut2);
	
	// Angle External Mode
	printf("External Sensor Mode\n");
	printf(" s-fcut1     =%.1f\n",prm->ext.spfcut1);
	printf(" s-fcut2     =%.1f\n",prm->ext.spfcut2);

	//-
	printf("\n");
	//-
	
	return XST_SUCCESS;
}

int motorfoc_ap_sts_disp(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;

	// status
	motorfoc_ap_sts_t * sts = &ApPtr->sts;
	
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;

    // not configured
	if (prm->sys.ts == 0.0f) return XST_FAILURE;
	
	//-
	printf("\n");
	//-
	
	// glb
	printf("Status Global\n");
	printf(" ip_ident = x%X\n",(unsigned)sts->glb.ip_ident);
	
	printf(" tsclocks  = %u  : ",(unsigned)sts->glb.tsclocks);
	printf(" motstatus = x%X : ",(unsigned)sts->glb.motstatus);
	
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_hwkill          ) ? "hwkill,"      :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_hwfail          ) ? "hwfail,"      :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_ovi_iphs_a      ) ? "ovi_iphs_a,"  :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_ovi_iphs_b      ) ? "ovi_iphs_b,"  :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_ovi_iphs_c      ) ? "ovi_iphs_c,"  :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_ovi_ibus_x      ) ? "ovi_ibus_x,"  :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_syserr          ) ? "syserr,"      :"");
	printf("%s",
		(sts->glb.motstatus & (MOTORFOC_BS_rpfmzmd1 | MOTORFOC_BS_rpfmzmd0)) == (MOTORFOC_BS_rpfmzmd1 | MOTORFOC_BS_rpfmzmd0) ? "rpfm_sqr," :
		(sts->glb.motstatus & (MOTORFOC_BS_rpfmzmd1 | MOTORFOC_BS_rpfmzmd0)) == (MOTORFOC_BS_rpfmzmd1                       ) ? "rpfm_hex," :
		(sts->glb.motstatus & (MOTORFOC_BS_rpfmzmd1 | MOTORFOC_BS_rpfmzmd0)) == (                       MOTORFOC_BS_rpfmzmd0) ? "rpfm_snx," : "rpfm_idl,");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_modtype0        ) ? "RPFM,"    :"PWM,");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_spd_fwdir       ) ? "spd_fwdir,"   :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_spd_moving      ) ? "spd_moving,"  :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_pwm_mdovf       ) ? "pwm_mdovf,"   :"");
	printf("%s", (sts->glb.motstatus & MOTORFOC_BS_ismodover       ) ? "ismodover,"   :"");

	printf("\n");

	printf( " I-Phs_a = %d %.3f A\n", (int)sts->phy.val_iphs_a, (float)sts->phy.val_iphs_a * prm->sys.iphs_gain);
	printf( " I-Phs_b = %d %.3f A\n", (int)sts->phy.val_iphs_b, (float)sts->phy.val_iphs_b * prm->sys.iphs_gain);
	printf( " I-Phs_c = %d %.3f A\n", (int)sts->phy.val_iphs_c, (float)sts->phy.val_iphs_c * prm->sys.iphs_gain);
	printf( " I-Bus_x = %d %.3f A\n", (int)sts->phy.val_ibus_x, (float)sts->phy.val_ibus_x * prm->sys.ibus_gain);
	printf( " V-Phs_a = %d %.3f A\n", (int)sts->phy.val_vphs_a, (float)sts->phy.val_vphs_a * prm->sys.vphs_gain);
	printf( " V-Phs_b = %d %.3f A\n", (int)sts->phy.val_vphs_b, (float)sts->phy.val_vphs_b * prm->sys.vphs_gain);
	printf( " V-Phs_c = %d %.3f A\n", (int)sts->phy.val_vphs_c, (float)sts->phy.val_vphs_c * prm->sys.vphs_gain);
	printf( " V-Bus_x = %d %.3f A\n", (int)sts->phy.val_vbus_x, (float)sts->phy.val_vbus_x * prm->sys.vbus_gain);
	printf( " V-Phs_n = %d %.3f A\n", (int)sts->phy.val_vphs_n, (float)sts->phy.val_vphs_n * prm->sys.vphs_gain);

	printf( " GU switch # = %u\n", (unsigned)sts->phy.guswcnt);

	// speed eval
	printf("Speed Eval\n");
	printf(" spd_speed = %d = %.1f Hz = %.1f RPM\n",
		(int)sts->spd.spd_speed,
		(float)sts->spd.spd_speed * K_SPD32_TO_HZ,
		(float)sts->spd.spd_speed * K_SPD32_TO_RPM);
		
	// rotor angle
	printf("Rotor Angle\n");
	printf(" rot_angle = %u = %.1f Deg\n",
		(unsigned)sts->rot.rot_angle,
		(float)sts->rot.rot_angle * K_ANG32_TO_DEG);
		
	// smo
	printf("Sliding Mode Observer\n");
	printf(" smo_es2_kflt = %u = %.1f Hz\n",
			(unsigned)sts->smo.smo_es2_kflt,
		motorfoc_lpf1_IK2Fcut(prm->sys.ts, sts->smo.smo_es2_kflt));
		
	printf(" smo_bemf_p = %u = %.1f Deg\n",
			(unsigned)sts->smo.smo_bemf_p,
		(float)sts->smo.smo_bemf_p * K_ANG32_TO_DEG);
		
	printf(" smo_bemf_m = %u = %e V\n",
			(unsigned)sts->smo.smo_bemf_m,
		(float)sts->smo.smo_bemf_m * prm->sys.vbus_gain);
		
	printf(" smo_angle = %u = %.1f Deg\n",
			(unsigned)sts->smo.smo_angle,
		(float)sts->smo.smo_angle * K_ANG32_TO_DEG);

		
	// rotor position
	printf("Rotor Position Act.\n");
	printf(" pos_position = %d = %.2f Rnd\n",
			(int)sts->pos.pos_position,
		(float)sts->pos.pos_position * K_POS_TO_ROUND);
		
	// Encoder
	printf("Encoder\n");
	printf(" enc_index = %u\n",(unsigned)sts->enc.enc_index);
	printf(" enc_phase = %d\n",(int)sts->enc.enc_phase);
	printf(" enc_phcpt = %d\n",(int)sts->enc.enc_phcpt);
	printf(" enc_angle = %u = %.1f Deg\n",
			(unsigned)sts->enc.enc_angle,
		(float)sts->enc.enc_angle * K_ANG32_TO_DEG);

	// Scalar mode
	printf("Scalar Mode\n");
	
	printf(" mrt_spdout = %d = %.1f Hz = %.1f RPM\n",
			(int)sts->mrt.mrt_spdout,
		(float)sts->mrt.mrt_spdout * K_SPD32_TO_HZ,
		(float)sts->mrt.mrt_spdout * K_SPD32_TO_RPM);
	printf(" mrt_angle = %u = %.1f Deg\n",
			(unsigned)sts->mrt.mrt_angle,
		(float)sts->mrt.mrt_angle * K_ANG32_TO_DEG);
		
	// Hall Sensor mode
	printf("Hall Sensor mode\n");
	printf(" hls_angle = %u = %.1f Deg\n",
			(unsigned)sts->hls.hls_angle,
		(float)sts->hls.hls_angle * K_ANG32_TO_DEG);
		
	// Resolver Sensor mode
	printf("Resolver Sensor mode\n");
	printf(" rsv_angle = %u = %.1f Deg\n",
			(unsigned)sts->rsv.rsv_angle,
		(float)sts->rsv.rsv_angle * K_ANG32_TO_DEG);
		
	// External Sensor mode
	printf("External Sensor mode\n");
	printf(" ext_angle = %u = %.1f Deg\n",
			(unsigned)sts->ext.ext_angle,
		(float)sts->ext.ext_angle * K_ANG32_TO_DEG);
		
	// Incoming current
	printf("Input Is\n");
	printf(" ismodval = %u = %.3f A\n",(unsigned)sts->iis.ismodval,
		(float)sts->iis.ismodval * prm->sys.iphs_gain);
	printf(" isangval = %u = %.1f Deg\n",(unsigned)sts->iis.isangval,
		(float)sts->iis.isangval * K_ANG32_TO_DEG);
		
	// Outgoing voltage
	printf("Output Vs\n");
	printf(" vsmodval = %u = %.3f V\n",
			(unsigned)sts->ovs.vsmodval,
		(float)sts->ovs.vsmodval * prm->sys.vbus_gain);
	printf(" vsangval = %u = %.1f Deg\n",
			(unsigned)sts->ovs.vsangval,
		(float)sts->ovs.vsangval * K_ANG32_TO_DEG);
		
	// sinwave modulation index
	float modixd = ((float)sts->ovs.vsmodval / (float)sts->phy.val_vbus_x) * 2.0f;
	printf("Mod Index\n");
	printf(" sin mdx = %.3f %%\n",modixd * 100.0f);
		
		
	// PWM
	float pwmmdxf = (ctl->pwm.pwm_presc > 0) ? (100.0f / (float)ctl->pwm.pwm_presc) : 0.0f;
	printf("PWM\n");
	printf(" pwm_mdval = %u = %.1f %%\n",
			(unsigned)sts->pwm.pwm_mdval,
		(float)sts->pwm.pwm_mdval * pwmmdxf);
		

	//-
	printf("\n");
	//-
	
	return XST_SUCCESS;
}

int motorfoc_ap_ctl_disp(motorfoc *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	motorfoc_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// parameters
	motorfoc_ap_prm_t * prm = &ApPtr->prm;
	
	// control
	motorfoc_ap_ctl_t * ctl = &ApPtr->ctl;
	
    // not configured
	if (prm->sys.ts == 0.0f) return XST_FAILURE;
	
	int i;

	//-
	printf("\n");
	//-
	
	// glb
	printf("Control Global\n");
	
	printf(" motcontrol = x%X : ",(unsigned)ctl->glb.motcontrol);
	
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_smo_usevsact    ) ? "smo_usevsact," :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_slp_mrt2spd     ) ? "slp_mrt2spd,"  :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_slp_spd2trq     ) ? "slp_spd2trq,"  :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_enc_xmcha       ) ? "enc_xmcha,"    :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_enc_xmchb       ) ? "enc_xmchb,"    :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_enc_xmchi       ) ? "enc_xmchi,"    :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_enc_inten       ) ? "enc_inten,"    :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_enc_rstang      ) ? "enc_rstang,"   :"");

	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_mrt_rstang      ) ? "mrt_rstang,"   :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_smo_rstang      ) ? "smo_rstang,"   :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_fbk_angdef      ) ? "fbk_angdef,"   :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_pi_deafmd       ) ? "pi_deafmd,"    :"");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_is3phases       ) ? "3ph,"          :"2ph,");

	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_modlevels0      ) ? "3," :"2,");
	
	printf("%s",
		(ctl->glb.motcontrol & MOTORFOC_BC_ROT_POS_MODE_MASK) == MOTORFOC_BC_ROT_POS_MODE_MANUAL    ? "man," :
		(ctl->glb.motcontrol & MOTORFOC_BC_ROT_POS_MODE_MASK) == MOTORFOC_BC_ROT_POS_MODE_EMULATOR  ? "scl," :
		(ctl->glb.motcontrol & MOTORFOC_BC_ROT_POS_MODE_MASK) == MOTORFOC_BC_ROT_POS_MODE_ENCODER   ? "enc," :
		(ctl->glb.motcontrol & MOTORFOC_BC_ROT_POS_MODE_MASK) == MOTORFOC_BC_ROT_POS_MODE_EVALUATOR ? "smo," :
		(ctl->glb.motcontrol & MOTORFOC_BC_ROT_POS_MODE_MASK) == MOTORFOC_BC_ROT_POS_MODE_HALLSNS   ? "hls," :
		(ctl->glb.motcontrol & MOTORFOC_BC_ROT_POS_MODE_MASK) == MOTORFOC_BC_ROT_POS_MODE_RESOLVER  ? "rsv," :
		(ctl->glb.motcontrol & MOTORFOC_BC_ROT_POS_MODE_MASK) == MOTORFOC_BC_ROT_POS_MODE_EXTERNAL  ? "ext," : "man,");

	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_modtype0        ) ? "RPFM," :"PWM,");
	printf("%s",(ctl->glb.motcontrol & MOTORFOC_BC_coilenab        ) ? "coilenab,"    :"");
	
	printf("\n");

	printf(" paipol = %u\n",ctl->glb.paipol);
	
	// Physical
	printf("Input Filter\n");

	printf(" xbus_k/fcut1 = %u = %.1f Hz\n",(unsigned)ctl->phy.xbus_fk1,motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->phy.xbus_fk1));
	printf(" xbus_k/fcut2 = %u = %.1f Hz\n",(unsigned)ctl->phy.xbus_fk2,motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->phy.xbus_fk2));

	printf(" xphs_k/fcut1 = %u = %.1f Hz\n",(unsigned)ctl->phy.xphs_fk1,motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->phy.xphs_fk1));
	printf(" xphs_k/fcut2 = %u = %.1f Hz\n",(unsigned)ctl->phy.xphs_fk2,motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->phy.xphs_fk2));
	
	// Bus current limit
	printf("I-Bus Limit = %u = %.3f A\n",(unsigned)ctl->phy.ibus_limit,(float)ctl->phy.ibus_limit * prm->sys.ibus_gain);

	// Phase current limit
	printf("I-Phs Limit = %u = %.3f A\n",(unsigned)ctl->phy.iphs_limit,(float)ctl->phy.iphs_limit * prm->sys.iphs_gain);

	printf("GU Dead Time = %u = %.1f ns\n",ctl->phy.deadtval,(float)(ctl->phy.deadtval + 1) / (float)InstancePtr->Config->systemclock * 1.0e9);
	
		
	// Speed eval
	printf("Speed Eval\n");
	printf(" spd_fktau1 = %u = %.1f Hz\n",
			(unsigned)ctl->spd.spd_fktau1,
		motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->spd.spd_fktau1));
	printf(" spd_fktau2 = %u = %.1f Hz\n",
			(unsigned)ctl->spd.spd_fktau2,
		motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->spd.spd_fktau2));
		
	
	// rotor angle
	printf("Rotor Angle\n");
	printf(" rot_angdef = %u = %.1f Deg\n",
			(unsigned)ctl->rot.rot_angdef,
		(float)ctl->rot.rot_angdef * K_ANG32_TO_DEG);
		
	
	// smo
	printf("SMO\n");
	printf(" smo_vs_mult  = %u = %e\n",
			(unsigned)ctl->smo.smo_vs_mult,
		(float)ctl->smo.smo_vs_mult / (float)(1 << InstancePtr->Config->smo_zs_dln2));
	printf(" smo_is_mult  = %u = %e\n",
			(unsigned)ctl->smo.smo_is_mult,
		(float)ctl->smo.smo_is_mult / (float)(1 << InstancePtr->Config->smo_zs_dln2));
	printf(" smo_zs_max   = %u =%.2f V\n",
			(unsigned)ctl->smo.smo_zs_max,
		(float)ctl->smo.smo_zs_max * prm->sys.vbus_gain);
	printf(" smo_es1_kflt = %u = %.1f Hz\n",
			(unsigned)ctl->smo.smo_es1_kflt,
		motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->smo.smo_es1_kflt));
	printf(" smo_es2_kflo = %u = %.1f Hz\n",
			(unsigned)ctl->smo.smo_es2_kflo,
		motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->smo.smo_es2_kflo));
	printf(" smo_es2_kfmx = %u =%e\n",
			(unsigned)ctl->smo.smo_es2_kfmx,
		(float)ctl->smo.smo_es2_kfmx / (float)(1 << InstancePtr->Config->smo_f2_dln2) * K_HZ_TO_SPD32);
	printf(" smo_angofs   = %.1f\n",(float)ctl->smo.smo_angofs * K_ANG16_TO_DEG);

	// Speed Loop
	printf("Speed Loop\n");

	printf(" slp_spdset = %d = %.1f Hz = %.1f RPM\n",
			(int)ctl->slp.slp_spdset,
		(float)ctl->slp.slp_spdset * K_SPD32_TO_HZ,
		(float)ctl->slp.slp_spdset * K_SPD32_TO_RPM);
		
	// gain conversion
	float kipr2slpgp = 1.0f * K_RPM_TO_SPD32 * prm->sys.iphs_gain / (float)(1 << InstancePtr->Config->slp_pro_dln2);
	float kipr2slpgi = 1.0f * K_RPM_TO_SPD32 * prm->sys.iphs_gain / (float)(1 << InstancePtr->Config->slp_int_dln2);
	
	printf(" slp_kmpro  = %08X = %e\n",(unsigned)ctl->slp.slp_kmpro.u, ctl->slp.slp_kmpro.f * kipr2slpgp);
	printf(" slp_kmint  = %08X = %e\n",(unsigned)ctl->slp.slp_kmint.u, ctl->slp.slp_kmint.f * kipr2slpgi);
		
	printf(" slp_outlim = %u = %.3f A\n",
			(unsigned)ctl->slp.slp_outlim,
		(float)ctl->slp.slp_outlim * prm->sys.iphs_gain);
		
	float kipr2slpxy = 1.0f / (float)MOTORFOC_SLP_YX_MODULO;
	printf(" slp_kmultx = %.3f\n",(float)ctl->slp.slp_kmultx * kipr2slpxy);
	printf(" slp_kmulty = %.3f\n",(float)ctl->slp.slp_kmulty * kipr2slpxy);
		
		
	// Hall Sensor
	printf("Hall Sensor\n");
	printf(" hls_ctolim = %u = %e\n",
			(unsigned)ctl->hls.hls_ctolim,
		(float)ctl->hls.hls_ctolim * prm->sys.ts);
	printf(" hls_dprangle : ");
	for(i=0;i<MOTORFOC_HLS_DPR_ANGLE_ELEM;i++)
		printf("%.1f ",(float)ctl->hls.hls_dprangle[i] * K_ANG16_TO_DEG);
	printf("\n");
		

		
	// Encoder
	printf("Encoder\n");
	printf(" enc_cyprnd = %u\n",(unsigned)ctl->enc.enc_cyprnd);
	printf(" enc_angphs = %u = %e Deg\n",
			(unsigned)ctl->enc.enc_angphs,
		(float)ctl->enc.enc_angphs * K_ANG32_TO_DEG);
		
	// Resolver
	printf("Resolver\n");
	printf(" rsv_m2rppk = %u\n",ctl->rsv.rsv_m2rppk);
	printf(" rsv_angofs = %d = %e Deg\n",ctl->rsv.rsv_angofs,
		(float)ctl->rsv.rsv_angofs * K_ANG16_TO_DEG);


	// Open Loop
	printf("Scalar Mode\n");

	printf(" mrt_speed  = %d = %.1f Hz = %.1f RPM\n",
			(int)ctl->mrt.mrt_speed,
		(float)ctl->mrt.mrt_speed * K_SPD32_TO_HZ,
		(float)ctl->mrt.mrt_speed * K_SPD32_TO_RPM);
		
	float kipr2mrtacc = K_SPD32_TO_RPM / prm->sys.ts / (float)(1 << InstancePtr->Config->mrt_accmax_dln2);
	
	printf(" mrt_accmax = %u = %.2f\n",
			(unsigned)ctl->mrt.mrt_accmax,
		(float)ctl->mrt.mrt_accmax * kipr2mrtacc);
	printf(" mrt_fktau1 = %u = %.1f Hz\n",
			(unsigned)ctl->mrt.mrt_fktau1,
		motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->mrt.mrt_fktau1));
	printf(" mrt_fktau2 = %u = %.1f Hz\n",
			(unsigned)ctl->mrt.mrt_fktau2,
		motorfoc_lpf1_IK2Fcut(prm->sys.ts, ctl->mrt.mrt_fktau2));
		
	// Incoming current limit
	printf("Input Is limit\n");
	printf(" ismodmax = %u = %.3f A\n",
			(unsigned)ctl->iis.ismodmax,
		(float)ctl->iis.ismodmax * prm->sys.iphs_gain);

	// Current Loop
	printf("Current Loop PI\n");
	printf(" pi_setvalx = %d = %.3f A\n",
			(int)ctl->pi.pi_setvalx,
		(float)ctl->pi.pi_setvalx * prm->sys.iphs_gain);
	printf(" pi_setvaly = %d = %.3f A\n",
			(int)ctl->pi.pi_setvaly,
		(float)ctl->pi.pi_setvaly * prm->sys.iphs_gain);
		
	float kipr2pikp = prm->sys.vbus_gain / prm->sys.iphs_gain / (float)(1 << InstancePtr->Config->pi_erpro_dln2);
	float kipr2piki = prm->sys.vbus_gain / prm->sys.iphs_gain / (float)(1 << InstancePtr->Config->pi_erint_dln2);

	printf(" pi_kmpro   = %u = %e\n",
			(unsigned)ctl->pi.pi_kmpro,
		(float)ctl->pi.pi_kmpro * kipr2pikp);

	printf(" pi_kmint   = %u = %e\n",
			(unsigned)ctl->pi.pi_kmint,
		(float)ctl->pi.pi_kmint * kipr2piki);
		
		
	// Modulator
	printf("Modulator\n");
	printf(" mod2angskw = %d = %e Deg\n",ctl->mod.mod2angskw,
		(float)ctl->mod.mod2angskw * K_ANG16_TO_DEG);

	// PWM
	printf("PWM\n");
	printf(" pwm_presc = %u = %.3f Hz\n",
			(unsigned)ctl->pwm.pwm_presc,
		(float)InstancePtr->Config->systemclock / (float)(ctl->pwm.pwm_presc + 1) / 2.0f);
	printf(" pwm_kmod  = %u = %.3f %%\n",
			(unsigned)ctl->pwm.pwm_kmod,
		ctl->pwm.pwm_kmod / 2.0f / (float)(ctl->pwm.pwm_presc) / (float)(1 << MOTORFOC_PWM_KMOD_DLN2) * 100.0f);
	printf(" pwm_mdmax = %u = %.1f %%\n",
			(unsigned)ctl->pwm.pwm_mdmax,
		ctl->pwm.pwm_mdmax / (float)(ctl->pwm.pwm_presc) * 100.0f);
	printf(" pwm_ctrl = x%X cmmofs=%u table=%u\n",
				(unsigned)ctl->pwm.pwm_ctrl,
				(ctl->pwm.pwm_ctrl & MOTORFOC_PWM_ctrl_cmmofs) != 0,
				(ctl->pwm.pwm_ctrl / MOTORFOC_PWM_ctrl_table) & 3);

	// RPFM
	
	int rpfm_lop = ((ctl->rpfm.rpfm3p_ctrl / MOTORFOC_RPFM3P_ctrl_mdpr) & 0x0F) + 1;
	int rpfm_hip = ((ctl->rpfm.rpfm3p_ctrl / MOTORFOC_RPFM3P_ctrl_mdkd) & 0xFF) + 1;
	float rpfmpw = 1.0f / (float)InstancePtr->Config->systemclock * (float)(rpfm_lop * rpfm_hip);
	
	printf("RPFM\n");
	printf(" rpfm3p_ctrl = x%X : %s%s  pw = %.3f uS\n",
			(unsigned)ctl->rpfm.rpfm3p_ctrl,
		(ctl->rpfm.rpfm3p_ctrl & MOTORFOC_RPFM3P_ctrl_v7h) ? "v7h,":"",
		(ctl->rpfm.rpfm3p_ctrl & MOTORFOC_RPFM3P_ctrl_v07n) ? "v07n":"",
		rpfmpw * 1.0e6);
	
	//-
	printf("\n");
	//-
	
	return XST_SUCCESS;
}

