/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2018, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief motorfoc ap lib
/// \file motorfoc_utl.c
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2018

#include "xparameters.h"
#include "xil_types.h"
#include "xstatus.h"

#include "motorfoc_utl.h"

#include <math.h>


int motorfoc_abs_int(const int v)
{
	return (v >= 0) ? (v) : (-v);
}

int motorfoc_max_int(const int a, const int b)
{
	return (a > b) ? a : b;
}

int motorfoc_min_int(const int a, const int b)
{
	return (a < b) ? a : b;
}

//-

#define MOTORFOC_sign_extend(v,b) (((v) & (1 << ((b)-1))) ? ((v) | (-(1 << (b)))) : ((v) & ((1 << (b))-1)))

//- sign extend value
int motorfoc_adjval_s18(const int v)
{
	return MOTORFOC_sign_extend(v,18);
}

// - adj floating point to integer value

int motorfoc_clip_round_int(const float v, const int l, const int h)
{
	if (v >= (float)h)
		return h;

	if (v <= (float)l)
		return l;

	int t = 0;

	if (v > 0.0f)
	{
		t = (int)(v + 0.5f);
	}

	if (v < 0.0f)
	{
		t = (int)(v - 0.5f);
	}

	if (t >= h)
		return h;

	if (t <= l)
		return l;

	return t;
}

int motorfoc_clip_round_s18(const float v)
{
	if (v == 0.0f)
		return 0;
	#define MIPL_S18_HI ((1 << 17) - 1)
	#define MIPL_S18_LO (-(1 << 17))
	if (v > 0.0f)
	{
		if (v >= (float)MIPL_S18_HI)
			return MIPL_S18_HI;
		int t = (int)(v + 0.5f);
		if (t >= MIPL_S18_HI)
			return MIPL_S18_HI;
		return t;
	}
	if (v <= (float)MIPL_S18_LO)
		return MIPL_S18_LO;
	int t = (int)(v - 0.5f);
	if (t <= MIPL_S18_LO)
		return MIPL_S18_LO;
	return t;
}

uint32_t motorfoc_clip_round_u17(const float v)
{
	if (v <= 0.0f)
		return 0;
	#define MIPL_U17_HI ((1 << 17) - 1)
	if (v >= (float)MIPL_U17_HI)
		return MIPL_U17_HI;
	uint32_t t = (uint32_t)(v + 0.5f);
	if (t >= MIPL_U17_HI)
		return MIPL_U17_HI;
	return t;
}

uint32_t motorfoc_clip_round_u32(const float v)
{
	if (v <= 0.0)
		return 0;
	#define MIPL_U32_HI 4294967295UL
	if (v >= (float)MIPL_U32_HI)
		return MIPL_U32_HI;
	float q = (float)(v + 0.5f);
	if (q >= (float)MIPL_U32_HI)
		return MIPL_U32_HI;
	uint32_t t = (uint32_t)q;
	return t;
}

//-

//- filter lpf1

#define LM_TWOPI_F  ((float)(M_PI * 2.0))


float motorfoc_lpf1_T2Fcut(float Ts, float T)
{
	float n = 1.0f / logf((1.0f + T) / T);
	float tau = n * Ts;
	return 1.0f / (LM_TWOPI_F * tau);
}

float motorfoc_lpf1_Fcut2T(float Ts, float Fcut)
{
	float tau = 1.0f / (LM_TWOPI_F * Fcut);
	float n = tau / Ts;
	return 1.0f / (expf(1.0f / n) - 1.0f);
}

float motorfoc_lpf1_T2K(float T)
{
	return 1.0f / (1.0f + T);
}

float motorfoc_lpf1_K2T(float K)
{
	return (1.0f - K) / K;
}

#define LPF1_LPF_MULK_VAL (1 << 17)

uint32_t motorfoc_lpf1_Fcut2IK(float Ts, float Fcut)
{
    float T = motorfoc_lpf1_Fcut2T(Ts, Fcut);
    float K = motorfoc_lpf1_T2K(T);
	uint32_t ik = motorfoc_clip_round_u17(K * (float)LPF1_LPF_MULK_VAL);
	return ik;
}

float motorfoc_lpf1_IK2Fcut(float Ts, uint32_t ik)
{
    float K = (float)ik / (float)LPF1_LPF_MULK_VAL;
	if (K >= 1.0f) K = 1.0f;
	if (K == 0.0f) return 0.0f;
    float T = motorfoc_lpf1_K2T(K);
    float Fcut = motorfoc_lpf1_T2Fcut(Ts, T);
    return Fcut;
}

//-

