/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ds1820 application engine
/// \file ds1820ape.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

#if SYS_DS1820_NUM > 0

#include "xx_timer.h"

int ds1820_ape_init(void)
{
	int rc = XST_SUCCESS;

	ds1820_env_t * ep = &g_ds1820_env_v;

	u16 id;
	for(id = 0;id < SYS_DS1820_NUM; id++)
	{
		ds1820 * InstancePtr = &g_ds1820_v[id];
		rc = ds1820_Initialize(InstancePtr,id);
		if (XST_SUCCESS != rc)
			return XST_FAILURE;
	}

	ep->tconvmax = DS1820_TCONV_MAX_MS * (xx_timer_cnt_pser_sec() / 1000);
	ep->deadtime = ep->tconvmax * 5;
	ep->idscan = 0;

	// printf("deadtime=%lu tconvmax=%lu\n",ep->deadtime,ep->tconvmax);

	int i;
	for(i = 0; i < DS1820_SENSORS_MAX; i++)
	{
		ds1820_sns_t * ps = & ep->sns[i];
		ps->alive = 0;
	}

	return XST_SUCCESS;
}

static void ds1820_ape_purge(void)
{
	u32 tnow = xx_timer_cnt_val();

	ds1820_env_t * ep = &g_ds1820_env_v;
	int i;
	for(i = 0; i < DS1820_SENSORS_MAX; i++)
	{
		ds1820_sns_t * ps = & ep->sns[i];
		if (ps->alive)
		{
			u32 age = tnow - ps->tstamp;
			if (age >= ep->deadtime)
			{
				ps->alive = 0;
				//printf("sns %d dead\n",i);
			}
		}
	}
}

static void ds1820_ape_update(void)
{
	ds1820_env_t * ep          = &g_ds1820_env_v;
	ds1820       * InstancePtr = &g_ds1820_v[ep->idscan];
	//ds1820ape_t  * dp          = &g_ds1820ape_v[ep->idscan];

	u32 tnow = xx_timer_cnt_val();
	float temp;
	ds1820_ap_temp_stdres(InstancePtr,&temp);

	int i;
	for(i = 0; i < DS1820_SENSORS_MAX; i++)
	{
		ds1820_sns_t * ps = & ep->sns[i];
		if (0 == memcmp(ps->ident,InstancePtr->ident,DS1820_IDENT_SIZE))
		{
			ps->temp   = temp;
			ps->tstamp = tnow;
			ps->alive  = 1;
			return;
		}
	}
	for(i = 0; i < DS1820_SENSORS_MAX; i++)
	{
		ds1820_sns_t * ps = & ep->sns[i];
		if (!ps->alive)
		{
			memcpy(ps->ident,InstancePtr->ident,DS1820_IDENT_SIZE);
			ps->temp   = temp;
			ps->tstamp = tnow;
			ps->alive  = 1;
			return;
		}
	}
}

int ds1820_ape_proc(void)
{
	int rc = XST_SUCCESS;

	ds1820_ape_purge();

	ds1820_env_t * ep = &g_ds1820_env_v;

	ep->idscan = (ep->idscan + 1) % SYS_DS1820_NUM;

	ds1820      * InstancePtr = &g_ds1820_v[ep->idscan];
	ds1820ape_t * dp          = &g_ds1820ape_v[ep->idscan];

	rc = ds1820_Process_Run(InstancePtr,DS1820_CMD_RUN);
	if (XST_SUCCESS != rc)
		return XST_FAILURE;


	switch(ds1820_Process_Event(InstancePtr))
	{
		default:
			{
			}
			break;
		case DS1820_EVENT_NONE:
			{
			}
			break;
		case DS1820_EVENT_TCONV_STARTED:
			{
				dp->tconvst = xx_timer_cnt_val();
			}
			break;
		case DS1820_EVENT_TCONV_RUNNING:
			{
				u32 te = xx_timer_cnt_val();
				u32 tx = te - dp->tconvst;
				if (tx >= ep->tconvmax)
				{
					ds1820_Process_Run(InstancePtr,DS1820_CMD_KICK);
				}
			}
			break;
		case DS1820_EVENT_DEVICE_FOUND:
			{
				//ds1820_ap_ident_disp(InstancePtr);
			}
			break;
		case DS1820_EVENT_TEMP_READED:
			{
				//ds1820_ap_temp_disp(InstancePtr);
				//printf("\n");
				ds1820_ape_update();
			}
			break;
		case DS1820_EVENT_END_OF_SCAN:
			{
				//printf("end-of-loop\n");
			}
			break;
	}

	return XST_SUCCESS;
}

int ds1820_ape_get_temp(u8 * idx, float * temp)
{

	ds1820_env_t * ep          = &g_ds1820_env_v;
	//ds1820       * InstancePtr = &g_ds1820_v[ep->idscan];
	//ds1820ape_t  * dp          = &g_ds1820ape_v[ep->idscan];

	int i;
	for(i = 0; i < DS1820_SENSORS_MAX; i++)
	{
		int k = (ep->idx_get + i + 1) % DS1820_SENSORS_MAX;
		ds1820_sns_t * ps = & ep->sns[k];
		if (ps->alive)
		{
			if (temp)
				*temp = ps->temp;

			if (idx)
				* idx = (u8)k;

			ep->idx_get = (u8)k;

			return XST_SUCCESS;
		}
	}
	return XST_NO_DATA;
}


#endif
