/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief application interactive
/// \file ethercat.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

#if SYS_ETHERCAT_NUM > 0

#include <stdio.h>

#define ETHERCAT_BASEADDR       (XPAR_ETHERCAT_USER_0_BASEADDR)
#define ETHERCAT_PRESENT        (1)

#define ECAT_PDI_OFS_IN  0x1300
#define ECAT_PDI_OFS_OUT 0x1000

#define ETHERCAT_MOT_CTL_SIZE (2+4+4+4)
#define ETHERCAT_MOT_STS_SIZE (2+4+4+4+4+4)

#define ETHERCAT_MOT_CTL_RAW_SIZE (ETHERCAT_MOT_CTL_SIZE * ETHERCAT_MOT_NUM + 0 + 4)
#define ETHERCAT_MOT_STS_RAW_SIZE (ETHERCAT_MOT_STS_SIZE * ETHERCAT_MOT_NUM + 4 + 0)

typedef union
{
	struct { u8 data[ETHERCAT_MOT_CTL_RAW_SIZE]; } in;
	struct { u8 data[ETHERCAT_MOT_STS_RAW_SIZE]; } out;
} ecat_raw_t;

static ecat_raw_t ecat_raw_v;

// Ethercat XDR

typedef union
{
	u32   u;
	float r;
} u32r32_t;

#if 0 // no more used
static u32 ethercat_xdr_get_u8(u8 *p)
{
	return p[0];
}
#endif

static u32 ethercat_xdr_get_u16(u8 *p)
{
	u32 v = 0;
	v |= p[1] << 8;
	v |= p[0];
	return v;
}

static u32 ethercat_xdr_get_u32(u8 *p)
{
	u32 v = 0;
	v |= p[3] << 24;
	v |= p[2] << 16;
	v |= p[1] << 8;
	v |= p[0];
	return v;
}

static float ethercat_xdr_get_r32(u8 *p)
{
	u32r32_t t;
	t.u = ethercat_xdr_get_u32(p);
	return t.r;
}

#if 0 // no more used
static void ethercat_xdr_put_u8(u8 *p, u32 v)
{
	p[0] = v;
}
#endif

static void ethercat_xdr_put_u16(u8 *p, u32 v)
{
	p[1] = v >> 8;
	p[0] = v;
}

static void ethercat_xdr_put_u32(u8 *p, u32 v)
{
	p[3] = v >> 24;
	p[2] = v >> 16;
	p[1] = v >> 8;
	p[0] = v;
}

static void ethercat_xdr_put_r32(u8 *p, float v)
{
	u32r32_t t;
	t.r = v;
	ethercat_xdr_put_u32(p,t.u);
}

#define MATH_PIF (3.1415926535897932384626433832795f)

// Init

int ethercat_init(void)
{
	if (!ETHERCAT_PRESENT)
		return XST_FAILURE;

	esc_env_t * pE = & g_esc_env_v;

	pE->warm = 0;
	pE->Last_AL_Control = ETHERCAT_ESC_AL_ST_MACH_INVALID;

	return XST_SUCCESS;
}

int ethercat_e2prdy(void)
{
	if (!ETHERCAT_PRESENT)
		return 0;

	// Test for EEPROM is loaded and process memory is accessible
	return 0x00 != (Xil_In8(ETHERCAT_BASEADDR + 0x0110) & 0x01);
}

// Diagnostic display

int ethercat_disp(void)
{
	if (!ETHERCAT_PRESENT)
		return XST_FAILURE;

	printf("\n");

	printf("ESC Information\n");
	printf(" Type            = %d\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0000));
	printf(" Revision        = %d\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0001));
	printf(" Build           = %d\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0002));
	printf(" FMMU's          = %d\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0004 ));
	printf(" SyncMan.        = %d\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0005 ));
	printf(" RAM Size        = %d\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0006 ));
	printf(" Port Desc       =x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0007 ));
	printf(" ESC Feat.       =x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0008 ));

	printf("Station Address\n");
	printf(" Conf. St. Addr. =x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0010));
	printf(" Conf. St. Alias =x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0012));

	printf("Write Protection\n");
	printf(" Write Reg. En.  =x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0020 ));
	printf(" Write Reg. Pr.  =x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0021 ));
	printf(" ESC Write  En.  =x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0030 ));
	printf(" ESC Write  Pr.  =x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0031 ));

	printf("Data Link Layer\n");
	printf(" ESC Reset ECAT  = x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0040 ));
	printf(" ESC Reset PDI   = x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0041 ));

	printf(" ESC DL Control  = x%08X\n",  (unsigned int)Xil_In32(ETHERCAT_BASEADDR + 0x0100 ));
	printf(" Phy R/W Offs.   = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0108 ));
	printf(" ESC DL Status   = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0110 ));

	printf("Application Layer\n");
	printf(" AL Control      = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0120 ));
	printf(" AL Status       = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0130 ));
	printf(" AL Status Code  = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0134 ));
	printf(" RUN LED Over.   = x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0138 ));
	printf(" ERR LED Over.   = x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0139 ));

	printf("PDI/ESC Configuration\n");
	printf(" PDI Control     = x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0140 ));
	printf(" ESC Config.     = x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0141 ));
	printf(" PDI Information = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x014E ));
	printf(" PDI Config.     = x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0150 ));
	printf(" S/L PDI Config. = x%02X\n",  Xil_In8(ETHERCAT_BASEADDR + 0x0151 ));
	printf(" Ext. PDI Conf.  = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0152 ));

	printf("Interrupts\n");
	printf(" ECAT Event Mask  = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0200 ));
	printf(" PDI AL Event Mask= x%08X\n",  (unsigned int)Xil_In32(ETHERCAT_BASEADDR + 0x0204 ));
	printf(" ECAT Event Req.  = x%04X\n",  Xil_In16(ETHERCAT_BASEADDR + 0x0210 ));
	printf(" PDI AL Event Req.= x%08X\n",  (unsigned int)Xil_In32(ETHERCAT_BASEADDR + 0x0220 ));

	int i;

	printf("Error Counters\n");
	for(i = 0; i < 4; i++)
	{
		printf(" RX Err Cnt [%d]: =x%04X\n",
			i,
			Xil_In16(ETHERCAT_BASEADDR + 0x0300 + i * 2 ));
	}
	for(i = 0; i < 4; i++)
	{
		printf(" FWD RX Err Cnt [%d]: =%d\n",
			i,
			Xil_In8(ETHERCAT_BASEADDR + 0x0308 + i ));
	}
	printf(" ECAT Proc EC.    =%d\n",  Xil_In8(ETHERCAT_BASEADDR + 0x030C ));
	printf(" PDI Error Cnt.   =%d\n",  Xil_In8(ETHERCAT_BASEADDR + 0x030D ));
	printf(" PDI Error Code   =%d\n",  Xil_In8(ETHERCAT_BASEADDR + 0x030E ));
	for(i = 0; i < 4; i++)
	{
		printf(" Link Lost  Cnt [%d]: =%d\n",
			i,
			Xil_In8(ETHERCAT_BASEADDR + 0x0310 + i ));
	}

	// WDG ....

	// SII EEprom ...

	// MMI ....

	// FMMU
	for(i = 0; i < 16; i++)
	{
		if (Xil_In8(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0xC ) > 0)
		{
			printf(" FMMU[%2d]: LSA = x%08X len=%d sb=%d eb=%d pha=x%X psb=%d ty=%d ac=%d\n",
				i,
				(unsigned int)Xil_In32(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0x0 ),
				Xil_In16(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0x4 ),
				Xil_In8(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0x6 ),
				Xil_In8(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0x7 ),
				Xil_In16(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0x8 ),
				Xil_In8(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0xA ),
				Xil_In8(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0xB ),
				Xil_In8(ETHERCAT_BASEADDR + 0x0600 + i * 16 + 0xC ));
		}

	}

	return XST_SUCCESS;
}

int ethercat_proc(void)
{
	if (!ETHERCAT_PRESENT)
		return XST_FAILURE;

	esc_env_t * pE = & g_esc_env_v;

	if (!ethercat_e2prdy())
	{
		pE->warm = 0;
		pE->Last_AL_Control = ETHERCAT_ESC_AL_ST_MACH_INVALID;
		return XST_SUCCESS;
	}


	// EtherCAT input
	{

		int i,o;

		// Read from ESC
		for(i=0;i<ETHERCAT_MOT_CTL_RAW_SIZE;i++)
		{
			ecat_raw_v.in.data[i] = Xil_In8(ETHERCAT_BASEADDR + ECAT_PDI_OFS_IN + i);
		}
		// XDR Input
		o = 0;
		for(i = 0; i < ETHERCAT_MOT_NUM; i++)
		{
			pE->ctl_v.motor[i].ctl      = ethercat_xdr_get_u16(&ecat_raw_v.in.data[o]); o += 2;
			pE->ctl_v.motor[i].current  = ethercat_xdr_get_r32(&ecat_raw_v.in.data[o]); o += 4;
			pE->ctl_v.motor[i].speed    = ethercat_xdr_get_r32(&ecat_raw_v.in.data[o]); o += 4;
			pE->ctl_v.motor[i].position = ethercat_xdr_get_r32(&ecat_raw_v.in.data[o]); o += 4;

			if (0 != memcmp(&pE->ctl_v.motor[i],&pE->ctl_o.motor[i],sizeof(eact_mot_ctl_t)))
			{
				pE->ctl_o.motor[i] = pE->ctl_v.motor[i];

				printf("\nM[%d]: ctl=x%04X, current = %g speed = %g positions = %g\n",
						i,
						pE->ctl_v.motor[i].ctl      ,
						pE->ctl_v.motor[i].current  ,
						pE->ctl_v.motor[i].speed    ,
						pE->ctl_v.motor[i].position );

			}
		}
	}

	// motor status/control
	#if SYS_MOTORFOC_NUM > 0
	{
		// -- motors
		{
			u16 id;
			for(id = 0; id < SYS_MOTORFOC_NUM && id < ETHERCAT_MOT_NUM; id++)
			{
				motorfoc *pM = &g_motorfoc_v[id];

				motorfoc_ap_sts_read(pM);
				motorfoc_ap_ctl_read(pM);

				// get main status
				pE->sts_v.motor[id].sts = 0; // starting value

				// ownership
				if (pM->ApPtr->ipdrvsource == MOTORFOC_AP_IPDRVSOURCE_LOCAL)
				{
					if ((pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_OWN_MSK) == ETHERCAT_MOT_CTL_CTL_OWN_REQ)
					{
						pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_OWN_ACK;
					}
				}

				// power-on control
				int power;
				motorfoc_ap_ctl_get_power(pM,&power);
				if (power)
					pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_PWR_ON;
				else
					pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_PWR_OFF;

				// modulator levels
				int modlevels;
				motorfoc_ap_prm_get_modlevels(pM,&modlevels);
				if (modlevels <= 2)
					pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_LVM_2;
				else
					pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_LVM_3;

				// modulator type
				int modulator;
				motorfoc_ap_sts_get_modulator(pM,&modulator);
				if (modulator == MOTORFOC_AP_MODULATOR_PWM)
					pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_MOD_PWM;
				else
					pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_MOD_RPFM;

				// rotor position mode
				int rotposmode;
				motorfoc_ap_ctl_get_rotposmode(pM,&rotposmode);
				switch(rotposmode)
				{
					default                              : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_ROT_MAN; break;
					case MOTORFOC_AP_ROTPOSMODE_MANUAL   : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_ROT_MAN; break;
					case MOTORFOC_AP_ROTPOSMODE_EMULATOR : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_ROT_MRT; break;
					case MOTORFOC_AP_ROTPOSMODE_ENCODER  : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_ROT_ENC; break;
					case MOTORFOC_AP_ROTPOSMODE_EVALUATOR: pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_ROT_SMO; break;
					case MOTORFOC_AP_ROTPOSMODE_HALLSNS  : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_ROT_HLS; break;
					case MOTORFOC_AP_ROTPOSMODE_RESOLVER : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_ROT_RSV; break;
					case MOTORFOC_AP_ROTPOSMODE_EXTERNAL : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_ROT_EXT; break;
				}

				// control mode
				int ctrlmode;
				int usemrt;
				motorfoc_ap_ctl_get_ctrlmode(pM,&ctrlmode,&usemrt);
				switch(ctrlmode)
				{
					case MOTORFOC_AP_CTRLMODE_TORQUE   : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_CTM_TRQ; break;
					case MOTORFOC_AP_CTRLMODE_SPEED    : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_CTM_SPD; break;
					//case MOTORFOC_AP_CTRLMODE_POSITION : pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_CTM_POS; break;
				}

				// failure
				int failure;
				motorfoc_ap_sts_get_failure(pM,&failure);
				if (failure)
					pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_FAIL_ON;
				else
					pE->sts_v.motor[id].sts |= ETHERCAT_MOT_STS_STS_FAIL_OFF;


				// motor status
				motorfoc_ap_sts_get_iscurr(pM,&pE->sts_v.motor[id].current);
				motorfoc_ap_sts_get_speedrpm(pM,&pE->sts_v.motor[id].speed);
				motorfoc_ap_sts_get_rotpos(pM,&pE->sts_v.motor[id].position);
				motorfoc_ap_sts_get_modidx(pM,&pE->sts_v.motor[id].modindex);
				motorfoc_ap_sts_get_vbus(pM,&pE->sts_v.motor[id].vbus);


				// motor control

				if ((pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_OWN_MSK) == ETHERCAT_MOT_CTL_CTL_OWN_REQ)
				{
					if ((pE->sts_v.motor[id].sts & ETHERCAT_MOT_STS_STS_OWN_MSK) == ETHERCAT_MOT_STS_STS_OWN_ACK)
					{
						if (pM->ApPtr->ipdrvsource == MOTORFOC_AP_IPDRVSOURCE_LOCAL)
						{
							// set rotor mode
							int rotposmode = 0;
							switch(pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_ROT_MSK)
							{
								default                           : rotposmode = MOTORFOC_AP_ROTPOSMODE_MANUAL   ; break;
								case ETHERCAT_MOT_CTL_CTL_ROT_MAN : rotposmode = MOTORFOC_AP_ROTPOSMODE_MANUAL   ; break;
								case ETHERCAT_MOT_CTL_CTL_ROT_MRT : rotposmode = MOTORFOC_AP_ROTPOSMODE_EMULATOR ; break;
								case ETHERCAT_MOT_CTL_CTL_ROT_ENC : rotposmode = MOTORFOC_AP_ROTPOSMODE_ENCODER  ; break;
								case ETHERCAT_MOT_CTL_CTL_ROT_SMO : rotposmode = MOTORFOC_AP_ROTPOSMODE_EVALUATOR; break;
								case ETHERCAT_MOT_CTL_CTL_ROT_HLS : rotposmode = MOTORFOC_AP_ROTPOSMODE_HALLSNS  ; break;
								case ETHERCAT_MOT_CTL_CTL_ROT_RSV : rotposmode = MOTORFOC_AP_ROTPOSMODE_RESOLVER ; break;
								case ETHERCAT_MOT_CTL_CTL_ROT_EXT : rotposmode = MOTORFOC_AP_ROTPOSMODE_EXTERNAL ; break;
							}
							motorfoc_ap_ctl_set_rotposmode(pM,rotposmode,1);

							// set control mode
							int ctrlmode = 0;
							int usemrt = 0;
							switch(pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_CTM_MSK)
							{
								case ETHERCAT_MOT_CTL_CTL_CTM_TRQ : ctrlmode = MOTORFOC_AP_CTRLMODE_TORQUE  ; break;
								case ETHERCAT_MOT_CTL_CTL_CTM_SPD : ctrlmode = MOTORFOC_AP_CTRLMODE_SPEED   ; break;
								//case ETHERCAT_MOT_CTL_CTL_CTM_POS : ctrlmode = MOTORFOC_AP_CTRLMODE_POSITION; break;
							}
							motorfoc_ap_ctl_set_ctrlmode(pM,ctrlmode,usemrt,1);

							// modulator level
							int modlevels;
							motorfoc_ap_prm_get_modlevels(pM,&modlevels);
							// requested value
							int modlvreq = ((pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_LVM_MSK) == ETHERCAT_MOT_CTL_CTL_LVM_3) ? 3 : 2;
							motorfoc_ap_prm_set_modlevels(pM,modlvreq);
							motorfoc_ap_prm_get_modlevels(pM,&modlvreq);
							if (modlevels != modlvreq)
							{
								motorfoc_ap_compile(pM);
								motorfoc_ap_ctl_write(pM);
							}

							// PWM/RPFM selector
							motorfoc_ap_ctl_set_modulator(pM,
									((pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_MOD_MSK) == ETHERCAT_MOT_CTL_CTL_MOD_PWM) ?
									MOTORFOC_AP_MODULATOR_PWM : MOTORFOC_AP_MODULATOR_RPFM, 1);

							int drive = (pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_FLX_MSK) == ETHERCAT_MOT_CTL_CTL_FLX_DRV;

							// encoder: drive/flux
							motorfoc_ap_ctl_set_enc_drive(pM,drive,1);

							// motor power control
							motorfoc_ap_ctl_set_power(pM,(pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_PWR_MSK) == ETHERCAT_MOT_CTL_CTL_PWR_ON,1);

							// current
							motorfoc_ap_ctl_set_pi_curset_pol(pM,pE->ctl_v.motor[id].current,drive? (MATH_PIF / 2.0f) : 0.0f,1);
							motorfoc_ap_ctl_set_slp_is2pi_pol(pM,pE->ctl_v.motor[id].current,drive? (MATH_PIF / 2.0f) : 0.0f,1);

							// speed (speed loop)
							motorfoc_ap_ctl_set_slp_speed(pM,pE->ctl_v.motor[id].speed,1);

							// speed (mrt)
							motorfoc_ap_ctl_set_mrt_speed(pM,pE->ctl_v.motor[id].speed,1);

							// position
							//motorfoc_ap_ctl_set_position(pM,pE->ctl_v.motor[id].position,1);

						}
					}
				}

			}
		}

	}

	// on chip temperature from XADC
	{
		pE->sts_v.octemp = 0.0F;

		// fmcmc1
		#if SYS_FMCMC1_NUM > 0
		{
			u16 id;
			for(id = 0;id < SYS_FMCMC1_NUM; id++)
			{
				fmcmc1_drv_Config *CfgPtr = fmcmc1_drv_LookupConfig(id);
				//-
				if (CfgPtr->xadc > 0)
				{
					u32 fmcmc1_base = CfgPtr->BaseAddress;
					pE->sts_v.octemp = (float)FMCMC1_DRV_xa_adi_mcelsius(FMCMC1_DRV_mRead_oc_temp_val(fmcmc1_base)) / 1.0e3F;
				}
			}
		}
		#endif

		// motcon2
		#if SYS_MOTCON2_NUM > 0
		{
			u16 id;
			for(id = 0;id < SYS_MOTCON2_NUM; id++)
			{
				motcon2_drv_Config *CfgPtr = motcon2_drv_LookupConfig(id);
				//-
				if (CfgPtr->xadc > 0)
				{
					u32 motcon2_base = CfgPtr->BaseAddress;
					pE->sts_v.octemp = (float)MOTCON2_DRV_xa_adi_mcelsius(MOTCON2_DRV_mRead_oc_temp_val(motcon2_base)) / 1.0e3F;
				}
			}
		}
		#endif

		#if SYS_QXADCPS_NUM > 0
		{
			pE->sts_v.octemp = g_qxadcps_oc_temp[0];
		}
		#endif

	}

	#endif

	{
		int i,o;

		// XDR Output
		o = 0;
		for(i = 0; i < ETHERCAT_MOT_NUM; i++)
		{
			ethercat_xdr_put_u16(&ecat_raw_v.out.data[o],pE->sts_v.motor[i].sts); o += 2;
			ethercat_xdr_put_r32(&ecat_raw_v.out.data[o],pE->sts_v.motor[i].current); o += 4;
			ethercat_xdr_put_r32(&ecat_raw_v.out.data[o],pE->sts_v.motor[i].speed); o += 4;
			ethercat_xdr_put_r32(&ecat_raw_v.out.data[o],pE->sts_v.motor[i].position); o += 4;
			ethercat_xdr_put_r32(&ecat_raw_v.out.data[o],pE->sts_v.motor[i].modindex); o += 4;
			ethercat_xdr_put_r32(&ecat_raw_v.out.data[o],pE->sts_v.motor[i].vbus); o += 4;
		}
		ethercat_xdr_put_r32(&ecat_raw_v.out.data[o],pE->sts_v.octemp); o += 4;

		// Write to ESC
		for(i = 0; i < ETHERCAT_MOT_STS_RAW_SIZE; i++)
		{
			Xil_Out8(ETHERCAT_BASEADDR + ECAT_PDI_OFS_OUT + i, ecat_raw_v.out.data[i]);
		}
	}


    // Simple EtherCAT Statemachine emulation
	// Read AL Control Register, mask error indication flag
	u8 AL_Control = Xil_In8(ETHERCAT_BASEADDR + 0x0120) & 0x0F;

	// Confirm at AL Status Register
	Xil_Out8(ETHERCAT_BASEADDR + 0x0130, AL_Control);

   	// Display EtherCAT state
	if(AL_Control != pE->Last_AL_Control || pE->warm == 0)
	{
		pE->Last_AL_Control = AL_Control;
		switch(AL_Control) {
			case ETHERCAT_ESC_AL_ST_MACH_INIT : // Init
						print("Init state \r\n");
						break;
			case ETHERCAT_ESC_AL_ST_MACH_PREOP : // Pre-Op
						print("Pre-Operational state \r\n");
						break;
			case ETHERCAT_ESC_AL_ST_MACH_BOOT : // Bootstrap
						print("Bootstrap state \r\n");
						break;
			case ETHERCAT_ESC_AL_ST_MACH_SAFE : // Safe-Op
						print("Safe-Op state \r\n");
						break;
			case ETHERCAT_ESC_AL_ST_MACH_OPER:  // Operational: running light
						print("Operational state \r\n");
						break;
			default: 	// Unknown
						print("Unknown state \r\n");
						break;
		}
	}


	// after 1st loop ... no more cold
	pE->warm = !0;

	return XST_SUCCESS;
}


#endif
