/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief Microblaze Ethernet driver
/// \file mb_ethmac.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "xparameters.h"

#if defined(__MICROBLAZE__)

#include <stdio.h>

#include "xstatus.h"
#include "xemaclite.h"

#include "board.h"

#include "xx_ethmac.h"

typedef struct emac_s
{
	XEmacLite emac;
	u8        mac_address[6];
} emac_t;

static emac_t g_emac;

#define XX_ETHMAC_DEF emac_t * ref = & g_emac
#define XX_ETHMAC_DEV          (&ref->emac)


//---------- phy setup ---------------------------

static XStatus xx_ethmac_phy_setup(void)
{
	XStatus rc = XST_SUCCESS;

	#if SYS_CARRIER_LYR178 > 0
	{
		XX_ETHMAC_DEF;
		u32 PhyAddress;
		u32 RegNum;
		u16 PhyData;
		//-
		PhyAddress = 7;
		RegNum     = 0x00;
		PhyData    = 0x2100 | 0x8000;
		rc  = XEmacLite_PhyWrite(XX_ETHMAC_DEV,PhyAddress,RegNum,PhyData);
	}
	#endif

	#if SYS_CARRIER_SP605 > 0
	{
		XX_ETHMAC_DEF;
		u32 PhyAddress;
		u32 RegNum;
		u16 PhyData;
		//-
		PhyAddress = 7;
		RegNum     = 0x00;
		PhyData    = 0x2100 | 0x8000;
		rc  = XEmacLite_PhyWrite(XX_ETHMAC_DEV,PhyAddress,RegNum,PhyData);
	}
	#endif

	#if SYS_CARRIER_LX150T > 0
	{
		XX_ETHMAC_DEF;
		u32 PhyAddress;
		u32 RegNum;
		u16 PhyData;
		//-
		PhyAddress = 1;
		RegNum     = 0x00;
		PhyData    = 0x2100;
		rc  = XEmacLite_PhyWrite(XX_ETHMAC_DEV,PhyAddress,RegNum,PhyData);
	}
	#endif

	#if SYS_CARRIER_LX75T > 0
	{
		XX_ETHMAC_DEF;
		u32 PhyAddress;
		u32 RegNum;
		u16 PhyData;
		//-
		PhyAddress = 0;
		RegNum     = 0x19;
		PhyData    = 0x8021;
		rc  = XEmacLite_PhyWrite(XX_ETHMAC_DEV,PhyAddress,RegNum,PhyData);
		//-
		PhyAddress = 1;
		RegNum     = 0x00;
		PhyData    = 0x2100;
		rc  = XEmacLite_PhyWrite(XX_ETHMAC_DEV,PhyAddress,RegNum,PhyData);
	}
	#endif

	#if SYS_CARRIER_KC705 > 0
	{
		XX_ETHMAC_DEF;
		u32 PhyAddress;
		u32 RegNum;
		u16 PhyData;
		//-
		PhyAddress = 7;
		RegNum     = 0x00;
		PhyData    = 0x2100 | 0x8000;
		rc  = XEmacLite_PhyWrite(XX_ETHMAC_DEV,PhyAddress,RegNum,PhyData);
	}
	#endif

	//-
	return rc;
}


//------------------------------------------------

int xx_ethmac_relinq_speed(void)
{
	return XST_SUCCESS;
}

int xx_ethmac_set_mac_addr(const u8* addr)
{
	XX_ETHMAC_DEF;
	memcpy(ref->mac_address,addr,6);
	return XST_SUCCESS;
}

int xx_ethmac_get_mac_addr(u8* addr)
{
	XX_ETHMAC_DEF;
	memcpy(addr,ref->mac_address,6);
	return XST_SUCCESS;
}

int xx_ethmac_recv_tgm(u8 *buf_pnt,u32 *buf_len,u8 *rok)
{
	XX_ETHMAC_DEF;
	u16 rrecv = XEmacLite_Recv(XX_ETHMAC_DEV,buf_pnt);
	* buf_len = rrecv;
	* rok = rrecv > 0;
	return XST_SUCCESS;
}

int xx_ethmac_send_tgm(u8 *buf_pnt,u32 buf_len,u8 *tok)
{
	XX_ETHMAC_DEF;
	int rc = XEmacLite_Send(XX_ETHMAC_DEV,buf_pnt,buf_len);
	* tok = rc == XST_SUCCESS;
	return XST_SUCCESS;
}

int xx_ethmac_init(void)
{
	XX_ETHMAC_DEF;
	int rc;

	//- Init h/w
	rc = XEmacLite_Initialize(XX_ETHMAC_DEV,XPAR_EMACLITE_0_DEVICE_ID);
	if (rc != XST_SUCCESS)
	{
		xil_printf("XEmacLite_Initialize()=%d\r\n",rc);
		return XST_FAILURE;
	}

	// set MAC address
	XEmacLite_SetMacAddress(XX_ETHMAC_DEV,ref->mac_address);

	// setup PHY
	rc = xx_ethmac_phy_setup();
	if (rc != XST_SUCCESS)
	{
		xil_printf("xx_ethmac_phy_setup()=%d\r\n",rc);
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

int xx_ethmac_down(void)
{
	return XST_SUCCESS;
}

#endif
