/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief Microblaze IIC driver
/// \file mb_iic.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "xparameters.h"

#if defined(__MICROBLAZE__)

#include <stdio.h>
#include "xstatus.h"

#include "board.h"

#include "xx_iic.h"

#ifdef SYS_CARRIER_LYR178

#include "xiic_l.h"


#define XX_IIC_DEV (XPAR_IIC_0_BASEADDR)

// LYR178
#define IIC_VSK_MUX_ADDR       (0xE4 / 2) // 0x72
#define IIC_VSK_MUX_DATA_NUL         0x00
#define IIC_VSK_MUX_DATA_FMC         0x06

#define IIC_VSK_POWER_ADDR           (0xA4 / 2) // 0x52
#define IIC_VSK_POT_PAGE_ADDR                8  // pot: RAM/NVRAM
#define IIC_VSK_POT_WR1_ADDR                 1  // pot: #1
#define IIC_VSK_POT_PAGE_RAM              0x80  // pot: RAM
#define IIC_VSK_POT_WR1_2P5               0x1E  // pot: 2.5 Volts


static XStatus xx_iic_dy_is_ready(void)
{
	Xuint8 StatusReg = XIic_ReadReg(XX_IIC_DEV, XIIC_SR_REG_OFFSET);
	if ((StatusReg & (XIIC_SR_RX_FIFO_EMPTY_MASK | XIIC_SR_TX_FIFO_EMPTY_MASK)) != (XIIC_SR_RX_FIFO_EMPTY_MASK | XIIC_SR_TX_FIFO_EMPTY_MASK))
		return XST_FAILURE;
	if (StatusReg & XIIC_SR_BUS_BUSY_MASK)
		return XST_DEVICE_BUSY;
	return XST_SUCCESS;
}

static XStatus xx_iic_dy_write(u8 ChipAddress, u8 *BufferPtr, u8 ByteCount, unsigned *Writted)
{
	// w4 ready for new command
	while( xx_iic_dy_is_ready() != XST_SUCCESS)
		;

	* Writted = XIic_DynSend(XX_IIC_DEV,ChipAddress,BufferPtr, ByteCount, XIIC_STOP);

	return XST_SUCCESS;
}

static XStatus xx_iic_dy_vsk_write_mux(u8 mux)
{
	XStatus Status;
	u8 buffer[16];
	unsigned Writted;

	// setup MUX
	buffer[0] = mux;
	if (XST_SUCCESS != (Status = xx_iic_dy_write(IIC_VSK_MUX_ADDR,buffer,1,&Writted)))
		return Status;
	return XST_SUCCESS;
}

int xx_iic_lyr178_fmc_pwon(void)
{
	XStatus Status;

	// MUX for FMC
	if (XST_SUCCESS != (Status = xx_iic_dy_vsk_write_mux(IIC_VSK_MUX_DATA_FMC)))
		return Status;

	{
		u8 buffer[16];
		unsigned Writted;

		// setup for POT
		buffer[0] = IIC_VSK_POT_PAGE_ADDR;
		buffer[1] = IIC_VSK_POT_PAGE_RAM;
		if (XST_SUCCESS != (Status = xx_iic_dy_write(IIC_VSK_POWER_ADDR,buffer,2,&Writted)))
			return Status;

		// FMC #2 @ 2.5V
		buffer[0] = IIC_VSK_POT_WR1_ADDR;
		buffer[1] = IIC_VSK_POT_WR1_2P5;
		if (XST_SUCCESS != (Status = xx_iic_dy_write(IIC_VSK_POWER_ADDR,buffer,2,&Writted)))
			return Status;
	}

	// reset MUX
	if (XST_SUCCESS != (Status = xx_iic_dy_vsk_write_mux(IIC_VSK_MUX_DATA_NUL)))
		return Status;

	return XST_SUCCESS;
}

#endif

int xx_iic_init(void)
{
#ifdef XX_IIC_DEV
	int rc;

	// initialize
	rc = XIic_DynInit(XX_IIC_DEV);
	if (rc != XST_SUCCESS)
	{
		xil_printf("XIic_DynInit()=%d\r\n",rc);
		return rc;
	}
#endif
	return XST_SUCCESS;
}

int xx_iic_down(void)
{
	return XST_SUCCESS;
}

#endif
