/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief motor control agent main program
/// \file mcamainc.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

	#if SW_ENA_PL_INIT > 0
	#include "ps7_init.h"
	#include "zq_qspipl.h"
	#endif


#define LINE_ERROR_STAMP()  {g_init_error_line = __LINE__;}

static XStatus motctl_initialize(void)
{
	XStatus Status;

	// default: board #0
	g_board_id = 0;

	// zynq qspips
	#ifdef XPAR_XQSPIPS_0_DEVICE_ID
		if (XST_SUCCESS != (Status = zq_qspips_initialize(&g_zq_qspips)))
		{
			LINE_ERROR_STAMP();
			return Status;
		}
	#endif

	// init environ
	nvenviron_initialize();

	// load environ
	nvenviron_load();

	if (XST_SUCCESS != (Status = xx_timer_init()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}

	if (XST_SUCCESS != (Status = xx_ethmac_init()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}

	if (XST_SUCCESS != (Status = xx_xcan_init()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}

	if (XST_SUCCESS != (Status = xx_iic_init()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}

	//------------------------------------------------------------------------

	#ifdef SYS_CARRIER_LYR178
	if (XST_SUCCESS != (Status = xx_iic_lyr178_fmc_pwon()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}
	#endif

	//------------------------------------------------------------------------

	#ifdef SYS_CARRIER_TE0720

	#if SYS_NETMOT_NUM > 0
	netmot_drv_fpga_setup_for_program_dev(netmot_drv_LookupConfig(0)->BaseAddress,1);
	#endif

	if (XST_SUCCESS != (Status = xx_iic_te0720_fmc_pwon()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}

	#if SYS_NETMOT_NUM > 0
	{
		u32 sd = xx_timer_cnt_val();
		u32 to = xx_timer_cnt_pser_sec() / 4;
		for(;;)
		{
			u32 df = xx_timer_cnt_val() - sd;
			if (df > to)
				break;
		}
	}
	netmot_drv_fpga_setup_for_program_dev(netmot_drv_LookupConfig(0)->BaseAddress,0);
	#endif

	#endif

	//------------------------------------------------------------------------

	#if SYS_ETHERCAT_NUM > 0
	if (XST_SUCCESS != (Status = ethercat_init()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}
	#endif

	//------------------------------------------------------------------------

	#if SYS_DS1820_NUM > 0
	if (XST_SUCCESS != (Status = ds1820_ape_init()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}
	#endif

	//------------------------------------------------------------------------

	#if SYS_IVPLOGGER_NUM > 0
	if (XST_SUCCESS != (Status = ivplogger_ape_init()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}
	#endif

	//------------------------------------------------------------------------

	#if SYS_QXADCPS_NUM > 0
	if (XST_SUCCESS != (Status = qxadcps_ape_init()))
	{
		LINE_ERROR_STAMP();
		return Status;
	}
	#endif

	//------------------------------------------------------------------------

	motbrddv_system_init();

	//------------------------------------------------------------------------

	return XST_SUCCESS;
}

static XStatus motctl_terminate(void)
{
	motbrddv_system_down();

	xx_iic_down();

	xx_xcan_down();

	xx_ethmac_down();

	xx_timer_down();

	return XST_SUCCESS;
}



int motctl_main(void)
{
	XStatus Status;

	// flush stdin
	while(con_tkey() >= 0)
		;

	// display ident
	motctl_disp_ident();
		
	// initialize
	if (XST_SUCCESS != (Status = motctl_initialize()))
	{
		xil_printf("\r\n""motctl_initialize() error @ %d\r\n",g_init_error_line);
		goto main_exit;
	}
	
	// =========================================

	for(;;)
	{
		#if SW_ENA_ETH_AGENT > 0
		if (g_link_phymode != LINK_PHYMODE_CAN)
		{
			mca_eth_agent();
			if (mca_eth_task_status() == MCA_ETH_TASK_STATUS_TERMINATE)
				break;
		}
		#endif

		#if SW_ENA_CAN_AGENT > 0
		if (g_link_phymode != LINK_PHYMODE_ETH)
		{
			mca_can_agent();
			if (mca_can_task_status() == MCA_CAN_TASK_STATUS_TERMINATE)
				break;
		}
		#endif
	}

	// =========================================

	if (XST_SUCCESS != (Status = motctl_interactive()))
	{
		xil_printf("\r\n""motctl_interactive error %d\r\n",Status);
		goto main_exit;
	}
	
	// =========================================
	
main_exit:
	motctl_terminate();


	print("\r\nThat's all folks!.\r\n");
	
	return Status;
}

//-

#if SW_ENA_OCM_RAM == 0

static void enable_caches(void)
{
	#ifdef XPAR_MICROBLAZE_USE_ICACHE
	Xil_ICacheEnable();
	#endif
	#ifdef XPAR_MICROBLAZE_USE_DCACHE
	Xil_DCacheEnable();
	#endif
}

#endif

static void disable_caches(void)
{
    Xil_DCacheDisable();
    Xil_ICacheDisable();
}

//
// the main program...
//

int main()
{
	#if SW_ENA_PL_INIT > 0
	ps7_init();
	#endif

	#if SW_ENA_OCM_RAM > 0
	Xil_DCacheFlush();
	Xil_DCacheDisable();
	#endif

	#if SW_ENA_PL_INIT > 0
	zq_qspipl_init();
	#endif

	#if SW_ENA_OCM_RAM == 0
	enable_caches();
	#endif

	motctl_main();

	disable_caches();
	return 0;
}
