/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ultra light tcp/ip test
/// \file utcpip.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

const utcpip_mac_addr_t utcpip_mac_brdaddr = { .b.b1 = 0xFF, .b.b2 = 0xFF, .b.b3 = 0xFF, .b.b4 = 0xFF, .b.b5 = 0xFF, .b.b6 = 0xFF };
const utcpip_mac_addr_t utcpip_mac_unkaddr = { .b.b1 = 0x00, .b.b2 = 0x00, .b.b3 = 0x00, .b.b4 = 0x00, .b.b5 = 0x00, .b.b6 = 0x00 };

const utcpip_in_addr_t  utcpip_ip_brdaddr = 0xFFFFFFFF;


static uint16_t utcpip_ntohs(uint16_t n)
{
	return (uint16_t)(((n >> 8) & 0x00FF) | ((n << 8) & 0xFF00));
}

#define utcpip_htons  utcpip_ntohs

//
// cksum eval
//

typedef uint32_t utcpip_cksum_var_t;

typedef struct
{
	utcpip_cksum_var_t cks;
} utcpip_cksum_env_t;


static void utcpip_cksum_init(utcpip_cksum_env_t *p)
{
	p->cks = 0;
}

static uint16_t utcpip_cksum_result(utcpip_cksum_env_t *p)
{
	return p->cks & 0xFFFF;
}

static void utcpip_cksum_finish(utcpip_cksum_env_t *p)
{
	uint32_t t;
	while(0 != (t = (p->cks >> 16)))
		p->cks = (p->cks & 0xFFFF) + t;
	p->cks = (~p->cks) & 0xFFFF;
}

static void utcpip_cksum_elab(utcpip_cksum_env_t *p,uint16_t v)
{
	p->cks += v;
}

static void utcpip_cksum_buffer(utcpip_cksum_env_t *p,void *d,int l)
{
	uint16_t *r = (uint16_t *)d;
	while(l > 0)
	{
		uint16_t t = *r;
		if (l == 1)
			t &= 0xFF00;
		utcpip_cksum_elab(p,t);
		l -= 2;
		r += 1;
	}
}

int utcpip_client(utcpip_dev_t *d)
{
	d->decoded = UTCPIP_RECEIVED_NONE;

	if (d->trx_msg_len < sizeof(utcpip_eth_hdr_t))
		return UTCPIP_RET_FAIL;
		
	utcpip_eth_hdr_t *pETH_I = (utcpip_eth_hdr_t *)d->trx_msg_pnt;
	
	if (0 != memcmp(&pETH_I->ether_dhost,&d->mac_address,6) && 0 != memcmp(&pETH_I->ether_dhost,&utcpip_mac_brdaddr,6))
		return UTCPIP_RET_OKAY;

	switch(utcpip_ntohs(pETH_I->ether_type))
	{
		default:
			{
			}
			break;
		case UTCPIP_ETHERTYPE_ARP:
			{
				if (d->trx_msg_len < (sizeof(utcpip_eth_hdr_t) + sizeof(utcpip_arp_t)))
					return UTCPIP_RET_FAIL;
					
				utcpip_arp_t *pARP_I = (utcpip_arp_t *)(pETH_I+1);
				
				uint16_t ar_hdr = utcpip_ntohs(pARP_I->ar_hdr);
				uint16_t ar_pro = utcpip_ntohs(pARP_I->ar_pro);
				uint8_t  ar_hln = pARP_I->ar_hln;
				uint8_t  ar_pln = pARP_I->ar_pln;
				uint16_t ar_op = utcpip_ntohs(pARP_I->ar_op);

				if (d->verbose >= 3)
				{
					xil_printf("\r\n ARP hdr=%d=%s pro=%d=%s op=%d=%s\r\n",
						ar_hdr,
						(ar_hdr == UTCPIP_ARP_HDR_ETHER) ? "ETH" : "",
						ar_pro,
						(ar_pro == UTCPIP_ETHERTYPE_IP) ? "IP" : "",
						ar_op,
						(ar_op == UTCPIP_ARP_OP_REQUEST) ? "RQ" :
						(ar_op == UTCPIP_ARP_OP_REPLY) ? "RY" : "");
				}

				if (ar_hdr == UTCPIP_ARP_HDR_ETHER && ar_pro == UTCPIP_ETHERTYPE_IP && ar_hln == 6 && ar_pln == 4)
				{
					if (ar_op == UTCPIP_ARP_OP_REQUEST && 0 == memcmp(&pARP_I->ar_var.ar_ether_ip.ar_tpa,&d->ip_address,4))
					{
						d->decoded = UTCPIP_RECEIVED_ARP_REQUEST;
						d->ctx.arp.eth_mac_address = pETH_I->ether_shost;
						d->ctx.arp.arp_mac_address = pARP_I->ar_var.ar_ether_ip.ar_sha;
						d->ctx.arp.arp_ip_address  = pARP_I->ar_var.ar_ether_ip.ar_spa;
					}
					if (ar_op == UTCPIP_ARP_OP_REPLY)
					{
						d->decoded = UTCPIP_RECEIVED_ARP_REPLY;
						d->ctx.arp.eth_mac_address = pETH_I->ether_shost;
						d->ctx.arp.arp_mac_address = pARP_I->ar_var.ar_ether_ip.ar_sha;
						d->ctx.arp.arp_ip_address  = pARP_I->ar_var.ar_ether_ip.ar_spa;
					}
				}
			}
			break;
		case UTCPIP_ETHERTYPE_IP:
			{
				if (d->trx_msg_len < (sizeof(utcpip_eth_hdr_t) + sizeof(utcpip_ip_t)))
					return UTCPIP_RET_FAIL;
					
				utcpip_ip_t *pIP_I = (utcpip_ip_t *)(pETH_I+1);
				
				uint8_t    ip_vhl = pIP_I->ip_vhl;
				//uint8_t    ip_tos = pIP_I->ip_tos;
				uint16_t   ip_len = utcpip_ntohs(pIP_I->ip_len);
				//uint16_t   ip_id = utcpip_ntohs(pIP_I->ip_id);
				//uint16_t   ip_off = utcpip_ntohs(pIP_I->ip_off);
				//uint8_t    ip_ttl = pIP_I->ip_ttl;
				uint8_t    ip_p = pIP_I->ip_p;
				//uint16_t   ip_sum = utcpip_ntohs(pIP_I->ip_sum);

				if (d->verbose >= 3)
				{
					xil_printf("\r\n IP vhl=%02X len=%d p=%d=%s\r\n",
						ip_vhl,
						ip_len,
						ip_p,
						(ip_p == UTCPIP_IPPROTO_ICMP) ? "ICMP" :
						(ip_p == UTCPIP_IPPROTO_UDP) ? "UDP" :
						(ip_p == UTCPIP_IPPROTO_TCP) ? "TCP" : "");
				}
					
				utcpip_cksum_env_t cks;
				utcpip_cksum_init(&cks);
				utcpip_cksum_buffer(&cks,pIP_I,20);
				utcpip_cksum_finish(&cks);
				if (0 == utcpip_cksum_result(&cks))
				{
					if (ip_vhl == 0x45 && ip_len >= 20 && (d->trx_msg_len - ip_len) >= 14)
					{
						if (pIP_I->ip_dst.s_addr == d->ip_address.s_addr || pIP_I->ip_dst.s_addr == utcpip_ip_brdaddr)
						{
							switch(ip_p)
							{
								default:
									{
									}
									break;
								case UTCPIP_IPPROTO_ICMP:
									{
										if (ip_len >= (20+4))
										{
											utcpip_icmp_t *pICMP_I = (utcpip_icmp_t *)(pIP_I+1);
											
											uint8_t    icmp_type = pICMP_I->icmp_type;
											uint8_t    icmp_code = pICMP_I->icmp_code;
											//uint16_t   icmp_cksum = utcpip_ntohs(pICMP_I->icmp_cksum);

											if (d->verbose >= 3)
											{
												xil_printf("\r\n ICMP type=%d=%s\r\n",
													icmp_type,
													(icmp_type == UTCPIP_ICMP_ECHO) ? "ECHO" :
													(icmp_type == UTCPIP_ICMP_ECHOREPLY) ? "ECHO_REPLY" : "");
											}
											
											int icmpl = ip_len-20;
	
											utcpip_cksum_init(&cks);
											utcpip_cksum_buffer(&cks,pICMP_I,icmpl);
											utcpip_cksum_finish(&cks);
											if (0 == utcpip_cksum_result(&cks))
											{
												if (icmp_type == UTCPIP_ICMP_ECHO && icmp_code == 0)
												{
													d->decoded = UTCPIP_RECEIVED_ICMP_ECHO_REQUEST;
													d->ctx.icmp.eth_mac_address = pETH_I->ether_shost;
													d->ctx.icmp.ip_len          = ip_len;
													d->ctx.icmp.ip_address      = pIP_I->ip_src;
													d->ctx.icmp.icmp_id         = utcpip_ntohs(pICMP_I->icmp_var.icmp_echo.icmp_id);
													d->ctx.icmp.icmp_seq        = utcpip_ntohs(pICMP_I->icmp_var.icmp_echo.icmp_seq);
													d->ctx.icmp.icmp_data       = &pICMP_I->icmp_var.icmp_echo.icmp_data[0];
												}
											}
										}
									}
									break;
								case UTCPIP_IPPROTO_UDP:
									{
										if (ip_len >= (20+8))
										{
											utcpip_udp_t *pUDP_I = (utcpip_udp_t *)(pIP_I+1);
											
											utcpip_in_port_t uh_sport = utcpip_ntohs(pUDP_I->uh_sport);
											utcpip_in_port_t uh_dport = utcpip_ntohs(pUDP_I->uh_dport);
											uint16_t         uh_ulen  = utcpip_ntohs(pUDP_I->uh_ulen);
											uint16_t         uh_sum   = utcpip_ntohs(pUDP_I->uh_sum);

											if (d->verbose >= 3)
											{
												xil_printf("\r\n UDP sp=%d dp=%d ln=%d sm=%d\r\n",
													uh_sport,
													uh_dport,
													uh_ulen,
													uh_sum);
											}
											
											int udpl = ip_len-20;
											
											// do not check for incoming cksum
											
											// match length	
											if (udpl == uh_ulen)
											{
												#if SW_ENA_ETH_AGENT > 0
												if (uh_dport == d->ctx.xnctl.xnctl_udp_dport)
												{
													if (uh_ulen >= 8)
													{
														// save XNCTL env
														d->ctx.xnctl.xnctl_mac_address = pETH_I->ether_shost;
														d->ctx.xnctl.xnctl_ip_address  = pIP_I->ip_src;
														d->ctx.xnctl.xnctl_udp_sport   = uh_sport;

														d->ctx.xnctl.xnctl_data = &pUDP_I->uh_data[0];
														d->ctx.xnctl.xnctl_len  = uh_ulen - 8;
														d->decoded = UTCPIP_RECEIVED_XNCTL_REQUEST;
													}
												}
												#endif
											}
										}
									}
									break;
							}
						}
					}
				}
			}
			break;
	}	
		
	return UTCPIP_RET_OKAY;
}


// ------------------------
// build ARP request telegram
// ------------------------

int utcpip_build_arp_request(utcpip_dev_t *d)
{
	utcpip_eth_hdr_t *pETH_O = (utcpip_eth_hdr_t *)d->trx_msg_pnt;
	utcpip_arp_t     *pARP_O = (utcpip_arp_t *)(pETH_O+1);

	// ethernet header
	pETH_O->ether_dhost = utcpip_mac_brdaddr;
	pETH_O->ether_shost = d->mac_address;
	pETH_O->ether_type  = utcpip_ntohs(UTCPIP_ETHERTYPE_ARP);

	pARP_O->ar_hdr = utcpip_ntohs(UTCPIP_ARP_HDR_ETHER);
	pARP_O->ar_pro = utcpip_ntohs(UTCPIP_ETHERTYPE_IP);
	pARP_O->ar_hln = 6;
	pARP_O->ar_pln = 4;
	pARP_O->ar_op  = utcpip_ntohs(UTCPIP_ARP_OP_REQUEST);

	pARP_O->ar_var.ar_ether_ip.ar_tha = utcpip_mac_unkaddr;
	pARP_O->ar_var.ar_ether_ip.ar_tpa = d->ctx.arp.arp_ip_address;

	pARP_O->ar_var.ar_ether_ip.ar_sha = d->mac_address;
	pARP_O->ar_var.ar_ether_ip.ar_spa = d->ip_address;

	d->trx_msg_len = sizeof(utcpip_eth_hdr_t) + sizeof(utcpip_arp_t);

	return UTCPIP_RET_OKAY;
}

// ------------------------
// build ARP reply telegram
// ------------------------

int utcpip_build_arp_reply(utcpip_dev_t *d)
{
	utcpip_eth_hdr_t *pETH_O = (utcpip_eth_hdr_t *)d->trx_msg_pnt;
	utcpip_arp_t     *pARP_O = (utcpip_arp_t *)(pETH_O+1);

	pETH_O->ether_dhost = d->ctx.arp.eth_mac_address;
	pETH_O->ether_shost = d->mac_address;
	pETH_O->ether_type  = utcpip_htons(UTCPIP_ETHERTYPE_ARP);

	pARP_O->ar_hdr = utcpip_htons(UTCPIP_ARP_HDR_ETHER);
	pARP_O->ar_pro = utcpip_htons(UTCPIP_ETHERTYPE_IP);
	pARP_O->ar_hln = 6;
	pARP_O->ar_pln = 4;
	pARP_O->ar_op  = utcpip_htons(UTCPIP_ARP_OP_REPLY);

	pARP_O->ar_var.ar_ether_ip.ar_tha = d->ctx.arp.arp_mac_address;
	pARP_O->ar_var.ar_ether_ip.ar_tpa = d->ctx.arp.arp_ip_address;

	pARP_O->ar_var.ar_ether_ip.ar_sha = d->mac_address;
	pARP_O->ar_var.ar_ether_ip.ar_spa = d->ip_address;

	d->trx_msg_len = sizeof(utcpip_eth_hdr_t) + sizeof(utcpip_arp_t);

	return UTCPIP_RET_OKAY;
}

// ------------------------
// build ICMP reply telegram
// ------------------------

int utcpip_build_icmp_echo_reply(utcpip_dev_t *d)
{
	utcpip_eth_hdr_t *pETH_O = (utcpip_eth_hdr_t *)d->trx_msg_pnt;
	utcpip_ip_t      *pIP_O   = (utcpip_ip_t *)(pETH_O+1);
	utcpip_icmp_t    *pICMP_O = (utcpip_icmp_t *)(pIP_O+1);

	pETH_O->ether_dhost = d->ctx.icmp.eth_mac_address;
	pETH_O->ether_shost = d->mac_address;
	pETH_O->ether_type  = utcpip_htons(UTCPIP_ETHERTYPE_IP);

	uint16_t ip_len = d->ctx.icmp.ip_len;
	d->reply_ip_id += 1;

	pIP_O->ip_vhl = 0x45;
	pIP_O->ip_tos = 0;
	pIP_O->ip_len = utcpip_htons(ip_len);
	pIP_O->ip_id  = utcpip_htons(d->reply_ip_id);
	pIP_O->ip_off = utcpip_htons(0);
	pIP_O->ip_ttl = 255;
	pIP_O->ip_p   = UTCPIP_IPPROTO_ICMP;
	pIP_O->ip_sum = utcpip_htons(0); // forward evaluation
	pIP_O->ip_dst = d->ctx.icmp.ip_address;
	pIP_O->ip_src = d->ip_address;

	pICMP_O->icmp_type  = UTCPIP_ICMP_ECHOREPLY;
	pICMP_O->icmp_code  = 0;
	pICMP_O->icmp_cksum = utcpip_htons(0); // forward evaluation

	pICMP_O->icmp_var.icmp_echo.icmp_id  = utcpip_htons(d->ctx.icmp.icmp_id);
	pICMP_O->icmp_var.icmp_echo.icmp_seq = utcpip_htons(d->ctx.icmp.icmp_seq);

	if ((&pICMP_O->icmp_var.icmp_echo.icmp_data[0]) != d->ctx.icmp.icmp_data)
	{
		memcpy(
				&pICMP_O->icmp_var.icmp_echo.icmp_data[0],
				d->ctx.icmp.icmp_data,
				d->ctx.icmp.ip_len - (20+8));
	}

	// IP cksum
	utcpip_cksum_env_t cks;
	utcpip_cksum_init(&cks);
	utcpip_cksum_buffer(&cks,pIP_O,20);
	utcpip_cksum_finish(&cks);
	pIP_O->ip_sum = utcpip_cksum_result(&cks);

	// ICMP cksum
	utcpip_cksum_init(&cks);
	utcpip_cksum_buffer(&cks,pICMP_O,d->ctx.icmp.ip_len-20);
	utcpip_cksum_finish(&cks);
	pICMP_O->icmp_cksum = utcpip_cksum_result(&cks);

	d->trx_msg_len = sizeof(utcpip_eth_hdr_t) + ip_len;

	return UTCPIP_RET_OKAY;
}


#if SW_ENA_ETH_AGENT > 0

// ------------------------
// build XNCTL reply telegram
// ------------------------

int utcpip_build_xnctl_reply(utcpip_dev_t *d)
{
	utcpip_eth_hdr_t *pETH_O  = (utcpip_eth_hdr_t *)d->trx_msg_pnt;
	utcpip_ip_t      *pIP_O   = (utcpip_ip_t *)(pETH_O+1);
	utcpip_udp_t     *pUDP_O  = (utcpip_udp_t *)(pIP_O+1);

	pETH_O->ether_dhost = d->ctx.xnctl.xnctl_mac_address;
	pETH_O->ether_shost = d->mac_address;
	pETH_O->ether_type  = utcpip_htons(UTCPIP_ETHERTYPE_IP);

	uint16_t ip_len = 20 + 8 + d->ctx.xnctl.xnctl_len;
	d->reply_ip_id += 1;

	pIP_O->ip_vhl = 0x45;
	pIP_O->ip_tos = 0;
	pIP_O->ip_len = utcpip_htons(ip_len);
	pIP_O->ip_id  = utcpip_htons(d->reply_ip_id);
	pIP_O->ip_off = utcpip_htons(0);
	pIP_O->ip_ttl = 255;
	pIP_O->ip_p   = UTCPIP_IPPROTO_UDP;
	pIP_O->ip_sum = utcpip_htons(0); // forward evaluation
	pIP_O->ip_dst = d->ctx.xnctl.xnctl_ip_address;
	pIP_O->ip_src = d->ip_address;

	uint16_t uh_ulen = 8 + d->ctx.xnctl.xnctl_len;

	pUDP_O->uh_dport    = utcpip_htons(d->ctx.xnctl.xnctl_udp_sport);
	pUDP_O->uh_sport    = utcpip_htons(d->ctx.xnctl.xnctl_udp_dport);
	pUDP_O->uh_ulen     = utcpip_htons(uh_ulen);
	pUDP_O->uh_sum      = utcpip_htons(0); // no cksum

	if ((&pUDP_O->uh_data[0]) != d->ctx.xnctl.xnctl_data)
	{
		memcpy(
			&pUDP_O->uh_data[0],
			d->ctx.xnctl.xnctl_data,
			d->ctx.xnctl.xnctl_len);
	}

	// IP cksum
	utcpip_cksum_env_t cks;
	utcpip_cksum_init(&cks);
	utcpip_cksum_buffer(&cks,pIP_O,20);
	utcpip_cksum_finish(&cks);
	pIP_O->ip_sum = utcpip_cksum_result(&cks);

	d->trx_msg_len = sizeof(utcpip_eth_hdr_t) + ip_len;

	return UTCPIP_RET_OKAY;
}

#endif
