/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ultra light tcp/ip test
/// \file utcpip.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

/// \fn int utcpip_client(utcpip_dev_t *d)
/// \brief tcp/ip protocol state machine function
/// \param d environment pointer
/// \return execution status UTCPIP_RET_OKAY, or UTCPIP_RET_FAIL


#ifndef UTCPIP_H_
#define UTCPIP_H_

#include "xparameters.h"

#define UTCPIP_PACK_STRUCT(x) __attribute__((packed)) x

typedef union
{
	struct { uint16_t w1,w2,w3; } UTCPIP_PACK_STRUCT(w);
	struct { uint8_t  b1,b2,b3,b4,b5,b6; } UTCPIP_PACK_STRUCT(b);
} UTCPIP_PACK_STRUCT(utcpip_mac_addr_t);


typedef uint32_t utcpip_in_addr_t;
typedef uint16_t utcpip_in_port_t;

typedef union
{
	utcpip_in_addr_t s_addr;
	struct { uint8_t  b1,b2,b3,b4; } UTCPIP_PACK_STRUCT(b);
} UTCPIP_PACK_STRUCT(utcpip_ip_addr_t);

#define UTCPIP_ETHERTYPE_IP  0x0800
#define UTCPIP_ETHERTYPE_ARP 0x0806

typedef struct
{
	utcpip_mac_addr_t ether_dhost;
	utcpip_mac_addr_t ether_shost;
	uint16_t          ether_type;
} UTCPIP_PACK_STRUCT(utcpip_eth_hdr_t);

typedef struct
{
	uint16_t ar_hdr;
	#define UTCPIP_ARP_HDR_ETHER 1
	uint16_t ar_pro;
	uint8_t  ar_hln;
	uint8_t  ar_pln;
	uint16_t ar_op;
	#define UTCPIP_ARP_OP_REQUEST 1
	#define UTCPIP_ARP_OP_REPLY   2
	union
	{
		struct
		{
			utcpip_mac_addr_t ar_sha;
			utcpip_ip_addr_t  ar_spa;
			utcpip_mac_addr_t ar_tha;
			utcpip_ip_addr_t  ar_tpa;
			
		} UTCPIP_PACK_STRUCT(ar_ether_ip);
	} UTCPIP_PACK_STRUCT(ar_var);
} UTCPIP_PACK_STRUCT(utcpip_arp_t);

typedef struct
{
	uint8_t          ip_vhl;
	uint8_t          ip_tos;
	uint16_t         ip_len;
	uint16_t         ip_id;
	uint16_t         ip_off;
	#define UTCPIP_IP_RF      0x8000
	#define UTCPIP_IP_DF      0x4000
	#define UTCPIP_IP_MF      0x2000
	#define UTCPIP_IP_OFFMASK 0x1FFF
	uint8_t          ip_ttl;
	uint8_t          ip_p;
	uint16_t         ip_sum;
	utcpip_ip_addr_t ip_src;
	utcpip_ip_addr_t ip_dst;
} UTCPIP_PACK_STRUCT(utcpip_ip_t);

#define UTCPIP_IPPROTO_ICMP  1
#define UTCPIP_IPPROTO_TCP   6
#define UTCPIP_IPPROTO_UDP  17

typedef struct
{
	uint8_t    icmp_type;
	#define UTCPIP_ICMP_ECHOREPLY 0
	#define UTCPIP_ICMP_ECHO      8
	uint8_t    icmp_code;
	uint16_t   icmp_cksum;
	union
	{
		struct
		{
			uint16_t icmp_id;
			uint16_t icmp_seq;
			#define UTCPIP_ICMP_ECHO_DATA_MAX (1500-8-20)
			uint8_t  icmp_data[UTCPIP_ICMP_ECHO_DATA_MAX];
		} UTCPIP_PACK_STRUCT(icmp_echo);
	} UTCPIP_PACK_STRUCT(icmp_var);
} UTCPIP_PACK_STRUCT(utcpip_icmp_t);

typedef struct
{
	utcpip_in_port_t uh_sport;
	utcpip_in_port_t uh_dport;
	uint16_t         uh_ulen;
	uint16_t         uh_sum;
	#define UTCPIP_UDP_DATA_MAX (1500-8-20)
	uint8_t          uh_data[UTCPIP_UDP_DATA_MAX];
} UTCPIP_PACK_STRUCT(utcpip_udp_t);

typedef struct
{
	uint16_t   tftp_op;
	#define UTCPIP_TFTP_RRQ   1
	#define UTCPIP_TFTP_WRQ   2
	#define UTCPIP_TFTP_DATA  3
	#define UTCPIP_TFTP_ACK   4
	#define UTCPIP_TFTP_ERROR 5
	union
	{
		struct
		{
			#define UTCPIP_TFTP_RRQ_COOKIE_MAX (1500-2-20)
			uint8_t  tftp_cookies[UTCPIP_TFTP_RRQ_COOKIE_MAX];
		} UTCPIP_PACK_STRUCT(tftp_rrq);
		struct
		{
			#define UTCPIP_TFTP_WRQ_COOKIE_MAX (1500-2-20)
			uint8_t  tftp_cookies[UTCPIP_TFTP_WRQ_COOKIE_MAX];
		} UTCPIP_PACK_STRUCT(tftp_wrq);
		struct
		{
			uint16_t tftp_block;
			#define UTCPIP_TFTP_DATA_MAX (1500-4-20)
			uint8_t  tftp_data[UTCPIP_TFTP_DATA_MAX];
		} UTCPIP_PACK_STRUCT(tftp_data);
		struct
		{
			uint16_t tftp_block;
		} UTCPIP_PACK_STRUCT(tftp_ack);
		struct
		{
			uint16_t tftp_error_code;
			#define UTCPIP_TFTP_ERR_CODE_UNDEF   0
			#define UTCPIP_TFTP_ERR_CODE_NOFOUND 1
			#define UTCPIP_TFTP_ERR_CODE_ACCFAIL 2
			#define UTCPIP_TFTP_ERR_CODE_FULL    3
			#define UTCPIP_TFTP_ERR_CODE_ILLEGAL 4
			#define UTCPIP_TFTP_ERR_CODE_UNKID   5
			#define UTCPIP_TFTP_ERR_CODE_FEXIST  6
			#define UTCPIP_TFTP_ERR_CODE_NOUSER  7
			
			#define UTCPIP_TFTP_ERROR_MAX (1500-4-20)
			uint8_t  tftp_error_msg[UTCPIP_TFTP_ERROR_MAX];
		} UTCPIP_PACK_STRUCT(tftp_error);
	} UTCPIP_PACK_STRUCT(tftp_var);
} UTCPIP_PACK_STRUCT(utcpip_tftp_t);


//
// device context
//

typedef struct utcpip_dev_s
{
	utcpip_mac_addr_t mac_address;
	utcpip_ip_addr_t  ip_address ;
	
	uint8_t         * trx_msg_pnt;
	uint16_t          trx_msg_len;
	
	int               verbose;
	
	uint16_t          reply_ip_id;
	

	// specific protocol data

	struct
	{
		// ARP specific data
		struct
		{
			utcpip_mac_addr_t eth_mac_address;
			utcpip_mac_addr_t arp_mac_address;
			utcpip_ip_addr_t  arp_ip_address;
		} arp;
		// ICMP specific data
		struct
		{
			utcpip_mac_addr_t  eth_mac_address;
			utcpip_ip_addr_t   ip_address;
			uint16_t           ip_len;
			uint16_t           icmp_id;
			uint16_t           icmp_seq;
			uint8_t          * icmp_data;
		} icmp;
		#if SW_ENA_ETH_AGENT > 0
		// XNCTL
		struct
		{
			utcpip_mac_addr_t xnctl_mac_address;
			utcpip_ip_addr_t  xnctl_ip_address;
			utcpip_in_port_t  xnctl_udp_sport;
			utcpip_in_port_t  xnctl_udp_dport;
			uint8_t          *xnctl_data;
			uint16_t          xnctl_len;
		} xnctl;
		#endif
	} ctx;

	uint8_t  decoded;
	#define UTCPIP_RECEIVED_NONE              0
	#define UTCPIP_RECEIVED_ARP_REQUEST       1
	#define UTCPIP_RECEIVED_ARP_REPLY         2
	#define UTCPIP_RECEIVED_ICMP_ECHO_REQUEST 3
	#define UTCPIP_RECEIVED_FTFP_WRQ          4
	#define UTCPIP_RECEIVED_FTFP_RRQ          5
	#define UTCPIP_RECEIVED_FTFP_DATA         6
	#define UTCPIP_RECEIVED_FTFP_ACK          7
	#define UTCPIP_RECEIVED_XNCTL_REQUEST     8

} utcpip_dev_t;

//
extern const utcpip_mac_addr_t utcpip_mac_brdaddr;
extern const utcpip_mac_addr_t utcpip_mac_unkaddr;
//

int utcpip_client(utcpip_dev_t *d);
int utcpip_build_arp_request(utcpip_dev_t *d);
int utcpip_build_arp_reply(utcpip_dev_t *d);
int utcpip_build_icmp_echo_reply(utcpip_dev_t *d);
int utcpip_build_xnctl_reply(utcpip_dev_t *d);

#define UTCPIP_RET_OKAY 0
#define UTCPIP_RET_FAIL (-1)

#endif /*UTCPIP_H_*/
