/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief Zynq Ethernet driver
/// \file zq_ethmac.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "xparameters.h"

#include "xstatus.h"
#include "xil_types.h"

#ifdef XPAR_XEMACPS_0_DEVICE_ID

#if defined (__arm__) || defined (__aarch64__) && defined(XPAR_XEMACPS_0_DEVICE_ID)

#include "board.h"

#include "xil_cache.h"
#include "xil_mmu.h"

#include "xemacps.h"
#include "xemacps_bd.h"
#include "xemacps_bdring.h"

#include "xil_printf.h"

#include "utcpip.h"
#include "conutil.h"
#include "xx_timer.h"
#include "xx_ethmac.h"


#if defined (__aarch64__) || defined (ARMR5)

	#define CRL_GEM0_REF_CTRL	(XPAR_PSU_CRL_APB_S_AXI_BASEADDR + 0x50)
	#define CRL_GEM1_REF_CTRL	(XPAR_PSU_CRL_APB_S_AXI_BASEADDR + 0x54)
	#define CRL_GEM2_REF_CTRL	(XPAR_PSU_CRL_APB_S_AXI_BASEADDR + 0x58)
	#define CRL_GEM3_REF_CTRL	(XPAR_PSU_CRL_APB_S_AXI_BASEADDR + 0x5C)

#endif

#if defined (__arm__) && !defined (ARMR5)


	#define SLCR_LOCK_ADDR                  (XPS_SYS_CTRL_BASEADDR + 0x4)
	#define SLCR_UNLOCK_ADDR                (XPS_SYS_CTRL_BASEADDR + 0x8)
	#define SLCR_GEM0_CLK_CTRL_ADDR         (XPS_SYS_CTRL_BASEADDR + 0x140)

	#define SLCR_GEM_10M_CLK_CTRL_VALUE     0x00803201
	#define SLCR_GEM_100M_CLK_CTRL_VALUE    0x00800501
	#define SLCR_GEM_1G_CLK_CTRL_VALUE      0x00800101

	#define SLCR_LOCK_KEY_VALUE             0x767B
	#define SLCR_UNLOCK_KEY_VALUE           0xDF0D

#endif

#if SW_ENA_OCM_RAM > 0
	#define EMAC_TLB_SIZE 64
#else
	#if defined (__aarch64__)
		#define EMAC_TLB_SIZE 0x200000
	#else
		#define EMAC_TLB_SIZE 0x100000
	#endif
#endif

//-
#define EMAC_TLB_ALIGN    __attribute__ ((aligned(EMAC_TLB_SIZE)))
#define EMAC_BD_ALIGN_BD  __attribute__ ((aligned(64)))
#define EMAC_TGM_ALIGN    __attribute__ ((aligned(64)))

// RX & RX descriptors
#define EMAC_N_RX_DESC 8
#define EMAC_N_TX_DESC 8

// MAC telegram size
#define EMAC_TGM_RAW_SIZE 1536

typedef struct
{
	u8  b[EMAC_TGM_RAW_SIZE];
} emac_tgm_t EMAC_TGM_ALIGN;

// buffer descriptors shared memory
typedef union
{
	struct
	{
		struct
		{
			XEmacPs_Bd work[EMAC_N_RX_DESC] EMAC_BD_ALIGN_BD;
			XEmacPs_Bd term                 EMAC_BD_ALIGN_BD;
		} Rx EMAC_BD_ALIGN_BD;

		struct
		{
			XEmacPs_Bd work[EMAC_N_TX_DESC] EMAC_BD_ALIGN_BD;
			XEmacPs_Bd term                 EMAC_BD_ALIGN_BD;
		} Tx EMAC_BD_ALIGN_BD;

	} bds EMAC_BD_ALIGN_BD;

#if SW_ENA_OCM_RAM == 0
	struct
	{
		u8 spare_4_align[EMAC_TLB_SIZE];
	};
#endif

} emac_bds_mem_t EMAC_TLB_ALIGN;


static emac_bds_mem_t emac_bds_mem_v EMAC_TLB_ALIGN;


// telegram buffers shared memory
typedef struct
{
	struct
	{
		emac_tgm_t Telegrams[EMAC_N_RX_DESC] EMAC_TGM_ALIGN;
	} Rx EMAC_TGM_ALIGN;

	struct
	{
		emac_tgm_t Telegrams[EMAC_N_TX_DESC] EMAC_TGM_ALIGN;
	} Tx EMAC_TGM_ALIGN;

} emac_tgm_mem_t EMAC_TGM_ALIGN;

static emac_tgm_mem_t emac_tgm_mem_v EMAC_TGM_ALIGN;



typedef struct
{
	XEmacPs DevEmac;
	u32     PhyAddress;
	u32     PhyIdent;
	const char * PhyMaker;
	const char * PhyDevice;

#define PHY_IDENT_MASK  0xFFFFFFF0

#define KSZ9031_IDENT   0x00221622
#define M88E1512_IDENT  0x01410DD1
#define M88E1116R_IDENT 0x01410E40
#define RTL8211E_IDENT  0x001CC915
#define DP83867_IDENT   0x2000A231

	u16     PhySpeed;

	u8      mac_address[6];
	u8      mac_factory;

	u8      tlb_initialized;

} xmc_t;

static xmc_t xmc_v;

#define XEMHD (&ref->DevEmac)
#define XX_ETHMAC_DEF xmc_t * ref = & xmc_v

static void xx_ethmac_memcpy_4(void* dst, const void* src, u32 cnt)
{
	if (cnt > 0)
	{
		u32  *pD = (u32 *)dst;
		const u32 *pS = (const u32 *)src;
		int   nW = (cnt + 3) >> 2;
		do {
			*pD++ = *pS++;
		} while(--nW > 0);
	}
}

static int xx_ethmac_tlb_init(xmc_t *ref)
{
#if SW_ENA_OCM_RAM == 0

	if (ref->tlb_initialized == 0)
	{
		#if defined (__aarch64__)
		Xil_SetTlbAttributes((UINTPTR)&emac_bds_mem_v, NORM_NONCACHE | INNER_SHAREABLE);
		#endif

		#if defined (__arm__) && defined (ARMR5)
		Xil_SetTlbAttributes((INTPTR)&emac_bds_mem_v, STRONG_ORDERD_SHARED | PRIV_RW_USER_RW);
		#endif

		#if defined (__arm__) && !defined (ARMR5)
		Xil_SetTlbAttributes((INTPTR)&emac_bds_mem_v, DEVICE_MEMORY);
		#endif

		ref->tlb_initialized = 1;
	}

#endif

	return XST_SUCCESS;
}

static void xx_ethmac_phy_setup(xmc_t *ref)
{
	#ifdef SYS_CARRIER_TE0720
	// weak pull-up on mdio (10k) !
	XEmacPs_SetMdioDivisor(XEMHD,MDC_DIV_16);

	// factory MAC
	{
		u16 PhyData;
		int i;
		for(i = 0; i < 3;i++)
		{
			XEmacPs_PhyRead(XEMHD,0x1A,9+i,&PhyData);
			ref->mac_address[i*2+0] = (unsigned char)(PhyData >> 8);
			ref->mac_address[i*2+1] = (unsigned char)(PhyData & 0xFF);
		}
		#if 0 // display factory MAC
		xil_printf("MAC: %02X.%02X.%02X-%02X.%02X.%02X\r\n",
				ref->mac_address[0],
				ref->mac_address[1],
				ref->mac_address[2],
				ref->mac_address[3],
				ref->mac_address[4],
				ref->mac_address[5]);
		#endif
		// check for valid
		if (memcmp(&ref->mac_address[0],&utcpip_mac_brdaddr,6) && memcmp(&ref->mac_address[0],&utcpip_mac_unkaddr,6))
		{
			ref->mac_factory = 1; // lock
		}
	}
	#endif

	for(ref->PhyAddress = 31;;ref->PhyAddress--)
	{
		#ifdef SYS_CARRIER_TE0720
		if (ref->PhyAddress == 26)
		{
			continue;
		}
		#endif

		// get phy ident word
		u16 PhyData;
		XEmacPs_PhyRead(XEMHD,ref->PhyAddress,2,&PhyData);
		ref->PhyIdent = (u32)PhyData << 16;
		XEmacPs_PhyRead(XEMHD,ref->PhyAddress,3,&PhyData);
		ref->PhyIdent |= PhyData & 0xFFFF;
		//xil_printf("XEmacPs_PhyIdent(%d)=x%08X\r\n",ref->PhyAddress,ref->PhyIdent);
		switch(ref->PhyIdent & PHY_IDENT_MASK)
		{
			case (KSZ9031_IDENT   & PHY_IDENT_MASK): ref->PhyMaker = "MICREL" ; ref->PhyDevice = "KSZ9031"  ; break;
			case (M88E1512_IDENT  & PHY_IDENT_MASK): ref->PhyMaker = "MARVELL"; ref->PhyDevice = "M88E1512" ; break;
			case (M88E1116R_IDENT & PHY_IDENT_MASK): ref->PhyMaker = "MARVELL"; ref->PhyDevice = "M88E1116R"; break;
			case (RTL8211E_IDENT  & PHY_IDENT_MASK): ref->PhyMaker = "Realtek"; ref->PhyDevice = "RTL8211E" ; break;
			case (DP83867_IDENT   & PHY_IDENT_MASK): ref->PhyMaker = "TI"     ; ref->PhyDevice = "DP83867"  ; break;
			default                                : ref->PhyMaker = NULL     ; ref->PhyDevice = NULL       ; break;
		}
		if (ref->PhyMaker != NULL)
		{
			break;
		}
		if (ref->PhyAddress == 0)
		{
			xil_printf("XEmacPs_PhyIdent(%d)=x%08X\r\n",ref->PhyAddress,ref->PhyIdent);
			break;
		}
	}

	//xil_printf("XEmacPs_PhyIdent(%d)=x%08X : %s %s\r\n",ref->PhyAddress,ref->PhyIdent,ref->PhyMaker,ref->PhyDevice);

	if ((ref->PhyIdent & PHY_IDENT_MASK) == (DP83867_IDENT & PHY_IDENT_MASK))
	{
		u32 one_sec = xx_timer_cnt_pser_sec();
		u32 time_st = xx_timer_cnt_val();
		int n;
		for(n = 0; n < 5;)
		{
			u32 time_df = xx_timer_cnt_val() - time_st;
			if (time_df >= one_sec)
			{
				time_st += one_sec;
				n++;
			}

			u16 PhyData;
			XEmacPs_PhyRead(XEMHD,ref->PhyAddress,1,&PhyData);
			if ((PhyData & 0x0004) != 0)
			{
				break;
			}
		}
	}
}

static void xx_ethmac_getspeedphy(xmc_t *ref)
{
	switch(ref->PhyIdent & PHY_IDENT_MASK)
	{
		case (KSZ9031_IDENT & PHY_IDENT_MASK):
			{
				u32 RegNum = 0x1F;
				u16 PhyData;
				XEmacPs_PhyRead(XEMHD,ref->PhyAddress,RegNum,&PhyData);
				if (0 != (PhyData & (1 << 6))) { ref->PhySpeed = 1000; break; }
				if (0 != (PhyData & (1 << 5))) { ref->PhySpeed = 100;break; }
				if (0 != (PhyData & (1 << 4))) { ref->PhySpeed = 10; break;	}
			}
			break;
		case (M88E1512_IDENT  & PHY_IDENT_MASK):
		case (M88E1116R_IDENT & PHY_IDENT_MASK):
		case (RTL8211E_IDENT  & PHY_IDENT_MASK):
		case (DP83867_IDENT   & PHY_IDENT_MASK):
		default:
			{
				u32 RegNum = 0x11;
				u16 PhyData;
				XEmacPs_PhyRead(XEMHD,ref->PhyAddress,RegNum,&PhyData);
				switch((PhyData >> 14) & 3)
				{
					case    2: ref->PhySpeed = 1000; break;
					case    1: ref->PhySpeed =  100; break;
					default  : ref->PhySpeed =   10; break;
				}

			}
			break;
	}
}

static void xx_ethmac_slcr_clock_setup(xmc_t *ref)
{
	#if defined (__aarch64__) || defined (ARMR5)
	{
		volatile unsigned int * p_gem_ref_ctrl = 0;

		#if defined(XPAR_PSU_ETHERNET_0_BASEADDR)
		if (ref->DevEmac.Config.BaseAddress == XPAR_PSU_ETHERNET_0_BASEADDR)
		{
			p_gem_ref_ctrl = (volatile unsigned int *)CRL_GEM0_REF_CTRL;
		}
		#endif
		#if defined(XPAR_PSU_ETHERNET_1_BASEADDR)
		if (ref->DevEmac.Config.BaseAddress == XPAR_PSU_ETHERNET_1_BASEADDR)
		{
			p_gem_ref_ctrl = (volatile unsigned int *)CRL_GEM1_REF_CTRL;
		}
		#endif
		#if defined(XPAR_PSU_ETHERNET_2_BASEADDR)
		if (ref->DevEmac.Config.BaseAddress == XPAR_PSU_ETHERNET_2_BASEADDR)
		{
			p_gem_ref_ctrl = (volatile unsigned int *)CRL_GEM2_REF_CTRL;
		}
		#endif
		#if defined(XPAR_PSU_ETHERNET_3_BASEADDR)
		if (ref->DevEmac.Config.BaseAddress == XPAR_PSU_ETHERNET_3_BASEADDR)
		{
			p_gem_ref_ctrl = (volatile unsigned int *)CRL_GEM3_REF_CTRL;
		}
		#endif
		if (p_gem_ref_ctrl)
		{

	#define GEMX_CTRL_CLK_10M_SET   (XPAR_XEMACPS_0_ENET_SLCR_10Mbps_DIV0   << 8) | (XPAR_XEMACPS_0_ENET_SLCR_10Mbps_DIV1   << 16)
	#define GEMX_CTRL_CLK_100M_SET  (XPAR_XEMACPS_0_ENET_SLCR_100Mbps_DIV0  << 8) | (XPAR_XEMACPS_0_ENET_SLCR_100Mbps_DIV1  << 16)
	#define GEMX_CTRL_CLK_1000M_SET (XPAR_XEMACPS_0_ENET_SLCR_1000Mbps_DIV0 << 8) | (XPAR_XEMACPS_0_ENET_SLCR_1000Mbps_DIV1 << 16)
	#define GEMX_CTRL_CLK_MASK      (0x3F << 8) | (0x3F << 16)

			u32 setvalue  = GEMX_CTRL_CLK_1000M_SET;
			switch(ref->PhySpeed)
			{
			case   10: setvalue = GEMX_CTRL_CLK_10M_SET; break;
			case  100: setvalue = GEMX_CTRL_CLK_100M_SET; break;
			case 1000: setvalue = GEMX_CTRL_CLK_1000M_SET; break;
			default: break;
			}
			u32 tmp = * p_gem_ref_ctrl;
			tmp &= ~GEMX_CTRL_CLK_MASK;
			tmp |= setvalue;
			* p_gem_ref_ctrl = tmp;
		}
	}
	#endif

	#if defined (__arm__) && !defined (ARMR5)
	{
		u32 tmp = SLCR_GEM_100M_CLK_CTRL_VALUE;
		switch(ref->PhySpeed)
		{
			case   10: tmp = SLCR_GEM_10M_CLK_CTRL_VALUE ; break;
			case  100: tmp = SLCR_GEM_100M_CLK_CTRL_VALUE; break;
			case 1000: tmp = SLCR_GEM_1G_CLK_CTRL_VALUE  ; break;
			default: break;
		}
		// SLCR unlock
		*(volatile unsigned int *)(SLCR_UNLOCK_ADDR) = SLCR_UNLOCK_KEY_VALUE;
		// GEM0 clock configuration
		*(volatile unsigned int *)(SLCR_GEM0_CLK_CTRL_ADDR) = tmp;
		// SLCR lock
		*(volatile unsigned int *)(SLCR_LOCK_ADDR) = SLCR_LOCK_KEY_VALUE;
	}
	#endif
}

static int xx_ethmac_dma_init(xmc_t *ref)
{
	XEmacPs_BdRing *RxRingPtr = &XEmacPs_GetRxRing(XEMHD);
	XEmacPs_BdRing *TxRingPtr = &XEmacPs_GetTxRing(XEMHD);

	//xil_printf("RxRingPtr: 0x%08X\r\n",RxRingPtr);
	//xil_printf("TxRingPtr: 0x%08X\r\n",TxRingPtr);

	//-
	LONG rc = XST_SUCCESS;
	// setup RxDB space

	XEmacPs_Bd BdTemplate;

	//-RX

	// ring for RX
	rc = XEmacPs_BdRingCreate(RxRingPtr,
				(UINTPTR)&emac_bds_mem_v.bds.Rx.work[0],
				(UINTPTR)&emac_bds_mem_v.bds.Rx.work[0],
				XEMACPS_DMABD_MINIMUM_ALIGNMENT,
				EMAC_N_RX_DESC);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_BdRingCreate(RX)=%d\r\n",rc);
		return XST_FAILURE;
	}
	//xil_printf("RxRingPtr->Separation=%d\r\n",RxRingPtr->Separation);

	XEmacPs_BdClear(&BdTemplate);
	//-
	rc = XEmacPs_BdRingClone(RxRingPtr,&BdTemplate,XEMACPS_RECV);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_BdRingClone(RX)=%d\r\n",rc);
		return XST_FAILURE;
	}

	//-TX

	// ring for TX
	rc = XEmacPs_BdRingCreate(TxRingPtr,
					(UINTPTR)&emac_bds_mem_v.bds.Tx.work[0],
					(UINTPTR)&emac_bds_mem_v.bds.Tx.work[0],
					XEMACPS_DMABD_MINIMUM_ALIGNMENT,
					EMAC_N_TX_DESC);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_BdRingCreate(TX)=%d\r\n",rc);
		return XST_FAILURE;
	}
	//xil_printf("TxRingPtr->Separation=%d\r\n",TxRingPtr->Separation);

	XEmacPs_BdClear(&BdTemplate);
	XEmacPs_BdSetStatus(&BdTemplate,XEMACPS_TXBUF_USED_MASK);
	//-
	rc = XEmacPs_BdRingClone(TxRingPtr,&BdTemplate,XEMACPS_SEND);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_BdRingClone(TX)=%d\r\n",rc);
		return XST_FAILURE;
	}

	// setup RX ring
	int i;
	for(i = 0;i < EMAC_N_RX_DESC;i++)
	{
		// peek BD
		XEmacPs_Bd * rxbd;
		rc = XEmacPs_BdRingAlloc(RxRingPtr,1,&rxbd);
		if (rc != XST_SUCCESS)
		{
			//xil_printf("XEmacPs_BdRingAlloc(RX)=%d\r\n",rc);
			return rc;
		}

		// setup BD
		XEmacPs_BdSetAddressRx(rxbd,(UINTPTR)&emac_tgm_mem_v.Rx.Telegrams[i].b);
		XEmacPs_BdSetLast(rxbd);

		// to H/W
		rc = XEmacPs_BdRingToHw(RxRingPtr,1,rxbd);
		if (rc != XST_SUCCESS)
		{
			//xil_printf("XEmacPs_BdRingToHw(RX)=%d\r\n",rc);
			return rc;
		}

		if (XEMHD->Version > 2)
		{
			Xil_DCacheFlushRange((INTPTR)rxbd,sizeof(XEmacPs_Bd));
		}
	}

	// GEM with priority support
	if (XEMHD->Version > 2)
	{
		XEmacPs_BdClear(&emac_bds_mem_v.bds.Rx.term);
		XEmacPs_BdSetAddressRx(&emac_bds_mem_v.bds.Rx.term, (XEMACPS_RXBUF_NEW_MASK | XEMACPS_RXBUF_WRAP_MASK));
		XEmacPs_WriteReg(XEMHD->Config.BaseAddress,XEMACPS_RXQ1BASE_OFFSET,(UINTPTR)&emac_bds_mem_v.bds.Rx.term);

		XEmacPs_BdClear(&emac_bds_mem_v.bds.Tx.term);
		XEmacPs_BdSetStatus(&emac_bds_mem_v.bds.Tx.term, (XEMACPS_TXBUF_USED_MASK | XEMACPS_TXBUF_WRAP_MASK));
		XEmacPs_WriteReg(XEMHD->Config.BaseAddress,XEMACPS_TXQBASE_OFFSET,(UINTPTR)&emac_bds_mem_v.bds.Tx.term);

		Xil_DCacheFlushRange((INTPTR)&emac_bds_mem_v.bds.Tx.term,sizeof(XEmacPs_Bd));
	}

	// Set the Queue pointers
	XEmacPs_SetQueuePtr(XEMHD, XEMHD->RxBdRing.BaseBdAddr, 0, XEMACPS_RECV);
	if (XEMHD->Version > 2) {
		XEmacPs_SetQueuePtr(XEMHD, XEMHD->TxBdRing.BaseBdAddr, 1, XEMACPS_SEND);
	} else {
		XEmacPs_SetQueuePtr(XEMHD, XEMHD->TxBdRing.BaseBdAddr, 0, XEMACPS_SEND);
	}

	// data sync barrier
	dsb();

	return XST_SUCCESS;
}


static int xx_ethmac_free_sent(xmc_t *ref)
{
	// transmit RING
	XEmacPs_BdRing * txring = & XEmacPs_GetTxRing(XEMHD);

	XEmacPs_Bd * txbdset;
	u32 nbd = XEmacPs_BdRingFromHwTx(txring,EMAC_N_TX_DESC,&txbdset);
	if (nbd <= 0)
		return XST_SUCCESS;

	XEmacPs_Bd * txbd = txbdset;
	u32 ui;
	for(ui = 0;ui < nbd;ui++)
	{
		txbd = XEmacPs_BdRingNext(txring,txbd);
	}

	LONG rc = XEmacPs_BdRingFree(txring,nbd,txbdset);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_BdRingFree()=%d\r\n",rc);
		return rc;
	}

	// xil_printf("TX free # %d BD\r\n",nbd);

	return rc;
}

//-

int xx_ethmac_relinq_speed(void)
{
	XX_ETHMAC_DEF;

	u16 oldspeed = ref->PhySpeed;

	// get current speed : (PHY point of view!)
	xx_ethmac_getspeedphy(ref);

	if (oldspeed != ref->PhySpeed)
	{
		// speed change: adj MAC PLL and MAC internal

		//xil_printf("EMAC PHY speed = %d\r\n",ref->PhySpeed);

		// clock setup
		xx_ethmac_slcr_clock_setup(ref);

		// set speed
		XEmacPs_SetOperatingSpeed(XEMHD,ref->PhySpeed);
	}
	return XST_SUCCESS;
}

int xx_ethmac_set_mac_addr(const u8* addr)
{
	XX_ETHMAC_DEF;
	if (!ref->mac_factory)
	{
		memcpy(ref->mac_address,addr,6);
	}
	return XST_SUCCESS;
}

int xx_ethmac_get_mac_addr(u8* addr)
{
	XX_ETHMAC_DEF;
	memcpy(addr,ref->mac_address,6);
	return XST_SUCCESS;
}

static int xx_ethmac_set_hw_mac_addr(void)
{
	XX_ETHMAC_DEF;

	u8 ix;
	for(ix = 1; ix <= XEMACPS_MAX_MAC_ADDR; ix++)
	{
		int rc = XEmacPs_SetMacAddress(XEMHD,ref->mac_address,ix);
		if (rc != XST_SUCCESS)
		{
			//xil_printf("XEmacPs_SetMacAddress()=%d\r\n",rc);
			return rc;
		}
	}
	return XST_SUCCESS;
}

int xx_ethmac_recv_tgm(u8 *buf_pnt,u32 *buf_len,u8 *rok)
{
	XX_ETHMAC_DEF;

	* rok = 0;

	if (XEMHD->Version > 2)
	{
		// sync
		Xil_DCacheInvalidateRange((UINTPTR)&emac_bds_mem_v.bds.Rx, sizeof(emac_bds_mem_v.bds.Rx));
	}

	// receiver RING
	XEmacPs_BdRing * rxring = & XEmacPs_GetRxRing(XEMHD);

	XEmacPs_Bd * rxbdset;
	u32 bd_prcd = XEmacPs_BdRingFromHwRx(rxring,1,&rxbdset);
	if (bd_prcd <= 0)
		return XST_SUCCESS;

	XEmacPs_Bd * cBD = rxbdset;

	u32 len = XEmacPs_BdGetLength(cBD);

	UINTPTR adr = XEmacPs_BdGetBufAddr(cBD);
	adr &= XEMACPS_RXBUF_ADD_MASK;

	u32 ll = * buf_len;
	if (ll > len)
		ll = len;

	// sync
	Xil_DCacheInvalidateRange((INTPTR)adr,EMAC_TGM_RAW_SIZE);

	xx_ethmac_memcpy_4(buf_pnt,(void *)adr,ll);
	* buf_len = ll;
	* rok = 1;

	XEmacPs_BdClearRxNew(cBD);
	cBD = XEmacPs_BdRingNext(rxring,cBD);

	XEmacPs_BdRingFree(rxring,bd_prcd,rxbdset);

	// setup
	u32 freebds = XEmacPs_BdRingGetFreeCnt(rxring);
	XEmacPs_Bd * rxbd;
	LONG rc = XEmacPs_BdRingAlloc(rxring,freebds,&rxbd);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_BdRingAlloc()=%d\r\n",rc);
		return rc;
	}
	//-
	rc = XEmacPs_BdRingToHw(rxring,freebds,rxbd);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_BdRingToHw()=%d\r\n",rc);
		return rc;
	}
    
	// data sync barrier
	dsb();

	if (XEMHD->Version > 2)
	{
		Xil_DCacheFlushRange((INTPTR)rxbd,sizeof(XEmacPs_Bd));
	}

	return XST_SUCCESS;
}

int xx_ethmac_send_tgm(u8 *buf_pnt,u32 buf_len,u8 *tok)
{
	XX_ETHMAC_DEF;

	// dma to user used BD's
	xx_ethmac_free_sent(ref);

	* tok = 0;
	// transmit RING
	XEmacPs_BdRing * txring = & XEmacPs_GetTxRing(XEMHD);

	XEmacPs_Bd * txbdset;
	LONG rc = XEmacPs_BdRingAlloc(txring,1,&txbdset);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_BdRingAlloc()=%d\r\n",rc);
		return rc;
	}

	//xil_printf("TX BD addr = %X\r\n",txbdset);

	XEmacPs_Bd * txbd = txbdset;

	int i;
	UINTPTR sta = (UINTPTR)&emac_bds_mem_v.bds.Tx.work[0];
	for(i = 0;i < EMAC_N_TX_DESC;i++)
	{
		if (sta == (UINTPTR)txbd)
		{
			break;
		}
		sta += txring->Separation;
	}

	if (i == EMAC_N_TX_DESC)
	{
		//xil_printf("Error get TX BD\r\n");
		return XST_FAILURE;
	}

	// xil_printf("Free tx desc %d\r\n",i);

	// copy user buffer to driver buffer
	xx_ethmac_memcpy_4(&emac_tgm_mem_v.Tx.Telegrams[i].b,buf_pnt,buf_len);

	// sync
	Xil_DCacheFlushRange((INTPTR)&emac_tgm_mem_v.Tx.Telegrams[i].b,EMAC_TGM_RAW_SIZE);

	// setup BD
	XEmacPs_BdSetAddressTx(txbd,(UINTPTR)&emac_tgm_mem_v.Tx.Telegrams[i].b);
	XEmacPs_BdSetLength(txbd,buf_len);
	XEmacPs_BdSetLast(txbd);
    
	// data sync barrier
	dsb();
    
	XEmacPs_BdClearTxUsed(txbd);

	rc = XEmacPs_BdRingToHw(txring,1,txbdset);
    
	// data sync barrier
	dsb();

	if (XEMHD->Version > 2)
	{
		Xil_DCacheFlushRange((INTPTR)txbdset,sizeof(XEmacPs_Bd));
	}

	XEmacPs_Transmit(XEMHD);

	* tok = 1;

	return rc;
}

int xx_ethmac_init(void)
{
	XX_ETHMAC_DEF;
	int rc;

	// invalid speed
	ref->PhySpeed = 0;

	// init TLB
	rc = xx_ethmac_tlb_init(ref);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("xx_ethmac_tlb_init()=%d\r\n",rc);
		return rc;
	}

	// get configuration info for EMAC
	XEmacPs_Config * xcf = XEmacPs_LookupConfig(XPAR_XEMACPS_0_DEVICE_ID);
	if (xcf == NULL)
		return XST_FAILURE;
	//xil_printf("id%d cfa=%x ea=%x\r\n",XPAR_XEMACPS_0_DEVICE_ID,xcf->BaseAddress,XPS_GEM0_BASEADDR);

	// initialize emac structure
	rc = XEmacPs_CfgInitialize(XEMHD,xcf,xcf->BaseAddress);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("XEmacPs_CfgInitialize()=%d\r\n",rc);
		return rc;
	}

	// PHY setup
	xx_ethmac_phy_setup(ref);
	//xil_printf("PHY address = %d\r\n",ref->PhyAddress);

	// initial speed
	xx_ethmac_relinq_speed();

	// Init DMA
	rc = xx_ethmac_dma_init(ref);
	if (rc != XST_SUCCESS)
	{
		//xil_printf("zq_ethmac_dma_init()=%d\r\n",rc);
		return rc;
	}

	// set MAC address
	rc = xx_ethmac_set_hw_mac_addr();
	if (rc != XST_SUCCESS)
	{
		//xil_printf("zq_ethmac_set_hw_mac_addr()=%d\r\n",rc);
		return rc;
	}

	// start
	XEmacPs_Start(XEMHD);


	return XST_SUCCESS;
}

int xx_ethmac_down(void)
{
	XX_ETHMAC_DEF;

	// stop
	XEmacPs_Stop(XEMHD);

	// reset
	XEmacPs_Reset(XEMHD);

	return XST_SUCCESS;
}

#endif

#else // XPAR_XEMACPS_0_DEVICE_ID


int xx_ethmac_relinq_speed(void)
{
	return XST_SUCCESS;
}

int xx_ethmac_set_mac_addr(const u8* addr)
{
	return XST_SUCCESS;
}

int xx_ethmac_get_mac_addr(u8* addr)
{
	return XST_SUCCESS;
}

int xx_ethmac_recv_tgm(u8 *buf_pnt,u32 *buf_len,u8 *rok)
{
	* buf_len = 0;
	* rok = 0;
	return XST_SUCCESS;
}

int xx_ethmac_send_tgm(u8 *buf_pnt,u32 buf_len,u8 *tok)
{
	* tok = 1;
	return XST_SUCCESS;
}

int xx_ethmac_init(void)
{
	return XST_SUCCESS;
}

int xx_ethmac_down(void)
{
	return XST_SUCCESS;
}

#endif // XPAR_XEMACPS_0_DEVICE_ID

