/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief Zynq IIC driver
/// \file zq_iic.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "xparameters.h"

#if defined (__arm__) || defined (__aarch64__)

#include "zq_iicps.h"
#include "xx_iic.h"

#include <stdio.h>

#if defined(XPAR_XGPIOPS_0_BASEADDR) || defined(XPAR_XGPIOPS_1_BASEADDR)
#include "xgpiops.h"
#endif

#include "xstatus.h"

#include "board.h"

#ifdef SYS_CARRIER_TE0720

#define XX_IIC_DEV  (XPAR_XIICPS_1_BASEADDR)
#define XX_IIC_CLKI (XPAR_PS7_I2C_1_I2C_CLK_FREQ_HZ)
#define XX_IIC_BAUD (100000)
#define XX_IIC_TMO  (255)

#define GPIOPS_DEV (XPAR_XGPIOPS_0_DEVICE_ID)
#define GPIOPS_ADR (XPAR_XGPIOPS_0_BASEADDR)

#define GPIOPS_II2_SW 0



// TE0720 I2C address


// zynq-uboot> i2c probe 0 7f
// Valid chip addresses: 1E 20 21 22 38 39 3C 3E 3F 57 6F
// zynq-uboot>

// 3E what ?
// 23 no reply ?

// TE0720 on board addresses
#define I2C_ADDR_TE0720_RTC_REG   0x6F // RTC registers
#define I2C_ADDR_TE0720_RTC_RAM   0x57 // RTC backed up RAM
#define I2C_ADDR_TE0720_MEMS_MAG  0x1E // MEMS Magnetometer
#define I2C_ADDR_TE0720_MEMS_ACC  0x18 // MEMS Accelerometer
#define I2C_ADDR_TE0720_SC_GPIO_1 0x20 // system controller emulated i2c gpio ext. (Sreg 1)
#define I2C_ADDR_TE0720_SC_GPIO_2 0x21 // system controller emulated i2c gpio ext. (Sreg 2)

// TE0701 carrier addresses
#define I2C_ADDR_TE0720_TE0701_FMC      0x22 // TE0701: Carrier controller FMC VBus
#define I2C_ADDR_TE0720_TE0701_xxx      0x23 // TE0701: Carrier controller
#define I2C_ADDR_TE0720_TE0701_ADV_REG  0x39 // TE0701: ADV7513 Registers
#define I2C_ADDR_TE0720_TE0701_ADV_PKM  0x38 // TE0701: ADV7513 Packet memory
#define I2C_ADDR_TE0720_TE0701_ADV_EDID 0x3F // TE0701: ADV7513 EDID Memory
#define I2C_ADDR_TE0720_TE0701_ADV_CDC  0x3C // TE0701: ADV7513 CEC Memory


// 7=EN_FMC_VADJ 1=ON, 4..6=VID0..2; 3=PG_C2M
#define I2C_TE0701_FMC_CONFIG (1*0x80 | 1*0x10 | 0*0x08);


#endif

#ifdef SYS_CARRIER_TE0720

#if defined(SYS_TE0701_NUM) && SYS_TE0701_NUM > 0

static int xx_iic_te0720_gpio_init(void)
{
	XGpioPs gpio_ps;

	#define GPIOPS_REF ( & gpio_ps )

	int rc = XST_SUCCESS;

	// gpio_ps
	{
		XGpioPs_Config * cfg = XGpioPs_LookupConfig(GPIOPS_DEV);
		if (NULL == cfg) return XST_FAILURE;

		rc = XGpioPs_CfgInitialize(GPIOPS_REF,cfg, GPIOPS_ADR);
		if (XST_SUCCESS != rc) return rc;
	}

    XGpioPs_SetDirectionPin(GPIOPS_REF,GPIOPS_II2_SW,1);
    XGpioPs_SetOutputEnablePin(GPIOPS_REF,GPIOPS_II2_SW,1);
    XGpioPs_WritePin(GPIOPS_REF,GPIOPS_II2_SW,0);

    return XST_SUCCESS;
}

#endif

int xx_iic_te0720_fmc_pwon(void)
{
	#if defined(SYS_TE0701_NUM) && SYS_TE0701_NUM > 0

	int rc;

	rc = xx_iic_te0720_gpio_init();
	if (XST_SUCCESS != rc) return rc;

	int ntry = 0;

	for(;;)
	{
		u8 buffer[1];
		buffer[0] = I2C_TE0701_FMC_CONFIG;

		rc = i2cps_send(XX_IIC_DEV,I2C_ADDR_TE0720_TE0701_FMC,&buffer[0],1,0);
		if (XST_SUCCESS == rc)
		{
			for(;;)
			{
				if (i2cps_is_send_done(XX_IIC_DEV))
				{
					return XST_SUCCESS;
				}

				if (i2cps_is_arb_lost(XX_IIC_DEV))
				{
					if (++ntry == 5)
					{
						// printf("IIC TE0701 FMC power-on fatal error\n");
						return XST_FAILURE;
					}
					// printf("IIC TE0701 FMC power-on retry %d\n",ntry);
					break;
				}
			}

		}
	}

	#endif

	return XST_SUCCESS;
}


#endif

int xx_iic_init(void)
{
#ifdef XX_IIC_DEV
	int rc;

	// initialize
	rc = i2cps_init(XX_IIC_DEV,XX_IIC_CLKI,XX_IIC_BAUD,XX_IIC_TMO);
	if (rc != XST_SUCCESS)
	{
		xil_printf("i2cps_init()=%d\r\n",rc);
		return rc;
	}
#endif
	return XST_SUCCESS;
}

int xx_iic_down(void)
{
	return XST_SUCCESS;
}

#endif
