/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ZYNQ IICPS alternaltive driver
/// \file zq_iicps.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "xparameters.h"

#if defined (__arm__) || defined (__aarch64__)

#if defined(XPAR_XIICPS_0_BASEADDR) || defined(XPAR_XIICPS_1_BASEADDR)


#include "xiicps_hw.h"
#include "xiicps.h"

#include "xstatus.h"

#include "zq_iicps.h"

int i2cps_get_isr(const u32 BaseAddress)
{
	return XIicPs_ReadReg(BaseAddress,XIICPS_ISR_OFFSET);
}

int i2cps_get_sr(const u32 BaseAddress)
{
	return XIicPs_ReadReg(BaseAddress,XIICPS_SR_OFFSET);
}

int i2cps_is_busy(const u32 BaseAddress)
{
	return (XIicPs_ReadReg(BaseAddress,XIICPS_SR_OFFSET) & XIICPS_SR_BA_MASK) != 0;
}

int i2cps_is_comp(const u32 BaseAddress)
{
	return (XIicPs_ReadReg(BaseAddress,XIICPS_ISR_OFFSET) & XIICPS_IXR_COMP_MASK) != 0;

}

int i2cps_get_baud(const u32 BaseAddress, const u32 inclockhz)
{
	u32 cr = XIicPs_ReadReg(BaseAddress,XIICPS_CR_OFFSET);
	int da = (cr & XIICPS_CR_DIV_A_MASK) >> XIICPS_CR_DIV_A_SHIFT;
	int db = (cr & XIICPS_CR_DIV_B_MASK) >> XIICPS_CR_DIV_B_SHIFT;
	int baud = inclockhz / (22 * (da + 1) * (db + 1));
	return baud;
}

int i2cps_set_baud(const u32 BaseAddress, const u32 inclockhz, const u32 baudrate)
{
	int ha = 4;
	int hb = 64;
	int he = 400000;
	{
		int a;
		for(a = 1; a <= 4; a++)
		{
			int b = inclockhz / (22 * baudrate * a);
			if (b >= 1 && b <= 64)
			{
				int k = inclockhz / (22 * a * b);
				int e = k - baudrate;
				if (e < 0) e = -e;
				if (e < he)
				{
					ha = a;
					hb = b;
					he = e;
				}
			}
		}
	}
	u32 cr = XIicPs_ReadReg(BaseAddress,XIICPS_CR_OFFSET);
	cr &= ~(XIICPS_CR_DIV_A_MASK|XIICPS_CR_DIV_B_MASK);
	cr |= (ha - 1) << XIICPS_CR_DIV_A_SHIFT;
	cr |= (hb - 1) << XIICPS_CR_DIV_B_SHIFT;
	XIicPs_WriteReg(BaseAddress,XIICPS_CR_OFFSET,cr);

	return XST_SUCCESS;
}

int i2cps_set_timeout(const u32 BaseAddress, const u8 timeout)
{
	// timeout value
	XIicPs_WriteReg(BaseAddress,XIICPS_TIME_OUT_OFFSET, timeout);

	return XST_SUCCESS;
}

int i2cps_init(const u32 BaseAddress, const u32 inclockhz, const u32 baudrate, const u8 timeout)
{
	// irq disable register: ALL off
	XIicPs_WriteReg(BaseAddress,XIICPS_IDR_OFFSET      , XIICPS_IXR_ALL_INTR_MASK);

	// clear FIFOs
	XIicPs_WriteReg(BaseAddress,XIICPS_CR_OFFSET       , XIICPS_CR_CLR_FIFO_MASK);

	// clear pending irq's
	u32 isr = XIicPs_ReadReg(BaseAddress,XIICPS_ISR_OFFSET);
	XIicPs_WriteReg(BaseAddress,XIICPS_ISR_OFFSET      , isr);

	// default CR register
	XIicPs_WriteReg(BaseAddress,XIICPS_CR_OFFSET      , XIICPS_CR_RESET_VALUE);

	// default timeout register
	XIicPs_WriteReg(BaseAddress,XIICPS_TIME_OUT_OFFSET, XIICPS_TO_RESET_VALUE);

	// timeout value
	XIicPs_WriteReg(BaseAddress,XIICPS_TIME_OUT_OFFSET, timeout);

	// set  baudrate
	i2cps_set_baud(BaseAddress,inclockhz,baudrate);

	return XST_SUCCESS;
}

int i2cps_is_arb_lost(const u32 BaseAddress)
{
	return (i2cps_get_isr(BaseAddress) & XIICPS_IXR_ARB_LOST_MASK) != 0;
}

int i2cps_is_send_done(const u32 BaseAddress)
{
	u32 sr  = i2cps_get_sr(BaseAddress);
	u32 isr = i2cps_get_isr(BaseAddress);

	return sr == 0 && isr == XIICPS_IXR_COMP_MASK;
}

int i2cps_send(const u32 BaseAddress, const u16 addr, const u8 * buffer, const u8 length, const u8 hold)
{
	// check arg
	if (buffer == NULL || length < 1 || length > XIICPS_FIFO_DEPTH)
		return XST_FAILURE;

	u32 ctrl  = XIicPs_ReadReg(BaseAddress, XIICPS_CR_OFFSET);
	// ctrl: MS=1, WR=0, NEA=1, ACKEN=1, CLR_FIFO=1
	ctrl |= XIICPS_CR_ACKEN_MASK | XIICPS_CR_CLR_FIFO_MASK | XIICPS_CR_NEA_MASK | XIICPS_CR_MS_MASK;
	ctrl &= ~XIICPS_CR_RD_WR_MASK;
	if (hold)
	{
		ctrl |= XIICPS_CR_HOLD_MASK;
	}
	else
	{
		ctrl &= ~XIICPS_CR_HOLD_MASK;
	}
	XIicPs_WriteReg(BaseAddress, XIICPS_CR_OFFSET, ctrl);

	// clear pending irq's
	u32 isr = XIicPs_ReadReg(BaseAddress,XIICPS_ISR_OFFSET);
	XIicPs_WriteReg(BaseAddress,XIICPS_ISR_OFFSET, isr);

	// load data buffer
	int i;
	for(i = 0; i < length; i++)
	{
		XIicPs_WriteReg(BaseAddress, XIICPS_DATA_OFFSET, buffer[i]);
	}

	// set slave address
	XIicPs_WriteReg(BaseAddress, XIICPS_ADDR_OFFSET, addr);

	return XST_SUCCESS;
}

int i2cps_recv_arm(const u32 BaseAddress, const u16 addr, const u8 length)
{
	// check arg
	if (length < 1 || length > XIICPS_FIFO_DEPTH)
		return XST_FAILURE;

	// transfer to go
	XIicPs_WriteReg(BaseAddress,XIICPS_TRANS_SIZE_OFFSET,length);

	// clear pending irq's
	u32 isr = XIicPs_ReadReg(BaseAddress,XIICPS_ISR_OFFSET);
	XIicPs_WriteReg(BaseAddress,XIICPS_ISR_OFFSET, isr);

	// control
	u32 ctrl  = XIicPs_ReadReg(BaseAddress, XIICPS_CR_OFFSET);
	// ctrl: MS=1, WR=1, NEA=1, ACKEN=1, CLR_FIFO=1
	ctrl |= XIICPS_CR_ACKEN_MASK | XIICPS_CR_CLR_FIFO_MASK | XIICPS_CR_NEA_MASK | XIICPS_CR_MS_MASK | XIICPS_CR_RD_WR_MASK;
	ctrl &= ~XIICPS_CR_HOLD_MASK;
	XIicPs_WriteReg(BaseAddress, XIICPS_CR_OFFSET, ctrl);

	// set slave address
	XIicPs_WriteReg(BaseAddress, XIICPS_ADDR_OFFSET, addr);

	return XST_SUCCESS;
}

int i2cps_recv_get(const u32 BaseAddress, u8 *buffer, const u8 length)
{
	// check arg
	if (buffer == NULL || length < 1 || length > XIICPS_FIFO_DEPTH)
		return XST_FAILURE;

	u32 sr  = XIicPs_ReadReg(BaseAddress,XIICPS_SR_OFFSET);
	u32 isr = XIicPs_ReadReg(BaseAddress,XIICPS_ISR_OFFSET);

	if (sr & XIICPS_SR_RXDV_MASK)
	{
		if (isr & (XIICPS_IXR_NACK_MASK | XIICPS_IXR_ARB_LOST_MASK | XIICPS_IXR_RX_OVR_MASK | XIICPS_IXR_RX_UNF_MASK))
		{
			return XST_FAILURE;
		}

		if ((isr & (XIICPS_IXR_DATA_MASK | XIICPS_IXR_COMP_MASK)) == (XIICPS_IXR_DATA_MASK | XIICPS_IXR_COMP_MASK))
		{
			u8 ts = XIicPs_ReadReg(BaseAddress,XIICPS_TRANS_SIZE_OFFSET);

			if (0 == ts)
			{
				int i;
				for(i = 0; i < length; i++)
				{
					buffer[i] = XIicPs_ReadReg(BaseAddress, XIICPS_DATA_OFFSET);
				}

				XIicPs_WriteReg(BaseAddress,XIICPS_ISR_OFFSET,isr);

				return XST_SUCCESS;
			}
			return XST_FAILURE;
		}
	}
	else
	{
		if (isr & (XIICPS_IXR_NACK_MASK | XIICPS_IXR_ARB_LOST_MASK | XIICPS_IXR_RX_OVR_MASK | XIICPS_IXR_RX_UNF_MASK))
		{
			return XST_FAILURE;
		}
	}

	return XST_NO_DATA;
}

#endif

#endif

