/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief zynq pl init from qspi partition
/// \file zq_qspipl.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020


#include "glovar.h"

#include "xparameters.h"
#include "xstatus.h"

#if SW_ENA_PL_INIT > 0


#ifdef XPAR_PS7_QSPI_0_BASEADDR
	#include "xqspips_hw.h"
	#define QSPI_DEVICE_BASEADDR   XPAR_PS7_QSPI_0_BASEADDR
	#define QSPI_LINEAR_BASEADDR   XPAR_PS7_QSPI_LINEAR_0_S_AXI_BASEADDR
#endif


// XDCFG & SLCR access definitions

#ifdef XPAR_PS7_DEV_CFG_0_BASEADDR
	#include "xdevcfg_hw.h"

	#define XDCFG_BASEADDR          XPAR_PS7_DEV_CFG_0_BASEADDR

	#define SlcrUnlock()			Xil_Out32(XPS_SYS_CTRL_BASEADDR + 0x08, 0xDF0DDF0D)
	#define SlcrLock()				Xil_Out32(XPS_SYS_CTRL_BASEADDR + 0x04, 0x767B767B)

	#define FPGA_RESET_REG			(XPS_SYS_CTRL_BASEADDR + 0x240)
	#define PS_LVL_SHFTR_EN			(XPS_SYS_CTRL_BASEADDR + 0x900)

	#define LVL_PL_PS               0x0000000F
	#define LVL_PS_PL               0x0000000A

#endif


#include "ps7_init.h"


// BOOT.BIN partition access

#define XIL_BOOT_HEADER_ARRAY_ADDRESS_OFFSET 0x9C

typedef union {

	struct {

		uint32_t PartitionDataSize;
		uint32_t WritteDataLength;
		uint32_t PartitionTotalSize;
		uint32_t LoadAddress;
		uint32_t ExecutionAddress;
		uint32_t PartitionOffset;
		uint32_t Attributes;
	#define XIL_BOOT_HEADER_ATTRIBUTE_PL_BITSTREAM 0x20
	#define XIL_BOOT_HEADER_ATTRIBUTE_PS_PARTITION 0x10
		uint32_t PartitionDataSectionCount;
		uint32_t Reserved[7];
		uint32_t Checksum;

	} fields;

	struct {
		uint32_t data[16];

	} raw;

} xil_boot_header_t;





static void zq_qspi_init(void)
{
	u32 ConfigReg;

	// Disable the device
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR, XQSPIPS_ER_OFFSET, 0);

	// De-assert slave select lines.
	ConfigReg = XQspiPs_ReadReg(QSPI_DEVICE_BASEADDR,XQSPIPS_CR_OFFSET);
	ConfigReg |= (XQSPIPS_CR_SSCTRL_MASK | XQSPIPS_CR_SSFORCE_MASK);
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR, XQSPIPS_CR_OFFSET, ConfigReg);

	// Set the RX and TX FIFO threshold to reset value (one)
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR,XQSPIPS_RXWR_OFFSET, XQSPIPS_RXWR_RESET_VALUE);

	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR,XQSPIPS_TXWR_OFFSET, XQSPIPS_TXWR_RESET_VALUE);

	// Clear the RX FIFO and drop any data.
	while ((XQspiPs_ReadReg(QSPI_DEVICE_BASEADDR,XQSPIPS_SR_OFFSET) & XQSPIPS_IXR_RXNEMPTY_MASK) != 0) {
		XQspiPs_ReadReg(QSPI_DEVICE_BASEADDR,XQSPIPS_RXD_OFFSET);
	}


	// Write default value to configuration register. Do not modify reserved bits.
	ConfigReg = XQspiPs_ReadReg(QSPI_DEVICE_BASEADDR, XQSPIPS_CR_OFFSET);
	ConfigReg |= XQSPIPS_CR_RESET_MASK_SET;
	ConfigReg &= ~XQSPIPS_CR_RESET_MASK_CLR;
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR, XQSPIPS_CR_OFFSET, ConfigReg);


	// Set the prescaler for QSPI clock
	#define XQSPIPS_CLK_PRESCALE_8		0x02 // PCLK/8 Prescaler

	ConfigReg = XQspiPs_ReadReg(QSPI_DEVICE_BASEADDR, XQSPIPS_CR_OFFSET);
	ConfigReg &= ~XQSPIPS_CR_PRESC_MASK;
	ConfigReg |= (u32) (XQSPIPS_CLK_PRESCALE_8 & XQSPIPS_CR_PRESC_MAXIMUM) << XQSPIPS_CR_PRESC_SHIFT;
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR, XQSPIPS_CR_OFFSET,  ConfigReg);

	// Enable linear mode
	ConfigReg = XQspiPs_ReadReg(QSPI_DEVICE_BASEADDR, XQSPIPS_CR_OFFSET);
	ConfigReg &= ~( XQSPIPS_CR_CPOL_MASK |  XQSPIPS_CR_CPHA_MASK | XQSPIPS_CR_SSFORCE_MASK | XQSPIPS_CR_MANSTRTEN_MASK | XQSPIPS_CR_HOLD_B_MASK);
	ConfigReg |= XQSPIPS_CR_HOLD_B_MASK;
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR, XQSPIPS_CR_OFFSET, ConfigReg);

	ConfigReg = XQspiPs_ReadReg(QSPI_DEVICE_BASEADDR,XQSPIPS_LQSPI_CR_OFFSET);
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR,XQSPIPS_LQSPI_CR_OFFSET,XQSPIPS_LQSPI_CR_RST_STATE);

	// Select Slave
	ConfigReg = XQspiPs_ReadReg(QSPI_DEVICE_BASEADDR,XQSPIPS_CR_OFFSET);
	ConfigReg &= ~XQSPIPS_CR_SSCTRL_MASK;
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR,XQSPIPS_CR_OFFSET, ConfigReg);

	#define LQSPI_CR_FAST_QUAD_READ		0x0000006B // Fast Quad Read output
	#define LQSPI_CR_1_DUMMY_BYTE		0x00000100 // 1 Dummy Byte between

	// Single linear read
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR,	XQSPIPS_LQSPI_CR_OFFSET, XQSPIPS_LQSPI_CR_LINEAR_MASK | LQSPI_CR_1_DUMMY_BYTE | LQSPI_CR_FAST_QUAD_READ);

	// Enable the controller
	XQspiPs_WriteReg(QSPI_DEVICE_BASEADDR, XQSPIPS_ER_OFFSET, XQSPIPS_ER_ENABLE_MASK);

}






static int zq_program_prolog(void)
{
	u32 PcapReg;

	// unlock SLCR
	SlcrUnlock();

	// unlock device
	XDcfg_WriteReg(XDCFG_BASEADDR,XDCFG_UNLOCK_OFFSET,XDCFG_UNLOCK_DATA);

	// assert reset : Disable AXI interface
	Xil_Out32(FPGA_RESET_REG, 0xFFFFFFFF);

	// Set Level Shifters DT618760 - PS to PL enabling
	Xil_Out32(PS_LVL_SHFTR_EN, LVL_PS_PL);

	// Get DEVCFG controller settings
	PcapReg = XDcfg_ReadReg(XDCFG_BASEADDR,XDCFG_CTRL_OFFSET);

	// Setting PCFG_PROG_B signal to high
	XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_CTRL_OFFSET,
			(PcapReg | XDCFG_CTRL_PCFG_PROG_B_MASK));

	// Setting PCFG_PROG_B signal to low
	XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_CTRL_OFFSET,
				(PcapReg & ~XDCFG_CTRL_PCFG_PROG_B_MASK));

	// Polling the PCAP_INIT status for Reset
	while(XDcfg_ReadReg(XDCFG_BASEADDR, XDCFG_STATUS_OFFSET) & XDCFG_STATUS_PCFG_INIT_MASK)
		;

	// Setting PCFG_PROG_B signal to high
	XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_CTRL_OFFSET,
			(PcapReg | XDCFG_CTRL_PCFG_PROG_B_MASK));

	// Polling the PCAP_INIT status for Set
	while(!(XDcfg_ReadReg(XDCFG_BASEADDR, XDCFG_STATUS_OFFSET) & XDCFG_STATUS_PCFG_INIT_MASK))
		;

	return XST_SUCCESS;
}

static void zq_program_epilog(void)
{
#ifdef PS7_POST_CONFIG
	ps7_post_config();
#else
	Xil_Out32(PS_LVL_SHFTR_EN, LVL_PL_PS);
	Xil_Out32(FPGA_RESET_REG, 0);
#endif
	SlcrLock();
}


int zq_qspipl_init(void)
{
	int rc;
	u32 bitstream_start;
	u32 bitstream_size;
	u8  plfound = 0;

	// init QSPI memory
	zq_qspi_init();

	// get header array address
	u32 haa;
	memcpy(&haa,(void *)(QSPI_LINEAR_BASEADDR + XIL_BOOT_HEADER_ARRAY_ADDRESS_OFFSET),4);

	#if 0
	xil_printf("offset = x%X\r\n",haa);
	#endif

	int part;
	for(part = 0; part < 16;part++)
	{
		xil_boot_header_t hdr;

		memcpy(&hdr,(void *)(QSPI_LINEAR_BASEADDR + haa + part * sizeof(hdr)),sizeof(hdr));

		int i;

		// eval cksum
		u32 cksum = 0;
		for(i = 0; i < 15; i++) {
			cksum += hdr.raw.data[i];
		}
		cksum ^= 0xFFFFFFFF;
		if (cksum != hdr.fields.Checksum)
			return XST_FAILURE;

		// dump header
		#if 0
		xil_printf("partiton # %d\r\n", part + 1);
		xil_printf("%-20s = %d\r\n","PartitionDataSize"        , hdr.fields.PartitionDataSize        );
		xil_printf("%-20s = %d\r\n","WritteDataLength"         , hdr.fields.WritteDataLength         );
		xil_printf("%-20s = %d\r\n","PartitionTotalSize"       , hdr.fields.PartitionTotalSize       );
		xil_printf("%-20s = %d\r\n","LoadAddress"              , hdr.fields.LoadAddress              );
		xil_printf("%-20s = %d\r\n","ExecutionAddress"         , hdr.fields.ExecutionAddress         );
		xil_printf("%-20s = %d\r\n","PartitionOffset"          , hdr.fields.PartitionOffset          );
		xil_printf("%-20s = x%X\r\n","Attributes"              , hdr.fields.Attributes               );
		xil_printf("%-20s = %d\r\n","PartitionDataSectionCount", hdr.fields.PartitionDataSectionCount);
		xil_printf("%-20s :"     ,"Reserved");
			for(i=0;i<7;i++) xil_printf("%d ", hdr.fields.Reserved[i]);
			xil_printf("\r\n");
			xil_printf("%-20s = x%08X\r\n","Checksum"              , hdr.fields.Checksum                 );
		#endif

		// test for end of table
		int eot = 1;
		for(i = 0; i < 15; i++) {
			if (hdr.raw.data[i]) {
				eot = 0;
				break;
			}
		}
		if (eot)
			break;

		// valid, non last header
		if (hdr.fields.Attributes & XIL_BOOT_HEADER_ATTRIBUTE_PL_BITSTREAM) {
			// bitstream found

			// plain bitstream
			if (hdr.fields.PartitionDataSize == hdr.fields.WritteDataLength && hdr.fields.PartitionDataSize == hdr.fields.PartitionTotalSize) {

				bitstream_start = hdr.fields.PartitionOffset   << 2;
				bitstream_size  = hdr.fields.PartitionDataSize << 2;

				plfound = 1;

				continue;
			}

			return XST_FAILURE;
		}

	}

	if (plfound)
	{
		//xil_printf("bitstream: start x%X size = %d\r\n", bitstream_start,bitstream_size);

		// prepare devcfg for PL program
		rc = zq_program_prolog();
		if (rc != XST_SUCCESS) return rc;

		// setup DMA transfer

		u32 SourcePtr      = (u32)(QSPI_LINEAR_BASEADDR + bitstream_start) | 1;
		u32 SrcWordLength  = (bitstream_size + 3) >> 2;

		XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_INT_STS_OFFSET,XDCFG_IXR_ALL_MASK);

		// DMA setup: (order is important!)
		XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_DMA_SRC_ADDR_OFFSET, SourcePtr);
		XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_DMA_DEST_ADDR_OFFSET, XDCFG_DMA_INVALID_ADDRESS);
		XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_DMA_SRC_LEN_OFFSET, SrcWordLength);
		XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_DMA_DEST_LEN_OFFSET, 0);


		// Polling the DMA complete
		while(!(XDcfg_ReadReg(XDCFG_BASEADDR, XDCFG_INT_STS_OFFSET) & XDCFG_IXR_DMA_DONE_MASK))
			;

		// reset flag
		XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_INT_STS_OFFSET,XDCFG_IXR_DMA_DONE_MASK);

		// wait for done
		while(!(XDcfg_ReadReg(XDCFG_BASEADDR, XDCFG_INT_STS_OFFSET) & XDCFG_IXR_PCFG_DONE_MASK))
			;

		// reset flag
		XDcfg_WriteReg(XDCFG_BASEADDR, XDCFG_INT_STS_OFFSET,XDCFG_IXR_PCFG_DONE_MASK);


		// exit PL program with AXI link-on
		zq_program_epilog();
	}

	return XST_SUCCESS;
}

#endif
