/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief zynq qspi_ps
/// \file zq_qspips.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

#ifdef XPAR_XQSPIPS_0_DEVICE_ID

// connection mode
#define QSPI_MODE_SINGLE_FLASH_CONNECTION  0

#define QSPI_MICRON_ID      0x20
#define QSPI_SPANSION_ID    0x01
#define QSPI_WINBOND_ID     0xEF
#define QSPI_MACRONIX_ID    0xC2


#define ZQ_QSPIPS_XX_TRACE()        printf("%s(%d)\n",__FUNCTION__,__LINE__)
#define ZQ_QSPIPS_XX_RET4ERROR()  { if (XST_SUCCESS != rc) { ZQ_QSPIPS_XX_TRACE(); return XST_FAILURE; } }

#define ZQ_QSPIPS_DEVICE           (&pDrv->dev)

int zq_qspips_flash_read_sr1(zq_qspips_t *pDrv, u8 * sr1)
{
	u8 local_buffer[8];

	// read status register 1
	local_buffer[0] = XQSPIPS_FLASH_OPCODE_RDSR1;
	local_buffer[1] = 0x00;
	int rc = XQspiPs_PolledTransfer(ZQ_QSPIPS_DEVICE, local_buffer, local_buffer, 2);
	ZQ_QSPIPS_XX_RET4ERROR();

	* sr1 = local_buffer[1];

	return XST_SUCCESS;
}

int zq_qspips_write_enable(zq_qspips_t *pDrv)
{
	u8 local_buffer[8];

	// write enable
	local_buffer[0] = XQSPIPS_FLASH_OPCODE_WREN;
	int rc = XQspiPs_PolledTransfer(ZQ_QSPIPS_DEVICE, local_buffer, NULL, 1);
	ZQ_QSPIPS_XX_RET4ERROR();

	return XST_SUCCESS;
}

static int zq_qspips_banksel(zq_qspips_t *pDrv, const u8 banksel)
{
	u8 local_buffer[8];
	int rc;

	//ZQ_QSPIPS_XX_TRACE();

	// only if flash size over 16MB
	if (pDrv->flashIdent->flash_size > (16*1024*1024))
	{
		switch(pDrv->flashIdent->id[0])
		{
			case QSPI_SPANSION_ID:
				{
					// write bank
					local_buffer[0] = XQSPIPS_FLASH_OPCODE_BRWR;
					local_buffer[1] = banksel;

					rc = XQspiPs_PolledTransfer(ZQ_QSPIPS_DEVICE, local_buffer, NULL, 2);
					ZQ_QSPIPS_XX_RET4ERROR();

					//ZQ_QSPIPS_XX_TRACE();
				}
				break;

			case QSPI_WINBOND_ID:
				{
					// write enable
					rc = zq_qspips_write_enable(pDrv);
					ZQ_QSPIPS_XX_RET4ERROR();

					// write extended address register
					local_buffer[0] = XQSPIPS_FLASH_OPCODE_EARWR;
					local_buffer[1] = banksel;

					rc = XQspiPs_PolledTransfer(ZQ_QSPIPS_DEVICE, local_buffer, NULL, 2);
					ZQ_QSPIPS_XX_RET4ERROR();

					//ZQ_QSPIPS_XX_TRACE();

				}
				break;

				// not yet implemented
			default:
				{
					rc = XST_FAILURE;
					ZQ_QSPIPS_XX_TRACE();
				}
				break;
		}
	}

	return XST_SUCCESS;
}


int zq_qspips_read(zq_qspips_t *pDrv, const u32 address, const u32 bytecount)
{
	if (bytecount > 0)
	{
		int rc;

		// adj bank/upper address
		rc = zq_qspips_banksel(pDrv, (address >> 24));
		ZQ_QSPIPS_XX_RET4ERROR();

		//ZQ_QSPIPS_XX_TRACE();

		// read block
		pDrv->flashBuffer[0] = XQSPIPS_FLASH_OPCODE_QUAD_READ;
		pDrv->flashBuffer[1] = (address >> 16) & 0xFF;
		pDrv->flashBuffer[2] = (address >>  8) & 0xFF;
		pDrv->flashBuffer[3] = (address >>  0) & 0xFF;

		rc = XQspiPs_PolledTransfer(ZQ_QSPIPS_DEVICE, pDrv->flashBuffer, pDrv->flashBuffer, 1 + 3 + 1 + bytecount);
		ZQ_QSPIPS_XX_RET4ERROR();

		//ZQ_QSPIPS_XX_TRACE();
	}

	return XST_SUCCESS;
}

int zq_qspips_sector_erase(zq_qspips_t *pDrv, const u32 address)
{
	u8 local_buffer[8];
	int rc = XST_FAILURE;

	// adj bank/upper address
	rc = zq_qspips_banksel(pDrv, (address >> 24));
	ZQ_QSPIPS_XX_RET4ERROR();


	//ZQ_QSPIPS_XX_TRACE();

	// write enable
	rc = zq_qspips_write_enable(pDrv);
	ZQ_QSPIPS_XX_RET4ERROR();

	//ZQ_QSPIPS_XX_TRACE();

	// sector erase
	local_buffer[0] = XQSPIPS_FLASH_OPCODE_SE;
	local_buffer[1] = (address >> 16) & 0xFF;
	local_buffer[2] = (address >>  8) & 0xFF;
	local_buffer[3] = (address >>  0) & 0xFF;

	rc = XQspiPs_PolledTransfer(ZQ_QSPIPS_DEVICE, local_buffer, NULL, 4);
	ZQ_QSPIPS_XX_RET4ERROR();

	return XST_SUCCESS;
}

int zq_qspips_page_program(zq_qspips_t *pDrv, const u32 address, const u32 bytecount)
{
	if (bytecount > 0)
	{
		int rc;

		// adj bank/upper address
		rc = zq_qspips_banksel(pDrv, (address >> 24));
		ZQ_QSPIPS_XX_RET4ERROR();

		//ZQ_QSPIPS_XX_TRACE();

		// write enable
		rc = zq_qspips_write_enable(pDrv);
		ZQ_QSPIPS_XX_RET4ERROR();

		//ZQ_QSPIPS_XX_TRACE();

		// page program
		pDrv->flashBuffer[0] = XQSPIPS_FLASH_OPCODE_PP;
		pDrv->flashBuffer[1] = (address >> 16) & 0xFF;
		pDrv->flashBuffer[2] = (address >>  8) & 0xFF;
		pDrv->flashBuffer[3] = (address >>  0) & 0xFF;

		rc = XQspiPs_PolledTransfer(ZQ_QSPIPS_DEVICE, pDrv->flashBuffer, NULL, 1 + 3 + bytecount);
		ZQ_QSPIPS_XX_RET4ERROR();
	}

	return XST_SUCCESS;
}

static const zq_qspips_flash_type_t zq_qspips_flash_types_v[] =
{
	{ "n25q128a13", {0x20,0xba,0x18,  0x10, 0x00, 0x00 }, 3, 64*1024, 256, 16*1024*1024 },
	{ "n25q128a11", {0x20,0xbb,0x18,  0x10, 0x00, 0x00 }, 3, 64*1024, 256, 16*1024*1024 },
	{ "s25fl128s" , {0x01,0x20,0x18,  0x4d, 0x01, 0x80 }, 6, 64*1024, 256, 16*1024*1024 },
	{ "s25fl256s1", {0x01,0x02,0x19,  0x4d, 0x01, 0x00 }, 5, 64*1024, 512, 32*1024*1024 },
	{ "w25q256",    {0xef,0x40,0x19,  0x00, 0x00, 0x00 }, 3, 64*1024, 512, 32*1024*1024 },

};

static int zq_qspips_read_ident(zq_qspips_t *pDrv)
{
	u8 local_buffer[8];
	int i;

	// Read flash ident

	local_buffer[0] = XQSPIPS_FLASH_OPCODE_RDID;
	local_buffer[1] = 0;
	local_buffer[2] = 0;
	local_buffer[3] = 0;

	int rc = XQspiPs_PolledTransfer(ZQ_QSPIPS_DEVICE, local_buffer, local_buffer, 1+6);
	ZQ_QSPIPS_XX_RET4ERROR();

	// search for flash
	for(i = 0; i < ((int)(sizeof(zq_qspips_flash_types_v)/sizeof(zq_qspips_flash_types_v[0])));i++)
	{
		const zq_qspips_flash_type_t * pD = &zq_qspips_flash_types_v[i];
		if (0 == memcmp(&local_buffer[1],pD->id,pD->id_len))
		{
			pDrv->flashIdent = pD;

			//printf("QSPI %s ss=%lu KB ns=%u fs=%lu MB\n",pD->name,pD->sec_size / 1024,pD->sectors,pD->flash_size / 1024 / 1024);

			return XST_SUCCESS;
		}
	}

	printf("QSPI flash ident : %02X %02X %02X : %02X %02X %02X not supported!\n",
			local_buffer[1],local_buffer[2],local_buffer[3],
			local_buffer[4],local_buffer[5],local_buffer[6]);

	//ZQ_QSPIPS_XX_TRACE();

	return XST_FAILURE;
}

int zq_qspips_initialize(zq_qspips_t *pDrv)
{
	// QSPI initialization

	// config record
	XQspiPs_Config * QspiConfig = XQspiPs_LookupConfig(XPAR_XQSPIPS_0_DEVICE_ID);
	if (NULL == QspiConfig)
		return XST_FAILURE;

	//ZQ_QSPIPS_XX_TRACE();

	// rewind initialization to cold state
	XQspiPs_Abort(ZQ_QSPIPS_DEVICE);

	//ZQ_QSPIPS_XX_TRACE();

	// initialize
	int rc = XQspiPs_CfgInitialize(ZQ_QSPIPS_DEVICE,QspiConfig, QspiConfig->BaseAddress);
	ZQ_QSPIPS_XX_RET4ERROR();

	//ZQ_QSPIPS_XX_TRACE();

	// Set Manual Chip select options and drive HOLD_B pin high.
	XQspiPs_SetOptions(ZQ_QSPIPS_DEVICE, XQSPIPS_FORCE_SSELECT_OPTION | XQSPIPS_HOLD_B_DRIVE_OPTION);

	//ZQ_QSPIPS_XX_TRACE();

	// Set the prescaler for QSPI clock
	XQspiPs_SetClkPrescaler(ZQ_QSPIPS_DEVICE, XQSPIPS_CLK_PRESCALE_8);

	//ZQ_QSPIPS_XX_TRACE();

	// Assert the FLASH chip select.
	XQspiPs_SetSlaveSelect(ZQ_QSPIPS_DEVICE);

	//ZQ_QSPIPS_XX_TRACE();

	// read flash ident
	rc = zq_qspips_read_ident(pDrv);
	ZQ_QSPIPS_XX_RET4ERROR();

	//printf("AQ: XPAR_PS7_QSPI_0_QSPI_MODE=%d\n", XPAR_PS7_QSPI_0_QSPI_MODE);

	switch(XPAR_PS7_QSPI_0_QSPI_MODE)
	{
		case QSPI_MODE_SINGLE_FLASH_CONNECTION:
			break;
		default:
			printf("AQ: Invalid QSPI_MODE = %d\n",XPAR_PS7_QSPI_0_QSPI_MODE);
			return XST_FAILURE;
	}

	//ZQ_QSPIPS_XX_TRACE();

	return XST_SUCCESS;
}

int zq_qspips_eth_process(zq_qspips_t *pDrv, u8 *p_data,u16 *p_len, u8 *p_xdn)
{
	if (*p_xdn)
		return XST_SUCCESS;

	u8  *pQuery = p_data;
	u16  lQuery = * p_len;
	u8  *pReply = p_data;
	u16 lReply  = 0;

#define zq_qspips_eth_xdr_ext_u8() ({ \
			u8 retval = 0; \
			if (lQuery == 0) \
				return XST_FAILURE; \
			retval = *pQuery++; \
			lQuery--; \
			retval; } )

#define zq_qspips_eth_xdr_ext_u16() ({ \
			u16 retval = 0; \
			retval = zq_qspips_eth_xdr_ext_u8(); \
			retval <<= 8; \
			retval |= zq_qspips_eth_xdr_ext_u8(); \
			retval; } )

#define zq_qspips_eth_xdr_ext_u32() ({ \
			u32 retval = 0; \
			retval = zq_qspips_eth_xdr_ext_u16(); \
			retval <<= 16; \
			retval |= zq_qspips_eth_xdr_ext_u16(); \
			retval; } )

#define zq_qspips_eth_xdr_ins_u8(value) { \
				*pReply++ = (value); \
				lReply ++ ; \
			}

#define zq_qspips_eth_xdr_ins_u16(value) { \
			u16 t16 = (value); \
			zq_qspips_eth_xdr_ins_u8(t16 >> 8); \
			zq_qspips_eth_xdr_ins_u8(t16); \
			}

#define zq_qspips_eth_xdr_ins_u32(value) { \
			u32 t32 = (value); \
			zq_qspips_eth_xdr_ins_u16(t32 >> 16); \
			zq_qspips_eth_xdr_ins_u16(t32); \
			}


	u8 tmp8;


	tmp8  = zq_qspips_eth_xdr_ext_u8();
	if (tmp8 != ZQ_QSPIPS_PROTO_TYPE)
	{
		return XST_FAILURE;
	}

	tmp8  = zq_qspips_eth_xdr_ext_u8();
	if (tmp8 != ZQ_QSPIPS_PROTO_VERSION)
	{
		return XST_FAILURE;
	}

	tmp8  = zq_qspips_eth_xdr_ext_u8();
	switch(tmp8)
	{
		case ZQ_QSPIPS_PROTO_CODE_FLS_IDENT:
			{
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_TYPE);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_VERSION);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_CODE_FLS_IDENT);
				zq_qspips_eth_xdr_ins_u32(pDrv->flashIdent->sec_size);
				zq_qspips_eth_xdr_ins_u32(pDrv->flashIdent->sectors);
				zq_qspips_eth_xdr_ins_u32(pDrv->flashIdent->flash_size);
				zq_qspips_eth_xdr_ins_u32(QSPI_FLASH_PAGE_SIZE);
			}
			break;

		case ZQ_QSPIPS_PROTO_CODE_FLS_STATUS:
			{
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_TYPE);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_VERSION);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_CODE_FLS_STATUS);
				u8 sr1;
				int rc = zq_qspips_flash_read_sr1(pDrv,&sr1);
				zq_qspips_eth_xdr_ins_u32(rc);
				u8 busy = (sr1 & 0x01) != 0;
				zq_qspips_eth_xdr_ins_u8(busy);
			}
			break;

		case ZQ_QSPIPS_PROTO_CODE_FLS_READ:
			{
				u32 address = zq_qspips_eth_xdr_ext_u32();
				u32 size    = zq_qspips_eth_xdr_ext_u32();

				if (size > ZQ_QSPIPS_PROTO_CODE_FLS_DATA_LENGTH)
					return XST_FAILURE;

				int rc = zq_qspips_read(pDrv,address,size);

				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_TYPE);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_VERSION);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_CODE_FLS_READ);
				zq_qspips_eth_xdr_ins_u32(rc);
			}
			break;

		case ZQ_QSPIPS_PROTO_CODE_FLS_WRITE:
			{
				u32 address = zq_qspips_eth_xdr_ext_u32();
				u32 size    = zq_qspips_eth_xdr_ext_u32();

				if (size > QSPI_FLASH_PAGE_SIZE)
					return XST_FAILURE;

				int rc = zq_qspips_page_program(pDrv,address,size);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_TYPE);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_VERSION);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_CODE_FLS_WRITE);
				zq_qspips_eth_xdr_ins_u32(rc);
			}
			break;

		case ZQ_QSPIPS_PROTO_CODE_FLS_SCT_ERASE:
			{
				u32 address = zq_qspips_eth_xdr_ext_u32();
				u32 size    = zq_qspips_eth_xdr_ext_u32();

				size = size; // avoid warning

				int rc = zq_qspips_sector_erase(pDrv,address);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_TYPE);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_VERSION);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_CODE_FLS_SCT_ERASE);
				zq_qspips_eth_xdr_ins_u32(rc);
			}
			break;

		case ZQ_QSPIPS_PROTO_CODE_FLS_GET:
			{
				u32 index = zq_qspips_eth_xdr_ext_u32();
				u32 size  = zq_qspips_eth_xdr_ext_u32();

				if ((index + size) > ZQ_QSPIPS_PROTO_CODE_FLS_DATA_LENGTH)
					return XST_FAILURE;

				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_TYPE);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_VERSION);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_CODE_FLS_GET);
				memcpy(pReply,&pDrv->flashBuffer[ZQ_QSPIPS_BUFFER_OFFSET_READ+index],size);
				pReply += size;
				lReply += size;
			}
			break;

		case ZQ_QSPIPS_PROTO_CODE_FLS_PUT:
			{
				u32 index = zq_qspips_eth_xdr_ext_u32();
				u32 size  = zq_qspips_eth_xdr_ext_u32();

				if ((index + size) > QSPI_FLASH_PAGE_SIZE)
					return XST_FAILURE;

				if (size > lQuery)
					return XST_FAILURE;

				memcpy(&pDrv->flashBuffer[ZQ_QSPIPS_BUFFER_OFFSET_WRITE+index],pQuery,size);
				pQuery += size;
				lQuery -= size;

				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_TYPE);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_VERSION);
				zq_qspips_eth_xdr_ins_u8(ZQ_QSPIPS_PROTO_CODE_FLS_PUT);
			}
			break;

		default:
			{
				return XST_FAILURE;
			}
			break;
	}

	*p_len = lReply;
	*p_xdn = 1;

	return XST_SUCCESS;
}

#endif
