/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief CAN driver
/// \file zq_xcan.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "xparameters.h"

#if defined (__arm__) || defined (__aarch64__)

#include "board.h"

#if defined(XPAR_PS7_CAN_0_DEVICE_ID)
	#define XX_XCAN_ID   XPAR_PS7_CAN_0_DEVICE_ID
	#define XX_XCAN_CLK  XPAR_XCANPS_0_CAN_CLK_FREQ_HZ
#endif

#include "xx_xcan.h"

#ifdef XX_XCAN_ID

#include "xcanps.h"

#include "xil_printf.h"

// CAN phy control
enum { XX_XCAN_PHY_STBY, XX_XCAN_PHY_ENABLE };


#ifdef SYS_CARRIER_ZC702

#include "xgpiops.h"

#define GPIOPS_DEV (XPAR_XGPIOPS_0_DEVICE_ID)
#define GPIOPS_ADR (XPAR_XGPIOPS_0_BASEADDR)

#define GPIOPS_CAN_0_STB 9

static int xx_xcan_phy_zc702(const u8 mode)
{
	XGpioPs gpio_ps;
	int rc;

	#define GPIOPS_REF ( & gpio_ps )

	// gpio_ps
	XGpioPs_Config * CfgPtr = XGpioPs_LookupConfig(GPIOPS_DEV);
	if (NULL == CfgPtr) return XST_FAILURE;

	rc = XGpioPs_CfgInitialize(GPIOPS_REF,CfgPtr, GPIOPS_ADR);
	if (XST_SUCCESS != rc) return rc;

    XGpioPs_SetDirectionPin(GPIOPS_REF,GPIOPS_CAN_0_STB,1);
    XGpioPs_SetOutputEnablePin(GPIOPS_REF,GPIOPS_CAN_0_STB,1);
    XGpioPs_WritePin(GPIOPS_REF,GPIOPS_CAN_0_STB,(mode == XX_XCAN_PHY_ENABLE) ? 0 : 1);

    return XST_SUCCESS;
}

#endif


// required baud rate
#define XX_XCAN_WANT_BAUDRATE 1000000

typedef struct
{
	XCanPs devcan;

	u32    cycles;

} xcan_t;

static xcan_t xcan_v;

#define XX_XCAN_DEF xcan_t * ref = & xcan_v
#define XCANHD      (&ref->devcan)


// bit size in quanta unit
#define xx_xcan_nquanta(TimeSegment2,TimeSegment1)                          (1 + ((TimeSegment2) + 1) + ((TimeSegment1) + 1))

// effective baurdate
#define xx_xcan_baudrate(CanClock,Prescaler,TimeSegment2,TimeSegment1)  	((CanClock) / (((Prescaler) + 1) * xx_xcan_nquanta(TimeSegment2,TimeSegment1)))

// integer prescaler (flor value)
#define xx_xcan_prescaler(CanClock,BaudRate,TimeSegment2,TimeSegment1)  	(((CanClock) / ((BaudRate) * xx_xcan_nquanta(TimeSegment2,TimeSegment1))) - 1)


// can controller mode to string value
static const char * xx_xcan_mode2str(const u8 mode)
{
	return
		(mode == XCANPS_MODE_CONFIG  ) ? "CONFIG" :
		(mode == XCANPS_MODE_SLEEP   ) ? "SLEEP" :
		(mode == XCANPS_MODE_NORMAL  ) ? "NORMAL" :
		(mode == XCANPS_MODE_LOOPBACK) ? "LOOPBACK" :
		(mode == XCANPS_MODE_SNOOP   ) ? "SNOOP" : "?";
}

// show some infos
int xx_xcan_show_info(void)
{
	XX_XCAN_DEF;

	u8 OperationMode;
	OperationMode = XCanPs_GetMode(XCANHD);
	xil_printf("XCanPs_GetMode()=%d=%s\r\n",OperationMode,xx_xcan_mode2str(OperationMode));

	u8 Prescaler = XCanPs_GetBaudRatePrescaler(XCANHD);
	xil_printf("XCanPs_GetBaudRatePrescaler()=%d\r\n",Prescaler);

	u8 SyncJumpWidth;
	u8 TimeSegment2;
	u8 TimeSegment1;
	XCanPs_GetBitTiming(XCANHD,&SyncJumpWidth,&TimeSegment2,&TimeSegment1);
	xil_printf("SyncJumpWidth=%d TimeSegment2=%d TimeSegment1=%d\r\n",SyncJumpWidth,TimeSegment2,TimeSegment1);

	xil_printf("BaudRate = %d\r\n",xx_xcan_baudrate(XX_XCAN_CLK,Prescaler,TimeSegment2,TimeSegment1));

	u32 buserstat = XCanPs_GetBusErrorStatus(XCANHD);
	xil_printf("XCanPs_GetBusErrorStatus()=%x\r\n",buserstat);

	u32 canstat = XCanPs_GetStatus(XCANHD);
	xil_printf("XCanPs_GetStatus()=%x\r\n",canstat);

	return XST_SUCCESS;
}

static int xx_xcan_phy(const u8 mode)
{
    int rc = XST_SUCCESS;

	#ifdef SYS_CARRIER_ZC702
    	rc = xx_xcan_phy_zc702(mode);
	#else
		(void)mode; // unused parameter
	#endif

    return rc;
}

static int xx_xcan_setbaud(const u32 BaudRate)
{
	XX_XCAN_DEF;
	int rc;

	// set word format
	u8 SyncJumpWidth = 1-1;
	u8 TimeSegment2  = 2-1;
	u8 TimeSegment1  = 5-1;

	// prescaler register
	u8 Prescaler = 3-1;

	// eval prescaler
	Prescaler = xx_xcan_prescaler(XX_XCAN_CLK,BaudRate,TimeSegment2,TimeSegment1);

	// baudrate @ eval prescaler
	int br0 = xx_xcan_baudrate(XX_XCAN_CLK,Prescaler,TimeSegment2,TimeSegment1) - BaudRate;
	if (br0 < 0) br0 = -br0;

	// baudrate @ next prescaler (maybe next value reduce error)
	Prescaler++;
	int br1 = xx_xcan_baudrate(XX_XCAN_CLK,Prescaler,TimeSegment2,TimeSegment1) - BaudRate;
	if (br1 < 0) br1 = -br1;

	// initial value is better ..
	if (br0 < br1) Prescaler --;

	//xil_printf("br0=%d br1=%d\r\n",br0,br1);

	// set Bit Timing
	rc = XCanPs_SetBitTiming(XCANHD,SyncJumpWidth,TimeSegment2,TimeSegment1);
	if (rc != XST_SUCCESS)
	{
		xil_printf("XCanPs_SetBitTiming()=%d\r\n",rc);
		return XST_FAILURE;
	}

	// Set Baud Rate Prescaler
	rc = XCanPs_SetBaudRatePrescaler(XCANHD,Prescaler);
	if (rc != XST_SUCCESS)
	{
		xil_printf("XCanPs_SetBaudRatePrescaler()=%d\r\n",rc);
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

int xx_xcan_recv_ready(void)
{
	XX_XCAN_DEF;

	if (XCanPs_IsRxEmpty(XCANHD))
		return XST_NO_DATA;
	return XST_SUCCESS;
}

int xx_xcan_send_full(void)
{
	XX_XCAN_DEF;

	if (XCanPs_IsTxFifoFull(XCANHD))
		return XST_FIFO_NO_ROOM;
	return XST_SUCCESS;
}

int xx_xcan_recv_frame(xx_xcan_frame_t *pf)
{
	XX_XCAN_DEF;
	int rc;

	rc = XCanPs_Recv(XCANHD,pf->raw_l);
	if (rc == XST_SUCCESS)
	{
		// expand 1st word
		pf->srr = (pf->raw_l[0] & XCANPS_IDR_SRR_MASK) >> XCANPS_IDR_SRR_SHIFT;
		pf->ide = (pf->raw_l[0] & XCANPS_IDR_IDE_MASK) >> XCANPS_IDR_IDE_SHIFT;
		pf->rtr = (pf->raw_l[0] & XCANPS_IDR_RTR_MASK);
		pf->id  =  (pf->raw_l[0] & XCANPS_IDR_ID1_MASK) >> XCANPS_IDR_ID1_SHIFT;
		if (pf->ide)
		{
			pf->id <<= 18;
			pf->id |=  (pf->raw_l[0] & XCANPS_IDR_ID2_MASK) >> XCANPS_IDR_ID2_SHIFT;
		}
		// expand 2nd word
		pf->dlc = (pf->raw_l[1] & XCANPS_DLCR_DLC_MASK) >> XCANPS_DLCR_DLC_SHIFT;
		pf->tst = (pf->raw_l[1] & XCANPS_DLCR_TIMESTAMP_MASK);

		return XST_SUCCESS;
	}

	return XST_NO_DATA;
}


int xx_xcan_send_frame(xx_xcan_frame_t *pf)
{
	XX_XCAN_DEF;
	int rc;

	pf->raw_l[0] = XCanPs_CreateIdValue(
					pf->ide ? (pf->id >> 18) : pf->id,
					pf->srr,pf->ide,pf->id,pf->rtr);
	pf->raw_l[1] = XCanPs_CreateDlcValue(pf->dlc);
	rc = XCanPs_Send(XCANHD,pf->raw_l);

	return rc;
}

int xx_xcan_init(void)
{
	XX_XCAN_DEF;
	int rc;

	// get CAN controller initialization info
	XCanPs_Config * CfgPtr = XCanPs_LookupConfig(XX_XCAN_ID);
	if (CfgPtr == NULL) return XST_FAILURE;

	// initialize
	rc = XCanPs_CfgInitialize(XCANHD,CfgPtr,CfgPtr->BaseAddr);
	if (rc != XST_SUCCESS) return rc;

	// set baudrate
	rc =  xx_xcan_setbaud(XX_XCAN_WANT_BAUDRATE);
	if (rc != XST_SUCCESS) return rc;

	// init for stdby
	rc = xx_xcan_phy(XX_XCAN_PHY_ENABLE);
	if (rc != XST_SUCCESS) return rc;

	// mode -> normal
	XCanPs_EnterMode(XCANHD,XCANPS_MODE_NORMAL);

	return XST_SUCCESS;
}

void xx_xcan_down(void)
{
	XX_XCAN_DEF;

	// stdby
	xx_xcan_phy(XX_XCAN_PHY_STBY);

	// reset can controller
	XCanPs_Reset(XCANHD);
}


#endif


#ifndef XX_XCAN_ID

int xx_xcan_recv_ready(void)
{
	return XST_NO_DATA;
}

int xx_xcan_send_full(void)
{
	return XST_FIFO_NO_ROOM;
}

int xx_xcan_recv_frame(xx_xcan_frame_t *pf)
{
	(void)pf; // unused
	return XST_NO_DATA;
}

int xx_xcan_send_frame(xx_xcan_frame_t *pf)
{
	(void)pf; // unused
	return XST_FIFO_NO_ROOM;
}

int xx_xcan_show_info(void)
{
	return XST_SUCCESS;
}

int xx_xcan_init(void)
{
	return XST_SUCCESS;
}

void xx_xcan_down(void)
{
}


#endif


#endif
