/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2013, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ZedBoard OLED display demo
/// \file zed_oled_demo.c
/// \version 0.1
/// \date nov  2, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2013


#include "xparameters.h"

#include "xtime_l.h"
#include "xuartps.h"

#include "zedodisp.h"
#include "zedodisp_ch.h"

#include <stdio.h>


#define ZEDODISP_DELAY_1_SEC      (COUNTS_PER_SECOND)

#define ZEDODISP_DELAY_1_MSEC     (COUNTS_PER_SECOND / (1000 /   1))
#define ZEDODISP_DELAY_2_MSEC     (COUNTS_PER_SECOND / (1000 /   2))
#define ZEDODISP_DELAY_100_MSEC   (COUNTS_PER_SECOND / (1000 / 100))

#define ZEDODISP_DELAY_3_USEC     (COUNTS_PER_SECOND / (1000000 /   3))
#define ZEDODISP_DELAY_10_USEC    (COUNTS_PER_SECOND / (1000000 /  10))

#define ZEDODISP_BASE             XPAR_ZEDODISP_0_BASEADDR


// local copy of display frame buffer
static union
{
	u8 b[ZEDODISP_PAGES][ZEDODISP_COLUMNS];
} local_frame_buffer;


// passive delay using ZYNQ high resolution timer
static void lc_sleep(const u32 tw)
{
	XTime ts;
	XTime_GetTime(&ts);

	for(;;) {
		XTime te;
		XTime_GetTime(&te);
		u32 td = te - ts;
		if (td >= tw)
			break;
	}
}

// wait for command execution acknowledge
static void lc_dtcmd_w4exec()
{
	while(ZEDODISP_ctrlst_is_busy(ZEDODISP_BASE))
		;
	while(!ZEDODISP_ctrlst_is_xack(ZEDODISP_BASE))
		;
}


// wait for command idle state
static void lc_dtcmd_w4rest()
{
	while(ZEDODISP_ctrlst_is_busy(ZEDODISP_BASE))
		;
	while(ZEDODISP_ctrlst_is_xack(ZEDODISP_BASE))
		;
}

// send data/command to display
static void lc_outdtcmd(const u8 cmd)
{
	ZEDODISP_ctrlst_dtcmd_set(ZEDODISP_BASE,cmd);
	lc_dtcmd_w4exec();
	ZEDODISP_ctrlst_dtcmd_idle(ZEDODISP_BASE);
	lc_dtcmd_w4rest();
}

// ASCII single char to pixmap (row,col) local frame buffer
static void lc_renderchr(const int r, const int c, const char v)
{
	int y = r % ZEDODISP_PAGES;
	int o = v;
	if (o < ZEDODISP_ch_t0_chlo)
		o = ZEDODISP_ch_t0_chlo;
	else if (o > ZEDODISP_ch_t0_chhi)
		o = ZEDODISP_ch_t0_chhi;
	o = (o - ZEDODISP_ch_t0_chlo) * ZEDODISP_ch_t0_size_x;
	int k;
	for(k = 0; k < ZEDODISP_ch_t0_size_x; k++)
	{
		int x = (c * ZEDODISP_ch_t0_size_x + k) % ZEDODISP_COLUMNS;
		int z = o + k;
		local_frame_buffer.b[y][x] = zedodisp_ch_t0_table[z];
	}
}

// ASCII char string to display pixmap
static void lc_renderstr(const int r, const char *s)
{
	int c,x;
	for(c = 0, x = 0; c < (ZEDODISP_COLUMNS / ZEDODISP_ch_t0_size_x);c++)
	{
		int v = ' ';
		if (s && s[x])
			v = s[x++];
		lc_renderchr(r,c,v);
	}
}


//
// demo main program
//

int main()
{

	printf("\n" "ZedBoard OLED display demo by QDESYS." "\n" "Enter <esc> <ctrl-c> <sp> to exit." "\n");

	// flush console
	while(XUartPs_IsReceiveData(STDIN_BASEADDRESS))
		XUartPs_RecvByte(STDIN_BASEADDRESS);


	// shutdown: initial state
	ZEDODISP_shutdown(ZEDODISP_BASE);

	// #1

	// VDD & Reset
	ZEDODISP_ctrlst_vdd(ZEDODISP_BASE,1);
	ZEDODISP_ctrlst_res(ZEDODISP_BASE,1);

	// wait 1 mSec
	lc_sleep(ZEDODISP_DELAY_1_MSEC);

	// #2

	ZEDODISP_ctrlst_res(ZEDODISP_BASE,0);

	lc_sleep(ZEDODISP_DELAY_3_USEC);

	ZEDODISP_ctrlst_res(ZEDODISP_BASE,1);

	ZEDODISP_ctrlst_dc(ZEDODISP_BASE,0);

	lc_outdtcmd(ZEDODISP_CMD_DISP_OFF);

	// wait 2 mSec
	lc_sleep(ZEDODISP_DELAY_2_MSEC);

	// #3

	lc_outdtcmd(0x8D); // Charge Pump Setting
	lc_outdtcmd(0x14); // Enable Charge Pump

	lc_outdtcmd(0xD9); // Set Pre-charge Period
	lc_outdtcmd(0xF1); // pre-charge period

	ZEDODISP_ctrlst_vbat(ZEDODISP_BASE,1); // vbat ON

	lc_sleep(ZEDODISP_DELAY_100_MSEC);

	// #4

	static u8 inicmds[] = ZEDODISP_CMD_INIT_SCN_0;
	//static u8 inicmds[] = ZEDODISP_CMD_INIT_SCN_180;

	int i;
	for(i = 0; i < ((int)(sizeof(inicmds)/sizeof(inicmds[0])));i++)
		lc_outdtcmd(inicmds[i]);


	lc_outdtcmd(ZEDODISP_CMD_DISP_ON);

	ZEDODISP_ctrlst_auto(ZEDODISP_BASE,1);

	// # end-of-initialization

	XTime time_start;
	XTime_GetTime(&time_start);

	int loops = 0;

	for(;;)
	{
		XTime time_now;
		XTime_GetTime(&time_now);

		XTime time_diff = time_now - time_start;

		u32 tmp = time_diff / ZEDODISP_DELAY_1_MSEC;

		int millisecond = tmp % 1000; tmp /= 1000;
		int second      = tmp % 60; tmp /= 60;
		int minute      = tmp % 60; tmp /= 60;
		int hour        = tmp % 24; tmp /= 24;

		loops += 1;

		// max execution time
		if (hour >= 1)
			break;

		int r;
		for(r = 0; r < ZEDODISP_PAGES; r++)
		{
			char s[32];

			switch(r)
			{
				case 0:
					sprintf(s,"Demo by QDESYS");
					break;
				case 1:
					sprintf(s,"Loops # %08d",loops);
					break;
				case 2:
					sprintf(s,"%02d:%02d:%02d.%03d",
						hour,
						minute,
						second,
						millisecond);
					break;
				case 3:
					sprintf(s,"%016lld", time_diff);
					break;
			}

			lc_renderstr(r,s);
		}
		ZEDODISP_fmem_copyto(ZEDODISP_BASE,&local_frame_buffer);


		// interrupt by UART stdin
		if (XUartPs_IsReceiveData(STDIN_BASEADDRESS))
		{
			int key = XUartPs_RecvByte(STDIN_BASEADDRESS);
			if (key == 27 || key == 3 || key == ' ')
				break;
		}

	}

	// shutdown
	ZEDODISP_shutdown(ZEDODISP_BASE);

	printf("\n" "That's All Folks!" "\n");

	return XST_SUCCESS;
}

