LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE ieee.math_real.ALL;

LIBRARY topic;
USE topic.topic_pkg.ALL;

ENTITY axis_traffic_gen IS
   GENERIC(
      p_data_width             : natural               := 32;
      p_insert_id_in_data      : boolean               := false;
      p_id                     : natural RANGE 0 TO 15 := 0;
      p_data_source            : string                := "RANDOM";                                           -- can also be "COUNTER" or "FILE"
      p_max_valid_window_width : natural               := 1;
      p_max_gap_width          : natural               := 1
   );
   PORT(
      i_clk         : IN  std_logic;
      i_rst         : IN  std_logic;
      -- AXI Stream Master:
      M_AXIS_TDATA  : OUT std_logic_vector(p_data_width - 1 DOWNTO 0);
      M_AXIS_TREADY : IN  std_logic;
      M_AXIS_TVALID : OUT std_logic
   );
END ENTITY;

ARCHITECTURE RTL OF axis_traffic_gen IS
   SIGNAL n_axis_tvalid : std_logic;
   SIGNAL r_data        : std_logic_vector(p_data_width - 1 DOWNTO 0);
BEGIN

   count_data : IF (p_data_source = "COUNTER") GENERATE
      PROCESS(i_clk, i_rst)
      BEGIN
         IF i_rst = '1' THEN
            r_data <= (OTHERS => '0');
         ELSIF rising_edge(i_clk) THEN
            IF M_AXIS_TREADY = '1' AND n_axis_tvalid = '1' THEN
               r_data <= std_logic_vector(unsigned(r_data) + 1);
            END IF;
         END IF;
      END PROCESS;
   END GENERATE;

   random_data : IF (p_data_source = "RANDOM") GENERATE
      PROCESS(i_clk, i_rst)
         VARIABLE v_seed1  : positive := 102458;
         VARIABLE v_seed2  : positive := 25349;
         VARIABLE v_random : real;
      BEGIN
         IF i_rst = '1' THEN
            uniform(v_seed1, v_seed2, v_random);
            FOR i IN 0 TO (r_data'length / 8) - 1 LOOP
               r_data(((i + 1) * 8) - 1 DOWNTO (i * 8)) <= t_byte(to_unsigned(integer(v_random * (real(i + 1) * 359.2)), 8));
            END LOOP;
         ELSIF rising_edge(i_clk) THEN
            IF M_AXIS_TREADY = '1' AND n_axis_tvalid = '1' THEN
               v_seed1 := v_seed1 + integer(v_random * 4.8);
               uniform(v_seed1, v_seed2, v_random);
               FOR i IN 0 TO (r_data'length / 8) - 1 LOOP
                  r_data(((i + 1) * 8) - 1 DOWNTO (i * 8)) <= t_byte(to_unsigned(integer(v_random * (real(i + 1) * 359.2)), 8));
               END LOOP;
            END IF;
         END IF;
      END PROCESS;
   END GENERATE;

   no_gaps : IF (p_max_valid_window_width = 0) GENERATE
      PROCESS(i_clk, i_rst)
      BEGIN
         IF i_rst = '1' THEN
            n_axis_tvalid <= '0';
         ELSIF rising_edge(i_clk) THEN
            n_axis_tvalid <= '1';
         END IF;
      END PROCESS;
   END GENERATE;

   gaps : IF (p_max_valid_window_width /= 0) GENERATE

      TYPE t_traffic_gen_states IS (GAP, SEND_DATA);
      SIGNAL sm_traffic_gen : t_traffic_gen_states := GAP;

      SIGNAL r_gap_width   : natural;
      SIGNAL r_gap_cnt     : natural;
      SIGNAL r_burst_width : natural;
      SIGNAL r_burst_cnt   : natural;

   BEGIN

      PROCESS(i_clk, i_rst)
         VARIABLE v_seed1  : positive := 102458;
         VARIABLE v_seed2  : positive := 25349;
         VARIABLE v_random : real;
      BEGIN
         IF i_rst = '1' THEN

            uniform(v_seed1, v_seed2, v_random);
            r_gap_width <= integer(v_random * real(p_max_gap_width));
            r_gap_cnt   <= 0;

            sm_traffic_gen <= GAP;
         ELSIF rising_edge(i_clk) THEN

            CASE (sm_traffic_gen) IS
               -- --------------------------------
               WHEN GAP =>
                  IF (r_gap_cnt = r_gap_width) THEN

                     v_seed1       := v_seed1 + integer(v_random * 3.4);
                     uniform(v_seed1, v_seed2, v_random);
                     r_burst_width <= integer(v_random * real(p_max_valid_window_width));
                     r_burst_cnt   <= 0;

                     sm_traffic_gen <= SEND_DATA;
                  ELSE
                     r_gap_cnt <= r_gap_cnt + 1;
                  END IF;
               -- --------------------------------
               WHEN SEND_DATA =>
                  IF (M_AXIS_TREADY = '1') THEN
                     IF (r_burst_cnt = r_burst_width) THEN

                        v_seed2     := v_seed2 + integer(v_random * 2.7);
                        uniform(v_seed1, v_seed2, v_random);
                        r_gap_width <= integer(v_random * real(p_max_gap_width));
                        r_gap_cnt   <= 0;

                        sm_traffic_gen <= GAP;
                     ELSE
                        r_burst_cnt <= r_burst_cnt + 1;
                     END IF;
                  END IF;
            END CASE;

         END IF;
      END PROCESS;

      n_axis_tvalid <= '1' WHEN sm_traffic_gen = SEND_DATA ELSE '0';

   END GENERATE;

   M_AXIS_TVALID <= n_axis_tvalid;
   M_AXIS_TDATA  <= std_logic_vector(to_unsigned(p_id, 4)) & std_logic_vector(r_data(p_data_width - 5 DOWNTO 0)) WHEN p_insert_id_in_data = true ELSE
    std_logic_vector(r_data);

END ARCHITECTURE RTL;
