LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE ieee.math_real.ALL;

LIBRARY topic;
USE topic.topic_pkg.ALL;
USE topic.topic_tb_pkg.ALL;

ENTITY axis_verifier IS
   GENERIC(
      p_data_width             : natural := 32;
      p_max_ready_window_width : natural := 1;
      p_max_gap_width          : natural := 1;
      p_check_data				 : boolean := true
   );
   PORT(
      i_clk             : IN  std_logic;
      i_rst             : IN  std_logic;
      -- AXI Stream Slave (expected data)
      S_AXIS_EXP_TDATA  : IN  std_logic_vector(p_data_width - 1 DOWNTO 0);
      S_AXIS_EXP_TREADY : OUT std_logic;
      S_AXIS_EXP_TVALID : IN  std_logic;
      -- AXI Stream Slave (actual data)
      S_AXIS_ACT_TDATA  : IN  std_logic_vector(p_data_width - 1 DOWNTO 0);
      S_AXIS_ACT_TREADY : OUT std_logic;
      S_AXIS_ACT_TVALID : IN  std_logic
   );
END ENTITY;

ARCHITECTURE RTL OF axis_verifier IS
   SIGNAL n_axis_tready : std_logic;
BEGIN


   no_gaps : IF (p_max_ready_window_width = 0) GENERATE
      PROCESS(i_clk, i_rst)
      BEGIN
         IF i_rst = '1' THEN
            n_axis_tready <= '0';
         ELSIF rising_edge(i_clk) THEN
            n_axis_tready <= '1';
         END IF;
      END PROCESS;
   END GENERATE;

   gaps : IF (p_max_ready_window_width /= 0) GENERATE

      TYPE t_traffic_gen_states IS (GAP, RECEIVE_DATA);
      SIGNAL sm_verifier : t_traffic_gen_states := GAP;

      SIGNAL r_gap_width   : natural;
      SIGNAL r_gap_cnt     : natural;
      SIGNAL r_burst_width : natural;
      SIGNAL r_burst_cnt   : natural;

   BEGIN

      PROCESS(i_clk, i_rst)
         VARIABLE v_seed1  : positive := 102458;
         VARIABLE v_seed2  : positive := 25349;
         VARIABLE v_random : real;
      BEGIN
         IF i_rst = '1' THEN

            uniform(v_seed1, v_seed2, v_random);
            r_gap_width <= integer(v_random * real(p_max_gap_width));
            r_gap_cnt   <= 0;

            sm_verifier <= GAP;
         ELSIF rising_edge(i_clk) THEN

            CASE (sm_verifier) IS
               -- --------------------------------
               WHEN GAP =>
                  IF (r_gap_cnt = r_gap_width) THEN

                     v_seed1       := v_seed1 + integer(v_random * 3.4);
                     uniform(v_seed1, v_seed2, v_random);
                     r_burst_width <= integer(v_random * real(p_max_ready_window_width));
                     r_burst_cnt   <= 0;

                     sm_verifier <= RECEIVE_DATA;
                  ELSE
                     r_gap_cnt <= r_gap_cnt + 1;
                  END IF;
               -- --------------------------------
               WHEN RECEIVE_DATA =>
                  IF (S_AXIS_ACT_TVALID = '1') THEN
                     IF (r_burst_cnt = r_burst_width) THEN

                        v_seed2     := v_seed2 + integer(v_random * 2.7);
                        uniform(v_seed1, v_seed2, v_random);
                        r_gap_width <= integer(v_random * real(p_max_gap_width));
                        r_gap_cnt   <= 0;

                        sm_verifier <= GAP;
                     ELSE
                        r_burst_cnt <= r_burst_cnt + 1;
                     END IF;

							IF (p_check_data) THEN
	                     IF (S_AXIS_EXP_TVALID = '0') THEN
	                        REPORT "ERROR: No data available to check against" SEVERITY failure;
	                     ELSE
	                        IF (S_AXIS_ACT_TDATA /= S_AXIS_EXP_TDATA) THEN
	                           REPORT "ERROR: Data from spacewire IP : 0x" &
	                           to_hstring(S_AXIS_ACT_TDATA) &
	                           " is different than expected: 0x" &
	                           to_hstring(S_AXIS_EXP_TDATA)
	                           SEVERITY failure;
	                        END IF;
	                     END IF;
	                  END IF;
                  END IF;
            END CASE;
         END IF;
      END PROCESS;

      n_axis_tready <= '1' WHEN sm_verifier = RECEIVE_DATA ELSE '0';

   END GENERATE;

   S_AXIS_ACT_TREADY <= n_axis_tready;
   S_AXIS_EXP_TREADY <= n_axis_tready;

END ARCHITECTURE RTL;
