LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE ieee.math_real.ALL;

USE std.textio.ALL;

LIBRARY topic;
USE topic.topic_pkg.ALL;

PACKAGE topic_tb_pkg IS

   -- ##########################################
   -- ########    TYPE DECLARATIONS    #########

   CONSTANT c_max_cmd_length      : integer := 16;                         -- in words
   CONSTANT c_max_cmd_word_length : integer := 32;                         -- in words

   CONSTANT c_read_no_check         : std_logic_vector(3 DOWNTO 0) := "0001"; -- read addr
   CONSTANT c_read_check_result     : std_logic_vector(3 DOWNTO 0) := "0010"; -- read addr exp_data
   CONSTANT c_read_data             : std_logic_vector(3 DOWNTO 0) := "0011"; -- read data
   CONSTANT c_req_irq_info          : std_logic_vector(3 DOWNTO 0) := "0100"; -- request interrupt info
   CONSTANT c_req_read_fifo_status  : std_logic_vector(3 DOWNTO 0) := "0101"; -- request read fifo status
   CONSTANT c_req_write_fifo_status : std_logic_vector(3 DOWNTO 0) := "0110"; -- request write fifo status
   CONSTANT c_write                 : std_logic_vector(3 DOWNTO 0) := "1000"; -- write addr
   CONSTANT c_write_data            : std_logic_vector(3 DOWNTO 0) := "1001"; -- write data
   CONSTANT c_clear_read_irq        : std_logic_vector(3 DOWNTO 0) := "1010"; -- clear read irq
   CONSTANT c_clear_write_irq       : std_logic_vector(3 DOWNTO 0) := "1011"; -- clear write irq
   CONSTANT c_set_read_threshold    : std_logic_vector(3 DOWNTO 0) := "1100"; -- set read threshold
   CONSTANT c_set_write_threshold   : std_logic_vector(3 DOWNTO 0) := "1101"; -- set write threshold
   CONSTANT c_set_read_irq          : std_logic_vector(3 DOWNTO 0) := "1110"; -- set read irq
   CONSTANT c_set_write_irq         : std_logic_vector(3 DOWNTO 0) := "1111"; -- set write irq     

   SHARED VARIABLE seed1, seed2 : positive;                                -- seed values for random generator

   SUBTYPE t_string32 IS string(1 TO 32);
   TYPE t_string32_array IS ARRAY (natural RANGE <>) OF t_string32;

   TYPE t_int_array IS ARRAY (natural RANGE <>) OF integer;
   
   TYPE t_time_array IS ARRAY (natural RANGE <>) OF time;

   TYPE t_cmd_record IS RECORD
      stim_file : string(1 TO 64);
      line_cnt  : integer;
      word      : t_string32_array(c_max_cmd_length - 1 DOWNTO 0);
      size      : t_int_array(c_max_cmd_length - 1 DOWNTO 0);
      cnt       : integer;
      valid     : boolean;
   END RECORD;

   TYPE t_stimulus_line IS RECORD
      word_cnt     : natural;
      words        : t_string32_array(c_max_cmd_length - 1 DOWNTO 0);
      words_length : t_int_array(c_max_cmd_length - 1 DOWNTO 0);
      line_nr      : natural;
      valid        : std_logic;
   END RECORD;

   TYPE t_spi_transaction IS RECORD
      valid     : std_logic;
      length    : positive RANGE 1 TO 32;
      mosi_data : t_byte_array(0 TO 31);
      miso_data : t_byte_array(0 TO 31);
   END RECORD;

   -- ########        END TYPES        #########
   -- ##########################################

   -- ###############################################
   -- ########    COMPONENT DECLARATIONS    #########

   COMPONENT tb_stim_reader IS
      GENERIC(
         STIM_FILE_NAME : string := ""
      );
      PORT(
         cmd_out       : OUT t_cmd_record;
         cmd_accept_in : IN  std_logic;
         eof           : OUT std_logic
      );
   END COMPONENT;

   COMPONENT tb_main_driver IS
      GENERIC(
         C_M_AXI_LITE_DATA_WIDTH : integer := 32;
         C_M_AXI_LITE_ADDR_WIDTH : integer := 32
      );
      PORT(
         clk                    : IN  std_logic;
         rst_n                  : IN  std_logic;
         cmd_in                 : IN  t_cmd_record;
         cmd_accept_out         : OUT std_logic;
         MTESTADDRESS           : OUT std_logic_vector(C_M_AXI_LITE_ADDR_WIDTH - 1 DOWNTO 0);
         MTESTPROTECTION        : OUT std_logic_vector(2 DOWNTO 0);
         TEST_DATA              : OUT std_logic_vector(C_M_AXI_LITE_DATA_WIDTH - 1 DOWNTO 0);
         RD_DATA                : IN  std_logic_vector(C_M_AXI_LITE_DATA_WIDTH - 1 DOWNTO 0);
         MTESTDATASIZE          : OUT std_logic_vector(11 DOWNTO 0);
         WRITE_BURST            : OUT std_logic;
         READ_BURST             : OUT std_logic;
         WRITE_BURST_DONE       : IN  std_logic;
         READ_BURST_DONE        : IN  std_logic;
         WRITE_BURST_CONCURRENT : OUT std_logic;
         WRITE_BURST_DATA_FIRST : OUT std_logic;
         REPORT_STATUS          : OUT std_logic
      );
   END COMPONENT;

   COMPONENT text_file_reader IS
      GENERIC(
         p_filename : string := ""
      );
      PORT(
         o_line      : OUT t_stimulus_line;
         o_eof       : OUT std_logic;
         i_next_line : IN  std_logic
      );
   END COMPONENT;

   COMPONENT linked_list_slv_fifo IS
      GENERIC(
         p_data_width : natural := 32;
         p_fwft       : boolean := true
      );
      PORT(
         i_clk   : IN  STD_LOGIC;
         i_rst   : IN  STD_LOGIC;
         i_data  : IN  STD_LOGIC_VECTOR(p_data_width - 1 DOWNTO 0);
         o_data  : OUT STD_LOGIC_VECTOR(p_data_width - 1 DOWNTO 0);
         i_put   : IN  STD_LOGIC;
         i_get   : IN  STD_LOGIC;
         o_empty : OUT STD_LOGIC
      );
   END COMPONENT;

   COMPONENT axis_traffic_gen IS
      GENERIC(
         p_data_width             : natural               := 32;
         p_insert_id_in_data      : boolean               := false;
         p_id                     : natural RANGE 0 TO 15 := 0;
         p_data_source            : string                := "RANDOM";     -- can also be "COUNTER"
         p_max_valid_window_width : natural               := 1;
         p_max_gap_width          : natural               := 1
      );
      PORT(
         i_clk         : IN  std_logic;
         i_rst         : IN  std_logic;
         -- AXI Stream Master:
         M_AXIS_TDATA  : OUT std_logic_vector(p_data_width - 1 DOWNTO 0);
         M_AXIS_TREADY : IN  std_logic;
         M_AXIS_TVALID : OUT std_logic
      );
   END COMPONENT;

   COMPONENT axis_verifier IS
      GENERIC(
         p_data_width             : natural := 32;
         p_max_ready_window_width : natural := 1;
         p_max_gap_width          : natural := 1;
         p_check_data				 : boolean := true
      );
      PORT(
         i_clk             : IN  std_logic;
         i_rst             : IN  std_logic;
         -- AXI Stream Slave (expected data)
         S_AXIS_EXP_TDATA  : IN  std_logic_vector(p_data_width - 1 DOWNTO 0);
         S_AXIS_EXP_TREADY : OUT std_logic;
         S_AXIS_EXP_TVALID : IN  std_logic;
         -- AXI Stream Slave (actual data)
         S_AXIS_ACT_TDATA  : IN  std_logic_vector(p_data_width - 1 DOWNTO 0);
         S_AXIS_ACT_TREADY : OUT std_logic;
         S_AXIS_ACT_TVALID : IN  std_logic
      );
   END COMPONENT;

   COMPONENT spi_master_bfm IS
      GENERIC(
         p_spi_clk_period : time := 100 ns
      );
      PORT(
         i_transaction      : IN  t_spi_transaction;
         o_transaction_done : OUT std_logic := '0';
         o_spi_sck          : OUT std_logic;
         o_spi_ss_n         : OUT std_logic;
         o_spi_mosi         : OUT std_logic;
         i_spi_miso         : IN  std_logic
      );
   END COMPONENT;

   COMPONENT spi_monitor IS
      GENERIC(
         p_logfile_name : string := "./spi_monitor_log.txt"
      );
      PORT(
         i_spi_sck               : IN  std_logic;
         i_spi_ss_n              : IN  std_logic;
         i_spi_mosi              : IN  std_logic;
         i_spi_miso              : IN  std_logic;
         o_monitored_transaction : OUT t_spi_transaction;
         i_read_transaction      : IN  std_logic
      );
   END COMPONENT;
   
   COMPONENT clock_reset_manager IS
      GENERIC(
         p_number_of_output_clocks : natural := 1;
         p_clk_periods             : t_time_array;
         p_enable_gsr              : boolean := true;
         p_gsr_duration            : time    := 100 ns
      );
      PORT(
         o_gsr         : OUT std_logic;
         o_clks        : OUT std_logic_vector(p_number_of_output_clocks - 1 DOWNTO 0);
         o_rsts        : OUT std_logic_vector(p_number_of_output_clocks - 1 DOWNTO 0);
         o_rsts_n      : OUT std_logic_vector(p_number_of_output_clocks - 1 DOWNTO 0);
         o_resets_done : OUT std_logic;
         i_stop_clocks : IN  std_logic
      );
   END COMPONENT;   

   -- ########   END COMPONENT DECLARATIONS   #######   
   -- ###############################################   

   -- ###############################################
   -- ########    FUNCTION DECLARATIONS    #########

   FUNCTION read_time(input_string : IN string) RETURN time;

   FUNCTION astr_to_slv(s : IN string) RETURN std_logic_vector;

   FUNCTION hstr_to_slv(s : IN string) RETURN std_logic_vector;

   FUNCTION char_to_sl(c : IN character) RETURN std_logic;

   FUNCTION bstr_to_slv(s : IN string) RETURN std_logic_vector;

   PROCEDURE proc_str_to_int(
      VARIABLE str    : IN string;
      VARIABLE int    : OUT integer;
      VARIABLE result : OUT boolean
   );

   FUNCTION func_lowercase(word : IN t_string32) RETURN t_string32;

   PROCEDURE proc_split_line(
      VARIABLE line_buf     : IN line;
      VARIABLE word_array   : OUT t_string32_array;
      VARIABLE size_array   : OUT t_int_array;
      VARIABLE word_cnt     : OUT integer;
      VARIABLE empty_line   : OUT boolean;
      VARIABLE comment_line : OUT boolean
   );

   PROCEDURE proc_get_value(
      VARIABLE str    : IN string;
      VARIABLE slv    : OUT std_logic_vector;
      VARIABLE result : OUT boolean;
      VARIABLE len    : OUT integer
   );

   FUNCTION to_hstring(value : std_logic_vector) RETURN string;

   FUNCTION to_intstring(value : std_logic_vector) RETURN string;

   FUNCTION int_to_string(input : integer) RETURN string;

   IMPURE FUNCTION randomize(dummy : integer) RETURN integer;
   IMPURE FUNCTION get_random_positive_integer(min_value : natural; max_value : positive) RETURN positive;
   IMPURE FUNCTION get_random_slv(max_value : positive; slv_width : positive) RETURN std_logic_vector;
   IMPURE FUNCTION get_random_time(max_value : time) RETURN time;

   FUNCTION get_unequal_compare_string(actual_data : std_logic_vector; expected_data : std_logic_vector) RETURN string;

   PROCEDURE str_read(FILE in_file : TEXT;
                      res_string   : OUT string);

   PROCEDURE do_spi_word_transaction(
      CONSTANT mosi_data : IN t_word;
      VARIABLE miso_data : OUT t_word;
      SIGNAL i_spi_clk   : IN std_logic;
      SIGNAL o_spi_mosi  : OUT std_logic;
      SIGNAL i_spi_miso  : IN std_logic);

   PROCEDURE monitor_spi_transaction(
      VARIABLE mosi_data                  : OUT t_word_array;
      VARIABLE miso_data                  : OUT t_word_array;
      VARIABLE number_of_bits_transferred : OUT natural;
      SIGNAL i_spi_clk                    : IN std_logic;
      SIGNAL i_spi_ss_n                   : IN std_logic;
      SIGNAL i_spi_mosi                   : IN std_logic;
      SIGNAL i_spi_miso                   : IN std_logic);
      
      FUNCTION str_pad(s : string; pad_char : character; len : integer) RETURN string;  

   -- ########   END FUNCTION DECLARATIONS   #######   
   -- ############################################### 
END PACKAGE;

PACKAGE BODY topic_tb_pkg IS
   -- read_time
   -- This function convert a string to time.
   FUNCTION read_time(input_string : IN string) RETURN time IS
      VARIABLE v_temp_line : line;                                         -- store line data
      VARIABLE v_delay     : time;                                         -- delay time
      VARIABLE v_result    : boolean;                                      -- result indicator
   BEGIN
      write(v_temp_line, input_string);
      read(v_temp_line, v_delay, v_result);
      IF (NOT (v_result)) THEN
         REPORT "ERROR function read_time" SEVERITY failure;
      ELSE
         RETURN v_delay;
      END IF;
   END FUNCTION read_time;

   FUNCTION astr_to_slv(s : IN string) RETURN std_logic_vector IS
      VARIABLE slv : std_logic_vector((s'high - s'low) * 8 + 7 DOWNTO 0);
      VARIABLE k   : integer;                                              -- position in std_logic_vector
   BEGIN
      k := 0;
      FOR i IN s'high DOWNTO s'low LOOP
         slv(k + 7 DOWNTO k) := std_logic_vector(to_unsigned(character'pos(s(i)), 8));
         k                   := k + 8;
      END LOOP;
      RETURN slv;
   END FUNCTION astr_to_slv;

   FUNCTION hstr_to_slv(s : IN string) RETURN std_logic_vector IS
      VARIABLE slv : std_logic_vector((s'high - s'low) * 4 + 3 DOWNTO 0) := (OTHERS => '0');
      VARIABLE k   : integer;                                              -- position in std_logic_vector
      VARIABLE j   : integer;                                              -- to filter underscores
      VARIABLE u   : integer;                                              -- help for position in std_logic_vector
      VARIABLE l   : integer;                                              -- help for position in std_logic_vector
   BEGIN
      k := 0;
      j := 0;                                                              -- tofilter underscores
      FOR i IN s'high DOWNTO s'low LOOP
         u := k + 3 - j;
         l := k - j;
         CASE s(i) IS
            WHEN '0'       => slv(u DOWNTO l) := "0000";
            WHEN '1'       => slv(u DOWNTO l) := "0001";
            WHEN '2'       => slv(u DOWNTO l) := "0010";
            WHEN '3'       => slv(u DOWNTO l) := "0011";
            WHEN '4'       => slv(u DOWNTO l) := "0100";
            WHEN '5'       => slv(u DOWNTO l) := "0101";
            WHEN '6'       => slv(u DOWNTO l) := "0110";
            WHEN '7'       => slv(u DOWNTO l) := "0111";
            WHEN '8'       => slv(u DOWNTO l) := "1000";
            WHEN '9'       => slv(u DOWNTO l) := "1001";
            WHEN 'a' | 'A' => slv(u DOWNTO l) := "1010";
            WHEN 'b' | 'B' => slv(u DOWNTO l) := "1011";
            WHEN 'c' | 'C' => slv(u DOWNTO l) := "1100";
            WHEN 'd' | 'D' => slv(u DOWNTO l) := "1101";
            WHEN 'e' | 'E' => slv(u DOWNTO l) := "1110";
            WHEN 'f' | 'F' => slv(u DOWNTO l) := "1111";
            WHEN 'z' | 'Z' => slv(u DOWNTO l) := "ZZZZ";
            WHEN 'x' | 'X' => slv(u DOWNTO l) := "XXXX";
            WHEN '-'       => slv(u DOWNTO i) := "----";
            WHEN '_'       => j := j + 4;                                  -- filter underscores
            WHEN ' '       => j := j + 4;                                  -- filter spaces
            WHEN OTHERS    => slv(u DOWNTO l) := "UUUU";
         END CASE;
         k := k + 4;
      END LOOP;

      RETURN slv;
   END FUNCTION hstr_to_slv;

   FUNCTION char_to_sl(c : IN character) RETURN std_logic IS
      VARIABLE sl : std_logic;
   BEGIN
      CASE c IS
         WHEN 'U' | 'u' => sl := 'U';
         WHEN 'X' | 'x' => sl := 'X';
         WHEN '0'       => sl := '0';
         WHEN '1'       => sl := '1';
         WHEN 'Z' | 'z' => sl := 'Z';
         WHEN 'W' | 'w' => sl := 'W';
         WHEN 'L' | 'l' => sl := 'L';
         WHEN 'H' | 'h' => sl := 'H';
         WHEN '-'       => sl := '-';
         WHEN OTHERS    => sl := 'X';
      END CASE;
      RETURN sl;
   END FUNCTION char_to_sl;

   FUNCTION bstr_to_slv(s : IN string) RETURN std_logic_vector IS
      VARIABLE slv : std_logic_vector((s'high - s'low) DOWNTO 0);
      VARIABLE k   : integer;                                              -- position in std_logic_vector
      VARIABLE j   : integer;                                              -- to filter underscores
   BEGIN
      k := 0;
      j := 0;
      FOR i IN s'high DOWNTO s'low LOOP
         IF (s(i) = '_') OR (s(i) = ' ') THEN                              -- filter underscores and spaces
            j := j + 1;
         ELSE
            slv(k - j) := char_to_sl(s(i));
         END IF;
         k := k + 1;
      END LOOP;
      RETURN slv;
   END FUNCTION bstr_to_slv;

   PROCEDURE proc_str_to_int(
      VARIABLE str    : IN string;
      VARIABLE int    : OUT integer;
      VARIABLE result : OUT boolean
   ) IS
      VARIABLE v_str    : string(1 TO c_max_cmd_word_length);              -- holds clean string value
      VARIABLE char_cnt : integer;                                         -- number of characters
      VARIABLE temp     : integer;
      VARIABLE tempint  : integer;

   BEGIN
      v_str    := (OTHERS => ' ');
      result   := true;
      char_cnt := 0;

      FOR i IN str'low TO str'high LOOP
         IF (str(i) /= ' ') THEN                                           -- skip spaces
            char_cnt        := char_cnt + 1;
            v_str(char_cnt) := str(i);
         END IF;
      END LOOP;

      tempint := (0);
      FOR i IN 1 TO char_cnt LOOP
         CASE v_str(i) IS
            WHEN '0'    => temp := 0;
            WHEN '1'    => temp := 1;
            WHEN '2'    => temp := 2;
            WHEN '3'    => temp := 3;
            WHEN '4'    => temp := 4;
            WHEN '5'    => temp := 5;
            WHEN '6'    => temp := 6;
            WHEN '7'    => temp := 7;
            WHEN '8'    => temp := 8;
            WHEN '9'    => temp := 9;
            WHEN OTHERS => result := false;
         END CASE;
         tempint := tempint + temp * 10 **(char_cnt - i);
      END LOOP;
      int     := tempint;

   END proc_str_to_int;

   FUNCTION func_lowercase(word : IN t_string32) RETURN t_string32 IS
      VARIABLE temp_word : string(word'RANGE);
      VARIABLE temp_int  : integer;
   BEGIN
      temp_word := (OTHERS => ' ');
      FOR i IN word'RANGE LOOP
         temp_int := character'pos(word(i));
         CASE temp_int IS
            WHEN 65 TO 90 => temp_word(i) := character'val(temp_int + 32);
            WHEN OTHERS   => temp_word(i) := word(i);
         END CASE;
      END LOOP;
      RETURN temp_word;
   END FUNCTION func_lowercase;

   PROCEDURE proc_split_line(
      VARIABLE line_buf     : IN line;
      VARIABLE word_array   : OUT t_string32_array;
      VARIABLE size_array   : OUT t_int_array;
      VARIABLE word_cnt     : OUT integer;
      VARIABLE empty_line   : OUT boolean;
      VARIABLE comment_line : OUT boolean
   ) IS
      VARIABLE line_char_cnt : integer;                                    -- counts characters in line
      VARIABLE word_char_cnt : integer;                                    -- counts characters in word
      VARIABLE line_word_cnt : integer;                                    -- counts words in line
      VARIABLE word          : t_string32;                                 -- string
      VARIABLE stop          : boolean;
   BEGIN
      comment_line  := false;
      line_word_cnt := 0;                                                  -- clear word counter for new line
      line_char_cnt := 0;                                                  -- start at first character in line
      stop          := false;

      -- clear all words in array
      FOR i IN word_array'RANGE LOOP
         word_array(i) := (OTHERS => ' ');
      END LOOP;

      -- loop while not end of line and number of words is less than max words
      -- in array and it is not an empty line
      WHILE ((line_char_cnt < line_buf'length) AND NOT (stop)) LOOP
         word_char_cnt := 0;                                               -- reset for new word
         word          := (OTHERS => ' ');                                 -- clear word

         -- read a character while not end of line
         WHILE (line_char_cnt < line_buf'length) LOOP
            line_char_cnt := line_char_cnt + 1;                            -- go to next character

            -- if character(s) is(are) '*' or "--" or "//" then rest is comment, stop
            IF ((line_buf(line_char_cnt) = '*') OR ((line_buf(line_char_cnt) = '-') AND (line_buf(line_char_cnt + 1) = '-')) OR ((line_buf(line_char_cnt) = '/') AND (line_buf(line_char_cnt + 1) = '/'))) THEN
               -- if no words and no characters are found then comment line
               comment_line := ((line_word_cnt = 0) AND (word_char_cnt = 0));
               stop         := true;                                       -- stop reading line
               EXIT;                                                       -- rest of line is comment exit loop
            END IF;

            -- check on word overflow
            IF (word_char_cnt = c_max_cmd_word_length) THEN
               stop := true;                                               -- stop reading line
               REPORT "ERROR: Word exceeds maximum of " & integer'image(c_max_cmd_word_length) & " characters" SEVERITY failure;
               EXIT;
               -- check on space or tab or word size then finish word
            ELSIF ((line_buf(line_char_cnt) = ' ') OR (line_buf(line_char_cnt) = ht)) THEN
               EXIT;                                                       -- ready with word exit loop

            ELSE                                                           -- found new character for word
               word_char_cnt       := word_char_cnt + 1;
               word(word_char_cnt) := line_buf(line_char_cnt);
            END IF;
         END LOOP;

         -- check on comment
         IF (word_char_cnt /= 0) THEN
            word_array(line_word_cnt) := func_lowercase(word);
            size_array(line_word_cnt) := word_char_cnt;
            line_word_cnt             := line_word_cnt + 1;
         END IF;

      END LOOP;

      empty_line := (line_word_cnt = 0);                                   -- set empty line to true when no words found, else false
      word_cnt   := line_word_cnt;                                         -- return number of words

   END PROCEDURE proc_split_line;

   PROCEDURE proc_get_value(
      VARIABLE str    : IN string;
      VARIABLE slv    : OUT std_logic_vector;
      VARIABLE result : OUT boolean;
      VARIABLE len    : OUT integer
   ) IS
      VARIABLE v_str         : string(1 TO c_max_cmd_word_length);         -- clean string value
      VARIABLE v_len         : integer;                                    -- length of slv
      VARIABLE v_result      : boolean;                                    -- needed for result of string to integer conversion 
      VARIABLE v_found_quote : boolean;                                    -- indcates first quote found 
      VARIABLE v_char_cnt    : integer;                                    -- counts characters in clean string
   BEGIN
      v_str         := (OTHERS => ' ');
      v_char_cnt    := 0;
      v_len         := 0;
      v_found_quote := false;

      -- create a clean string without spaces and underscores, find number of characters
      FOR i IN str'low TO str'high LOOP
         IF ((str(i) /= ' ') AND (str(i) /= '_') AND (str(i) /= CR)) THEN  -- skip space and underscores
            IF (str(i) = '"') THEN                                         -- " -- check for first quotes in string
               IF NOT (v_found_quote) THEN                                 -- skip last quote
                  v_found_quote := true;
                  proc_str_to_int(str(str'low TO i - 2), v_len, v_result); -- get bit length
                  v_str(1)      := str(i - 1);                             -- first char indicates format
                  v_char_cnt    := 2;                                      -- continue with charater 2 in result string
               END IF;
            ELSE
               v_char_cnt        := v_char_cnt + 1;
               v_str(v_char_cnt) := str(i);
            END IF;
         END IF;
      END LOOP;

      -- set std logic vector output to undefined
      FOR i IN slv'RANGE LOOP
         slv(i) := '0';
      END LOOP;
      len := 0;

      -- determine format
      IF (v_found_quote) THEN
         IF ((v_str(1) = 'x') OR (v_str(1) = 'X')) THEN                    -- found #x"<hex value>"
            slv(v_char_cnt * 4 - 5 DOWNTO 0) := hstr_to_slv(v_str(2 TO v_char_cnt)); -- remove x
            len                              := (v_char_cnt - 1) * 4;
         ELSIF ((v_str(1) = 'b') OR (v_str(1) = 'B')) THEN                 -- found #b"<hex value>"
            slv(v_char_cnt - 2 DOWNTO 0) := bstr_to_slv(v_str(2 TO v_char_cnt)); -- remove b"
            len                          := v_char_cnt - 1;
         ELSIF ((v_str(1) = 'a') OR (v_str(1) = 'A')) THEN                 -- found #a"<ascii value>"
            slv(v_char_cnt - 2 DOWNTO 0) := astr_to_slv(v_str(2 TO v_char_cnt)); -- remove a"
            len                          := (v_char_cnt - 1) * 8;
         END IF;
      ELSE
         IF ((v_str(1) = '0') AND ((v_str(2) = 'x') OR (v_str(2) = 'X'))) THEN -- found 0x<hex value>
            slv(v_char_cnt * 4 - 9 DOWNTO 0) := hstr_to_slv(v_str(3 TO v_char_cnt)); -- remove 0x
            len                              := (v_char_cnt - 2) * 4;
         ELSIF ((v_str(1) = '0') AND ((v_str(2) = 'b') OR (v_str(2) = 'B'))) THEN -- found 0b<bin value>
            slv(v_char_cnt - 3 DOWNTO 0) := bstr_to_slv(v_str(3 TO v_char_cnt)); -- remove 0b
            len                          := v_char_cnt - 2;
         ELSE
            slv(v_char_cnt - 1 DOWNTO 0) := bstr_to_slv(v_str(1 TO v_char_cnt));
            len                          := v_char_cnt;
         END IF;
      END IF;

      result := NOT (v_char_cnt = 0);                                      -- correct result unless empty string

      -- print value found
      -- if DEBUG then
      -- report "global_pkg - proc_get_value -> String value found: " & v_str(1 to v_char_cnt);
      -- end if;
   END PROCEDURE proc_get_value;

   FUNCTION to_hstring(value : std_logic_vector) RETURN string IS
      CONSTANT ne     : integer := (value'length + 3) / 4;
      VARIABLE pad    : std_logic_vector(0 TO (ne * 4 - value'length) - 1);
      VARIABLE ivalue : std_logic_vector(0 TO ne * 4 - 1);
      VARIABLE result : string(1 TO ne);
      VARIABLE quad   : std_logic_vector(0 TO 3);
   BEGIN
      IF value'length < 1 THEN
         -- return NUS;
         RETURN "  ";
      ELSE
         IF value(value'left) = 'Z' THEN
            pad := (OTHERS => 'Z');
         ELSE
            pad := (OTHERS => '0');
         END IF;
         ivalue := pad & value;
         FOR i IN 0 TO ne - 1 LOOP
            quad := To_X01Z(ivalue(4 * i TO 4 * i + 3));
            CASE quad IS
               WHEN x"0"   => result(i + 1) := '0';
               WHEN x"1"   => result(i + 1) := '1';
               WHEN x"2"   => result(i + 1) := '2';
               WHEN x"3"   => result(i + 1) := '3';
               WHEN x"4"   => result(i + 1) := '4';
               WHEN x"5"   => result(i + 1) := '5';
               WHEN x"6"   => result(i + 1) := '6';
               WHEN x"7"   => result(i + 1) := '7';
               WHEN x"8"   => result(i + 1) := '8';
               WHEN x"9"   => result(i + 1) := '9';
               WHEN x"A"   => result(i + 1) := 'A';
               WHEN x"B"   => result(i + 1) := 'B';
               WHEN x"C"   => result(i + 1) := 'C';
               WHEN x"D"   => result(i + 1) := 'D';
               WHEN x"E"   => result(i + 1) := 'E';
               WHEN x"F"   => result(i + 1) := 'F';
               WHEN "ZZZZ" => result(i + 1) := 'Z';
               WHEN OTHERS => result(i + 1) := 'X';
            END CASE;
         END LOOP;
         RETURN result;
      END IF;
   END FUNCTION to_hstring;

   FUNCTION to_intstring(value : std_logic_vector) RETURN string IS
      -- constant ne     : integer := (value'length+3)/4;
      CONSTANT ne : integer := (value'length);

      -- variable pad    : std_logic_vector(0 to (ne*4 - value'length) - 1);
      -- variable ivalue : std_logic_vector(0 to ne*4 - 1);
      VARIABLE result : string(1 TO ne);
      -- variable quad   : std_logic_vector(0 to 3);

      VARIABLE v_int_result : integer;

   BEGIN
      IF value'length < 1 THEN
         -- return NUS;
         RETURN "  ";
      ELSE
         FOR i IN 0 TO ne - 1 LOOP
            IF (i = 0) THEN
               IF (value(i) = '1') THEN
                  v_int_result := v_int_result + 1;
               ELSE
                  v_int_result := v_int_result;
               END IF;
            ELSE
               IF (value(i) = '1') THEN
                  v_int_result := v_int_result + (i * 2);
               END IF;
            END IF;
         END LOOP;

         result := integer'image(v_int_result);

         RETURN result;
      END IF;
   END FUNCTION to_intstring;

   FUNCTION int_to_string(input : integer) RETURN string IS
      VARIABLE my_string : line;
      VARIABLE length    : integer;
   BEGIN
      length    := integer(input / 10 + 1);
      my_string := NEW string(1 TO length);
      write(my_string, input);
      RETURN my_string.ALL;
   END int_to_string;

   IMPURE FUNCTION randomize(dummy : integer) RETURN integer IS
      VARIABLE v_rand : real;                                              -- random real-number value in range 0 to 1.0  
   BEGIN
      uniform(seed1, seed2, v_rand);
      RETURN 1;
   END FUNCTION;

   IMPURE FUNCTION get_random_positive_integer(min_value : natural; max_value : positive) RETURN positive IS
      VARIABLE v_rand : real;                                              -- random real-number value in range 0 to 1.0  
      VARIABLE v_tmp  : integer;
   BEGIN
      uniform(seed1, seed2, v_rand);                                       -- generate random number
      v_tmp := integer(v_rand * real(max_value));
      IF (v_tmp < min_value) THEN
         RETURN min_value;
      ELSE
         RETURN v_tmp;
      END IF;
   END FUNCTION;

   IMPURE FUNCTION get_random_slv(max_value : positive; slv_width : positive) RETURN std_logic_vector IS
   BEGIN
      RETURN std_logic_vector(to_unsigned(get_random_positive_integer(0, max_value), slv_width));
   END FUNCTION;

   IMPURE FUNCTION get_random_time(max_value : time) RETURN time IS
      VARIABLE v_rand : real;                                              -- random real-number value in range 0 to 1.0  
   BEGIN
      uniform(seed1, seed2, v_rand);
      RETURN v_rand * max_value;
   END FUNCTION;

   FUNCTION get_unequal_compare_string(actual_data : std_logic_vector; expected_data : std_logic_vector) RETURN string IS
   BEGIN
      RETURN "Data: " & to_hstring(actual_data) & " does not match expected data: " & to_hstring(expected_data);
   END FUNCTION;

   PROCEDURE str_read(FILE in_file : TEXT;
                      res_string   : OUT string) IS
      VARIABLE l         : line;
      VARIABLE c         : character;
      VARIABLE is_string : boolean;

   BEGIN
      readline(in_file, l);
      -- clear the contents of the result string
      FOR i IN res_string'RANGE LOOP
         res_string(i) := ' ';
      END LOOP;
      -- read all characters of the line, up to the length  
      -- of the results string
      FOR i IN res_string'RANGE LOOP
         read(l, c, is_string);
         res_string(i) := c;
         IF NOT is_string THEN                                             -- found end of line
            EXIT;
         END IF;
      END LOOP;

   END str_read;

   PROCEDURE do_spi_word_transaction(
      CONSTANT mosi_data : IN t_word;
      VARIABLE miso_data : OUT t_word;
      SIGNAL i_spi_clk   : IN std_logic;
      SIGNAL o_spi_mosi  : OUT std_logic;
      SIGNAL i_spi_miso  : IN std_logic
   ) IS
   BEGIN

      FOR i IN 15 DOWNTO 0 LOOP

         -- Put data on MOSI on falling edge
         WAIT UNTIL falling_edge(i_spi_clk);
         o_spi_mosi <= mosi_data(i);

         -- Capture data on MISO on rising edge
         WAIT UNTIL rising_edge(i_spi_clk);
         miso_data(i) := i_spi_miso;
      END LOOP;

   END PROCEDURE;

   PROCEDURE monitor_spi_transaction(
      VARIABLE mosi_data                  : OUT t_word_array;
      VARIABLE miso_data                  : OUT t_word_array;
      VARIABLE number_of_bits_transferred : OUT natural;
      SIGNAL i_spi_clk                    : IN std_logic;
      SIGNAL i_spi_ss_n                   : IN std_logic;
      SIGNAL i_spi_mosi                   : IN std_logic;
      SIGNAL i_spi_miso                   : IN std_logic
   ) IS
      VARIABLE v_bit_cnt : natural := 0;
   BEGIN

      -- wait for the slave select line to deassert
      WAIT UNTIL i_spi_ss_n = '0';
      v_bit_cnt := 0;

      WHILE true LOOP
         WAIT UNTIL rising_edge(i_spi_clk) OR rising_edge(i_spi_ss_n);

         IF rising_edge(i_spi_clk) THEN
            mosi_data(v_bit_cnt / 16)(15 - (v_bit_cnt MOD 16)) := i_spi_mosi;
            miso_data(v_bit_cnt / 16)(15 - (v_bit_cnt MOD 16)) := i_spi_miso;
            v_bit_cnt                                     := v_bit_cnt + 1;
         ELSE
            EXIT;                                                          -- end of SPI transaction (break while loop)
         END IF;
      END LOOP;

      number_of_bits_transferred := v_bit_cnt;

   END PROCEDURE;
   
      FUNCTION str_pad(s : string; pad_char : character; len : integer) RETURN string IS
      VARIABLE v_ret : string(1 TO len);
   BEGIN
      ASSERT len >= s'length REPORT "Specified length must be longer than the specified string" SEVERITY failure;

      FOR i IN 1 TO s'high LOOP
         v_ret(i) := s(i);
      END LOOP;

      FOR i IN s'high + 1 TO len LOOP
         v_ret(i) := pad_char;
      END LOOP;

      RETURN v_ret;
   END FUNCTION str_pad;   

END PACKAGE BODY;
