-- ----------------------------------------------------------------------------
-- Copyright Message
-- ----------------------------------------------------------------------------
--
-- Topic Embedded Systems confidential and proprietary.
-- COPYRIGHT (c) 2009 Topic Embedded Systems B.V.
--
-- All rights are reserved. Reproduction or dissemination
-- in whole or in part is prohibited without the prior written
-- consent of the copyright holder.
--
-- ----------------------------------------------------------------------------
-- Design Information
-- ----------------------------------------------------------------------------
--
-- File : tb_control_driver.vhd
--
-- Author : $Author$
--
-- Organisation : Topic Embedded Systems
--
-- Date : $Date$
--
-- Revision : $Revision$
--
-- Project : Dyplo
--
-- Description :  This unit is part a VHDL test environment and performs
--                register read and writes. Read are checked
--                on expected result and errors are logged.
-- ----------------------------------------------------------------------------
-- Design Notes
-- ----------------------------------------------------------------------------
--
-- <Important notes about design>
-- <signals are active-low or active-high>
-- <main clock signal>
-- <main reset signal>
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

use work.tb_env_pkg.all;
use std.textio.all;

entity tb_control_driver is
port
(
  clk                         : in    std_logic;
  rst_n                       : in    std_logic;

  cmd_in                      : in    cmd_record;
  cmd_accept_out              : out   std_logic;

  cmd_valid                   : out   std_logic;
  cmd                         : out   std_logic_vector(99 downto 0);
  cmd_ready                   : in    std_logic;

  waiting_on_sync             : out   std_logic;
  continue_after_sync         : in    std_logic
);
end tb_control_driver;


architecture behaviour of tb_control_driver is
  constant CMD_DATA_WIDTH             : integer := 100;

  type sm_control_driver_type is (IDLE, PARSE_CMD, WAIT_FOR_READY, SYNCING);
  signal sm_control_driver       : sm_control_driver_type := IDLE;
begin

  p_handle_cmd: process(clk)
    variable v_value                    : std_logic_vector(31 downto 0);    -- temporary storage for value
    variable v_result                   : boolean;
    variable v_result_len               : integer;
    variable v_string                   : string(1 to CMD_WORD_SIZE);       -- temp variable to convert string signal to variable
    variable v_log_line                 : line;  -- store log line
  begin
    if(rising_edge(clk)) then
      if(rst_n = '0') then

        cmd        <= (others => '0');
        cmd_valid  <= '0';

        waiting_on_sync <= '0';
        cmd_accept_out  <= '0';

      else
        case(sm_control_driver) is

          when IDLE =>
            if(cmd_in.valid = true) then
              sm_control_driver   <= PARSE_CMD;
            else
              cmd_accept_out      <= '0'; --release command
            end if;

          when PARSE_CMD =>
            if (cmd_in.word(1)(1 to 5) = "reset") then  -- global reset
              cmd_accept_out      <= '1';  -- do accept command
              sm_control_driver   <= IDLE;

            elsif (cmd_in.word(1)(1 to 4) = "ctrl") then  -- node command

              if (cmd_in.word(2)(1 to 5) = "reset") then  -- node reset
                write(v_log_line, string'("        Reset control Driver"));
                writeline(log_file, v_log_line);
                cmd_accept_out   <= '1';  -- do accept command

              elsif (cmd_in.word(2)(1 to 4) = "sync") then  -- node synq
                write(v_log_line, string'("Control waiting on sync"));
                writeline(log_file, v_log_line);

                waiting_on_sync     <= '1';
                sm_control_driver   <= SYNCING;

              --define conf or data
              elsif (cmd_in.word(2)(1 to 10) = "write_conf") or (cmd_in.word(2)(1 to 9) = "read_conf") then

                cmd((CMD_DATA_WIDTH-1) downto 0) <= (others => '0');

                --define mode
                if (cmd_in.word(2)(1 to 10) = "write_conf") then
                  cmd(3 downto 0) <= c_write_conf;

                elsif (cmd_in.word(2)(1 to 9) = "read_conf") then
                  if (cmd_in.cnt = 4) then
                    cmd(3 downto 0) <= c_read_conf_no_check;
                  elsif (cmd_in.cnt = 6) then
                    cmd(3 downto 0) <= c_read_conf_check_result;
                  end if;
                end if;

                --read arguments
                for i in 3 to (cmd_in.cnt-1) loop
                  v_string(1 to cmd_in.size(i))  := cmd_in.word(i)(1 to cmd_in.size(i));
                  proc_get_value (
                      str           => v_string(1 to cmd_in.size(i)),
                      slv           => v_value,
                      result        => v_result,
                      len           => v_result_len
                  );
                  if not(v_result) then
                      write(v_log_line, string'("ERROR:  Control, Unknown value for command: <param>=<value> !"));
                      writeline(log_file, v_log_line);
                      write(v_log_line, string'("        Found: " & cmd_in.word(i)(3 to cmd_in.size(i))));
                      writeline(log_file, v_log_line);
                      write(v_log_line, string'("        Expected: hexadecimal or binary value e.g. 0xABCD or 0b1011010000100100 or X1011010000HLL100"));
                      writeline(log_file, v_log_line);
                      write(v_log_line, string'("        File: " & cmd_in.stim_file & " (line " & integer'image(cmd_in.line_cnt) & ")."));
                      writeline(log_file, v_log_line);
                      report "ERROR: Control, Unknown value .=<value> !";
                      report "Found: " & cmd_in.word(i)(1 to cmd_in.size(i));
                      report "Expected: hexadecimal or binary value e.g. 0xABCD or 0b1011010000100100 or X1011010000HLL100";
                      report "File: " & cmd_in.stim_file & " (line " & integer'image(cmd_in.line_cnt) & ")."
                      severity failure;
                  end if;

                  if (i=3) then                                     --address / IRQ ID
                    cmd(35 downto 4)   <= v_value(31 downto 0);
                  elsif (i=4) then                                  --data / expected_data / dataFileID(name)
                    cmd(67 downto 36)  <= v_value(31 downto 0);
                  elsif (i=5) then                                  -- length in words
                    cmd(99 downto 68)  <= v_value(31 downto 0);
                  end if;

                end loop;

                cmd_valid      <= '1';
                sm_control_driver   <= WAIT_FOR_READY;

              else
                write(v_log_line, string'("ERROR:  Control, Unknown command: fifo <command> !"));
                writeline(log_file, v_log_line);
                write(v_log_line, string'("        Found: " & cmd_in.word(2)));
                writeline(log_file, v_log_line);
                write(v_log_line, string'("        File: " & cmd_in.stim_file & " (line " & integer'image(cmd_in.line_cnt) & ")."));
                writeline(log_file, v_log_line);
                report "ERROR: Control, Unknown command: fifo <command> !";
                report "Found: " & cmd_in.word(2);
                report "File: " & cmd_in.stim_file
                & " (line " & integer'image(cmd_in.line_cnt) & ")."
                severity failure;
              end if;

            else
              write(v_log_line, string'("ERROR:  Unknown command!"));
              writeline(log_file, v_log_line);
              write(v_log_line, string'("        Found: " & cmd_in.word(1)));
              writeline(log_file, v_log_line);
              report "ERROR: Unknown command!";
              report "Found: " & cmd_in.word(2);
              report "Expected: node#, reset";
              report "File: " & cmd_in.stim_file
              & " (line " & integer'image(cmd_in.line_cnt) & ")."
              severity failure;
            end if;

          when WAIT_FOR_READY =>
            if(cmd_ready = '1') then
              cmd_valid      <= '0';

              cmd_accept_out      <= '1';  -- do accept command
              sm_control_driver   <= IDLE;
            end if;

          when SYNCING =>
            if(continue_after_sync = '1') then
              waiting_on_sync <= '0';

              cmd_accept_out      <= '1';  -- do accept command
              sm_control_driver   <= IDLE;
            end if;
        end case;

      end if;
    end if;
  end process p_handle_cmd;

end behaviour;
