-- 	File: tb_dyplo_axi_top.vhd
--
--	© COPYRIGHT 2014 TOPIC EMBEDDED PRODUCTS B.V. ALL RIGHTS RESERVED.
--
--	This file contains confidential and proprietary information of
--	Topic Embedded Products B.V. and is protected under Dutch and
--	International copyright and other international intellectual property laws.
--
--	Disclaimer
--
--	This disclaimer is not a license and does not grant any rights to the
--	materials distributed herewith. Except as otherwise provided in a valid
--	license issued to you by Topic Embedded Products B.V., and to the maximum
--	extend permitted by applicable law:
--
--	1.	Dyplo is furnished on an "as is", as available basis. Topic makes no
--	warranty, express or implied, with respect to the capability of Dyplo. All
--	warranties of any type, express or implied, including the warranties of
--	merchantability, fitness for a particular purpose and non-infringement of
--	third party rights are expressly disclaimed.
--
--	2.	Topic's maximum total liability shall be limited to general money
--	damages in an amount not to exceed the total amount paid for in the year
--	in which the damages have occurred.  Under no circumstances including
--	negligence shall Topic be liable for direct, indirect, incidental, special,
--	consequential or punitive damages, or for loss of profits, revenue, or data,
--	that are directly or indirectly related to the use of, or the inability to
--	access and use Dyplo and related services, whether in an action in contract,
--	tort, product liability, strict liability, statute or otherwise even if
--	Topic has been advised of the possibility of those damages.
--
--	This copyright notice and disclaimer must be retained as part of this file at all times.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library bfm;
use bfm.bfm_axi4_full_master.all;
use bfm.bfm_axi4_full_slave.all;

library axi_vip;
use axi_vip.axi_vip_components.all;

library dyplo;
use dyplo.dyplo_types.all;
use dyplo.dyplo_config.all;
use dyplo.dyplo_components.all;

use work.tb_env_pkg.all;
use work.tb_stim_reader;
use work.tb_fifo_driver;

entity tb_dyplo_axi_top is
	generic(
		-- AMBA AHB or AXI4 interfaces
		CONTROL_INTERFACE_TYPE    : string                        := "AXI4";
		-- Dyplo related configuration parameters
		CPU_TO_FPGA_STREAMS       : integer range 0 to 16         := 4;
		FPGA_TO_CPU_STREAMS       : integer range 0 to 16         := 4;
		IO_NODES                  : integer                       := 1;
		PR_NODES                  : integer                       := 0;
		FIXED_NODES               : integer                       := 0;
		DMA_NODES                 : integer                       := 0;
		CONNECT_LANES             : integer                       := 1;
		MONITOR                   : boolean                       := true;
		-- Static bitstream id (for matching partial bitstreams with the static)
		STATIC_BITSTREAM_ID       : std_logic_vector(15 downto 0) := X"1234"; -- (for test)
		-- IO node 1
		IO_NODE_1_INSTREAMS       : integer                       := 1;
		IO_NODE_1_OUTSTREAMS      : integer                       := 1;
		-- IO node 2
		IO_NODE_2_INSTREAMS       : integer                       := 1;
		IO_NODE_2_OUTSTREAMS      : integer                       := 1;
		-- IO node 3
		IO_NODE_3_INSTREAMS       : integer                       := 1;
		IO_NODE_3_OUTSTREAMS      : integer                       := 1;
		-- IO node 4
		IO_NODE_4_INSTREAMS       : integer                       := 1;
		IO_NODE_4_OUTSTREAMS      : integer                       := 1;
		-- Generics for DMA interfaces
		DMA_INTERFACE_TYPE        : string                        := "AXI4";
		DMA_ADDR_WIDTH            : integer range 10 to 64        := 32;
		DMA_ENABLE_DEDICATED_MODE : boolean                       := false
	);
end tb_dyplo_axi_top;

architecture rtl of tb_dyplo_axi_top is

	constant c_control_bus_data_width   : integer := 32;
	constant c_control_bus_addr_width   : integer := 32;
	constant c_control_bus_axi_id_width : integer := 4;

	constant c_dma_bus_data_width : integer := 64;

	signal tb_ACLK        : std_logic := '0';
	signal tb_ACLK_dma    : std_logic := '0';
	signal tb_ARESETN     : std_logic := '0';
	signal tb_ARESETN_dma : std_logic := '0';

	signal io_clk_i     : std_logic := '0';
	signal tb_dyplo_CLK : std_logic := '0';
	signal config_clk 	: std_logic := '0';

	signal GSR : std_logic := '1';

	-- fifo cmds
	signal fifo_cmd_valid : std_logic_vector(NO_FIFOS - 1 downto 0);
	signal fifo_cmd       : fifo_cmd_type;
	signal fifo_cmd_ready : std_logic_vector(NO_FIFOS - 1 downto 0);

	signal fifo_waiting_on_sync     : std_logic_vector(NO_FIFOS - 1 downto 0);
	signal fifo_continue_after_sync : std_logic_vector(NO_FIFOS - 1 downto 0);

	signal fifo_eof : std_logic_vector(NO_FIFOS - 1 downto 0);
	-- fifo cmds

	-- control node
	signal stim_control_cmd_i        : cmd_record;
	signal stim_control_cmd_accept_i : std_logic;

	signal control_cmd_valid_i : std_logic;
	signal control_cmd_i       : std_logic_vector(99 downto 0);
	signal control_cmd_ready_i : std_logic;

	signal control_waiting_on_sync_i     : std_logic;
	signal control_continue_after_sync_i : std_logic;

	signal stim_control_eof_i : std_logic;
	-- control node

	-- cpu node
	signal stim_cpu_cmd_i        : cmd_record;
	signal stim_cpu_cmd_accept_i : std_logic;

	signal cpu_cmd_valid_i : std_logic;
	signal cpu_cmd_i       : std_logic_vector(99 downto 0);
	signal cpu_cmd_ready_i : std_logic;

	signal cpu_waiting_on_sync_i     : std_logic;
	signal cpu_continue_after_sync_i : std_logic;

	signal stim_cpu_eof_i : std_logic;
	-- cpu node

	-- hdl nodes
	signal stim_hdl_cmd_i        : cmd_record;
	signal stim_hdl_cmd_accept_i : std_logic;

	signal hdl_cmd_valid_i : std_logic;
	signal hdl_cmd_i       : std_logic_vector(99 downto 0);
	signal hdl_cmd_ready_i : std_logic;

	signal hdl_waiting_on_sync_i     : std_logic;
	signal hdl_continue_after_sync_i : std_logic;

	signal stim_hdl_eof_i : std_logic;
	-- hdl nodes

	signal dyplo_irq : std_logic;

	component axi_2_amba is
		port(
			AXI4_araddr     : in  STD_LOGIC_VECTOR(31 downto 0);
			AXI4_arburst    : in  STD_LOGIC_VECTOR(1 downto 0);
			AXI4_arcache    : in  STD_LOGIC_VECTOR(3 downto 0);
			AXI4_arid       : in  STD_LOGIC_VECTOR(3 downto 0);
			AXI4_arlen      : in  STD_LOGIC_VECTOR(7 downto 0);
			AXI4_arlock     : in  STD_LOGIC;
			AXI4_arprot     : in  STD_LOGIC_VECTOR(2 downto 0);
			AXI4_arready    : out STD_LOGIC;
			AXI4_arsize     : in  STD_LOGIC_VECTOR(2 downto 0);
			AXI4_arvalid    : in  STD_LOGIC;
			AXI4_awaddr     : in  STD_LOGIC_VECTOR(31 downto 0);
			AXI4_awburst    : in  STD_LOGIC_VECTOR(1 downto 0);
			AXI4_awcache    : in  STD_LOGIC_VECTOR(3 downto 0);
			AXI4_awid       : in  STD_LOGIC_VECTOR(3 downto 0);
			AXI4_awlen      : in  STD_LOGIC_VECTOR(7 downto 0);
			AXI4_awlock     : in  STD_LOGIC;
			AXI4_awprot     : in  STD_LOGIC_VECTOR(2 downto 0);
			AXI4_awready    : out STD_LOGIC;
			AXI4_awsize     : in  STD_LOGIC_VECTOR(2 downto 0);
			AXI4_awvalid    : in  STD_LOGIC;
			AXI4_bid        : out STD_LOGIC_VECTOR(3 downto 0);
			AXI4_bready     : in  STD_LOGIC;
			AXI4_bresp      : out STD_LOGIC_VECTOR(1 downto 0);
			AXI4_bvalid     : out STD_LOGIC;
			AXI4_rdata      : out STD_LOGIC_VECTOR(31 downto 0);
			AXI4_rid        : out STD_LOGIC_VECTOR(3 downto 0);
			AXI4_rlast      : out STD_LOGIC;
			AXI4_rready     : in  STD_LOGIC;
			AXI4_rresp      : out STD_LOGIC_VECTOR(1 downto 0);
			AXI4_rvalid     : out STD_LOGIC;
			AXI4_wdata      : in  STD_LOGIC_VECTOR(31 downto 0);
			AXI4_wlast      : in  STD_LOGIC;
			AXI4_wready     : out STD_LOGIC;
			AXI4_wstrb      : in  STD_LOGIC_VECTOR(3 downto 0);
			AXI4_wvalid     : in  STD_LOGIC;
			M_AHB_haddr     : out STD_LOGIC_VECTOR(31 downto 0);
			M_AHB_hburst    : out STD_LOGIC_VECTOR(2 downto 0);
			M_AHB_hmastlock : out STD_LOGIC;
			M_AHB_hprot     : out STD_LOGIC_VECTOR(3 downto 0);
			M_AHB_hrdata    : in  STD_LOGIC_VECTOR(31 downto 0);
			M_AHB_hready    : in  STD_LOGIC;
			M_AHB_hresp     : in  STD_LOGIC;
			M_AHB_hsize     : out STD_LOGIC_VECTOR(2 downto 0);
			M_AHB_htrans    : out STD_LOGIC_VECTOR(1 downto 0);
			M_AHB_hwdata    : out STD_LOGIC_VECTOR(31 downto 0);
			M_AHB_hwrite    : out STD_LOGIC;
			s_axi_aclk      : in  STD_LOGIC;
			s_axi_aresetn   : in  STD_LOGIC
		);
	end component;

	component ahblite_axi_bridge_0 is
		port(
			s_ahb_hclk       : IN  STD_LOGIC;
			s_ahb_hresetn    : IN  STD_LOGIC;
			s_ahb_hsel       : IN  STD_LOGIC;
			s_ahb_haddr      : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
			s_ahb_hprot      : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
			s_ahb_htrans     : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
			s_ahb_hsize      : IN  STD_LOGIC_VECTOR(2 DOWNTO 0);
			s_ahb_hwrite     : IN  STD_LOGIC;
			s_ahb_hburst     : IN  STD_LOGIC_VECTOR(2 DOWNTO 0);
			s_ahb_hwdata     : IN  STD_LOGIC_VECTOR(63 DOWNTO 0);
			s_ahb_hready_out : OUT STD_LOGIC;
			s_ahb_hready_in  : IN  STD_LOGIC;
			s_ahb_hrdata     : OUT STD_LOGIC_VECTOR(63 DOWNTO 0);
			s_ahb_hresp      : OUT STD_LOGIC;
			m_axi_awid       : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
			m_axi_awlen      : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
			m_axi_awsize     : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
			m_axi_awburst    : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
			m_axi_awcache    : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
			m_axi_awaddr     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
			m_axi_awprot     : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
			m_axi_awvalid    : OUT STD_LOGIC;
			m_axi_awready    : IN  STD_LOGIC;
			m_axi_awlock     : OUT STD_LOGIC;
			m_axi_wdata      : OUT STD_LOGIC_VECTOR(63 DOWNTO 0);
			m_axi_wstrb      : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
			m_axi_wlast      : OUT STD_LOGIC;
			m_axi_wvalid     : OUT STD_LOGIC;
			m_axi_wready     : IN  STD_LOGIC;
			m_axi_bid        : IN  STD_LOGIC_VECTOR(11 DOWNTO 0);
			m_axi_bresp      : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
			m_axi_bvalid     : IN  STD_LOGIC;
			m_axi_bready     : OUT STD_LOGIC;
			m_axi_arid       : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
			m_axi_arlen      : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
			m_axi_arsize     : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
			m_axi_arburst    : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
			m_axi_arprot     : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
			m_axi_arcache    : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
			m_axi_arvalid    : OUT STD_LOGIC;
			m_axi_araddr     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
			m_axi_arlock     : OUT STD_LOGIC;
			m_axi_arready    : IN  STD_LOGIC;
			m_axi_rid        : IN  STD_LOGIC_VECTOR(11 DOWNTO 0);
			m_axi_rdata      : IN  STD_LOGIC_VECTOR(63 DOWNTO 0);
			m_axi_rresp      : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
			m_axi_rvalid     : IN  STD_LOGIC;
			m_axi_rlast      : IN  STD_LOGIC;
			m_axi_rready     : OUT STD_LOGIC
		);
	end component;

	-- ----------------------------------------------------------------------------
	-- I/O INTERFACES                                                           --
	-- ----------------------------------------------------------------------------

	-- IO node 1
	signal io_node_1_rst_i                 : std_logic := '1';
	signal io_node_1_rst_ack_i             : std_logic := '0';
	signal io_node_1_out_stream_1_tvalid_i : std_logic;
	signal io_node_1_out_stream_1_tready_i : std_logic;
	signal io_node_1_out_stream_1_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_1_out_stream_1_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_1_out_stream_1_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_1_out_stream_2_tvalid_i : std_logic;
	signal io_node_1_out_stream_2_tready_i : std_logic;
	signal io_node_1_out_stream_2_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_1_out_stream_2_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_1_out_stream_2_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_1_out_stream_3_tvalid_i : std_logic;
	signal io_node_1_out_stream_3_tready_i : std_logic;
	signal io_node_1_out_stream_3_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_1_out_stream_3_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_1_out_stream_3_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_1_out_stream_4_tvalid_i : std_logic;
	signal io_node_1_out_stream_4_tready_i : std_logic;
	signal io_node_1_out_stream_4_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_1_out_stream_4_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_1_out_stream_4_tuser_i  : std_logic_vector(3 downto 0);

	signal io_node_1_in_stream_1_tvalid_i : std_logic;
	signal io_node_1_in_stream_1_tready_i : std_logic;
	signal io_node_1_in_stream_1_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_1_in_stream_1_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_1_in_stream_1_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_1_in_stream_2_tvalid_i : std_logic;
	signal io_node_1_in_stream_2_tready_i : std_logic;
	signal io_node_1_in_stream_2_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_1_in_stream_2_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_1_in_stream_2_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_1_in_stream_3_tvalid_i : std_logic;
	signal io_node_1_in_stream_3_tready_i : std_logic;
	signal io_node_1_in_stream_3_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_1_in_stream_3_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_1_in_stream_3_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_1_in_stream_4_tvalid_i : std_logic;
	signal io_node_1_in_stream_4_tready_i : std_logic;
	signal io_node_1_in_stream_4_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_1_in_stream_4_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_1_in_stream_4_tuser_i  : std_logic_vector(3 downto 0);

	-- IO node 2
	signal io_node_2_rst_i                 : std_logic := '1';
	signal io_node_2_rst_ack_i             : std_logic := '0';
	signal io_node_2_out_stream_1_tvalid_i : std_logic;
	signal io_node_2_out_stream_1_tready_i : std_logic;
	signal io_node_2_out_stream_1_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_2_out_stream_1_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_2_out_stream_1_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_2_out_stream_2_tvalid_i : std_logic;
	signal io_node_2_out_stream_2_tready_i : std_logic;
	signal io_node_2_out_stream_2_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_2_out_stream_2_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_2_out_stream_2_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_2_out_stream_3_tvalid_i : std_logic;
	signal io_node_2_out_stream_3_tready_i : std_logic;
	signal io_node_2_out_stream_3_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_2_out_stream_3_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_2_out_stream_3_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_2_out_stream_4_tvalid_i : std_logic;
	signal io_node_2_out_stream_4_tready_i : std_logic;
	signal io_node_2_out_stream_4_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_2_out_stream_4_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_2_out_stream_4_tuser_i  : std_logic_vector(3 downto 0);

	signal io_node_2_in_stream_1_tvalid_i : std_logic;
	signal io_node_2_in_stream_1_tready_i : std_logic;
	signal io_node_2_in_stream_1_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_2_in_stream_1_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_2_in_stream_1_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_2_in_stream_2_tvalid_i : std_logic;
	signal io_node_2_in_stream_2_tready_i : std_logic;
	signal io_node_2_in_stream_2_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_2_in_stream_2_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_2_in_stream_2_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_2_in_stream_3_tvalid_i : std_logic;
	signal io_node_2_in_stream_3_tready_i : std_logic;
	signal io_node_2_in_stream_3_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_2_in_stream_3_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_2_in_stream_3_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_2_in_stream_4_tvalid_i : std_logic;
	signal io_node_2_in_stream_4_tready_i : std_logic;
	signal io_node_2_in_stream_4_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_2_in_stream_4_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_2_in_stream_4_tuser_i  : std_logic_vector(3 downto 0);

	-- IO node 3
	signal io_node_3_rst_i                 : std_logic := '1';
	signal io_node_3_rst_ack_i             : std_logic := '0';
	signal io_node_3_out_stream_1_tvalid_i : std_logic;
	signal io_node_3_out_stream_1_tready_i : std_logic;
	signal io_node_3_out_stream_1_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_3_out_stream_1_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_3_out_stream_1_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_3_out_stream_2_tvalid_i : std_logic;
	signal io_node_3_out_stream_2_tready_i : std_logic;
	signal io_node_3_out_stream_2_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_3_out_stream_2_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_3_out_stream_2_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_3_out_stream_3_tvalid_i : std_logic;
	signal io_node_3_out_stream_3_tready_i : std_logic;
	signal io_node_3_out_stream_3_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_3_out_stream_3_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_3_out_stream_3_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_3_out_stream_4_tvalid_i : std_logic;
	signal io_node_3_out_stream_4_tready_i : std_logic;
	signal io_node_3_out_stream_4_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_3_out_stream_4_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_3_out_stream_4_tuser_i  : std_logic_vector(3 downto 0);

	signal io_node_3_in_stream_1_tvalid_i : std_logic;
	signal io_node_3_in_stream_1_tready_i : std_logic;
	signal io_node_3_in_stream_1_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_3_in_stream_1_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_3_in_stream_1_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_3_in_stream_2_tvalid_i : std_logic;
	signal io_node_3_in_stream_2_tready_i : std_logic;
	signal io_node_3_in_stream_2_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_3_in_stream_2_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_3_in_stream_2_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_3_in_stream_3_tvalid_i : std_logic;
	signal io_node_3_in_stream_3_tready_i : std_logic;
	signal io_node_3_in_stream_3_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_3_in_stream_3_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_3_in_stream_3_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_3_in_stream_4_tvalid_i : std_logic;
	signal io_node_3_in_stream_4_tready_i : std_logic;
	signal io_node_3_in_stream_4_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_3_in_stream_4_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_3_in_stream_4_tuser_i  : std_logic_vector(3 downto 0);

	-- IO node 4
	signal io_node_4_rst_i                 : std_logic := '1';
	signal io_node_4_rst_ack_i             : std_logic := '0';
	signal io_node_4_out_stream_1_tvalid_i : std_logic;
	signal io_node_4_out_stream_1_tready_i : std_logic;
	signal io_node_4_out_stream_1_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_4_out_stream_1_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_4_out_stream_1_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_4_out_stream_2_tvalid_i : std_logic;
	signal io_node_4_out_stream_2_tready_i : std_logic;
	signal io_node_4_out_stream_2_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_4_out_stream_2_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_4_out_stream_2_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_4_out_stream_3_tvalid_i : std_logic;
	signal io_node_4_out_stream_3_tready_i : std_logic;
	signal io_node_4_out_stream_3_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_4_out_stream_3_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_4_out_stream_3_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_4_out_stream_4_tvalid_i : std_logic;
	signal io_node_4_out_stream_4_tready_i : std_logic;
	signal io_node_4_out_stream_4_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_4_out_stream_4_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_4_out_stream_4_tuser_i  : std_logic_vector(3 downto 0);

	signal io_node_4_in_stream_1_tvalid_i : std_logic;
	signal io_node_4_in_stream_1_tready_i : std_logic;
	signal io_node_4_in_stream_1_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_4_in_stream_1_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_4_in_stream_1_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_4_in_stream_2_tvalid_i : std_logic;
	signal io_node_4_in_stream_2_tready_i : std_logic;
	signal io_node_4_in_stream_2_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_4_in_stream_2_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_4_in_stream_2_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_4_in_stream_3_tvalid_i : std_logic;
	signal io_node_4_in_stream_3_tready_i : std_logic;
	signal io_node_4_in_stream_3_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_4_in_stream_3_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_4_in_stream_3_tuser_i  : std_logic_vector(3 downto 0);
	signal io_node_4_in_stream_4_tvalid_i : std_logic;
	signal io_node_4_in_stream_4_tready_i : std_logic;
	signal io_node_4_in_stream_4_tdata_i  : std_logic_vector(63 downto 0);
	signal io_node_4_in_stream_4_tstrb_i  : std_logic_vector(1 downto 0) := (others => '1');
	signal io_node_4_in_stream_4_tuser_i  : std_logic_vector(3 downto 0);

	signal uart_to_uart : std_logic;

	signal phase_shift_1 : boolean := true;
	signal phase_shift_2 : boolean := true;
	signal phase_shift_3 : boolean := true;

	-- AMBA AHB Slave signals ------------------------------------------------------
	signal S_AHB_HADDR  : std_logic_vector((c_control_bus_addr_width - 1) downto 0);
	signal S_AHB_HTRANS : std_logic_vector(1 downto 0);
	signal S_AHB_HWRITE : std_logic;
	signal S_AHB_HSIZE  : std_logic_vector(2 downto 0);
	signal S_AHB_HBURST : std_logic_vector(2 downto 0);
	signal S_AHB_HPROT  : std_logic_vector(3 downto 0);
	signal S_AHB_HWDATA : std_logic_vector(c_control_bus_data_width - 1 downto 0);
	signal S_AHB_HSEL   : std_logic := '1';
	signal S_AHB_HRDATA : std_logic_vector(c_control_bus_data_width - 1 downto 0);
	signal S_AHB_HREADY : std_logic;
	signal S_AHB_HRESP  : std_logic_vector(1 downto 0);

	-- AMBA AHB Master signals ------------------------------------------------------
	signal M_AHB_DMA_FROM_DYPLO_HADDR  : std_logic_vector((DMA_ADDR_WIDTH - 1) downto 0);
	signal M_AHB_DMA_FROM_DYPLO_HTRANS : std_logic_vector(1 downto 0);
	signal M_AHB_DMA_FROM_DYPLO_HWRITE : std_logic;
	signal M_AHB_DMA_FROM_DYPLO_HSIZE  : std_logic_vector(2 downto 0);
	signal M_AHB_DMA_FROM_DYPLO_HBURST : std_logic_vector(2 downto 0);
	signal M_AHB_DMA_FROM_DYPLO_HPROT  : std_logic_vector(3 downto 0);
	signal M_AHB_DMA_FROM_DYPLO_HWDATA : std_logic_vector(c_dma_bus_data_width - 1 downto 0);
	signal M_AHB_DMA_FROM_DYPLO_HSEL   : std_logic;
	signal M_AHB_DMA_FROM_DYPLO_HRDATA : std_logic_vector(c_dma_bus_data_width - 1 downto 0);
	signal M_AHB_DMA_FROM_DYPLO_HREADY : std_logic;
	signal M_AHB_DMA_FROM_DYPLO_HRESP  : std_logic_vector(1 downto 0) := (others => '0');

	signal dyplo_irq_i : std_logic;

	signal axi_wr_cmd_valid : std_logic;
	signal axi_wr_address   : std_logic_vector(31 downto 0);
	signal axi_wr_length    : std_logic_vector(7 downto 0);
	signal axi_wr_data      : std_logic_vector(32767 downto 0);
	signal axi_wr_cmd_done  : std_logic;
	signal axi_rd_cmd_valid : std_logic;
	signal axi_rd_address   : std_logic_vector(31 downto 0);
	signal axi_rd_length    : std_logic_vector(7 downto 0);
	signal axi_rd_cmd_done  : std_logic;
	signal axi_rd_data      : std_logic_vector(32767 downto 0);

begin
	fifo_drivers_i : for i in 0 to NO_FIFOS - 1 generate
		signal cmd        : cmd_record;
		signal cmd_accept : std_logic;
	begin

		-- Stimuli Reader
		u_stim_reader : entity tb_stim_reader
			generic map(
				STIM_FILE_NAME => LOG_FILE_DIR & "/fifo" & integer'image(i) & "_stimuli.txt")
			port map(
				cmd_out       => cmd,
				cmd_accept_in => cmd_accept,
				eof           => fifo_eof(i)
			);

		-- Fifo Driver
		u_fifo_driver : entity work.tb_fifo_driver
			generic map(
				fifo_id => i
			)
			port map(
				clk                 => tb_ACLK,
				rst_n               => tb_ARESETN,
				cmd_in              => cmd,
				cmd_accept_out      => cmd_accept,
				fifo_cmd_valid      => fifo_cmd_valid(i),
				fifo_cmd            => fifo_cmd(i),
				fifo_cmd_ready      => fifo_cmd_ready(i),
				waiting_on_sync     => fifo_waiting_on_sync(i),
				continue_after_sync => fifo_continue_after_sync(i)
			);

	end generate fifo_drivers_i;

	-- Stimuli Readers for Control node
	u_stim_reader_ctrl : entity tb_stim_reader
		generic map(
			STIM_FILE_NAME => LOG_FILE_DIR & "/control_node_stimuli.txt")
		port map(
			cmd_out       => stim_control_cmd_i,
			cmd_accept_in => stim_control_cmd_accept_i,
			eof           => stim_control_eof_i
		);

	-- Control Driver
	u_ctrl_driver : entity work.tb_control_driver
		port map(
			clk                 => tb_ACLK,
			rst_n               => tb_ARESETN,
			cmd_in              => stim_control_cmd_i,
			cmd_accept_out      => stim_control_cmd_accept_i,
			cmd_valid           => control_cmd_valid_i,
			cmd                 => control_cmd_i,
			cmd_ready           => control_cmd_ready_i,
			waiting_on_sync     => control_waiting_on_sync_i,
			continue_after_sync => control_continue_after_sync_i
		);

	-- Stimuli Readers for CPU node
	u_stim_reader_cpu : entity tb_stim_reader
		generic map(
			STIM_FILE_NAME => LOG_FILE_DIR & "/cpu_node_stimuli.txt")
		port map(
			cmd_out       => stim_cpu_cmd_i,
			cmd_accept_in => stim_cpu_cmd_accept_i,
			eof           => stim_cpu_eof_i
		);

	-- CPU Driver
	u_cpu_driver : entity work.tb_cpu_driver
		port map(
			clk                 => tb_ACLK,
			rst_n               => tb_ARESETN,
			cmd_in              => stim_cpu_cmd_i,
			cmd_accept_out      => stim_cpu_cmd_accept_i,
			cmd_valid           => cpu_cmd_valid_i,
			cmd                 => cpu_cmd_i,
			cmd_ready           => cpu_cmd_ready_i,
			waiting_on_sync     => cpu_waiting_on_sync_i,
			continue_after_sync => cpu_continue_after_sync_i
		);

	-- Stimuli Readers for HDL node
	u_stim_reader_hdl : entity tb_stim_reader
		generic map(
			STIM_FILE_NAME => LOG_FILE_DIR & "/hdl_node_stimuli.txt")
		port map(
			cmd_out       => stim_hdl_cmd_i,
			cmd_accept_in => stim_hdl_cmd_accept_i,
			eof           => stim_hdl_eof_i
		);

	-- HDL Driver
	u_hdl_driver : entity work.tb_hdl_driver
		port map(
			clk                 => tb_ACLK,
			rst_n               => tb_ARESETN,
			cmd_in              => stim_hdl_cmd_i,
			cmd_accept_out      => stim_hdl_cmd_accept_i,
			cmd_valid           => hdl_cmd_valid_i,
			cmd                 => hdl_cmd_i,
			cmd_ready           => hdl_cmd_ready_i,
			waiting_on_sync     => hdl_waiting_on_sync_i,
			continue_after_sync => hdl_continue_after_sync_i
		);

	u_scheduler : entity work.tb_scheduler
		port map(
			clk                         => tb_ACLK,
			rst_n                       => tb_ARESETN,
			-- control node
			control_cmd_valid           => control_cmd_valid_i,
			control_cmd                 => control_cmd_i,
			control_cmd_ready           => control_cmd_ready_i,
			control_waiting_on_sync     => control_waiting_on_sync_i,
			control_continue_after_sync => control_continue_after_sync_i,
			control_eof                 => stim_control_eof_i,
			-- cpu node
			cpu_cmd_valid               => cpu_cmd_valid_i,
			cpu_cmd                     => cpu_cmd_i,
			cpu_cmd_ready               => cpu_cmd_ready_i,
			cpu_waiting_on_sync         => cpu_waiting_on_sync_i,
			cpu_continue_after_sync     => cpu_continue_after_sync_i,
			cpu_eof                     => stim_cpu_eof_i,
			-- fifo's
			fifo_cmd_valid              => fifo_cmd_valid,
			fifo_cmd                    => fifo_cmd,
			fifo_cmd_ready              => fifo_cmd_ready,
			fifo_waiting_on_sync        => fifo_waiting_on_sync,
			fifo_continue_after_sync    => fifo_continue_after_sync,
			fifo_eof                    => fifo_eof,
			-- hdl nodes
			hdl_cmd_valid               => hdl_cmd_valid_i,
			hdl_cmd                     => hdl_cmd_i,
			hdl_cmd_ready               => hdl_cmd_ready_i,
			hdl_waiting_on_sync         => hdl_waiting_on_sync_i,
			hdl_continue_after_sync     => hdl_continue_after_sync_i,
			hdl_eof                     => stim_hdl_eof_i,
			-- dyplo irq
			dyplo_irq                   => dyplo_irq_i,
			-- interface to AXI VIP commander
			wr_cmd_valid                => axi_wr_cmd_valid,
			wr_cmd_address              => axi_wr_address,
			wr_cmd_length               => axi_wr_length,
			wr_cmd_data                 => axi_wr_data,
			wr_cmd_done                 => axi_wr_cmd_done,
			rd_cmd_valid                => axi_rd_cmd_valid,
			rd_cmd_address              => axi_rd_address,
			rd_cmd_length               => axi_rd_length,
			rd_cmd_done                 => axi_rd_cmd_done,
			rd_cmd_data                 => axi_rd_data
		);

	dyplo_irq_i <= '0' when CONTROL_INTERFACE_TYPE = "AMBA_AHB" else dyplo_irq;

	dyplo_top_i : entity dyplo.dyplo_top
		generic map(
			CONTROL_INTERFACE_TYPE    => CONTROL_INTERFACE_TYPE,
			CONTROL_DATA_WIDTH        => c_control_bus_data_width,
			CONTROL_ADDR_WIDTH        => c_control_bus_addr_width,
			CONTROL_AXI_ID_WIDTH      => c_control_bus_axi_id_width,
			-- Static bitstream identifier
			STATIC_BITSTREAM_ID       => STATIC_BITSTREAM_ID,
			-- Dyplo related configuration parameters
			CPU_TO_FPGA_STREAMS       => CPU_TO_FPGA_STREAMS,
			FPGA_TO_CPU_STREAMS       => FPGA_TO_CPU_STREAMS,
			IO_NODES                  => IO_NODES,
			PR_NODES                  => PR_NODES,
			FIXED_NODES               => FIXED_NODES,
			DMA_NODES                 => DMA_NODES,
			CONNECT_LANES             => CONNECT_LANES,
			MONITOR                   => MONITOR,
			-- IO node related configuration parameters
			-- IO node 1
			IO_NODE_1_INSTREAMS       => IO_NODE_1_INSTREAMS,
			IO_NODE_1_OUTSTREAMS      => IO_NODE_1_OUTSTREAMS,
			-- IO node 2
			IO_NODE_2_INSTREAMS       => IO_NODE_2_INSTREAMS,
			IO_NODE_2_OUTSTREAMS      => IO_NODE_2_OUTSTREAMS,
			-- IO node 3
			IO_NODE_3_INSTREAMS       => IO_NODE_3_INSTREAMS,
			IO_NODE_3_OUTSTREAMS      => IO_NODE_3_OUTSTREAMS,
			-- IO node 4
			IO_NODE_4_INSTREAMS       => IO_NODE_4_INSTREAMS,
			IO_NODE_4_OUTSTREAMS      => IO_NODE_4_OUTSTREAMS,
			-- DMA related generics
			DMA_INTERFACE_TYPE        => DMA_INTERFACE_TYPE,
			DMA_ADDR_WIDTH            => DMA_ADDR_WIDTH,
			DMA_DATA_WIDTH            => c_dma_bus_data_width
		)
		port map(
			-- ----------------------------------------------------------------------------
			-- I/O INTERFACES                                                           --
			-- ----------------------------------------------------------------------------
			io_clk                        => io_clk_i,
			-- IO node 1
			io_node_1_rst                 => io_node_1_rst_i,
			io_node_1_out_stream_1_tvalid => io_node_1_out_stream_1_tvalid_i,
			io_node_1_out_stream_1_tready => io_node_1_out_stream_1_tready_i,
			io_node_1_out_stream_1_tdata  => io_node_1_out_stream_1_tdata_i,
			io_node_1_out_stream_1_tstrb  => io_node_1_out_stream_1_tstrb_i,
			io_node_1_out_stream_1_tuser  => io_node_1_out_stream_1_tuser_i,
			io_node_1_out_stream_2_tvalid => io_node_1_out_stream_2_tvalid_i,
			io_node_1_out_stream_2_tready => io_node_1_out_stream_2_tready_i,
			io_node_1_out_stream_2_tdata  => io_node_1_out_stream_2_tdata_i,
			io_node_1_out_stream_2_tstrb  => io_node_1_out_stream_2_tstrb_i,
			io_node_1_out_stream_2_tuser  => io_node_1_out_stream_2_tuser_i,
			io_node_1_out_stream_3_tvalid => io_node_1_out_stream_3_tvalid_i,
			io_node_1_out_stream_3_tready => io_node_1_out_stream_3_tready_i,
			io_node_1_out_stream_3_tdata  => io_node_1_out_stream_3_tdata_i,
			io_node_1_out_stream_3_tstrb  => io_node_1_out_stream_3_tstrb_i,
			io_node_1_out_stream_3_tuser  => io_node_1_out_stream_3_tuser_i,
			io_node_1_out_stream_4_tvalid => io_node_1_out_stream_4_tvalid_i,
			io_node_1_out_stream_4_tready => io_node_1_out_stream_4_tready_i,
			io_node_1_out_stream_4_tdata  => io_node_1_out_stream_4_tdata_i,
			io_node_1_out_stream_4_tstrb  => io_node_1_out_stream_4_tstrb_i,
			io_node_1_out_stream_4_tuser  => io_node_1_out_stream_4_tuser_i,
			io_node_1_in_stream_1_tvalid  => io_node_1_in_stream_1_tvalid_i,
			io_node_1_in_stream_1_tready  => io_node_1_in_stream_1_tready_i,
			io_node_1_in_stream_1_tdata   => io_node_1_in_stream_1_tdata_i,
			io_node_1_in_stream_1_tstrb   => io_node_1_in_stream_1_tstrb_i,
			io_node_1_in_stream_1_tuser   => io_node_1_in_stream_1_tuser_i,
			io_node_1_in_stream_2_tvalid  => io_node_1_in_stream_2_tvalid_i,
			io_node_1_in_stream_2_tready  => io_node_1_in_stream_2_tready_i,
			io_node_1_in_stream_2_tdata   => io_node_1_in_stream_2_tdata_i,
			io_node_1_in_stream_2_tstrb   => io_node_1_in_stream_2_tstrb_i,
			io_node_1_in_stream_2_tuser   => io_node_1_in_stream_2_tuser_i,
			io_node_1_in_stream_3_tvalid  => io_node_1_in_stream_3_tvalid_i,
			io_node_1_in_stream_3_tready  => io_node_1_in_stream_3_tready_i,
			io_node_1_in_stream_3_tdata   => io_node_1_in_stream_3_tdata_i,
			io_node_1_in_stream_3_tstrb   => io_node_1_in_stream_3_tstrb_i,
			io_node_1_in_stream_3_tuser   => io_node_1_in_stream_3_tuser_i,
			io_node_1_in_stream_4_tvalid  => io_node_1_in_stream_4_tvalid_i,
			io_node_1_in_stream_4_tready  => io_node_1_in_stream_4_tready_i,
			io_node_1_in_stream_4_tdata   => io_node_1_in_stream_4_tdata_i,
			io_node_1_in_stream_4_tstrb   => io_node_1_in_stream_4_tstrb_i,
			io_node_1_in_stream_4_tuser   => io_node_1_in_stream_4_tuser_i,
			-- IO node 2
			io_node_2_rst                 => io_node_2_rst_i,
			io_node_2_out_stream_1_tvalid => io_node_2_out_stream_1_tvalid_i,
			io_node_2_out_stream_1_tready => io_node_2_out_stream_1_tready_i,
			io_node_2_out_stream_1_tdata  => io_node_2_out_stream_1_tdata_i,
			io_node_2_out_stream_1_tstrb  => io_node_2_out_stream_1_tstrb_i,
			io_node_2_out_stream_1_tuser  => io_node_2_out_stream_1_tuser_i,
			io_node_2_out_stream_2_tvalid => io_node_2_out_stream_2_tvalid_i,
			io_node_2_out_stream_2_tready => io_node_2_out_stream_2_tready_i,
			io_node_2_out_stream_2_tdata  => io_node_2_out_stream_2_tdata_i,
			io_node_2_out_stream_2_tstrb  => io_node_2_out_stream_2_tstrb_i,
			io_node_2_out_stream_2_tuser  => io_node_2_out_stream_2_tuser_i,
			io_node_2_out_stream_3_tvalid => io_node_2_out_stream_3_tvalid_i,
			io_node_2_out_stream_3_tready => io_node_2_out_stream_3_tready_i,
			io_node_2_out_stream_3_tdata  => io_node_2_out_stream_3_tdata_i,
			io_node_2_out_stream_3_tstrb  => io_node_2_out_stream_3_tstrb_i,
			io_node_2_out_stream_3_tuser  => io_node_2_out_stream_3_tuser_i,
			io_node_2_out_stream_4_tvalid => io_node_2_out_stream_4_tvalid_i,
			io_node_2_out_stream_4_tready => io_node_2_out_stream_4_tready_i,
			io_node_2_out_stream_4_tdata  => io_node_2_out_stream_4_tdata_i,
			io_node_2_out_stream_4_tstrb  => io_node_2_out_stream_4_tstrb_i,
			io_node_2_out_stream_4_tuser  => io_node_2_out_stream_4_tuser_i,
			io_node_2_in_stream_1_tvalid  => io_node_2_in_stream_1_tvalid_i,
			io_node_2_in_stream_1_tready  => io_node_2_in_stream_1_tready_i,
			io_node_2_in_stream_1_tdata   => io_node_2_in_stream_1_tdata_i,
			io_node_2_in_stream_1_tstrb   => io_node_2_in_stream_1_tstrb_i,
			io_node_2_in_stream_1_tuser   => io_node_2_in_stream_1_tuser_i,
			io_node_2_in_stream_2_tvalid  => io_node_2_in_stream_2_tvalid_i,
			io_node_2_in_stream_2_tready  => io_node_2_in_stream_2_tready_i,
			io_node_2_in_stream_2_tdata   => io_node_2_in_stream_2_tdata_i,
			io_node_2_in_stream_2_tstrb   => io_node_2_in_stream_2_tstrb_i,
			io_node_2_in_stream_2_tuser   => io_node_2_in_stream_2_tuser_i,
			io_node_2_in_stream_3_tvalid  => io_node_2_in_stream_3_tvalid_i,
			io_node_2_in_stream_3_tready  => io_node_2_in_stream_3_tready_i,
			io_node_2_in_stream_3_tdata   => io_node_2_in_stream_3_tdata_i,
			io_node_2_in_stream_3_tstrb   => io_node_2_in_stream_3_tstrb_i,
			io_node_2_in_stream_3_tuser   => io_node_2_in_stream_3_tuser_i,
			io_node_2_in_stream_4_tvalid  => io_node_2_in_stream_4_tvalid_i,
			io_node_2_in_stream_4_tready  => io_node_2_in_stream_4_tready_i,
			io_node_2_in_stream_4_tdata   => io_node_2_in_stream_4_tdata_i,
			io_node_2_in_stream_4_tstrb   => io_node_2_in_stream_4_tstrb_i,
			io_node_2_in_stream_4_tuser   => io_node_2_in_stream_4_tuser_i,
			-- IO node 3
			io_node_3_rst                 => io_node_3_rst_i,
			io_node_3_out_stream_1_tvalid => io_node_3_out_stream_1_tvalid_i,
			io_node_3_out_stream_1_tready => io_node_3_out_stream_1_tready_i,
			io_node_3_out_stream_1_tdata  => io_node_3_out_stream_1_tdata_i,
			io_node_3_out_stream_1_tstrb  => io_node_3_out_stream_1_tstrb_i,
			io_node_3_out_stream_1_tuser  => io_node_3_out_stream_1_tuser_i,
			io_node_3_out_stream_2_tvalid => io_node_3_out_stream_2_tvalid_i,
			io_node_3_out_stream_2_tready => io_node_3_out_stream_2_tready_i,
			io_node_3_out_stream_2_tdata  => io_node_3_out_stream_2_tdata_i,
			io_node_3_out_stream_2_tstrb  => io_node_3_out_stream_2_tstrb_i,
			io_node_3_out_stream_2_tuser  => io_node_3_out_stream_2_tuser_i,
			io_node_3_out_stream_3_tvalid => io_node_3_out_stream_3_tvalid_i,
			io_node_3_out_stream_3_tready => io_node_3_out_stream_3_tready_i,
			io_node_3_out_stream_3_tdata  => io_node_3_out_stream_3_tdata_i,
			io_node_3_out_stream_3_tstrb  => io_node_3_out_stream_3_tstrb_i,
			io_node_3_out_stream_3_tuser  => io_node_3_out_stream_3_tuser_i,
			io_node_3_out_stream_4_tvalid => io_node_3_out_stream_4_tvalid_i,
			io_node_3_out_stream_4_tready => io_node_3_out_stream_4_tready_i,
			io_node_3_out_stream_4_tdata  => io_node_3_out_stream_4_tdata_i,
			io_node_3_out_stream_4_tstrb  => io_node_3_out_stream_4_tstrb_i,
			io_node_3_out_stream_4_tuser  => io_node_3_out_stream_4_tuser_i,
			io_node_3_in_stream_1_tvalid  => io_node_3_in_stream_1_tvalid_i,
			io_node_3_in_stream_1_tready  => io_node_3_in_stream_1_tready_i,
			io_node_3_in_stream_1_tdata   => io_node_3_in_stream_1_tdata_i,
			io_node_3_in_stream_1_tstrb   => io_node_3_in_stream_1_tstrb_i,
			io_node_3_in_stream_1_tuser   => io_node_3_in_stream_1_tuser_i,
			io_node_3_in_stream_2_tvalid  => io_node_3_in_stream_2_tvalid_i,
			io_node_3_in_stream_2_tready  => io_node_3_in_stream_2_tready_i,
			io_node_3_in_stream_2_tdata   => io_node_3_in_stream_2_tdata_i,
			io_node_3_in_stream_2_tstrb   => io_node_3_in_stream_2_tstrb_i,
			io_node_3_in_stream_2_tuser   => io_node_3_in_stream_2_tuser_i,
			io_node_3_in_stream_3_tvalid  => io_node_3_in_stream_3_tvalid_i,
			io_node_3_in_stream_3_tready  => io_node_3_in_stream_3_tready_i,
			io_node_3_in_stream_3_tdata   => io_node_3_in_stream_3_tdata_i,
			io_node_3_in_stream_3_tstrb   => io_node_3_in_stream_3_tstrb_i,
			io_node_3_in_stream_3_tuser   => io_node_3_in_stream_3_tuser_i,
			io_node_3_in_stream_4_tvalid  => io_node_3_in_stream_4_tvalid_i,
			io_node_3_in_stream_4_tready  => io_node_3_in_stream_4_tready_i,
			io_node_3_in_stream_4_tdata   => io_node_3_in_stream_4_tdata_i,
			io_node_3_in_stream_4_tstrb   => io_node_3_in_stream_4_tstrb_i,
			io_node_3_in_stream_4_tuser   => io_node_3_in_stream_4_tuser_i,
			-- IO node 4
			io_node_4_rst                 => io_node_4_rst_i,
			io_node_4_out_stream_1_tvalid => io_node_4_out_stream_1_tvalid_i,
			io_node_4_out_stream_1_tready => io_node_4_out_stream_1_tready_i,
			io_node_4_out_stream_1_tdata  => io_node_4_out_stream_1_tdata_i,
			io_node_4_out_stream_1_tstrb  => io_node_4_out_stream_1_tstrb_i,
			io_node_4_out_stream_1_tuser  => io_node_4_out_stream_1_tuser_i,
			io_node_4_out_stream_2_tvalid => io_node_4_out_stream_2_tvalid_i,
			io_node_4_out_stream_2_tready => io_node_4_out_stream_2_tready_i,
			io_node_4_out_stream_2_tdata  => io_node_4_out_stream_2_tdata_i,
			io_node_4_out_stream_2_tstrb  => io_node_4_out_stream_2_tstrb_i,
			io_node_4_out_stream_2_tuser  => io_node_4_out_stream_2_tuser_i,
			io_node_4_out_stream_3_tvalid => io_node_4_out_stream_3_tvalid_i,
			io_node_4_out_stream_3_tready => io_node_4_out_stream_3_tready_i,
			io_node_4_out_stream_3_tdata  => io_node_4_out_stream_3_tdata_i,
			io_node_4_out_stream_3_tstrb  => io_node_4_out_stream_3_tstrb_i,
			io_node_4_out_stream_3_tuser  => io_node_4_out_stream_3_tuser_i,
			io_node_4_out_stream_4_tvalid => io_node_4_out_stream_4_tvalid_i,
			io_node_4_out_stream_4_tready => io_node_4_out_stream_4_tready_i,
			io_node_4_out_stream_4_tdata  => io_node_4_out_stream_4_tdata_i,
			io_node_4_out_stream_4_tstrb  => io_node_4_out_stream_4_tstrb_i,
			io_node_4_out_stream_4_tuser  => io_node_4_out_stream_4_tuser_i,
			io_node_4_in_stream_1_tvalid  => io_node_4_in_stream_1_tvalid_i,
			io_node_4_in_stream_1_tready  => io_node_4_in_stream_1_tready_i,
			io_node_4_in_stream_1_tdata   => io_node_4_in_stream_1_tdata_i,
			io_node_4_in_stream_1_tstrb   => io_node_4_in_stream_1_tstrb_i,
			io_node_4_in_stream_1_tuser   => io_node_4_in_stream_1_tuser_i,
			io_node_4_in_stream_2_tvalid  => io_node_4_in_stream_2_tvalid_i,
			io_node_4_in_stream_2_tready  => io_node_4_in_stream_2_tready_i,
			io_node_4_in_stream_2_tdata   => io_node_4_in_stream_2_tdata_i,
			io_node_4_in_stream_2_tstrb   => io_node_4_in_stream_2_tstrb_i,
			io_node_4_in_stream_2_tuser   => io_node_4_in_stream_2_tuser_i,
			io_node_4_in_stream_3_tvalid  => io_node_4_in_stream_3_tvalid_i,
			io_node_4_in_stream_3_tready  => io_node_4_in_stream_3_tready_i,
			io_node_4_in_stream_3_tdata   => io_node_4_in_stream_3_tdata_i,
			io_node_4_in_stream_3_tstrb   => io_node_4_in_stream_3_tstrb_i,
			io_node_4_in_stream_3_tuser   => io_node_4_in_stream_3_tuser_i,
			io_node_4_in_stream_4_tvalid  => io_node_4_in_stream_4_tvalid_i,
			io_node_4_in_stream_4_tready  => io_node_4_in_stream_4_tready_i,
			io_node_4_in_stream_4_tdata   => io_node_4_in_stream_4_tdata_i,
			io_node_4_in_stream_4_tstrb   => io_node_4_in_stream_4_tstrb_i,
			io_node_4_in_stream_4_tuser   => io_node_4_in_stream_4_tuser_i,
			S_AXI_ACLK                    => tb_ACLK,
			S_AXI_ARESETN                 => tb_ARESETN,
			S_AXI_AWADDR                  => M_AXI_1_AWADDR,
			S_AXI_AWVALID                 => M_AXI_1_AWVALID,
			S_AXI_WDATA                   => M_AXI_1_WDATA,
			S_AXI_WSTRB                   => M_AXI_1_WSTRB,
			S_AXI_WVALID                  => M_AXI_1_WVALID,
			S_AXI_BREADY                  => M_AXI_1_BREADY,
			S_AXI_ARADDR                  => M_AXI_1_ARADDR,
			S_AXI_ARVALID                 => M_AXI_1_ARVALID,
			S_AXI_RREADY                  => M_AXI_1_RREADY,
			S_AXI_ARREADY                 => M_AXI_1_ARREADY,
			S_AXI_RDATA                   => M_AXI_1_RDATA,
			S_AXI_RRESP                   => M_AXI_1_RRESP,
			S_AXI_RVALID                  => M_AXI_1_RVALID,
			S_AXI_WREADY                  => M_AXI_1_WREADY,
			S_AXI_BRESP                   => M_AXI_1_BRESP,
			S_AXI_BVALID                  => M_AXI_1_BVALID,
			S_AXI_AWREADY                 => M_AXI_1_AWREADY,
			S_AXI_AWID                    => M_AXI_1_AWID,
			S_AXI_AWLEN                   => M_AXI_1_AWLEN,
			S_AXI_AWSIZE                  => M_AXI_1_AWSIZE,
			S_AXI_AWBURST                 => M_AXI_1_AWBURST,
			S_AXI_AWLOCK                  => M_AXI_1_AWLOCK,
			S_AXI_AWCACHE                 => M_AXI_1_AWCACHE,
			S_AXI_AWPROT                  => M_AXI_1_AWPROT,
			S_AXI_WLAST                   => M_AXI_1_WLAST,
			S_AXI_BID                     => M_AXI_1_BID,
			S_AXI_ARID                    => M_AXI_1_ARID,
			S_AXI_ARLEN                   => M_AXI_1_ARLEN,
			S_AXI_ARSIZE                  => M_AXI_1_ARSIZE,
			S_AXI_ARBURST                 => M_AXI_1_ARBURST,
			S_AXI_ARLOCK                  => M_AXI_1_ARLOCK,
			S_AXI_ARCACHE                 => M_AXI_1_ARCACHE,
			S_AXI_ARPROT                  => M_AXI_1_ARPROT,
			S_AXI_RID                     => M_AXI_1_RID,
			S_AXI_RLAST                   => M_AXI_1_RLAST,
			dyplo_irq                     => dyplo_irq,
			icap_clk 					  => config_clk,
			dna_port_clk				  => config_clk,
			-- ----------- AXI-4 Full Interface -----------
			-- AXI Global System Signals
			M_AXI_0_ACLK                  => tb_ACLK_dma,
			M_AXI_0_ARESETN               => tb_ARESETN_dma,
			-- AXI Write Address Channel Signals
			M_AXI_0_AWID                  => M_AXI_DMA_FROM_DYPLO_AWID,
			M_AXI_0_AWADDR                => M_AXI_DMA_FROM_DYPLO_AWADDR(DMA_ADDR_WIDTH - 1 downto 0),
			M_AXI_0_AWLEN                 => M_AXI_DMA_FROM_DYPLO_AWLEN,
			M_AXI_0_AWSIZE                => M_AXI_DMA_FROM_DYPLO_AWSIZE,
			M_AXI_0_AWBURST               => M_AXI_DMA_FROM_DYPLO_AWBURST,
			M_AXI_0_AWLOCK                => M_AXI_DMA_FROM_DYPLO_AWLOCK(0),
			M_AXI_0_AWCACHE               => M_AXI_DMA_FROM_DYPLO_AWCACHE,
			M_AXI_0_AWPROT                => M_AXI_DMA_FROM_DYPLO_AWPROT,
			M_AXI_0_AWVALID               => M_AXI_DMA_FROM_DYPLO_AWVALID,
			M_AXI_0_AWREADY               => M_AXI_DMA_FROM_DYPLO_AWREADY,
			-- AXI Write Channel Signals
			M_AXI_0_WDATA                 => M_AXI_DMA_FROM_DYPLO_WDATA,
			M_AXI_0_WSTRB                 => M_AXI_DMA_FROM_DYPLO_WSTRB,
			M_AXI_0_WLAST                 => M_AXI_DMA_FROM_DYPLO_WLAST,
			M_AXI_0_WVALID                => M_AXI_DMA_FROM_DYPLO_WVALID,
			M_AXI_0_WREADY                => M_AXI_DMA_FROM_DYPLO_WREADY,
			-- AXI Write Response Channel Signals
			M_AXI_0_BID                   => M_AXI_DMA_FROM_DYPLO_BID,
			M_AXI_0_BRESP                 => M_AXI_DMA_FROM_DYPLO_BRESP,
			M_AXI_0_BVALID                => M_AXI_DMA_FROM_DYPLO_BVALID,
			M_AXI_0_BREADY                => M_AXI_DMA_FROM_DYPLO_BREADY,
			-- AXI Read Address Channel Signals
			M_AXI_0_ARID                  => M_AXI_DMA_FROM_DYPLO_ARID,
			M_AXI_0_ARADDR                => M_AXI_DMA_FROM_DYPLO_ARADDR(DMA_ADDR_WIDTH - 1 downto 0),
			M_AXI_0_ARLEN                 => M_AXI_DMA_FROM_DYPLO_ARLEN,
			M_AXI_0_ARSIZE                => M_AXI_DMA_FROM_DYPLO_ARSIZE,
			M_AXI_0_ARBURST               => M_AXI_DMA_FROM_DYPLO_ARBURST,
			M_AXI_0_ARLOCK                => M_AXI_DMA_FROM_DYPLO_ARLOCK(0),
			M_AXI_0_ARCACHE               => M_AXI_DMA_FROM_DYPLO_ARCACHE,
			M_AXI_0_ARPROT                => M_AXI_DMA_FROM_DYPLO_ARPROT,
			M_AXI_0_ARVALID               => M_AXI_DMA_FROM_DYPLO_ARVALID,
			M_AXI_0_ARREADY               => M_AXI_DMA_FROM_DYPLO_ARREADY,
			-- AXI Read Data Channel Signals
			M_AXI_0_RID                   => M_AXI_DMA_FROM_DYPLO_RID,
			M_AXI_0_RDATA                 => M_AXI_DMA_FROM_DYPLO_RDATA,
			M_AXI_0_RRESP                 => M_AXI_DMA_FROM_DYPLO_RRESP,
			M_AXI_0_RLAST                 => M_AXI_DMA_FROM_DYPLO_RLAST,
			M_AXI_0_RVALID                => M_AXI_DMA_FROM_DYPLO_RVALID,
			M_AXI_0_RREADY                => M_AXI_DMA_FROM_DYPLO_RREADY,
			M_AXI_1_ACLK                  => '0',
			M_AXI_1_ARESETN               => '0',
			M_AXI_1_AWREADY               => '0',
			M_AXI_1_WREADY                => '0',
			M_AXI_1_BID                   => (others => '0'),
			M_AXI_1_BVALID                => '0',
			M_AXI_1_BRESP                 => (others => '0'),
			M_AXI_1_ARREADY               => '0',
			M_AXI_1_RID                   => (others => '0'),
			M_AXI_1_RVALID                => '0',
			M_AXI_1_RDATA                 => (others => '0'),
			M_AXI_1_RRESP                 => (others => '0'),
			M_AXI_1_RLAST                 => '0',
			M_AXI_2_ACLK                  => '0',
			M_AXI_2_ARESETN               => '0',
			M_AXI_2_AWREADY               => '0',
			M_AXI_2_WREADY                => '0',
			M_AXI_2_BID                   => (others => '0'),
			M_AXI_2_BVALID                => '0',
			M_AXI_2_BRESP                 => (others => '0'),
			M_AXI_2_ARREADY               => '0',
			M_AXI_2_RID                   => (others => '0'),
			M_AXI_2_RVALID                => '0',
			M_AXI_2_RDATA                 => (others => '0'),
			M_AXI_2_RRESP                 => (others => '0'),
			M_AXI_2_RLAST                 => '0',
			M_AXI_3_ACLK                  => '0',
			M_AXI_3_ARESETN               => '0',
			M_AXI_3_AWREADY               => '0',
			M_AXI_3_WREADY                => '0',
			M_AXI_3_BID                   => (others => '0'),
			M_AXI_3_BVALID                => '0',
			M_AXI_3_BRESP                 => (others => '0'),
			M_AXI_3_ARREADY               => '0',
			M_AXI_3_RID                   => (others => '0'),
			M_AXI_3_RVALID                => '0',
			M_AXI_3_RDATA                 => (others => '0'),
			M_AXI_3_RRESP                 => (others => '0'),
			M_AXI_3_RLAST                 => '0',
			-- Dyplo_Clock signal
			dyplo_clock                   => tb_dyplo_CLK,
			-- AMBA AHB Slave signals ------------------------------------------------------
			S_AHB_HCLK                    => tb_ACLK,
			S_AHB_HRESETN                 => tb_ARESETN,
			S_AHB_HADDR                   => S_AHB_HADDR,
			S_AHB_HTRANS                  => S_AHB_HTRANS,
			S_AHB_HWRITE                  => S_AHB_HWRITE,
			S_AHB_HSIZE                   => S_AHB_HSIZE,
			S_AHB_HBURST                  => S_AHB_HBURST,
			S_AHB_HPROT                   => S_AHB_HPROT,
			S_AHB_HWDATA                  => S_AHB_HWDATA,
			S_AHB_HSEL                    => S_AHB_HSEL,
			S_AHB_HREADYIN                => S_AHB_HREADY,
			S_AHB_HRDATA                  => S_AHB_HRDATA,
			S_AHB_HREADYOUT               => S_AHB_HREADY,
			S_AHB_HRESP                   => S_AHB_HRESP,
			M_AHB_0_HCLK                  => tb_ACLK_dma,
			M_AHB_0_HRESETN               => tb_ARESETN_dma,
			M_AHB_0_HRDATA                => M_AHB_DMA_FROM_DYPLO_HRDATA,
			M_AHB_0_HREADY                => M_AHB_DMA_FROM_DYPLO_HREADY,
			M_AHB_0_HRESP                 => M_AHB_DMA_FROM_DYPLO_HRESP,
			M_AHB_0_HADDR                 => M_AHB_DMA_FROM_DYPLO_HADDR,
			M_AHB_0_HTRANS                => M_AHB_DMA_FROM_DYPLO_HTRANS,
			M_AHB_0_HWRITE                => M_AHB_DMA_FROM_DYPLO_HWRITE,
			M_AHB_0_HSIZE                 => M_AHB_DMA_FROM_DYPLO_HSIZE,
			M_AHB_0_HBURST                => M_AHB_DMA_FROM_DYPLO_HBURST,
			M_AHB_0_HPROT                 => M_AHB_DMA_FROM_DYPLO_HPROT,
			M_AHB_0_HWDATA                => M_AHB_DMA_FROM_DYPLO_HWDATA,
			M_AHB_0_HSEL                  => M_AHB_DMA_FROM_DYPLO_HSEL,
			M_AHB_1_HCLK                  => '0',
			M_AHB_1_HRESETN               => '0',
			M_AHB_1_HRDATA                => (others => '0'),
			M_AHB_1_HREADY                => '0',
			M_AHB_1_HRESP                 => (others => '0'),
			M_AHB_2_HCLK                  => '0',
			M_AHB_2_HRESETN               => '0',
			M_AHB_2_HRDATA                => (others => '0'),
			M_AHB_2_HREADY                => '0',
			M_AHB_2_HRESP                 => (others => '0'),
			M_AHB_3_HCLK                  => '0',
			M_AHB_3_HRESETN               => '0',
			M_AHB_3_HRDATA                => (others => '0'),
			M_AHB_3_HREADY                => '0',
			M_AHB_3_HRESP                 => (others => '0')
		);

	tb_ACLK <= not tb_ACLK after bfm.bfm_axi4_full_master.clk_per / 2 when GSR = '0';

	io_clk_i     <= not io_clk_i after bfm.bfm_axi4_full_master.clk_per / 2 when GSR = '0' and phase_shift_1 = false;
	tb_ACLK_dma  <= not tb_ACLK_dma after bfm.bfm_axi4_full_master.clk_per / 2 when GSR = '0' and phase_shift_2 = false;
	tb_dyplo_CLK <= not tb_dyplo_CLK after bfm.bfm_axi4_full_master.clk_per / 2 when GSR = '0' and phase_shift_3 = false;
	config_clk   <= not config_clk after 20 ns / 2 when GSR = '0';

	process
	begin
		wait for 100 ns;
		GSR <= '0';
		wait;
	end process;

	process
	begin
		phase_shift_1 <= true;
		wait until GSR = '0';
		wait for 1 ns;
		phase_shift_1 <= false;
		wait;
	end process;

	process
	begin
		phase_shift_2 <= true;
		wait until GSR = '0';
		wait for 2 ns;
		phase_shift_2 <= false;
		wait;
	end process;

	process
	begin
		phase_shift_3 <= true;
		wait until GSR = '0';
		wait for 3 ns;
		phase_shift_3 <= false;
		wait;
	end process;

	process
	begin
		tb_ARESETN <= '0';
		wait for 970 ns;
		tb_ARESETN <= '1';
		wait;
	end process;

	process
	begin
		tb_ARESETN_dma <= '0';
		wait for 512 ns;
		tb_ARESETN_dma <= '1';
		wait;
	end process;

	-- This connections are for test 010_cpu_io_cpu
	io_node_1_in_stream_1_tvalid_i  <= io_node_1_out_stream_1_tvalid_i;
	io_node_1_out_stream_1_tready_i <= io_node_1_in_stream_1_tready_i;
	io_node_1_in_stream_1_tdata_i   <= io_node_1_out_stream_1_tdata_i;
	io_node_1_in_stream_1_tstrb_i   <= io_node_1_out_stream_1_tstrb_i;
	io_node_1_in_stream_1_tuser_i   <= io_node_1_out_stream_1_tuser_i;
	io_node_1_rst_ack_i             <= io_node_1_rst_i;
	-- This connections are for test 011_cpu_io_io_cpu
	io_node_3_in_stream_1_tvalid_i  <= io_node_2_out_stream_1_tvalid_i;
	io_node_2_out_stream_1_tready_i <= io_node_3_in_stream_1_tready_i;
	io_node_3_in_stream_1_tdata_i   <= io_node_2_out_stream_1_tdata_i;
	io_node_3_in_stream_1_tstrb_i   <= io_node_2_out_stream_1_tstrb_i;
	io_node_3_in_stream_1_tuser_i   <= io_node_2_out_stream_1_tuser_i;
	io_node_2_rst_ack_i             <= io_node_2_rst_i;
	io_node_3_rst_ack_i             <= io_node_3_rst_i;

	-- This connections to dyplo_uart and back are for test 014_cpu_io_uart_io_cpu
	uart_i : entity work.dyplo_uart
		port map(
			-- Miscellaneous
			clk        => io_clk_i,
			rst        => io_node_4_rst_i,
			rst_ack    => io_node_4_rst_ack_i,
			en         => '1',
			-- Processor interface bus
			-- Streaming data to serial port TxD channel
			txd_tvalid => io_node_4_out_stream_1_tvalid_i,
			txd_tready => io_node_4_out_stream_1_tready_i,
			txd_tdata  => io_node_4_out_stream_1_tdata_i(31 downto 0),
			txd        => uart_to_uart,
			-- Streaming data from FPGA fabric to Dyplo block
			rxd_tvalid => io_node_4_in_stream_1_tvalid_i,
			rxd_tready => io_node_4_in_stream_1_tready_i,
			rxd_tdata  => io_node_4_in_stream_1_tdata_i(31 downto 0),
			rxd        => uart_to_uart
		);

	io_node_4_in_stream_1_tuser_i <= (others => '0');
	io_node_4_in_stream_1_tstrb_i <= "01";

	MASTER_GP : axi_vip_master
		port map(
			aclk           => tb_ACLK,
			aresetn        => tb_ARESETN,
			m_axi_awaddr   => M_AXI_GP0_AWADDR,
			m_axi_awlen    => M_AXI_GP0_AWLEN,
			m_axi_awburst  => M_AXI_GP0_AWBURST,
			m_axi_awlock   => M_AXI_GP0_AWLOCK,
			m_axi_awcache  => M_AXI_GP0_AWCACHE,
			m_axi_awprot   => M_AXI_GP0_AWPROT,
			m_axi_awregion => M_AXI_GP0_AWREGION,
			m_axi_awqos    => M_AXI_GP0_AWQOS,
			m_axi_awvalid  => M_AXI_GP0_AWVALID,
			m_axi_awready  => M_AXI_GP0_AWREADY,
			m_axi_wdata    => M_AXI_GP0_WDATA,
			m_axi_wstrb    => M_AXI_GP0_WSTRB,
			m_axi_wlast    => M_AXI_GP0_WLAST,
			m_axi_wvalid   => M_AXI_GP0_WVALID,
			m_axi_wready   => M_AXI_GP0_WREADY,
			m_axi_bresp    => M_AXI_GP0_BRESP,
			m_axi_bvalid   => M_AXI_GP0_BVALID,
			m_axi_bready   => M_AXI_GP0_BREADY,
			m_axi_araddr   => M_AXI_GP0_ARADDR,
			m_axi_arlen    => M_AXI_GP0_ARLEN,
			m_axi_arburst  => M_AXI_GP0_ARBURST,
			m_axi_arlock   => M_AXI_GP0_ARLOCK,
			m_axi_arcache  => M_AXI_GP0_ARCACHE,
			m_axi_arprot   => M_AXI_GP0_ARPROT,
			m_axi_arregion => M_AXI_GP0_ARREGION,
			m_axi_arqos    => M_AXI_GP0_ARQOS,
			m_axi_arvalid  => M_AXI_GP0_ARVALID,
			m_axi_arready  => M_AXI_GP0_ARREADY,
			m_axi_rdata    => M_AXI_GP0_RDATA,
			m_axi_rresp    => M_AXI_GP0_RRESP,
			m_axi_rlast    => M_AXI_GP0_RLAST,
			m_axi_rvalid   => M_AXI_GP0_RVALID,
			m_axi_rready   => M_AXI_GP0_RREADY
		);

	MASTER_GP_commander : axi_vip_master_commander
		port map(
			i_wr_cmd_valid   => axi_wr_cmd_valid,
			i_wr_cmd_address => axi_wr_address,
			i_wr_cmd_length  => axi_wr_length,
			i_wr_cmd_data    => axi_wr_data,
			o_wr_cmd_done    => axi_wr_cmd_done,
			i_rd_cmd_valid   => axi_rd_cmd_valid,
			i_rd_cmd_address => axi_rd_address,
			i_rd_cmd_length  => axi_rd_length,
			o_rd_cmd_done    => axi_rd_cmd_done,
			o_rd_cmd_data    => axi_rd_data
		);

	SLAVE_HP : axi_vip_slave
		port map(
			aclk          => tb_ACLK_dma,
			aresetn       => tb_ARESETN,
			s_axi_awid    => S_AXI_HP0_AWID,
			s_axi_awaddr  => S_AXI_HP0_awaddr,
			s_axi_awlen   => S_AXI_HP0_awlen,
			s_axi_awsize  => S_AXI_HP0_awsize,
			s_axi_awburst => S_AXI_HP0_awburst,
			s_axi_awlock  => S_AXI_HP0_awlock,
			s_axi_awcache => S_AXI_HP0_awcache,
			s_axi_awprot  => S_AXI_HP0_awprot,
			s_axi_awvalid => S_AXI_HP0_awvalid,
			s_axi_awready => S_AXI_HP0_awready,
			s_axi_wdata   => S_AXI_HP0_wdata,
			s_axi_wstrb   => S_AXI_HP0_wstrb,
			s_axi_wlast   => S_AXI_HP0_wlast,
			s_axi_wvalid  => S_AXI_HP0_wvalid,
			s_axi_wready  => S_AXI_HP0_wready,
			s_axi_bid     => S_AXI_HP0_bid,
			s_axi_bresp   => S_AXI_HP0_bresp,
			s_axi_bvalid  => S_AXI_HP0_bvalid,
			s_axi_bready  => S_AXI_HP0_bready,
			s_axi_arid    => S_AXI_HP0_arid,
			s_axi_araddr  => S_AXI_HP0_araddr,
			s_axi_arlen   => S_AXI_HP0_arlen,
			s_axi_arsize  => S_AXI_HP0_arsize,
			s_axi_arburst => S_AXI_HP0_arburst,
			s_axi_arlock  => S_AXI_HP0_arlock,
			s_axi_arcache => S_AXI_HP0_arcache,
			s_axi_arprot  => S_AXI_HP0_arprot,
			s_axi_arvalid => S_AXI_HP0_arvalid,
			s_axi_arready => S_AXI_HP0_arready,
			s_axi_rid     => S_AXI_HP0_rid,
			s_axi_rdata   => S_AXI_HP0_rdata,
			s_axi_rresp   => S_AXI_HP0_rresp,
			s_axi_rlast   => S_AXI_HP0_rlast,
			s_axi_rvalid  => S_AXI_HP0_rvalid,
			s_axi_rready  => S_AXI_HP0_rready
		);

	SLAVE_HP_commander : axi_vip_slave_commander;

	axi_2_amba_gen : if (CONTROL_INTERFACE_TYPE = "AMBA_AHB") generate
		axi_2_amba_i : axi_2_amba
			port map(
				AXI4_araddr     => M_AXI_2_araddr,
				AXI4_arburst    => M_AXI_2_arburst,
				AXI4_arcache    => M_AXI_2_arcache,
				AXI4_arid       => M_AXI_2_arid,
				AXI4_arlen      => M_AXI_2_arlen,
				AXI4_arlock     => M_AXI_2_arlock,
				AXI4_arprot     => M_AXI_2_arprot,
				AXI4_arready    => M_AXI_2_arready,
				AXI4_arsize     => M_AXI_2_arsize,
				AXI4_arvalid    => M_AXI_2_arvalid,
				AXI4_awaddr     => M_AXI_2_awaddr,
				AXI4_awburst    => M_AXI_2_awburst,
				AXI4_awcache    => M_AXI_2_awcache,
				AXI4_awid       => M_AXI_2_awid,
				AXI4_awlen      => M_AXI_2_awlen,
				AXI4_awlock     => M_AXI_2_awlock,
				AXI4_awprot     => M_AXI_2_awprot,
				AXI4_awready    => M_AXI_2_AWREADY,
				AXI4_awsize     => M_AXI_2_awsize,
				AXI4_awvalid    => M_AXI_2_awvalid,
				AXI4_bid        => M_AXI_2_bid,
				AXI4_bready     => M_AXI_2_bready,
				AXI4_bresp      => M_AXI_2_bresp,
				AXI4_bvalid     => M_AXI_2_bvalid,
				AXI4_rdata      => M_AXI_2_rdata,
				AXI4_rid        => M_AXI_2_rid,
				AXI4_rlast      => M_AXI_2_rlast,
				AXI4_rready     => M_AXI_2_rready,
				AXI4_rresp      => M_AXI_2_rresp,
				AXI4_rvalid     => M_AXI_2_rvalid,
				AXI4_wdata      => M_AXI_2_wdata,
				AXI4_wlast      => M_AXI_2_wlast,
				AXI4_wready     => M_AXI_2_wready,
				AXI4_wstrb      => M_AXI_2_wstrb,
				AXI4_wvalid     => M_AXI_2_wvalid,
				M_AHB_haddr     => S_AHB_HADDR,
				M_AHB_hburst    => S_AHB_HBURST,
				M_AHB_hmastlock => open,
				M_AHB_hprot     => S_AHB_HPROT,
				M_AHB_hrdata    => S_AHB_HRDATA,
				M_AHB_hready    => S_AHB_HREADY,
				M_AHB_hresp     => S_AHB_HRESP(0),
				M_AHB_hsize     => S_AHB_HSIZE,
				M_AHB_htrans    => S_AHB_HTRANS,
				M_AHB_hwdata    => S_AHB_HWDATA,
				M_AHB_hwrite    => S_AHB_HWRITE,
				s_axi_aclk      => tb_ACLK,
				s_axi_aresetn   => tb_ARESETN
			);

		S_AHB_HRESP(1) <= '0';

		M_AXI_2_araddr    <= M_AXI_GP0_araddr;
		M_AXI_2_arburst   <= M_AXI_GP0_arburst;
		M_AXI_2_arcache   <= M_AXI_GP0_arcache;
		M_AXI_2_arid      <= M_AXI_GP0_arid;
		M_AXI_2_arlen     <= M_AXI_GP0_arlen;
		M_AXI_2_arlock    <= M_AXI_GP0_arlock(0);
		M_AXI_2_arprot    <= M_AXI_GP0_arprot;
		M_AXI_GP0_arready <= M_AXI_2_arready;
		M_AXI_2_arsize    <= M_AXI_GP0_arsize;
		M_AXI_2_arvalid   <= M_AXI_GP0_arvalid;
		M_AXI_2_awaddr    <= M_AXI_GP0_awaddr;
		M_AXI_2_awburst   <= M_AXI_GP0_awburst;
		M_AXI_2_awcache   <= M_AXI_GP0_awcache;
		M_AXI_2_awid      <= M_AXI_GP0_awid;
		M_AXI_2_awlen     <= M_AXI_GP0_awlen;
		M_AXI_2_awlock    <= M_AXI_GP0_awlock(0);
		M_AXI_2_awprot    <= M_AXI_GP0_awprot;
		M_AXI_GP0_awready <= M_AXI_2_AWREADY;
		M_AXI_2_awsize    <= M_AXI_GP0_awsize;
		M_AXI_2_awvalid   <= M_AXI_GP0_awvalid;
		M_AXI_GP0_bid     <= M_AXI_2_bid;
		M_AXI_2_bready    <= M_AXI_GP0_bready;
		M_AXI_GP0_bresp   <= M_AXI_2_bresp;
		M_AXI_GP0_bvalid  <= M_AXI_2_bvalid;
		M_AXI_GP0_rdata   <= M_AXI_2_rdata;
		M_AXI_GP0_rid     <= M_AXI_2_rid;
		M_AXI_GP0_rlast   <= M_AXI_2_rlast;
		M_AXI_2_rready    <= M_AXI_GP0_rready;
		M_AXI_GP0_rresp   <= M_AXI_2_rresp;
		M_AXI_GP0_rvalid  <= M_AXI_2_rvalid;
		M_AXI_2_wdata     <= M_AXI_GP0_wdata;
		M_AXI_2_wlast     <= M_AXI_GP0_wlast;
		M_AXI_GP0_wready  <= M_AXI_2_wready;
		M_AXI_2_wstrb     <= M_AXI_GP0_wstrb;
		M_AXI_2_wvalid    <= M_AXI_GP0_wvalid;

	end generate;

	axi_interfaces_gen : if (CONTROL_INTERFACE_TYPE = "AXI4") generate
		M_AXI_1_araddr    <= M_AXI_GP0_araddr;
		M_AXI_1_arburst   <= M_AXI_GP0_arburst;
		M_AXI_1_arcache   <= M_AXI_GP0_arcache;
		M_AXI_1_arid      <= M_AXI_GP0_arid;
		M_AXI_1_arlen     <= M_AXI_GP0_arlen;
		M_AXI_1_arlock    <= M_AXI_GP0_arlock(0);
		M_AXI_1_arprot    <= M_AXI_GP0_arprot;
		M_AXI_GP0_arready <= M_AXI_1_arready;
		M_AXI_1_arsize    <= M_AXI_GP0_arsize;
		M_AXI_1_arvalid   <= M_AXI_GP0_arvalid;
		M_AXI_1_awaddr    <= M_AXI_GP0_awaddr;
		M_AXI_1_awburst   <= M_AXI_GP0_awburst;
		M_AXI_1_awcache   <= M_AXI_GP0_awcache;
		M_AXI_1_awid      <= M_AXI_GP0_awid;
		M_AXI_1_awlen     <= M_AXI_GP0_awlen;
		M_AXI_1_awlock    <= M_AXI_GP0_awlock(0);
		M_AXI_1_awprot    <= M_AXI_GP0_awprot;
		M_AXI_GP0_awready <= M_AXI_1_AWREADY;
		M_AXI_1_awsize    <= M_AXI_GP0_awsize;
		M_AXI_1_awvalid   <= M_AXI_GP0_awvalid;
		M_AXI_GP0_bid     <= M_AXI_1_bid;
		M_AXI_1_bready    <= M_AXI_GP0_bready;
		M_AXI_GP0_bresp   <= M_AXI_1_bresp;
		M_AXI_GP0_bvalid  <= M_AXI_1_bvalid;
		M_AXI_GP0_rdata   <= M_AXI_1_rdata;
		M_AXI_GP0_rid     <= M_AXI_1_rid;
		M_AXI_GP0_rlast   <= M_AXI_1_rlast;
		M_AXI_1_rready    <= M_AXI_GP0_rready;
		M_AXI_GP0_rresp   <= M_AXI_1_rresp;
		M_AXI_GP0_rvalid  <= M_AXI_1_rvalid;
		M_AXI_1_wdata     <= M_AXI_GP0_wdata;
		M_AXI_1_wlast     <= M_AXI_GP0_wlast;
		M_AXI_GP0_wready  <= M_AXI_1_wready;
		M_AXI_1_wstrb     <= M_AXI_GP0_wstrb;
		M_AXI_1_wvalid    <= M_AXI_GP0_wvalid;
	end generate;

	axi_interface_gen : if (DMA_INTERFACE_TYPE = "AXI4") generate
		S_AXI_HP0_awid               <= M_AXI_DMA_FROM_DYPLO_AWID;
		S_AXI_HP0_awlen              <= M_AXI_DMA_FROM_DYPLO_AWLEN;
		S_AXI_HP0_awsize             <= M_AXI_DMA_FROM_DYPLO_AWSIZE;
		S_AXI_HP0_awburst            <= M_AXI_DMA_FROM_DYPLO_AWBURST;
		S_AXI_HP0_awcache            <= M_AXI_DMA_FROM_DYPLO_AWCACHE;
		S_AXI_HP0_awaddr             <= M_AXI_DMA_FROM_DYPLO_AWADDR;
		S_AXI_HP0_awprot             <= M_AXI_DMA_FROM_DYPLO_AWPROT;
		S_AXI_HP0_awvalid            <= M_AXI_DMA_FROM_DYPLO_AWVALID;
		M_AXI_DMA_FROM_DYPLO_AWREADY <= S_AXI_HP0_awready;
		S_AXI_HP0_awlock             <= M_AXI_DMA_FROM_DYPLO_AWLOCK;
		S_AXI_HP0_wdata              <= M_AXI_DMA_FROM_DYPLO_WDATA;
		S_AXI_HP0_wstrb              <= M_AXI_DMA_FROM_DYPLO_WSTRB;
		S_AXI_HP0_wlast              <= M_AXI_DMA_FROM_DYPLO_WLAST;
		S_AXI_HP0_wvalid             <= M_AXI_DMA_FROM_DYPLO_WVALID;
		M_AXI_DMA_FROM_DYPLO_WREADY  <= S_AXI_HP0_wready;
		S_AXI_HP0_bready             <= M_AXI_DMA_FROM_DYPLO_BREADY;
		M_AXI_DMA_FROM_DYPLO_BID     <= S_AXI_HP0_bid;
		M_AXI_DMA_FROM_DYPLO_BRESP   <= S_AXI_HP0_bresp;
		M_AXI_DMA_FROM_DYPLO_BVALID  <= S_AXI_HP0_bvalid;
		S_AXI_HP0_arid               <= M_AXI_DMA_FROM_DYPLO_ARID;
		S_AXI_HP0_arlen              <= M_AXI_DMA_FROM_DYPLO_ARLEN;
		S_AXI_HP0_arsize             <= M_AXI_DMA_FROM_DYPLO_ARSIZE;
		S_AXI_HP0_arburst            <= M_AXI_DMA_FROM_DYPLO_ARBURST;
		S_AXI_HP0_arprot             <= M_AXI_DMA_FROM_DYPLO_ARPROT;
		S_AXI_HP0_arcache            <= M_AXI_DMA_FROM_DYPLO_ARCACHE;
		S_AXI_HP0_arvalid            <= M_AXI_DMA_FROM_DYPLO_ARVALID;
		S_AXI_HP0_araddr             <= M_AXI_DMA_FROM_DYPLO_ARADDR;
		S_AXI_HP0_arlock             <= M_AXI_DMA_FROM_DYPLO_ARLOCK;
		M_AXI_DMA_FROM_DYPLO_ARREADY <= S_AXI_HP0_arready;
		S_AXI_HP0_rready             <= M_AXI_DMA_FROM_DYPLO_RREADY;
		M_AXI_DMA_FROM_DYPLO_RID     <= S_AXI_HP0_rid;
		M_AXI_DMA_FROM_DYPLO_RDATA   <= S_AXI_HP0_rdata;
		M_AXI_DMA_FROM_DYPLO_RRESP   <= S_AXI_HP0_rresp;
		M_AXI_DMA_FROM_DYPLO_RVALID  <= S_AXI_HP0_rvalid;
		M_AXI_DMA_FROM_DYPLO_RLAST   <= S_AXI_HP0_rlast;
	end generate;

	ahb_2_axi_gen : if (DMA_INTERFACE_TYPE = "AMBA_AHB") generate
		ahb_2_axi_i : ahblite_axi_bridge_0
			port map(
				s_ahb_hclk       => tb_ACLK_dma,
				s_ahb_hresetn    => tb_ARESETN_dma,
				s_ahb_hsel       => M_AHB_DMA_FROM_DYPLO_HSEL,
				s_ahb_haddr      => M_AHB_DMA_FROM_DYPLO_HADDR(31 downto 0),
				s_ahb_hprot      => M_AHB_DMA_FROM_DYPLO_HPROT,
				s_ahb_htrans     => M_AHB_DMA_FROM_DYPLO_HTRANS,
				s_ahb_hsize      => M_AHB_DMA_FROM_DYPLO_HSIZE,
				s_ahb_hwrite     => M_AHB_DMA_FROM_DYPLO_HWRITE,
				s_ahb_hburst     => M_AHB_DMA_FROM_DYPLO_HBURST,
				s_ahb_hwdata     => M_AHB_DMA_FROM_DYPLO_HWDATA,
				s_ahb_hready_in  => M_AHB_DMA_FROM_DYPLO_HREADY,
				s_ahb_hready_out => M_AHB_DMA_FROM_DYPLO_HREADY,
				s_ahb_hrdata     => M_AHB_DMA_FROM_DYPLO_HRDATA,
				s_ahb_hresp      => M_AHB_DMA_FROM_DYPLO_HRESP(1),
				m_axi_awready    => S_AXI_HP0_awready,
				m_axi_wready     => S_AXI_HP0_wready,
				m_axi_bid        => S_AXI_HP0_bid,
				m_axi_bresp      => S_AXI_HP0_bresp,
				m_axi_bvalid     => S_AXI_HP0_bvalid,
				m_axi_arready    => S_AXI_HP0_arready,
				m_axi_rid        => S_AXI_HP0_rid,
				m_axi_rdata      => S_AXI_HP0_rdata,
				m_axi_rresp      => S_AXI_HP0_rresp,
				m_axi_rvalid     => S_AXI_HP0_rvalid,
				m_axi_rlast      => S_AXI_HP0_rlast,
				m_axi_awid       => S_AXI_HP0_awid,
				m_axi_awlen      => S_AXI_HP0_awlen,
				m_axi_awsize     => S_AXI_HP0_awsize,
				m_axi_awburst    => S_AXI_HP0_awburst,
				m_axi_awcache    => S_AXI_HP0_awcache,
				m_axi_awaddr     => S_AXI_HP0_awaddr(31 downto 0),
				m_axi_awprot     => S_AXI_HP0_awprot,
				m_axi_awvalid    => S_AXI_HP0_awvalid,
				m_axi_awlock     => S_AXI_HP0_awlock(0),
				m_axi_wdata      => S_AXI_HP0_wdata,
				m_axi_wstrb      => S_AXI_HP0_wstrb,
				m_axi_wlast      => S_AXI_HP0_wlast,
				m_axi_wvalid     => S_AXI_HP0_wvalid,
				m_axi_bready     => S_AXI_HP0_bready,
				m_axi_arid       => S_AXI_HP0_arid,
				m_axi_arlen      => S_AXI_HP0_arlen,
				m_axi_arsize     => S_AXI_HP0_arsize,
				m_axi_arburst    => S_AXI_HP0_arburst,
				m_axi_arprot     => S_AXI_HP0_arprot,
				m_axi_arcache    => S_AXI_HP0_arcache,
				m_axi_arvalid    => S_AXI_HP0_arvalid,
				m_axi_araddr     => S_AXI_HP0_araddr(31 downto 0),
				m_axi_arlock     => S_AXI_HP0_arlock(0),
				m_axi_rready     => S_AXI_HP0_rready
			);
	end generate;

end;
