LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

LIBRARY topic;
USE topic.topic_pkg.ALL;

ENTITY axil_pwm_fan_controller IS
   GENERIC(
      p_number_of_fans        : positive RANGE 1 TO 8 := 1;
      p_clkin_freq            : positive              := 100_000_000; -- Frequency of i_clk in Hz
      p_pwm_freq              : positive              := 30; -- Frequency of PWM signal in Hz
      p_pulse_stretch_timeout : positive              := 100_000_000; -- Timeout for pulse_stretching (if not tacho pulses are received)
      p_axi_data_width        : integer               := 32;
      p_axi_addr_width        : integer               := 32
   );
   PORT(
      -- ---------- AXI-4 Lite interface -------
      S_AXI_ACLK    : IN  std_logic;
      S_AXI_ARESETN : IN  std_logic;
      S_AXI_AWADDR  : IN  std_logic_vector(p_axi_addr_width - 1 DOWNTO 0);
      S_AXI_AWVALID : IN  std_logic;
      S_AXI_AWREADY : OUT std_logic;
      S_AXI_WDATA   : IN  std_logic_vector(p_axi_data_width - 1 DOWNTO 0);
      S_AXI_WSTRB   : IN  std_logic_vector(3 DOWNTO 0);
      S_AXI_WVALID  : IN  std_logic;
      S_AXI_WREADY  : OUT std_logic;
      S_AXI_BRESP   : OUT std_logic_vector(1 DOWNTO 0);
      S_AXI_BVALID  : OUT std_logic;
      S_AXI_BREADY  : IN  std_logic;
      S_AXI_ARADDR  : IN  std_logic_vector(p_axi_addr_width - 1 DOWNTO 0);
      S_AXI_ARVALID : IN  std_logic;
      S_AXI_ARREADY : OUT std_logic;
      S_AXI_RDATA   : OUT std_logic_vector(p_axi_data_width - 1 DOWNTO 0);
      S_AXI_RRESP   : OUT std_logic_vector(1 DOWNTO 0);
      S_AXI_RVALID  : OUT std_logic;
      S_AXI_RREADY  : IN  std_logic;
      -- fan interface
      o_fan_pwm     : OUT std_logic_vector(p_number_of_fans - 1 DOWNTO 0);
      i_fan_tacho   : IN  std_logic_vector(p_number_of_fans - 1 DOWNTO 0);
      o_fan_error   : OUT std_logic
   );
END ENTITY;

ARCHITECTURE Behavioral OF axil_pwm_fan_controller IS

   SIGNAL n_regs_from_slave : t_slv32_array(0 TO p_number_of_fans);
   ALIAS n_fan_enables      : std_logic_vector(31 DOWNTO 0) IS n_regs_from_slave(0);
   ALIAS n_req_fan_speeds   : t_slv32_array(0 TO p_number_of_fans - 1) IS n_regs_from_slave(1 TO p_number_of_fans);

   SIGNAL n_regs_to_slave : t_slv32_array(0 TO p_number_of_fans);
   ALIAS n_fan_errors     : std_logic_vector(31 DOWNTO 0) IS n_regs_to_slave(0);
   ALIAS n_act_fan_speeds : t_slv32_array(0 TO p_number_of_fans - 1) IS n_regs_to_slave(1 TO p_number_of_fans);

   SIGNAL n_S_AXI_ARESET : std_logic;
   SIGNAL r_in_reset : std_logic := '1';

BEGIN

   n_S_AXI_ARESET <= (NOT S_AXI_ARESETN) OR r_in_reset;
   o_fan_error    <= '0' WHEN n_fan_errors = (n_fan_errors'RANGE => '0') ELSE '1';
   
   -- Process to check for first write to fan_enables signal
   -- This will release the reset of the pwm_fan_controllers
   PROCESS(S_AXI_ACLK)
      VARIABLE v_fan_enables_prev : std_logic_vector(31 downto 0);
   BEGIN
      IF (rising_edge(S_AXI_ACLK)) THEN
         IF (S_AXI_ARESETN = '0') THEN
            r_in_reset <= '1';
         ELSE
            IF(v_fan_enables_prev /= n_fan_enables) THEN
               r_in_reset <= '0';
            END IF;
         END IF;
         
         v_fan_enables_prev := n_fan_enables;
         
      END IF;
   END PROCESS;   

   fan_controllers : FOR i IN 0 TO p_number_of_fans - 1 GENERATE
   BEGIN
      fan_controller_x : pwm_fan_controller
         GENERIC MAP(
            p_clkin_freq            => p_clkin_freq,
            p_pwm_freq              => p_pwm_freq,
            p_pwm_precision         => 255,
            p_pulse_stretch_timeout => 100_000_000
         )
         PORT MAP(
            i_clk           => S_AXI_ACLK,
            i_rst           => n_S_AXI_ARESET,
            i_fan_enable    => n_fan_enables(i),
            i_fan_req_speed => n_req_fan_speeds(i)(7 DOWNTO 0),
            o_fan_act_speed => n_act_fan_speeds(i),
            o_fan_pwm       => o_fan_pwm(i),
            i_fan_tacho     => i_fan_tacho(i),
            o_fan_error     => n_fan_errors(i)
         );
   END GENERATE;

   AXI_REG : axil_slave_to_reg
      GENERIC MAP(
         PNiRegs            => 1 + p_number_of_fans,
         PNoRegs            => 1 + p_number_of_fans,
         -- AXI bus related configuration parameters
         C_S_AXI_DATA_WIDTH => p_axi_data_width,
         C_S_AXI_ADDR_WIDTH => p_axi_addr_width
      )
      PORT MAP(
         -- AXI_lite: Control interface
         S_AXI_ACLK    => S_AXI_ACLK,
         S_AXI_ARESETN => S_AXI_ARESETN,
         S_AXI_AWADDR  => S_AXI_AWADDR,
         S_AXI_AWVALID => S_AXI_AWVALID,
         S_AXI_AWREADY => S_AXI_AWREADY,
         S_AXI_WDATA   => S_AXI_WDATA,
         S_AXI_WSTRB   => S_AXI_WSTRB,
         S_AXI_WVALID  => S_AXI_WVALID,
         S_AXI_WREADY  => S_AXI_WREADY,
         S_AXI_BRESP   => S_AXI_BRESP,
         S_AXI_BVALID  => S_AXI_BVALID,
         S_AXI_BREADY  => S_AXI_BREADY,
         S_AXI_ARADDR  => S_AXI_ARADDR,
         S_AXI_ARVALID => S_AXI_ARVALID,
         S_AXI_ARREADY => S_AXI_ARREADY,
         S_AXI_RDATA   => S_AXI_RDATA,
         S_AXI_RRESP   => S_AXI_RRESP,
         S_AXI_RVALID  => S_AXI_RVALID,
         S_AXI_RREADY  => S_AXI_RREADY,
         iData         => n_regs_to_slave,
         oData         => n_regs_from_slave
      );

END Behavioral;
