library ieee;
use ieee.std_logic_1164.ALL;
use ieee.numeric_std.ALL;

library topic;
use topic.topic_pkg.all;

entity axil_slave_to_reg is
   generic(
      PNiRegs            : integer := 8;
      PNoRegs            : integer := 8;
      -- AXI bus related configuration parameters
      C_S_AXI_DATA_WIDTH : integer := 32;
      C_S_AXI_ADDR_WIDTH : integer := 32
   );
   port(
      ------------ AXI-4 Lite Interface -------
      S_AXI_ACLK    : in  std_logic;
      S_AXI_ARESETN : in  std_logic;
      S_AXI_AWADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH - 1 downto 0);
      S_AXI_AWVALID : in  std_logic;
      S_AXI_AWREADY : out std_logic;
      S_AXI_WDATA   : in  std_logic_vector(C_S_AXI_DATA_WIDTH - 1 downto 0);
      S_AXI_WSTRB   : in  std_logic_vector(3 downto 0);
      S_AXI_WVALID  : in  std_logic;
      S_AXI_WREADY  : out std_logic;
      S_AXI_BRESP   : out std_logic_vector(1 downto 0);
      S_AXI_BVALID  : out std_logic;
      S_AXI_BREADY  : in  std_logic;
      S_AXI_ARADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH - 1 downto 0);
      S_AXI_ARVALID : in  std_logic;
      S_AXI_ARREADY : out std_logic;
      S_AXI_RDATA   : out std_logic_vector(C_S_AXI_DATA_WIDTH - 1 downto 0);
      S_AXI_RRESP   : out std_logic_vector(1 downto 0);
      S_AXI_RVALID  : out std_logic;
      S_AXI_RREADY  : in  std_logic;
      ------------ ADC Data input -------------- 
      iData         : in  t_slv32_array(0 to PNiRegs - 1);
      oData         : out t_slv32_array(0 to PNoRegs - 1)
   );
end entity;

architecture Behavioral of axil_slave_to_reg is
   constant cAddrtotal : integer := (PNiRegs + PNoRegs);

   signal written_addr : integer range 0 to cAddrtotal;
   signal read_addr    : integer range 0 to cAddrtotal;

   -- Write Handling State machine
   type sm_write_action_type is (WAIT_FOR_AWVALID, WAIT_FOR_WVALID, WAIT_FOR_BREADY);
   signal sm_write_action : sm_write_action_type;

   -- Read Handling State machine
   type sm_read_action_type is (WAIT_FOR_ARVALID, SET_RVALID_AND_RDATA, WAIT_FOR_RREADY);
   signal sm_read_action : sm_read_action_type;

   signal rOData : t_slv32_array(0 to PNoRegs - 1);
   signal rIData : t_slv32_array(0 to PNiRegs - 1);

begin

   -----------------------------------------------------------------------------
   -- AXI WRITE BUS HANDLING STATEMACHINE                                     --
   -----------------------------------------------------------------------------
   write_action : process(S_AXI_ACLK) is
   begin
      if (rising_edge(S_AXI_ACLK)) then
         if (S_AXI_ARESETN = '0') then
            written_addr    <= 0;
            S_AXI_AWREADY   <= '0';
            S_AXI_WREADY    <= '0';
            S_AXI_BRESP     <= (others => '0');
            S_AXI_BVALID    <= '0';
            sm_write_action <= WAIT_FOR_AWVALID;

         else
            case (sm_write_action) is
               --------------------------------------------------------------
               when WAIT_FOR_AWVALID =>
                  S_AXI_BVALID <= '0';

                  if (S_AXI_AWVALID = '1') then
                     written_addr    <= to_integer(unsigned(S_AXI_AWADDR(C_S_AXI_ADDR_WIDTH - 1 downto 2)));
                     S_AXI_AWREADY   <= '1';
                     -- next state
                     sm_write_action <= WAIT_FOR_WVALID;
                  end if;
               --------------------------------------------------------------  
               when WAIT_FOR_WVALID =>
                  S_AXI_AWREADY <= '0';

                  if (S_AXI_WVALID = '1') then
                     --            checks if address is valid. else do nothing
                     if (written_addr < (PNoRegs)) THEN
                        rOData(written_addr) <= S_AXI_WDATA;
                     end if;

                     S_AXI_WREADY    <= '1';
                     S_AXI_BVALID    <= '1';
                     -- next state
                     sm_write_action <= WAIT_FOR_BREADY;
                  end if;
               --------------------------------------------------------------
               when WAIT_FOR_BREADY =>
                  S_AXI_WREADY <= '0';

                  if (S_AXI_BREADY = '1') then
                     S_AXI_BVALID <= '0';

                     -- next state
                     sm_write_action <= WAIT_FOR_AWVALID;
                  end if;
            --------------------------------------------------------------		  
            end case;
         end if;
      end if;
   end process write_action;

   -----------------------------------------------------------------------------
   -- AXI READ BUS HANDLING STATEMACHINE                                      --
   -----------------------------------------------------------------------------
   read_action : process(S_AXI_ACLK)
   begin
      if (rising_edge(S_AXI_ACLK)) then
         if (S_AXI_ARESETN = '0') then
            read_addr      <= 0;
            S_AXI_RDATA    <= (others => '0');
            S_AXI_RRESP    <= "00";
            S_AXI_ARREADY  <= '0';
            S_AXI_RVALID   <= '0';
            sm_read_action <= WAIT_FOR_ARVALID;
         else
            case (sm_read_action) is
               --------------------------------------------------------------
               when WAIT_FOR_ARVALID =>
                  if (S_AXI_ARVALID = '1') then
                     read_addr      <= to_integer(unsigned(S_AXI_ARADDR(C_S_AXI_ADDR_WIDTH - 1 downto 2)));
                     S_AXI_ARREADY  <= '1';
                     -- next state
                     sm_read_action <= SET_RVALID_AND_RDATA;
                  end if;
               --------------------------------------------------------------
               when SET_RVALID_AND_RDATA =>
                  if (read_addr < (cAddrtotal)) THEN
                     --            checks if address is valid. else do nothing
                     --               if address is valid, check if address is write / read 
                     --               and put the right register on the output
                     if (read_addr < PNoRegs) THEN
                        S_AXI_RDATA <= rOData(read_addr);
                     else
                        S_AXI_RDATA <= rIData(read_addr - PNoRegs);
                     end if;
                  else
                     S_AXI_RDATA <= (others => '0');
                  end if;

                  S_AXI_ARREADY  <= '0';
                  S_AXI_RVALID   <= '1';
                  -- next state  
                  sm_read_action <= WAIT_FOR_RREADY;
               --------------------------------------------------------------
               when WAIT_FOR_RREADY =>
                  if (S_AXI_RREADY = '1') then
                     S_AXI_RVALID   <= '0';
                     sm_read_action <= WAIT_FOR_ARVALID;
                  end if;
            --------------------------------------------------------------       
            end case;
         end if;
      end if;
   end process read_action;

   --sets the output of the registers on the outputs
   inp : process(S_AXI_ACLK)
   begin
      if (rising_edge(S_AXI_ACLK)) then
         for I in 0 to (PNoRegs - 1) loop
            oData(I) <= rOData(I);
         end loop;
      end if;
   end process;

   --sets the input in the registers 
   outp : process(S_AXI_ACLK)
   begin
      if (rising_edge(S_AXI_ACLK)) then
         for N in 0 to (PNiRegs - 1) loop
            rIData(N) <= iData(N);
         end loop;
      end if;
   end process;

end Behavioral;
